@@ -2,7 +2,7 @@
 #define _EEG_h_
 /* EEG.h
  *
- * Copyright (C) 2011-2012,2014,2015,2017,2018 Paul Boersma
+ * Copyright (C) 2011-2012,2014-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@ autoEEG EEG_readFromBdfFile (MelderFile file);
 autoEEG EEGs_concatenate (OrderedOf<structEEG>* me);
 
 void EEG_init (EEG me, double tmin, double tmax);
-integer EEG_getChannelNumber (EEG me, const char32 *channelName);
-void EEG_setChannelName (EEG me, integer channelNumber, const char32 *a_name);
+integer EEG_getChannelNumber (EEG me, conststring32 channelName);
+void EEG_setChannelName (EEG me, integer channelNumber, conststring32 newName);
 static inline integer EEG_getNumberOfCapElectrodes (EEG me) {
 	return (my numberOfChannels - 1) & ~ 15L;   // BUG
 }
@@ -42,20 +42,20 @@ static inline integer EEG_getNumberOfExtraSensors (EEG me) {
 static inline integer EEG_getNumberOfExternalElectrodes (EEG me) {
 	return my numberOfChannels - EEG_getNumberOfCapElectrodes (me) - EEG_getNumberOfExtraSensors (me);
 }
-void EEG_setExternalElectrodeNames (EEG me, const char32 *nameExg1, const char32 *nameExg2, const char32 *nameExg3, const char32 *nameExg4,
-	const char32 *nameExg5, const char32 *nameExg6, const char32 *nameExg7, const char32 *nameExg8);
+void EEG_setExternalElectrodeNames (EEG me, conststring32 nameExg1, conststring32 nameExg2, conststring32 nameExg3, conststring32 nameExg4,
+	conststring32 nameExg5, conststring32 nameExg6, conststring32 nameExg7, conststring32 nameExg8);
 void EEG_detrend (EEG me);
 void EEG_filter (EEG me, double lowFrequency, double lowWidth, double highFrequency, double highWidth, bool doNotch50Hz);
-void EEG_subtractReference (EEG me, const char32 *channelNumber1, const char32 *channelNumber2);
+void EEG_subtractReference (EEG me, conststring32 channelName1, conststring32 channelName2);
 void EEG_subtractMeanChannel (EEG me, integer fromChannel, integer toChannel);
 void EEG_setChannelToZero (EEG me, integer channelNumber);
-void EEG_setChannelToZero (EEG me, const char32 *channelName);
-void EEG_removeTriggers (EEG me, kMelder_string which, const char32 *criterion);
+void EEG_setChannelToZero (EEG me, conststring32 channelName);
+void EEG_removeTriggers (EEG me, kMelder_string which, conststring32 criterion);
 autoEEG EEG_extractChannel (EEG me, integer channelNumber);
-autoEEG EEG_extractChannel (EEG me, const char32 *channelName);
-autoEEG EEG_extractChannels (EEG me, numvec channelNumbers);
+autoEEG EEG_extractChannel (EEG me, conststring32 channelName);
+autoEEG EEG_extractChannels (EEG me, constVEC channelNumbers);
 void EEG_removeChannel (EEG me, integer channelNumber);
-void EEG_removeChannel (EEG me, const char32 *channelName);
+void EEG_removeChannel (EEG me, conststring32 channelName);
 static inline autoSound EEG_extractSound (EEG me) { return Data_copy (my sound.get()); }
 static inline autoTextGrid EEG_extractTextGrid (EEG me) { return Data_copy (my textgrid.get()); }
 autoEEG EEG_extractPart (EEG me, double tmin, double tmax, bool preserveTimes);
