@@ -1,6 +1,6 @@
 /* ERPWindow.cpp
  *
- * Copyright (C) 2012,2013,2014,2015,2016,2017 Paul Boersma
+ * Copyright (C) 2012-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -151,14 +151,14 @@ static BiosemiLocationData biosemiCapCoordinates32 [1+32] =
 
 void ERP_drawScalp_garnish (Graphics graphics, double vmin, double vmax, enum kGraphics_colourScale colourScale) {
 	integer n = 201;
-	autonummat legend (n, 2, kTensorInitializationType::RAW);
+	autoMAT legend (n, 2, kTensorInitializationType::RAW);
 	for (integer irow = 1; irow <= n; irow ++) {
 		for (integer icol = 1; icol <= 2; icol ++) {
 			legend [irow] [icol] = (irow - 1) / (n - 1.0);
 		}
 	}
 	Graphics_setColourScale (graphics, colourScale);
-	Graphics_image (graphics, legend.at, 1, 2, 0.85, 0.98, 1, n, -0.8, +0.8, 0.0, 1.0);
+	Graphics_image (graphics, legend.all(), 0.85, 0.98, -0.8, +0.8, 0.0, 1.0);
 	Graphics_setColourScale (graphics, kGraphics_colourScale::GREY);
 	Graphics_rectangle (graphics, 0.85, 0.98, -0.8, +0.8);
 	Graphics_setTextAlignment (graphics, Graphics_RIGHT, Graphics_TOP);
@@ -174,8 +174,8 @@ void ERP_drawScalp (ERP me, Graphics graphics, double tmin, double tmax, double
 	//Graphics_fillRectangle (graphics, -1.1, 1.1, -1.01, 1.19);
 	//Graphics_setColour (graphics, Graphics_BLACK);
 	integer numberOfDrawableChannels =
-			my ny >= 64 && Melder_equ (my channelNames [64], U"O2") ? 64 :
-			my ny >= 32 && Melder_equ (my channelNames [32], U"Cz") ? 32 :
+			my ny >= 64 && Melder_equ (my channelNames [64].get(), U"O2") ? 64 :
+			my ny >= 32 && Melder_equ (my channelNames [32].get(), U"Cz") ? 32 :
 			0;
 	BiosemiLocationData *biosemiLocationData = numberOfDrawableChannels == 64 ? biosemiCapCoordinates64 : numberOfDrawableChannels == 32 ? biosemiCapCoordinates32 : 0;
 	for (integer ichan = 1; ichan <= numberOfDrawableChannels; ichan ++) {
@@ -189,13 +189,13 @@ void ERP_drawScalp (ERP me, Graphics graphics, double tmin, double tmax, double
 	}
 	integer n = 201;
 	double d = 2.0 / (n - 1);
-	autonumvec mean (numberOfDrawableChannels, kTensorInitializationType::RAW);
+	autoVEC mean (numberOfDrawableChannels, kTensorInitializationType::RAW);
 	for (integer ichan = 1; ichan <= numberOfDrawableChannels; ichan ++) {
 		mean [ichan] = tmin == tmax ?
 				Sampled_getValueAtX (me, tmin, ichan, 0, true) :
 				Vector_getMean (me, tmin, tmax, ichan);
 	}
-	autonummat image (n, n, kTensorInitializationType::RAW);
+	autoMAT image (n, n, kTensorInitializationType::RAW);
 	for (integer irow = 1; irow <= n; irow ++) {
 		double y = -1.0 + (irow - 1) * d;
 		for (integer icol = 1; icol <= n; icol ++) {
@@ -232,7 +232,7 @@ void ERP_drawScalp (ERP me, Graphics graphics, double tmin, double tmax, double
 			}
 		}
 	}
-	Graphics_image (graphics, image.at, 1, n, -1.0-0.5/n, 1.0+0.5/n, 1, n, -1.0-0.5/n, 1.0+0.5/n, vmin, vmax);
+	Graphics_image (graphics, image.all(), -1.0-0.5/n, 1.0+0.5/n, -1.0-0.5/n, 1.0+0.5/n, vmin, vmax);
 	Graphics_setColourScale (graphics, kGraphics_colourScale::GREY);
 	Graphics_setLineWidth (graphics, 2.0);
 	/*
@@ -277,8 +277,8 @@ void structERPWindow :: v_drawSelectionViewer () {
 	Graphics_fillRectangle (our graphics.get(), -1.1, 1.1, -1.01, 1.19);
 	Graphics_setColour (our graphics.get(), Graphics_BLACK);
 	integer numberOfDrawableChannels =
-			erp -> ny >= 64 && Melder_equ (erp -> channelNames [64], U"O2") ? 64 :
-			erp -> ny >= 32 && Melder_equ (erp -> channelNames [32], U"Cz") ? 32 :
+			erp -> ny >= 64 && Melder_equ (erp -> channelNames [64].get(), U"O2") ? 64 :
+			erp -> ny >= 32 && Melder_equ (erp -> channelNames [32].get(), U"Cz") ? 32 :
 			0;
 	BiosemiLocationData *biosemiLocationData = numberOfDrawableChannels == 64 ? biosemiCapCoordinates64 : numberOfDrawableChannels == 32 ? biosemiCapCoordinates32 : 0;
 	for (integer ichan = 1; ichan <= numberOfDrawableChannels; ichan ++) {
@@ -292,14 +292,14 @@ void structERPWindow :: v_drawSelectionViewer () {
 	}
 	integer n = 201;
 	double d = 2.0 / (n - 1);
-	autonumvec means (numberOfDrawableChannels, kTensorInitializationType::RAW);
+	autoVEC means (numberOfDrawableChannels, kTensorInitializationType::RAW);
 	for (integer ichan = 1; ichan <= numberOfDrawableChannels; ichan ++) {
 		means [ichan] =
 			our startSelection == our endSelection ?
 				Sampled_getValueAtX (erp, our startSelection, ichan, 0, true) :
 				Vector_getMean (erp, our startSelection, our endSelection, ichan);
 	}
-	autonummat image (n, n, kTensorInitializationType::RAW);
+	autoMAT image (n, n, kTensorInitializationType::RAW);
 	for (integer irow = 1; irow <= n; irow ++) {
 		double y = -1.0 + (irow - 1) * d;
 		for (integer icol = 1; icol <= n; icol ++) {
@@ -357,7 +357,7 @@ void structERPWindow :: v_drawSelectionViewer () {
 		}
 	}
 	Graphics_setColourScale (our graphics.get(), our p_scalp_colourScale);
-	Graphics_image (our graphics.get(), image.at, 1, n, -1.0-0.5/n, 1.0+0.5/n, 1, n, -1.0-0.5/n, 1.0+0.5/n, minimum, maximum);
+	Graphics_image (our graphics.get(), image.all(), -1.0-0.5/n, 1.0+0.5/n, -1.0-0.5/n, 1.0+0.5/n, minimum, maximum);
 	Graphics_setColourScale (our graphics.get(), kGraphics_colourScale::GREY);
 	Graphics_setLineWidth (our graphics.get(), 2.0);
 	/*
@@ -393,7 +393,8 @@ void structERPWindow :: v_drawSelectionViewer () {
 
 OPTIONMENU_ENUM_VARIABLE (kGraphics_colourScale, v_prefs_scalpColourSpace)
 void structERPWindow :: v_prefs_addFields (EditorCommand cmd) {
-	OPTIONMENU_ENUM_FIELD (v_prefs_scalpColourSpace, U"Scalp colour space", kGraphics_colourScale, kGraphics_colourScale::BLUE_TO_RED)
+	OPTIONMENU_ENUM_FIELD (kGraphics_colourScale, v_prefs_scalpColourSpace,
+			U"Scalp colour space", kGraphics_colourScale::BLUE_TO_RED)
 }
 void structERPWindow :: v_prefs_setValues (EditorCommand cmd) {
 	SET_ENUM (v_prefs_scalpColourSpace, kGraphics_colourScale, p_scalp_colourScale)
@@ -403,7 +404,7 @@ void structERPWindow :: v_prefs_getValues (EditorCommand /* cmd */) {
 	FunctionEditor_redraw (this);
 }
 
-autoERPWindow ERPWindow_create (const char32 *title, ERP data) {
+autoERPWindow ERPWindow_create (conststring32 title, ERP data) {
 	Melder_assert (data);
 	try {
 		autoERPWindow me = Thing_new (ERPWindow);
