@@ -2,7 +2,7 @@
 #define _FFNet_h_
 /* FFNet.h
  *
- * Copyright (C) 1997-207 David Weenink
+ * Copyright (C) 1997-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,48 +34,48 @@
 #include "Minimizers.h"
 #include "PatternList.h"
 #include "TableOfReal.h"
+#include "melder.h"
 
 #include "FFNet_def.h"
 
 /* Parameters:
- *  integer nLayers	    : the #layers in the net (exclusive the inputs)
- *    nUnitsInLayer	    : array[0..nLayers] the #units in each layer
- *						  nUnitsInLayer[0] : #inputs
- *						  nUnitsInLayer[nLayers] :#outputs
- *    nWeights		    : the total #weights in the net (inclusive bias)
- *  double *w		    : array[1..nWeights] with the connection strengths
- *    *activity		    : array[1..nNodes] with activities
+ *  integer numberOfLayers	    : the #layers in the net (exclusive the inputs)
+ *    numberOfUnitsInLayer	    : array[1..numberOfLayers] the #units in each layer
+ *						  numberOfUnitsInLayer[numberOfLayers] :#outputs
+ *    numberOfWeights		    : the total #weights in the net (inclusive bias)
+ *  double *w		    : array[1..numberOfWeights] with the connection strengths
+ *    *activity		    : array[1..numberOfNodes] with activities
  *  outputLabels	    : labels belonging to the outputs
  *  BOOKKEEPING:
- *  integer nNodes		    : total #nodes: bias modelled as unit with constant activity)
- *    *isbias		    : array[1..nNodes] set 1 if node is bias else 0
- *    *nodeFirst	    : array[1..nNodes] first node connected to this unit
- *    *nodeLast:	    : array[1..nNodes] last node connected to this unit
- *    *wFirst		    : array[1..nNodes] first index in *w for this unit
- *    *wLast		    : array[1..nNodes] last (inclusive the bias)
+ *  integer numberOfNodes: total #nodes: bias modelled as unit with constant activity)
+ *    *isbias		    : array[1..numberOfNodes] set 1 if node is bias else 0
+ *    *nodeFirst	    : array[1..numberOfNodes] first node connected to this unit
+ *    *nodeLast:	    : array[1..numberOfNodes] last node connected to this unit
+ *    *wFirst		    : array[1..numberOfNodes] first index in *w for this unit
+ *    *wLast		    : array[1..numberOfNodes] last (inclusive the bias)
  *  LEARNING:
- *  int *wSelected		: array[1..nWeights] weights selected for minimization
- *  double *deriv	    : array[1..nNodes] derivative of nonlinearity at node
- *    *error		    : array[1..nNodes] the error at node
- *    *dw		    	: array[1..nWeights] total derivative for weights
- *    *dwi		   		: array[1..nWeights] derivative per pattern
- *  integer dimension		: dimension of minimizer space (<= my nWeights)
- *  integer nPatterns	    : the #patterns to be learned
- * double **inputPattern: matrix[1..nPatterns][1..nInputs]
- * double **targetActivation: matrix[1..nPatterns][1..nOutputs]
- * double accumulatedCost : accumulated costs of testing/training with patterns
+ *  int *wSelected		: array[1..numberOfWeights] weights selected for minimization
+ *  double *deriv	    : array[1..numberOfNodes] derivative of nonlinearity at node
+ *    *error		    : array[1..numberOfNodes] the error at node
+ *    *dw		    	: array[1..numberOfWeights] total derivative for weights
+ *    *dwi		   		: array[1..numberOfWeights] derivative per pattern
+ *  integer dimension		: dimension of minimizer space (<= my numberOfWeights)
+ *  integer numberOfPatterns	    : the #patterns to be learned
+ *  MAT inputPattern: matrix[1..numberOfPatterns][1..numberOfInputs]
+ *  MAT targetActivation: matrix[1..numberOfPatterns][1..numberOfOutputs]
+ *  double accumulatedCost : accumulated costs of testing/training with patterns
  *
- * A network consists of nLayers layers. Layer numbering is from 0...nLayers.
- * Layer 0 is the input layer,  the highest numbered layer is the output layer
- *   (nLayers <= 4)
+ * A network consists of numberOfLayers layers. Layer numbering is from 1...numberOfLayers.
+ * The highest numbered layer is the output layer
+ *   (numberOfLayers <= 4)
  * Each layer consists of a number of units. The biases of all the units in a layer
  * are modelled with connections to an extra unit in the lower layer (with constant
  * activity 1.0). Nodes refers to 'units' + 'bias units'.
- * The variable 'nNodes' is the total number of nodes (inclusive bias nodes).
+ * The variable 'numberOfNodes' is the total number of nodes (inclusive bias nodes).
  * E.g. the topology (2,3,4), i.e., 2 inputs, 3 units in the first layer
  * and 4 units in the second layer (outputs) is modelled
  * with (2+1)+ (3+1)+ (4) = 11 nodes.
- * The numbering of the weights is as follows (indices 1..nWeights):
+ * The numbering of the weights is as follows (indices 1..numberOfWeights):
  * E.g., topology (I,H,O) (I inputs, H hidden units and O output units)
  * There are a total of H* (I+1) + O* (H+1) weights in this net.
  * w[1] - w[I]                        : I (1)->H (1), I (2)->H (1) ... I (I)->H (1)
@@ -98,17 +98,17 @@
  *
  * A number of auxiliary arrays for efficient calculations have been setup.
  * For a node k we need to know:
- * 1. isbias[1..nNodes]        : usage: if (isbias[k]) ...
- *      true if node k is a bias node. There are nLayers bias nodes
- * 2. nodeFirst[1..nNodes]       : usage is j=nodeFirst[k];
+ * 1. isbias[1..numberOfNodes]        : usage: if (isbias[k]) ...
+ *      true if node k is a bias node. There are numberOfLayers bias nodes
+ * 2. nodeFirst[1..numberOfNodes]       : usage is j=nodeFirst[k];
  *      j is the first node that is connected to k .
- * 3. nodeLast[1..nNodes]        : usage is j=nodeLast[k]
+ * 3. nodeLast[1..numberOfNodes]        : usage is j=nodeLast[k]
  *      j is the  last node that is connected to k (bias included).
  * For the calculation of the errors,  during learning,  in unit k we need to
  * know which weights from the preceeding layer connect to it.
- * 4. wFirst[1..nNodes] : usage j=wFirst[k]
+ * 4. wFirst[1..numberOfNodes] : usage j=wFirst[k]
  *      w[j] is first weight to node k.
- * 5. wLast[1..nNodes]  : usage j=wLast[k]
+ * 5. wLast[1..numberOfNodes]  : usage j=wLast[k]
  *      w[j] is last weight to node k.
  */
 
@@ -122,7 +122,7 @@ void FFNet_init (FFNet me, integer numberOfInputs, integer nodesInLayer1, intege
 autoFFNet FFNet_create (integer numberOfInputs, integer numberInLayer1, integer numberInLayer2,
 	integer numberOfOutputs, bool outputsAreLinear);
 
-char32 * FFNet_createNameFromTopology (FFNet me);
+autostring32 FFNet_createNameFromTopology (FFNet me);
 /* Create names as <inputs>-<outputs>, <inputs>-<hidden>-<outputs>,
 	<inputs>-<hidden1>-<hidden2>-<outputs> for 1, 2 or 3 layer networks.
 */
@@ -156,16 +156,16 @@ const char32* FFNet_getCategoryOfOutputUnit (FFNet me, integer outputUnit);
 
 integer FFNet_getOutputUnitOfCategory (FFNet me, const char32* category);
 
-void FFNet_propagateToLayer (FFNet me, const double input[], double activity[], integer layer);
+void FFNet_propagateToLayer (FFNet me, constVEC input, VEC activity, integer layer);
 /* propagate the input through the net to layer and calculate the activities */
 
-void FFNet_propagate (FFNet me, const double input[], double output[]);
+void FFNet_propagate (FFNet me, constVEC input, autoVEC *output);
 /* step (1) feed forward input from "input layer" to "output layer"
  * if output != nullptr the output activity is copied into output.
  * postcondition: my activities defined
  */
 
-double FFNet_computeError (FFNet me, const double target[]);
+double FFNet_computeError (FFNet me, constVEC target);
 /* step (2) calculate error on output nodes w.r.t. desired output */
 /* step (3) backpropagate this error to previous nodes */
 /* precondition: step (1) */
@@ -186,9 +186,9 @@ integer FFNet_dimensionOfSearchSpace (FFNet me);
 /* count the selected weights */
 
 integer FFNet_getNumberOfWeights (FFNet me);
-/* return my nWeights */
+/* return my numberOfWeights */
 
-void FFNet_weightConnectsUnits (FFNet me, integer index, integer *fromUnit, integer *toUnit, integer *layer);
+void FFNet_weightConnectsUnits (FFNet me, integer index, integer *out_fromUnit, integer *out_toUnit, integer *out_layer);
 /*
  * w[index] connects unit fromUnit in "layer-1" with unit toUnit in "layer".
  *  fromUnit returns 0 then w[index] is bias.
@@ -196,14 +196,11 @@ void FFNet_weightConnectsUnits (FFNet me, integer index, integer *fromUnit, inte
 
 integer FFNet_getNodeNumberFromUnitNumber (FFNet me, integer unit, integer layer);
 
-void FFNet_nodeToUnitInLayer (FFNet me, integer node, integer *unit, integer *layer);
-/* translate node index to unit "unit" in layer "layer" */
-
 integer FFNet_getNumberOfLayers (FFNet me);
 
 integer FFNet_getNumberOfUnits (FFNet me);
 
-integer FFNet_getNumberOfHiddenLayers (FFNet me);
+integer FFNet_getNumberOfHiddenumberOfLayers (FFNet me);
 
 integer FFNet_getNumberOfUnitsInLayer (FFNet me, int layer);
 
