@@ -168,13 +168,13 @@ DO
 
 DIRECT (INTEGER_FFNet_getNumberOfLayers) {
 	INTEGER_ONE (FFNet)
-		integer result = my nLayers;
-	INTEGER_ONE_END (U" layer", (my nLayers > 1 ? U"s" : U""))
+		integer result = my numberOfLayers;
+	INTEGER_ONE_END (U" layer", (my numberOfLayers > 1 ? U"s" : U""))
 }
 
 DIRECT (INTEGER_FFNet_getNumberOfOutputs) {
 	INTEGER_ONE (FFNet)
-		integer result = my nUnitsInLayer[my nLayers];
+		integer result = my numberOfUnitsInLayer[my numberOfLayers];
 	INTEGER_ONE_END (U" units")
 }
 
@@ -184,13 +184,13 @@ FORM (INTEGER_FFNet_getNumberOfHiddenUnits, U"FFNet: Get number of hidden units"
 	OK
 DO
 	INTEGER_ONE (FFNet)
-		integer result = layer > 0 && layer <= my nLayers - 1 ? my nUnitsInLayer[layer] : 0;
+		integer result = layer > 0 && layer <= my numberOfLayers - 1 ? my numberOfUnitsInLayer[layer] : 0;
 	INTEGER_ONE_END (U" units")
 }
 
 DIRECT (INTEGER_FFNet_getNumberOfInputs) {
 	INTEGER_ONE (FFNet)
-		integer result = my nUnitsInLayer[0];
+		integer result = my numberOfInputs;
 	INTEGER_ONE_END (U" units")
 }
 
@@ -199,13 +199,18 @@ FORM (INTEGER_FFNet_getNumberOfHiddenWeights, U"FFNet: Get number of hidden weig
 	OK
 DO
 	INTEGER_ONE (FFNet)
-		integer result = (layer > 0 && layer <= my nLayers - 1) ? (my nUnitsInLayer[layer] * (my nUnitsInLayer[layer - 1] + 1)) : 0;
+		integer result = 0;
+		if (layer <= my numberOfLayers - 1) {
+			integer numberOfUnitsInPreviousLayer = ( layer == 1 ? my numberOfInputs : my numberOfUnitsInLayer[layer - 1] );
+			result = my numberOfUnitsInLayer[layer] * (numberOfUnitsInPreviousLayer + 1);
+		}
 	INTEGER_ONE_END (U" weights (including biases)")
 }
 	
 DIRECT (INTEGER_FFNet_getNumberOfOutputWeights) {
 	INTEGER_ONE (FFNet)
-		integer result = my nUnitsInLayer[my nLayers] * (my nUnitsInLayer[my nLayers - 1] + 1);
+		integer numberOfUnitsInPreviousLayer = ( my numberOfLayers == 1 ? my numberOfInputs : my numberOfUnitsInLayer[my numberOfLayers - 1] );
+		integer result = my numberOfUnitsInLayer[my numberOfLayers] * (numberOfUnitsInPreviousLayer + 1);
 	INTEGER_ONE_END (U" weights");
 }
 
@@ -214,7 +219,7 @@ FORM (INFO_FFNet_getCategoryOfOutputUnit, U"FFNet: Get category of output unit",
 	OK
 DO
 	STRING_ONE (FFNet)
-		const char32 *result = FFNet_getCategoryOfOutputUnit (me, outputUnit);
+		conststring32 result = FFNet_getCategoryOfOutputUnit (me, outputUnit);
 	STRING_ONE_END
 }
 
@@ -247,7 +252,7 @@ FORM (REAL_FFNet_getWeight, U"FFNet: Get weight", nullptr) {
 DO
 	NUMBER_ONE (FFNet)
 		double result = FFNet_getWeight (me, layer, unitTo, unitFrom);
-	NUMBER_ONE_END (U"(weight between unit ", unitTo, U" in layer ", layer, U", and unit ", unitFrom, U"in layer ", layer - 1, U")")
+	NUMBER_ONE_END (U"(weight between unit ", unitTo, U" in layer ", layer, U", and unit ", unitFrom, U" in layer ", layer - 1, U")")
 }
 
 DIRECT (REAL_FFNet_getMinimum) {
@@ -313,7 +318,7 @@ FORM (NEW_FFNet_extractWeights, U"FFNet: Extract weights", U"FFNet: Extract weig
 DO
 	CONVERT_EACH (FFNet)
 		autoTableOfReal result = FFNet_extractWeights (me, layer);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 FORM (NEW_FFNet_weightsToMatrix, U"FFNet: Weights to Matrix ", nullptr) {
@@ -323,7 +328,7 @@ FORM (NEW_FFNet_weightsToMatrix, U"FFNet: Weights to Matrix ", nullptr) {
 DO
 	CONVERT_EACH (FFNet)
 		autoMatrix result = FFNet_weightsToMatrix (me, layer, false);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (HINT_hint_FFNet_PatternList_classify) {
@@ -346,7 +351,7 @@ FORM (NEW1_FFNet_ActivationList_to_Categories, U"FFNet & ActivationList: To Cate
 DO
 	CONVERT_TWO (FFNet, ActivationList)
 		autoCategories result = FFNet_ActivationList_to_Categories (me, you, categorizationgMethod);
-	CONVERT_TWO_END (my name, U"_", you -> name)
+	CONVERT_TWO_END (my name.get(), U"_", your name.get())
 }
 
 /******************* FFNet && Eigen ******************************************/
@@ -370,7 +375,7 @@ DO
 DIRECT (NEW1_FFNet_Categories_to_ActivationList) {
 	CONVERT_TWO (FFNet, Categories)
 		autoActivationList result = FFNet_Categories_to_ActivationList (me, you);
-	CONVERT_TWO_END (my name);
+	CONVERT_TWO_END (my name.get());
 }
 
 /************************* FFNet && Matrix **********************************/
@@ -381,7 +386,7 @@ FORM (NEW1_FFNet_weightsFromMatrix, U"Replace weights by values from Matrix", nu
 DO
 	CONVERT_TWO (FFNet, Matrix)
 		autoFFNet result = FFNet_weightsFromMatrix (me, you, layer);
-	CONVERT_TWO_END (my name);
+	CONVERT_TWO_END (my name.get());
 }
 
 /************************* FFNet && PatternList **********************************/
@@ -403,7 +408,7 @@ FORM (NEW1_FFNet_PatternList_to_Categories, U"FFNet & PatternList: To Categories
 DO
 	GRAPHICS_TWO (FFNet, PatternList)
 		autoCategories result = FFNet_PatternList_to_Categories (me, you, categorizationgMethod);
-	CONVERT_TWO_END (my name, U"_", you -> name)
+	CONVERT_TWO_END (my name.get(), U"_", your name.get())
 }
 
 FORM (NEW1_FFNet_PatternList_to_ActivationList, U"To activations in layer", nullptr) {
@@ -412,7 +417,7 @@ FORM (NEW1_FFNet_PatternList_to_ActivationList, U"To activations in layer", null
 DO
 	GRAPHICS_TWO (FFNet, PatternList)
 		autoActivationList result = FFNet_PatternList_to_ActivationList (me, you, layer);
-	CONVERT_TWO_END (my name, U"_", you -> name)
+	CONVERT_TWO_END (my name.get(), U"_", your name.get())
 }
 
 /*********** FFNet & PatternList & ActivationList **********************************/
@@ -551,7 +556,8 @@ FORM (NEW1_PatternList_Categories_to_FFNet, U"PatternList & Categories: To FFNet
 DO
 	CONVERT_TWO (PatternList, Categories)
 		autoFFNet result = PatternList_Categories_to_FFNet (me, you, numberOfUnitsInHiddenLayer1, numberOfUnitsInHiddenLayer2);
-	CONVERT_TWO_END (result -> name)
+		autostring32 name = result -> name.move();
+	CONVERT_TWO_END (name.get())
 }
 
 void praat_uvafon_FFNet_init () {
