@@ -40,7 +40,6 @@ autoCepstrogram Cepstrogram_create (double tmin, double tmax, integer nt, double
 	double qmin, double qmax, integer nq, double dq, double q1) {
 	try {
 		autoCepstrogram me = Thing_new (Cepstrogram);
-
 		Matrix_init (me.get(), tmin, tmax, nt, dt, t1, qmin, qmax, nq, dq, q1);
 		return me;
 	} catch (MelderError) {
@@ -52,7 +51,6 @@ autoPowerCepstrogram PowerCepstrogram_create (double tmin, double tmax, integer
 	double qmin, double qmax, integer nq, double dq, double q1) {
 	try {
 		autoPowerCepstrogram me = Thing_new (PowerCepstrogram);
-
 		Matrix_init (me.get(), tmin, tmax, nt, dt, t1, qmin, qmax, nq, dq, q1);
 		return me;
 	} catch (MelderError) {
@@ -73,14 +71,15 @@ void PowerCepstrogram_paint (PowerCepstrogram me, Graphics g, double tmin, doubl
 	for (integer i = 1; i <= my ny; i ++) {
 		for (integer j = 1; j <= my nx; j ++) {
 			double val = TO10LOG (my z [i] [j]);
-			min = val < min ? val : min;
-			max = val > max ? val : max;
+			if (val < min) min = val;
+			if (val > max) max = val;
 			thy z [i] [j] = val;
 		}
 	}
 	double dBminimum = dBmaximum - dynamicRangedB;
 	if (autoscaling) {
-		dBminimum = min; dBmaximum = max;
+		dBminimum = min;
+		dBmaximum = max;
 	}
 
 	for (integer j = 1; j <= my nx; j ++) {
@@ -91,18 +90,14 @@ void PowerCepstrogram_paint (PowerCepstrogram me, Graphics g, double tmin, doubl
 			}
 		}
 		double factor = dynamicCompression * (max - lmax);
-		for (integer i = 1; i <= my ny; i ++) {
-			thy z [i] [j] += factor;
-		}
+		thy z.column(j) += factor;
 	}
 	
 	Graphics_setInner (g);
 	Graphics_setWindow (g, tmin, tmax, qmin, qmax);
-	Graphics_image (g, thy z,
-		itmin, itmax,
+	Graphics_image (g, thy z.part (ifmin, ifmax, itmin, itmax),
 		Matrix_columnToX (thee.get(), itmin - 0.5),
 		Matrix_columnToX (thee.get(), itmax + 0.5),
-		ifmin, ifmax,
 		Matrix_rowToY (thee.get(), ifmin - 0.5),
 		Matrix_rowToY (thee.get(), ifmax + 0.5),
 		dBminimum, dBmaximum);
@@ -121,13 +116,9 @@ void PowerCepstrogram_subtractTilt_inplace (PowerCepstrogram me, double qstartFi
 	try {
 		autoPowerCepstrum thee = PowerCepstrum_create (my ymax, my ny);
 		for (integer i = 1; i <= my nx; i ++) {
-			for (integer j = 1; j <= my ny; j ++) {
-				thy z [1] [j] = my z [j] [i];
-			}
+			thy z.row (1) <<= my z.column (i);
 			PowerCepstrum_subtractTilt_inplace (thee.get(), qstartFit, qendFit, lineType, fitMethod);
-			for (integer j = 1; j <= my ny; j ++) {
-				my z [j] [i] = thy z [1] [j];
-			}
+			my z.column (i) <<= thy z.row (1);
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": no tilt subtracted (inline).");
@@ -150,9 +141,7 @@ autoTable PowerCepstrogram_to_Table_hillenbrand (PowerCepstrogram me, double pit
 		autoTable thee = Table_createWithColumnNames (my nx, U"time quefrency cpp f0");
 		autoPowerCepstrum him = PowerCepstrum_create (my ymax, my ny);
 		for (integer i = 1; i <= my nx; i ++) {
-			for (integer j = 1; j <= my ny; j ++) {
-				his z [1] [j] = my z [j] [i];
-			}
+			his z.row (1) <<= my z.column (i);
 			double qpeak, cpp = PowerCepstrum_getPeakProminence_hillenbrand (him.get(), pitchFloor, pitchCeiling, &qpeak);
 			double time = Sampled_indexToX (me, i);
 			Table_setNumericValue (thee.get(), i, 1, time);
@@ -171,9 +160,7 @@ autoTable PowerCepstrogram_to_Table_cpp (PowerCepstrogram me, double pitchFloor,
 		autoTable thee = Table_createWithColumnNames (my nx, U"time quefrency cpp f0 rnr");
 		autoPowerCepstrum him = PowerCepstrum_create (my ymax, my ny);
 		for (integer i = 1; i <= my nx; i ++) {
-			for (integer j = 1; j <= my ny; j ++) {
-				his z [1] [j] = my z [j] [i];
-			}
+			his z.row (1) <<= my z.column (i);
 			double qpeak, cpp = PowerCepstrum_getPeakProminence (him.get(), pitchFloor, pitchCeiling, interpolation,
 				qstartFit, qendFit, lineType, fitMethod, & qpeak);
 			double rnr = PowerCepstrum_getRNR (him.get(), pitchFloor, pitchCeiling, deltaF0);
@@ -196,35 +183,23 @@ autoPowerCepstrogram PowerCepstrogram_smooth (PowerCepstrogram me, double timeAv
 		// 1. average across time
 		integer numberOfFrames = Melder_ifloor (timeAveragingWindow / my dx);
 		if (numberOfFrames > 1) {
-			autoNUMvector<double> qin (1, my nx);
-			autoNUMvector<double> qout (1, my nx);
+			autoVEC qin = newVECraw (my nx);
+			autoVEC qout = newVECraw (my nx);
 			for (integer iq = 1; iq <= my ny; iq ++) {
-				for (integer iframe = 1; iframe <= my nx; iframe ++) {
-					//qin[iframe] = TO10LOG (my z[iq][iframe]);
-					qin [iframe] = thy z [iq] [iframe];
-				}
-				NUMvector_smoothByMovingAverage (qin.peek(), my nx, numberOfFrames, qout.peek());
-				for (integer iframe = 1; iframe <= my nx; iframe ++) {
-					//thy z[iq][iframe] = FROMLOG (qout[iframe]); // inverse 
-					thy z [iq] [iframe] = qout [iframe]; // inverse 
-				}
+				qin.all() <<= thy z.row (iq);   // ppgb: why this extra copying?
+				VECsmoothByMovingAverage_preallocated (qout.get(), qin.get(), numberOfFrames);
+				thy z.row (iq) <<= qout.all();
 			}
 		}
 		// 2. average across quefrencies
 		integer numberOfQuefrencyBins = Melder_ifloor (quefrencyAveragingWindow / my dy);
 		if (numberOfQuefrencyBins > 1) {
-			autoNUMvector<double> qin (1, thy ny);
-			autoNUMvector<double> qout (1, thy ny);
+			autoVEC qin = newVECraw (thy ny);
+			autoVEC qout = newVECraw (thy ny);
 			for (integer iframe = 1; iframe <= my nx; iframe ++) {
-				for (integer iq = 1; iq <= thy ny; iq ++) {
-					//qin[iq] = TO10LOG (my z[iq][iframe]);
-					qin [iq] = thy z [iq] [iframe];
-				}
-				NUMvector_smoothByMovingAverage (qin.peek(), thy ny, numberOfQuefrencyBins, qout.peek());
-				for (integer iq = 1; iq <= thy ny; iq ++) {
-					//thy z[iq][iframe] = FROMLOG (qout[iq]);
-					thy z [iq] [iframe] = qout [iq];
-				}
+				qin.get() <<= thy z.column (iframe);
+				VECsmoothByMovingAverage_preallocated (qout.get(), qin.get(), numberOfQuefrencyBins);
+				thy z.column (iframe) <<= qout.get();
 			}
 		}
 		return thee;
@@ -248,9 +223,7 @@ autoPowerCepstrum PowerCepstrogram_to_PowerCepstrum_slice (PowerCepstrogram me,
 		integer iframe = Sampled_xToNearestIndex (me, time);
 		iframe = iframe < 1 ? 1 : iframe > my nx ? my nx : iframe;
 		autoPowerCepstrum thee = PowerCepstrum_create (my ymax, my ny);
-		for (integer i = 1; i <= my ny; i ++) {
-			thy z [1] [i] = my z [i] [iframe];
-		}
+		thy z.row (1) <<= my z.column (iframe);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": Cepstrum not extracted.");
@@ -298,15 +271,14 @@ autoPowerCepstrogram Sound_to_PowerCepstrogram (Sound me, double pitchFloor, dou
 			Sound_into_Sound (sound.get(), sframe.get(), t - windowDuration / 2);
 			Vector_subtractMean (sframe.get());
 			Sounds_multiply (sframe.get(), window.get());
-			autoSpectrum spec = Sound_to_Spectrum (sframe.get(), 1); // FFT yes
+			autoSpectrum spec = Sound_to_Spectrum (sframe.get(), true);   // FFT yes
 			autoPowerCepstrum cepstrum = Spectrum_to_PowerCepstrum (spec.get());
-			for (integer i = 1; i <= nq; i ++) {
+			for (integer i = 1; i <= nq; i ++)
 				thy z [i] [iframe] = cepstrum -> z [1] [i];
-			}
-			if ((iframe % 10) == 1) {
+
+			if ((iframe % 10) == 1)
 				Melder_progress ((double) iframe / nFrames, U"PowerCepstrogram analysis of frame ",
 					iframe, U" out of ", nFrames, U".");
-			}
 		}
 		return thee;
 	} catch (MelderError) {
@@ -319,18 +291,16 @@ autoCepstrum Spectrum_to_Cepstrum_hillenbrand (Spectrum me) {
 	try {
 		autoNUMfft_Table fftTable;
 		// originalNumberOfSamplesProbablyOdd irrelevant
-		if (my x1 != 0.0) {
-			Melder_throw (U"A Fourier-transformable Spectrum must have a first frequency of 0 Hz, not ", my x1, U" Hz.");
-		}
+		Melder_require (my x1 == 0.0,
+			U"A Fourier-transformable Spectrum should have a first frequency of 0 Hz, not ", my x1, U" Hz.");
 		integer numberOfSamples = my nx - 1;
 		autoCepstrum thee = Cepstrum_create (0.5 / my dx, my nx);
 		NUMfft_Table_init (& fftTable, my nx);
-		autoNUMvector<double> amp (1, my nx);
+		autoVEC amp = newVECraw (my nx);
 		
-		for (integer i = 1; i <= my nx; i ++) {
+		for (integer i = 1; i <= my nx; i ++)
 			amp [i] = my v_getValueAtSample (i, 0, 2);
-		}
-		NUMfft_forward (& fftTable, amp.peek());
+		NUMfft_forward (& fftTable, amp.get());
 		
 		for (integer i = 1; i <= my nx; i ++) {
 			double val = amp [i] / numberOfSamples;// scaling 1/n because ifft(fft(1))= n;
@@ -346,18 +316,18 @@ autoCepstrum Spectrum_to_Cepstrum_hillenbrand (Spectrum me) {
 //    re   im    re     im                   re      im
 // ((fft[1],0) (fft[2],fft[3]), (,), (,), (fft[nfft], 0))  nfft even
 // ((fft[1],0) (fft[2],fft[3]), (,), (,), (fft[nfft-1], fft[nfft]))  nfft uneven
-static void complexfftoutput_to_power (double *fft, integer nfft, double *dbs, bool to_db) {
+static void complexfftoutput_to_power (constVEC fft, VEC dbs, bool to_db) {
 	double valsq = fft [1] * fft [1];
 	dbs[1] = to_db ? TOLOG (valsq) : valsq;
-	integer nfftdiv2p1 = (nfft + 2) / 2;
-	integer nend = nfft % 2 == 0 ? nfftdiv2p1 : nfftdiv2p1 + 1;
+	integer nfftdiv2p1 = (fft.size + 2) / 2;
+	integer nend = fft.size % 2 == 0 ? nfftdiv2p1 : nfftdiv2p1 + 1;
 	for (integer i = 2; i < nend; i ++) {
 		double re = fft [i + i - 2], im = fft [i + i - 1];
 		valsq = re * re + im * im;
 		dbs [i] = to_db ? TOLOG (valsq) : valsq;
 	}
-	if (nfft % 2 == 0) {
-		valsq = fft [nfft] * fft [nfft];
+	if (fft.size % 2 == 0) {
+		valsq = fft [fft.size] * fft [fft.size];
 		dbs[nfftdiv2p1] = to_db ? TOLOG (valsq) : valsq;
 	}
 }
@@ -365,69 +335,54 @@ static void complexfftoutput_to_power (double *fft, integer nfft, double *dbs, b
 autoPowerCepstrogram Sound_to_PowerCepstrogram_hillenbrand (Sound me, double pitchFloor, double dt) {
 	try {
 		// minimum analysis window has 3 periods of lowest pitch
-		double analysisWidth = 3.0 / pitchFloor;
-		if (analysisWidth > my dx * my nx) {
-			analysisWidth = my dx * my nx;
-		}
+		const double physicalDuration = my dx * my nx;
+		const double analysisWidth = std::min (3.0 / pitchFloor, physicalDuration);
+
 		double samplingFrequency = 1.0 / my dx;
 		autoSound thee;
-		if (samplingFrequency > 30000) {
+		if (samplingFrequency > 30000.0) {
 			samplingFrequency = samplingFrequency / 2.0;
 			thee = Sound_resample (me, samplingFrequency, 1);
 		} else {
 			thee = Data_copy (me);
 		}
 		// pre-emphasis with fixed coefficient 0.9
-		for (integer i = thy nx; i > 1; i --) {
+		for (integer i = thy nx; i > 1; i --)
 			thy z [1] [i] -= 0.9 * thy z [1] [i - 1];
-		}
-		integer nosInWindow = (integer) floorl (analysisWidth * samplingFrequency), numberOfFrames;
-		if (nosInWindow < 8) {
-			Melder_throw (U"Analysis window too short.");
-		}
+
+		integer nosInWindow = Melder_ifloor (analysisWidth * samplingFrequency), numberOfFrames;
+		Melder_require (nosInWindow >= 8, U"Analysis window too short.");
+
 		double t1;
 		Sampled_shortTermAnalysis (thee.get(), analysisWidth, dt, & numberOfFrames, & t1);
-		autoNUMvector<double> hamming (1, nosInWindow);
-		for (integer i = 1; i <= nosInWindow; i ++) {
-			hamming [i] = 0.54 -0.46 * cos(2 * NUMpi * (i - 1) / (nosInWindow - 1));
-		}
+		autoVEC hamming = newVECraw (nosInWindow);
+		for (integer i = 1; i <= nosInWindow; i ++) 
+			hamming [i] = 0.54 - 0.46 * cos (2.0 * NUMpi * (i - 1) / (nosInWindow - 1));
+
 		integer nfft = 8; // minimum possible
 		while (nfft < nosInWindow) { nfft *= 2; }
-		integer nfftdiv2 = nfft / 2;
-		autoNUMvector<double> fftbuf (1, nfft); // "complex" array
-		autoNUMvector<double> spectrum (1, nfftdiv2 + 1); // +1 needed 
+		const integer nfftdiv2 = nfft / 2;
+		autoVEC fftbuf = newVECzero (nfft); // "complex" array
+		autoVEC spectrum = newVECzero (nfftdiv2 + 1); // +1 needed 
 		autoNUMfft_Table fftTable;
 		NUMfft_Table_init (& fftTable, nfft); // sound to spectrum
 		
-		double qmax = 0.5 * nfft / samplingFrequency, dq = qmax / (nfftdiv2 + 1);
+		const double qmax = 0.5 * nfft / samplingFrequency, dq = qmax / (nfftdiv2 + 1);
 		autoPowerCepstrogram him = PowerCepstrogram_create (my xmin, my xmax, numberOfFrames, dt, t1, 0, qmax, nfftdiv2+1, dq, 0);
 		
 		autoMelderProgress progress (U"Cepstrogram analysis");
 		
 		for (integer iframe = 1; iframe <= numberOfFrames; iframe ++) {
-			double tbegin = t1 + (iframe - 1) * dt - analysisWidth / 2;
-			tbegin = tbegin < thy xmin ? thy xmin : tbegin;
-			integer istart = Sampled_xToLowIndex (thee.get(), tbegin);   // ppgb: afronding naar beneden?
-			istart = istart < 1 ? 1 : istart;
-			integer iend = istart + nosInWindow - 1;
-			iend = iend > thy nx ? thy nx : iend;
-			for (integer i = 1; i <= nosInWindow; i ++) {
-				fftbuf [i] = thy z [1] [istart + i - 1] * hamming [i];
-			}
-			for (integer i = nosInWindow + 1; i <= nfft; i ++) { 
-				fftbuf [i] = 0;
-			}
-			NUMfft_forward (&fftTable, fftbuf.peek());
-			complexfftoutput_to_power (fftbuf.peek(), nfft, spectrum.peek(), true); // log10(|fft|^2)
-			// subtract average
-			double specmean = spectrum [1];
-			for (integer i = 2; i <= nfftdiv2 + 1; i ++) {
-				specmean += spectrum [i];
-			}
-			specmean /= nfftdiv2 + 1;
-			for (integer i = 1; i <= nfftdiv2 + 1; i ++) {
-				spectrum [i] -= specmean;
-			}
+			const double tbegin = std::max (thy xmin, t1 + (iframe - 1) * dt - analysisWidth / 2.0);
+			const integer istart = std::max (integer (1), Sampled_xToLowIndex (thee.get(), tbegin));   // ppgb: afronding naar beneden?
+			const integer iend = istart + nosInWindow - 1;   // ppgb: crash if not provably iend <= thy nx
+			fftbuf.part (1, nosInWindow) <<= thy z.row (1).part (istart, iend) * hamming.all();
+			fftbuf.part (nosInWindow + 1, nfft) <<= 0.0;
+			
+			NUMfft_forward (& fftTable, fftbuf.get());
+			complexfftoutput_to_power (fftbuf.get(), spectrum.get(), true); // log10(|fft|^2)
+		
+			VECcentre_inplace (spectrum.get()); // subtract average
 			/*
 			 * Here we diverge from Hillenbrand as he takes the fft of half of the spectral values.
 			 * H. forgets that the actual spectrum has nfft/2+1 values. Thefore, we take the inverse
@@ -438,17 +393,16 @@ autoPowerCepstrogram Sound_to_PowerCepstrogram_hillenbrand (Sound me, double pit
 			fftbuf [1] = spectrum [1];
 			for (integer i = 2; i < nfftdiv2 + 1; i ++) {
 				fftbuf [i+i-2] = spectrum [i];
-				fftbuf [i+i-1] = 0;
+				fftbuf [i+i-1] = 0.0;
 			}
 			fftbuf [nfft] = spectrum [nfftdiv2 + 1];
-			NUMfft_backward (& fftTable, fftbuf.peek());
-			for (integer i = 1; i <= nfftdiv2 + 1; i ++) {
+			NUMfft_backward (& fftTable, fftbuf.get());
+			for (integer i = 1; i <= nfftdiv2 + 1; i ++)
 				his z [i] [iframe] = fftbuf [i] * fftbuf [i];
-			}
-			if ((iframe % 10) == 1) {
+
+			if (iframe % 10 == 1)
 				Melder_progress ((double) iframe / numberOfFrames, U"Cepstrogram analysis of frame ",
 					 iframe, U" out of ", numberOfFrames, U".");
-			}
 		}
 		return him;
 	} catch (MelderError) {
@@ -459,9 +413,9 @@ autoPowerCepstrogram Sound_to_PowerCepstrogram_hillenbrand (Sound me, double pit
 double PowerCepstrogram_getCPPS (PowerCepstrogram me, bool subtractTiltBeforeSmoothing, double timeAveragingWindow, double quefrencyAveragingWindow, double pitchFloor, double pitchCeiling, double deltaF0, int interpolation, double qstartFit, double qendFit, int lineType, int fitMethod) {
 	try {
 		autoPowerCepstrogram flattened;
-		if (subtractTiltBeforeSmoothing) {
+		if (subtractTiltBeforeSmoothing)
 			flattened = PowerCepstrogram_subtractTilt (me, qstartFit, qendFit, lineType, fitMethod);
-		}
+
 		autoPowerCepstrogram smooth = PowerCepstrogram_smooth (flattened ? flattened.get() : me, timeAveragingWindow, quefrencyAveragingWindow);
 		autoTable table = PowerCepstrogram_to_Table_cpp (smooth.get(), pitchFloor, pitchCeiling, deltaF0, interpolation, qstartFit, qendFit, lineType, fitMethod);
 		double cpps = Table_getMean (table.get(), 3);
@@ -474,9 +428,9 @@ double PowerCepstrogram_getCPPS (PowerCepstrogram me, bool subtractTiltBeforeSmo
 double PowerCepstrogram_getCPPS_hillenbrand (PowerCepstrogram me, bool subtractTiltBeforeSmoothing, double timeAveragingWindow, double quefrencyAveragingWindow, double pitchFloor, double pitchCeiling) {
 	try {
 		autoPowerCepstrogram him;
-		if (subtractTiltBeforeSmoothing) {
+		if (subtractTiltBeforeSmoothing)
 			him = PowerCepstrogram_subtractTilt (me, 0.001, 0, 1, 1);
-		}
+
 		autoPowerCepstrogram smooth = PowerCepstrogram_smooth (subtractTiltBeforeSmoothing ? him.get() : me, timeAveragingWindow, quefrencyAveragingWindow);
 		autoTable table = PowerCepstrogram_to_Table_hillenbrand (smooth.get(), pitchFloor, pitchCeiling);
 		double cpps = Table_getMean (table.get(), 3);
