@@ -1,6 +1,6 @@
 /* LPC.cpp
  *
- * Copyright (C) 1994-2017 David Weenink
+ * Copyright (C) 1994-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,21 +60,20 @@ void structLPC :: v_info () {
 	MelderInfo_writeLine (U"First frame at: ", x1, U" (s).");
 }
 
-void LPC_Frame_init (LPC_Frame me, int nCoefficients) {
-	if (nCoefficients != 0) {
-		my a = NUMvector<double> (1, nCoefficients);
-	}
+void LPC_Frame_init (LPC_Frame me, integer nCoefficients) {
+	if (nCoefficients != 0)
+		my a = newVECzero (nCoefficients);
 	my nCoefficients = nCoefficients;
 }
 
-void LPC_init (LPC me, double tmin, double tmax, integer nt, double dt, double t1, int predictionOrder, double samplingPeriod) {
+void LPC_init (LPC me, double tmin, double tmax, integer nt, double dt, double t1, integer predictionOrder, double samplingPeriod) {
 	my samplingPeriod = samplingPeriod;
 	my maxnCoefficients = predictionOrder;
 	Sampled_init (me, tmin, tmax, nt, dt, t1);
 	my d_frames = NUMvector<structLPC_Frame> (1, nt);
 }
 
-autoLPC LPC_create (double tmin, double tmax, integer nt, double dt, double t1, int predictionOrder, double samplingPeriod) {
+autoLPC LPC_create (double tmin, double tmax, integer nt, double dt, double t1, integer predictionOrder, double samplingPeriod) {
 	try {
 		autoLPC me = Thing_new (LPC);
 		LPC_init (me.get(), tmin, tmax, nt, dt, t1, predictionOrder, samplingPeriod);
@@ -90,17 +89,18 @@ void LPC_drawGain (LPC me, Graphics g, double tmin, double tmax, double gmin, do
 		tmax = my xmax;
 	}
 	integer itmin, itmax;
-	if (! Sampled_getWindowSamples (me, tmin, tmax, & itmin, & itmax)) {
+	if (! Sampled_getWindowSamples (me, tmin, tmax, & itmin, & itmax))
 		return;
-	}
-	autoNUMvector<double> gain (itmin, itmax);
 
-	for (integer iframe = itmin; iframe <= itmax; iframe ++) {
-		gain [iframe] = my d_frames [iframe]. gain;
-	}
-	if (gmax <= gmin) {
-		NUMvector_extrema (gain.peek(), itmin, itmax, & gmin, & gmax);
-	}
+	integer numberOfSelected = itmax - itmin + 1;
+	autoVEC gain = newVECraw (numberOfSelected);
+
+	for (integer iframe = itmin; iframe <= itmax; iframe ++)
+		gain [iframe - itmin + 1] = my d_frames [iframe]. gain;
+
+	if (gmax <= gmin)
+		NUMextrema (gain.get(), & gmin, & gmax);
+
 	if (gmax == gmin) {
 		gmin = 0;
 		gmax += 0.5;
@@ -110,7 +110,7 @@ void LPC_drawGain (LPC me, Graphics g, double tmin, double tmax, double gmin, do
 	Graphics_setWindow (g, tmin, tmax, gmin, gmax);
 	for (integer iframe = itmin; iframe <= itmax; iframe ++) {
 		double x = Sampled_indexToX (me, iframe);
-		Graphics_speckle (g, x, gain[iframe]);
+		Graphics_speckle (g, x, gain [iframe - itmin + 1]);
 	}
 	Graphics_unsetInner (g);
 	if (garnish) {
@@ -122,7 +122,7 @@ void LPC_drawGain (LPC me, Graphics g, double tmin, double tmax, double gmin, do
 	}
 }
 
-void LPC_drawPoles (LPC me, Graphics g, double time, int garnish) {
+void LPC_drawPoles (LPC me, Graphics g, double time, bool garnish) {
 	autoPolynomial p = LPC_to_Polynomial (me, time);
 	autoRoots r = Polynomial_to_Roots (p.get());
 	Roots_draw (r.get(), g, -1.0, 1.0, -1.0, 1.0, U"+", 12, garnish);
@@ -133,9 +133,7 @@ autoMatrix LPC_downto_Matrix_lpc (LPC me) {
 		autoMatrix thee = Matrix_create (my xmin, my xmax, my nx, my dx, my x1, 0.5, 0.5 + my maxnCoefficients, my maxnCoefficients, 1.0, 1.0);
 		for (integer j = 1; j <= my nx; j ++) {
 			LPC_Frame lpc = & my d_frames [j];
-			for (integer i = 1; i <= lpc -> nCoefficients; i ++) {
-				thy z [i] [j] = lpc -> a [i];
-			}
+			thy z.column (j) <<= lpc-> a.get();
 		}
 		return thee;
 	} catch (MelderError) {
@@ -146,13 +144,13 @@ autoMatrix LPC_downto_Matrix_lpc (LPC me) {
 autoMatrix LPC_downto_Matrix_rc (LPC me) {
 	try {
 		autoMatrix thee = Matrix_create (my xmin, my xmax, my nx, my dx, my x1, 0.5, 0.5 + my maxnCoefficients, my maxnCoefficients, 1.0, 1.0);
-		autoNUMvector<double> rc (1, my maxnCoefficients);
+		autoVEC rc = newVECzero (my maxnCoefficients);
 		for (integer j = 1; j <= my nx; j ++) {
 			LPC_Frame lpc = & my d_frames [j];
-			NUMlpc_lpc_to_rc (lpc -> a, lpc -> nCoefficients, rc.peek());
-			for (integer i = 1; i <= lpc -> nCoefficients; i ++) {
-				thy z [i] [j] = rc [i];
-			}
+			VECrc_from_lpc (rc.part (1, lpc -> nCoefficients), lpc -> a.part (1, lpc -> nCoefficients));
+			if (lpc -> nCoefficients < my maxnCoefficients)
+				rc.part (lpc -> nCoefficients + 1, my maxnCoefficients) <<= 0.0;
+			thy z.column (j) <<= rc.get();
 		}
 		return thee;
 	} catch (MelderError) {
@@ -163,15 +161,14 @@ autoMatrix LPC_downto_Matrix_rc (LPC me) {
 autoMatrix LPC_downto_Matrix_area (LPC me) {
 	try {
 		autoMatrix thee = Matrix_create (my xmin, my xmax, my nx, my dx, my x1, 0.5, 0.5 + my maxnCoefficients, my maxnCoefficients, 1.0, 1.0);
-		autoNUMvector<double> rc (1, my maxnCoefficients);
-		autoNUMvector<double> area (1, my maxnCoefficients);
+		autoVEC rc = newVECraw (my maxnCoefficients);
+		autoVEC area = newVECraw (my maxnCoefficients);
 		for (integer j = 1; j <= my nx; j ++) {
 			LPC_Frame lpc = & my d_frames [j];
-			NUMlpc_lpc_to_rc (lpc -> a, lpc -> nCoefficients, rc.peek());
-			NUMlpc_rc_to_area (rc.peek(), lpc -> nCoefficients, area.peek());
-			for (integer i = 1; i <= lpc -> nCoefficients; i ++) {
-				thy z [i] [j] = area [i];
-			}
+			VECrc_from_lpc (rc.part (1, lpc -> nCoefficients), lpc -> a.part (1, lpc -> nCoefficients));
+			VECarea_from_rc (area.part (1, lpc -> nCoefficients), rc.part (1, lpc -> nCoefficients));
+			if (lpc -> nCoefficients < my maxnCoefficients) area.part (lpc -> nCoefficients + 1, my maxnCoefficients) <<= 0.0;
+			thy z.column (j) <<= area.get();
 		}
 		return thee;
 	} catch (MelderError) {
