@@ -1,6 +1,6 @@
 /* Sound_and_LPC_robust.cpp
  *
- * Copyright (C) 1994-2017 David Weenink
+ * Copyright (C) 1994-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,155 +36,111 @@
 
 struct huber_struct {
 	autoSound e;
-	double k, tol, tol_svd;
+	double k_stdev, tol, tol_svd;
 	integer iter, itermax;
 	int wantlocation, wantscale;
 	double location, scale;
 	integer n, p;
-	double *w, *work;
-	double *a;
-	double **covar, *c;
+	autoVEC w, work, a, c;
+	autoMAT covar;
 	autoSVD svd;
 };
 
 static void huber_struct_init (struct huber_struct *hs, double windowDuration, integer p, double samplingFrequency, double location, int wantlocation) {
-	hs -> w = hs -> work = hs -> a = hs -> c = nullptr;
-	hs -> covar = nullptr;
-	hs -> svd = autoSVD();
 	hs -> e = Sound_createSimple (1, windowDuration, samplingFrequency);
+	hs -> k_stdev = hs -> tol = hs -> tol_svd = hs -> scale = 0.0;
+	hs -> iter = 1;
+	hs -> itermax = 1;
+	hs -> wantlocation = wantlocation;
+	if (! wantlocation) hs -> location = location;
+	hs -> wantscale = 1;
 	integer n = hs -> e -> nx;
 	hs -> n = n;
 	hs -> p = p;
-	hs -> w = NUMvector<double> (1, n);
-	hs -> work = NUMvector<double> (1, n);
-	hs -> a = NUMvector<double> (1, p);
-	hs -> covar = NUMmatrix<double> (1, p, 1, p);
-	hs -> c = NUMvector<double> (1, p);
+	hs -> w = newVECzero (n);
+	hs -> work = newVECraw (n);
+	hs -> a = newVECraw (p);
+	hs -> c = newVECzero (p);
+	hs -> covar = newMATzero (p, p);
 	hs -> svd = SVD_create (p, p);
-	hs -> wantlocation = wantlocation;
-	if (! wantlocation) {
-		hs -> location = location;
-	}
-	hs -> wantscale = 1;
 }
 
-static void huber_struct_destroy (struct huber_struct *hs) {
-	hs -> e.reset();
-	hs -> svd.reset();
-	NUMvector_free<double> (hs -> w, 1);
-	NUMvector_free<double> (hs -> work, 1);
-	NUMvector_free<double> (hs -> a, 1);
-	NUMmatrix_free<double> (hs -> covar, 1, 1);
-	NUMvector_free<double> (hs -> c, 1);
-}
-
-static void huber_struct_getWeights (struct huber_struct *hs, double *e) {
-	double ks = hs -> k * hs -> scale;
-	double *w = hs -> w;
+static void huber_struct_getWeights (struct huber_struct *hs, constVEC e) {
+	Melder_assert (e.size == hs -> n);
+	double kstdev = hs -> k_stdev * hs -> scale;
 
 	for (integer i = 1 ; i <= hs -> n; i ++) {
-		double ei = e [i] - hs -> location;
-		w [i] = ei > -ks && ei < ks ? 1.0 : ks / fabs (ei);
+		double abs_ei = fabs (e [i] - hs -> location);
+		hs -> w [i] = abs_ei < kstdev ? 1.0 : kstdev / abs_ei;
 	}
 }
 
-static void huber_struct_getWeightedCovars (struct huber_struct *hs, double *s) {
-	integer p = hs -> p, n = hs -> n;
-	double *w = hs -> w, **covar = hs -> covar, *c = hs -> c;
+static void huber_struct_getWeightedCovars (struct huber_struct *hs, VEC s) {
+	Melder_assert (s.size == hs -> n);
+	integer p = hs -> p;
 
 	for (integer i = 1; i <= p; i ++) {
 		for (integer j = i; j <= p; j ++) {
-			double tmp = 0;
-			for (integer k = p + 1; k <= n; k ++) {
-				tmp += s [k - j] * s [k - i] * w [k];
-			}
-			covar [i] [j] = covar [j] [i] = tmp;
+			longdouble cv1 = 0.0;
+			for (integer k = p + 1; k <= s.size; k ++)
+				cv1 += s [k - j] * s [k - i] *  hs -> w [k];
+			hs -> covar [i] [j] = hs -> covar [j] [i] = (double) cv1;
 		}
 
-		double tmp = 0;
-		for (integer k = p + 1; k <= n; k ++) {
-			tmp += s [k - i] * s [k] * w [k];
-		}
-		c [i] = -tmp;
+		longdouble cv2 = 0.0;
+		for (integer k = p + 1; k <= s.size; k ++)
+			cv2 += s [k - i] * s [k] *  hs -> w [k];
+		hs -> c [i] = - cv2;
 	}
 }
 
 static void huber_struct_solvelpc (struct huber_struct *hs) {
 	SVD me = hs -> svd.get();
-	double **covar = hs -> covar;
-
-	for (integer i = 1; i <= my numberOfRows; i ++) {
-		for (integer j = 1; j <= my numberOfColumns; j ++) {
-			my u [i] [j] = covar [i] [j];
-		}
-	}
-
+	my u.get() <<= hs -> covar.get();
 	SVD_setTolerance (me, hs -> tol_svd);
 	SVD_compute (me);
 
-	//integer nzeros = SVD_zeroSmallSingularValues (me, 0);
-
-	SVD_solve (me, hs -> c, hs -> a);
+	hs -> a = SVD_solve (me, hs -> c.get());
+	//hs -> a.all() <<= x.all();
 }
 
 void LPC_Frames_Sound_huber (LPC_Frame me, Sound thee, LPC_Frame him, struct huber_struct *hs) {
-	integer p = my nCoefficients > his nCoefficients ? his nCoefficients : my nCoefficients;
-	integer n = hs -> e -> nx > thy nx ? thy nx : hs -> e -> nx;
-	double *e = hs -> e -> z [1], *s = thy z [1];
+	integer p = std::min (my nCoefficients, his nCoefficients);
 
 	hs -> iter = 0;
 	hs -> scale = 1e308;
 	hs -> p = p;
 
-	double s0;
+	double scale0;
 	do {
 		Sound hse = hs -> e.get();
-		for (integer i = 1; i <= thy nx; i ++) {
-			hse -> z [1] [i] = thy z [1] [i];
-		}
+		hse -> z.row (1) <<= thy z.row (1);
+		
 		LPC_Frame_Sound_filterInverse (him, hse, 1);
 
-		s0 = hs -> scale;
+		scale0 = hs -> scale;
+		VEC work = hs -> work.get();
+		NUMstatistics_huber (hs -> e -> z.row (1), & hs -> location, hs -> wantlocation, & hs -> scale, hs -> wantscale, hs -> k_stdev, hs -> tol, 5);
 
-		NUMstatistics_huber (e, n, & (hs -> location), hs -> wantlocation, & (hs -> scale), hs -> wantscale, hs -> k, hs -> tol, hs -> work);
-
-		huber_struct_getWeights (hs, e);
-		huber_struct_getWeightedCovars (hs, s);
+		huber_struct_getWeights (hs, hs -> e -> z.row (1));
+		huber_struct_getWeightedCovars (hs, thy z.row (1));
 
 		// Solve C a = [-] c */
 		try {
 			huber_struct_solvelpc (hs);
 		} catch (MelderError) {
 			// Copy the starting lpc coeffs */
-			for (integer i = 1; i <= p; i ++) {
-				his a [i] = my a [i];
-			}
+			his a.part (1, p) <<= my a.part (1, p); 
 			throw MelderError();
 		}
-		for (integer i = 1; i <= p; i ++) {
-			his a [i] = hs -> a [i];
-		}
-
-		(hs -> iter) ++;
-	} while ( (hs -> iter < hs -> itermax) && (fabs (s0 - hs -> scale) > hs -> tol * s0));
+		his a.part (1, p) <<= hs -> a.part (1, p);
+		hs -> iter ++;
+	} while (hs -> iter < hs -> itermax && fabs (scale0 - hs -> scale) > hs -> tol * scale0);
 }
 
-#if 0
-	autoSound e;
-	double k, tol, tol_svd;
-	integer iter, itermax;
-	bool wantlocation, wantscale;
-	double location, scale;
-	integer n, p;
-	double *w, *work;
-	double *a;
-	double **covar, *c;
-	autoSVD svd;
-#endif
-	
-autoLPC LPC_Sound_to_LPC_robust (LPC thee, Sound me, double analysisWidth, double preEmphasisFrequency, double k,
+autoLPC LPC_Sound_to_LPC_robust (LPC thee, Sound me, double analysisWidth, double preEmphasisFrequency, double k_stdev,
 	int itermax, double tol, bool wantlocation) {
-	struct huber_struct struct_huber { autoSound(), 0.0, 0.0, 0.0, 0, 0, 0, 0, 0.0, 0.0, 0, 0, nullptr, nullptr, nullptr, nullptr, nullptr, autoSVD() };
+	struct huber_struct struct_huber;
 	try {
 		double t1, samplingFrequency = 1.0 / my dx, tol_svd = 0.000001;
 		double location = 0, windowDuration = 2 * analysisWidth; /* Gaussian window */
@@ -201,9 +157,9 @@ autoLPC LPC_Sound_to_LPC_robust (LPC thee, Sound me, double analysisWidth, doubl
 		autoSound sframe = Sound_createSimple (1, windowDuration, samplingFrequency);
 		autoSound window = Sound_createGaussian (windowDuration, samplingFrequency);
 		autoLPC him = Data_copy (thee);
-		huber_struct_init (&struct_huber, windowDuration, p, samplingFrequency, location, wantlocation);
+		huber_struct_init (& struct_huber, windowDuration, p, samplingFrequency, location, wantlocation);
 
-		struct_huber.k = k;
+		struct_huber.k_stdev = k_stdev;
 		struct_huber.tol = tol;
 		struct_huber.tol_svd = tol_svd;
 		struct_huber.itermax = itermax;
@@ -229,19 +185,16 @@ autoLPC LPC_Sound_to_LPC_robust (LPC thee, Sound me, double analysisWidth, doubl
 
 			iter += struct_huber.iter;
 
-			if ( (i % 10) == 1) {
-				Melder_progress ((double) i / numberOfFrames, U"LPC analysis of frame ", i, U" out of ", numberOfFrames, U".");
-			}
+			if (i % 10 == 1)
+				Melder_progress ((double) i / numberOfFrames,
+					U"LPC analysis of frame ", i, U" out of ", numberOfFrames, U".");
 		}
 
 		if (frameErrorCount) Melder_warning (U"Results of ", frameErrorCount,
 			U" frame(s) out of ", numberOfFrames, U" could not be optimised.");
-		MelderInfo_writeLine (U"Number of iterations: ", iter,
-			U"\n   Average per frame: ", (double) iter / numberOfFrames);
-		huber_struct_destroy (&struct_huber);
+		//Melder_casual (U"Number of iterations: ", iter, U"\n   Average per frame: ", (double) iter / numberOfFrames);
 		return him;
 	} catch (MelderError) {
-		huber_struct_destroy (&struct_huber);
 		Melder_throw (me, U": no robust LPC created.");
 	}
 }
@@ -254,11 +207,10 @@ autoFormant Sound_to_Formant_robust (Sound me, double dt_in, double numberOfForm
 	integer predictionOrder = Melder_ifloor (2 * numberOfFormants);
 	try {
 		autoSound sound;
-		if (maximumFrequency <= 0.0 || fabs (maximumFrequency / nyquist - 1.0) < 1.0e-12) {
+		if (maximumFrequency <= 0.0 || fabs (maximumFrequency / nyquist - 1.0) < 1.0e-12)
 			sound = Data_copy (me);   // will be modified
-		} else {
+		else
 			sound = Sound_resample (me, maximumFrequency * 2.0, 50);
-		}
 
 		autoLPC lpc = Sound_to_LPC_auto (sound.get(), predictionOrder, halfdt_window, dt, preEmphasisFrequency);
 		autoLPC lpcr = LPC_Sound_to_LPC_robust (lpc.get(), sound.get(), halfdt_window, preEmphasisFrequency, k, itermax, tol, wantlocation);
