@@ -1,6 +1,6 @@
 /* Tube.cpp
  *
- * Copyright (C) 1994-2012, 2015-2016 David Weenink
+ * Copyright (C) 1994-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,46 +49,36 @@ Thing_implement (Tube, Sampled, 0);
 void structTube :: v_info () {
 	structDaata :: v_info ();
 	MelderInfo_writeLine (U"Time domain: ", xmin, U" to ", xmax, U" seconds");
-	MelderInfo_writeLine (U"Maximum number of segments: ", maxnSegments);
+	MelderInfo_writeLine (U"Maximum number of segments: ", maxNumberOfSegments);
 	MelderInfo_writeLine (U"Number of frames: ", nx);
 	MelderInfo_writeLine (U"Time step: ", dx, U" seconds");
 	MelderInfo_writeLine (U"First frame at: ", x1, U" seconds");
 }
 
-void Tube_Frame_init (Tube_Frame me, integer nSegments, double length) {
-	my nSegments = nSegments;
+void Tube_Frame_init (Tube_Frame me, integer numberOfSegments, double length) {
+	my numberOfSegments = numberOfSegments;
 	my length = length;
-	if (nSegments <= 0) {
-		Melder_throw (U"Number of segments should be a natural number.");
-	}
-	my c = NUMvector<double> (1, nSegments);
+	Melder_require (numberOfSegments > 0, 
+		U"Number of segments should be a natural number.");
+	my c = newVECzero (numberOfSegments);
 }
 
 /* Gray & Markel (1979), LPTRN */
 void Tube_Frames_rc_into_area (Tube_Frame me, Tube_Frame thee) {
-	if (my nSegments > thy nSegments) {
+	if (my numberOfSegments > thy numberOfSegments)
 		Melder_throw (U"Number of segments to big.");
-	}
-
-	double s = 0.0001; /* 1.0 cm^2 at glottis */
-	double *rc = my c, *area = thy c;
-	for (integer i = my nSegments; i > 0; i --) {
-		s *= (1.0 + rc [i]) / (1.0 - rc [i]);
-		area [i] = s;
-	}
+	VECarea_from_rc (thy c.get(), my c.get());
 }
 
 static void Tube_setLengths (Tube me, double length) {
 	for (integer i = 1; i <= my nx; i ++) {
 		Tube_Frame f = & my frame [i];
-		if (f) {
-			f -> length = length;
-		}
+		if (f) f -> length = length;
 	}
 }
 
-void Tube_init (Tube me, double tmin, double tmax, integer nt, double dt, double t1, integer maxnSegments, double defaultLength) {
-	my maxnSegments = maxnSegments;
+void Tube_init (Tube me, double tmin, double tmax, integer nt, double dt, double t1, integer maxNumberOfSegments, double defaultLength) {
+	my maxNumberOfSegments = maxNumberOfSegments;
 	Sampled_init (me, tmin, tmax, nt, dt, t1);
 	my frame = NUMvector<structTube_Frame> (1, nt);
 	Tube_setLengths (me, defaultLength);
@@ -96,14 +86,14 @@ void Tube_init (Tube me, double tmin, double tmax, integer nt, double dt, double
 
 Thing_implement (Area, Tube, 0);
 
-void Area_init (Area me, double tmin, double tmax, integer nt, double dt, double t1, integer maxnSegments, double defaultLength) {
-	Tube_init (me, tmin, tmax, nt, dt, t1, maxnSegments, defaultLength);
+void Area_init (Area me, double tmin, double tmax, integer nt, double dt, double t1, integer maxNumberOfSegments, double defaultLength) {
+	Tube_init (me, tmin, tmax, nt, dt, t1, maxNumberOfSegments, defaultLength);
 }
 
-autoArea Area_create (double tmin, double tmax, integer nt, double dt, double t1, integer maxnSegments, double defaultLength) {
+autoArea Area_create (double tmin, double tmax, integer nt, double dt, double t1, integer maxNumberOfSegments, double defaultLength) {
 	try {
 		autoArea me = Thing_new (Area);
-		Area_init (me.get(), tmin, tmax, nt, dt, t1, maxnSegments, defaultLength);
+		Area_init (me.get(), tmin, tmax, nt, dt, t1, maxNumberOfSegments, defaultLength);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Area not crteated.");
@@ -112,8 +102,8 @@ autoArea Area_create (double tmin, double tmax, integer nt, double dt, double t1
 
 Thing_implement (RC, Tube, 0);
 
-void RC_init (RC me, double tmin, double tmax, integer nt, double dt, double t1, integer maxnSegments, double defaultLength) {
-	Tube_init (me, tmin, tmax, nt, dt, t1, maxnSegments, defaultLength);
+void RC_init (RC me, double tmin, double tmax, integer nt, double dt, double t1, integer maxNumberOfSegments, double defaultLength) {
+	Tube_init (me, tmin, tmax, nt, dt, t1, maxNumberOfSegments, defaultLength);
 }
 
 autoRC RC_create (double tmin, double tmax, integer nt, double dt, double t1, integer maxnCoefficients, double defaultLength) {
