@@ -20,7 +20,7 @@
 #define ooSTRUCT FeatureWeights
 oo_DEFINE_CLASS (FeatureWeights, Daata)
 
-	oo_AUTO_OBJECT (TableOfReal, 0, fweights)
+	oo_OBJECT (TableOfReal, 0, fweights)
 
 	#if oo_DECLARING
 		void v_info ()
