@@ -43,8 +43,8 @@ Thing_implement (FileInMemory, Daata, 0);
 
 void structFileInMemory :: v_info () {
 	our structDaata :: v_info ();
-	MelderInfo_writeLine (U"File name: ", our d_path);
-	MelderInfo_writeLine (U"Id: ", our d_id);
+	MelderInfo_writeLine (U"File name: ", our d_path.get());
+	MelderInfo_writeLine (U"Id: ", our d_id.get());
 	MelderInfo_writeLine (U"Number of bytes: ", our d_numberOfBytes);
 }
 
@@ -59,7 +59,7 @@ autoFileInMemory FileInMemory_create (MelderFile file) {
 		my d_id = Melder_dup (MelderFile_name (file));
 		my d_numberOfBytes = length;
 		my _dontOwnData = false;
-		my d_data = NUMvector <unsigned char> (0, my d_numberOfBytes);   // includes room for a final null byte in case the file happens to contain text
+		my d_data = NUMvector <uint8> (0, my d_numberOfBytes);   // includes room for a final null byte in case the file happens to contain text
 		MelderFile_open (file);
 		for (integer i = 0; i < my d_numberOfBytes; i++) {
 			unsigned int number = bingetu8 (file -> filePointer);
@@ -73,7 +73,7 @@ autoFileInMemory FileInMemory_create (MelderFile file) {
 	}
 }
 
-autoFileInMemory FileInMemory_createWithData (integer numberOfBytes, const char *data, bool isStaticData, const char32 *path, const char32 *id) {
+autoFileInMemory FileInMemory_createWithData (integer numberOfBytes, const char *data, bool isStaticData, conststring32 path, conststring32 id) {
 	try {
 		autoFileInMemory me = Thing_new (FileInMemory);
 		my d_path = Melder_dup (path);
@@ -84,7 +84,7 @@ autoFileInMemory FileInMemory_createWithData (integer numberOfBytes, const char
 			my d_data = reinterpret_cast<unsigned char *> (const_cast<char *> (data)); // ... just a link
 		} else {
 			my _dontOwnData = false;
-			my d_data =  NUMvector <unsigned char> ((integer) 0, numberOfBytes);
+			my d_data = NUMvector <unsigned char> ((integer) 0, numberOfBytes);
 			NUMvector_copyElements <unsigned char> (reinterpret_cast<unsigned char *> (const_cast<char *> (data)), my d_data, 0, numberOfBytes);
 		} 
 		return me;
@@ -93,12 +93,11 @@ autoFileInMemory FileInMemory_createWithData (integer numberOfBytes, const char
 	}
 }
 
-void FileInMemory_setId (FileInMemory me, const char32 *newId) {
-	Melder_free (my d_id);
+void FileInMemory_setId (FileInMemory me, conststring32 newId) {
 	my d_id = Melder_dup (newId);
 }
 
-void FileInMemory_showAsCode (FileInMemory me, const char32 *name, integer numberOfBytesPerLine)
+void FileInMemory_showAsCode (FileInMemory me, conststring32 name, integer numberOfBytesPerLine)
 {
 	if (numberOfBytesPerLine < 1) {
 		numberOfBytesPerLine = 20;
@@ -107,11 +106,12 @@ void FileInMemory_showAsCode (FileInMemory me, const char32 *name, integer numbe
 	for (integer i = 0; i < my d_numberOfBytes; i++) {
 		unsigned char number = my d_data [i];
 		MelderInfo_write ((i % numberOfBytesPerLine == 0 ? U"\t\t\t" : U""), number, U",",
-			((i % numberOfBytesPerLine == (numberOfBytesPerLine - 1)) ? U"\n" : U" "));
+			i % numberOfBytesPerLine == numberOfBytesPerLine - 1 ? U"\n" : U" ");
 	}
 	MelderInfo_writeLine ((my d_numberOfBytes - 1) % numberOfBytesPerLine == (numberOfBytesPerLine - 1) ? U"\t\t\t0};" : U"0};");
 	MelderInfo_write (U"\t\tautoFileInMemory ", name, U" = FileInMemory_createWithData (");
-	MelderInfo_writeLine (my d_numberOfBytes, U", reinterpret_cast<const char *> (&", name, U"_data), true, \n\t\t\tU\"", my d_path, U"\", \n\t\t\tU\"", my d_id, U"\");");
+	MelderInfo_writeLine (my d_numberOfBytes, U", reinterpret_cast<const char *> (&",
+		name, U"_data), true, \n\t\t\tU\"", my d_path.get(), U"\", \n\t\t\tU\"", my d_id.get(), U"\");");
 }
 
 /* End of file FileInMemory.cpp */
