@@ -76,7 +76,7 @@ autoFileInMemorySet FileInMemorySets_merge (OrderedOf<structFileInMemorySet>& li
 	}
 }
 
-autoFileInMemorySet FileInMemorySet_createFromDirectoryContents (const char32 *dirpath, const char32 *fileGlobber) {
+autoFileInMemorySet FileInMemorySet_createFromDirectoryContents (conststring32 dirpath, conststring32 fileGlobber) {
 	try {
 		structMelderDir parent { };
 		Melder_pathToDir (dirpath, & parent);
@@ -87,7 +87,7 @@ autoFileInMemorySet FileInMemorySet_createFromDirectoryContents (const char32 *d
 		autoFileInMemorySet me = FileInMemorySet_create ();
 		for (integer i = 1; i <= thy numberOfStrings; i ++) {
 			structMelderFile file { };
-			MelderDir_getFile (& parent, thy strings [i], & file);
+			MelderDir_getFile (& parent, thy strings [i].get(), & file);
 			autoFileInMemory fim = FileInMemory_create (& file);
 			my addItem_move (fim.move());
 		}
@@ -111,12 +111,12 @@ autoFileInMemorySet FilesInMemory_to_FileInMemorySet (OrderedOf<structFileInMemo
 	
 }
 
-autoFileInMemorySet FileInMemorySet_extractFiles (FileInMemorySet me, kMelder_string which, const char32 *criterion) {
+autoFileInMemorySet FileInMemorySet_extractFiles (FileInMemorySet me, kMelder_string which, conststring32 criterion) {
 	try {
 		autoFileInMemorySet thee = Thing_new (FileInMemorySet);
 		for (integer ifile = 1; ifile <= my size; ifile ++) {
 			FileInMemory fim = static_cast <FileInMemory> (my at [ifile]);
-			if (Melder_stringMatchesCriterion (fim -> d_path, which, criterion, true)) {
+			if (Melder_stringMatchesCriterion (fim -> d_path.get(), which, criterion, true)) {
 				autoFileInMemory item = Data_copy (fim);
 				thy addItem_move (item.move());
 			}
@@ -127,12 +127,12 @@ autoFileInMemorySet FileInMemorySet_extractFiles (FileInMemorySet me, kMelder_st
 	}
 }
 
-autoFileInMemorySet FileInMemorySet_listFiles (FileInMemorySet me, kMelder_string which, const char32 *criterion) {
+autoFileInMemorySet FileInMemorySet_listFiles (FileInMemorySet me, kMelder_string which, conststring32 criterion) {
 	try {
 		autoFileInMemorySet thee = Thing_new (FileInMemorySet);
 		for (integer ifile = 1; ifile <= my size; ifile ++) {
 			FileInMemory fim = static_cast<FileInMemory> (my at [ifile]);
-			if (Melder_stringMatchesCriterion (fim -> d_path, which, criterion, true)) {
+			if (Melder_stringMatchesCriterion (fim -> d_path.get(), which, criterion, true)) {
 				thy addItem_ref (fim);
 			}
 		}
@@ -142,7 +142,7 @@ autoFileInMemorySet FileInMemorySet_listFiles (FileInMemorySet me, kMelder_strin
 	}
 }
 
-void FileInMemorySet_showAsCode (FileInMemorySet me, const char32 *name, integer numberOfBytesPerLine) {
+void FileInMemorySet_showAsCode (FileInMemorySet me, conststring32 name, integer numberOfBytesPerLine) {
 	autoMelderString one_fim;
 	MelderInfo_writeLine (U"#include \"FileInMemorySet.h\"");
 	MelderInfo_writeLine (U"#include \"melder.h\"\n");
@@ -162,7 +162,7 @@ void FileInMemorySet_showAsCode (FileInMemorySet me, const char32 *name, integer
 	MelderInfo_writeLine (U"}\n\n");
 }
 
-void FileInMemorySet_showOneFileAsCode (FileInMemorySet me, integer index, const char32 *name, integer numberOfBytesPerLine)
+void FileInMemorySet_showOneFileAsCode (FileInMemorySet me, integer index, conststring32 name, integer numberOfBytesPerLine)
 {
 	if (index < 1 || index > my size) return;
 	MelderInfo_writeLine (U"#include \"FileInMemory.h\"");
@@ -181,11 +181,11 @@ void FileInMemorySet_showOneFileAsCode (FileInMemorySet me, integer index, const
 	MelderInfo_writeLine (U"autoFileInMemory ", name, U" = create_new_object ();");
 }
 
-integer FileInMemorySet_getIndexFromId (FileInMemorySet me, const char32 *id) {
+integer FileInMemorySet_getIndexFromId (FileInMemorySet me, conststring32 id) {
 	integer index = 0;
 	for (integer i = 1; i <= my size; i ++) {
 		FileInMemory fim = (FileInMemory) my at [i];
-		if (Melder_equ (id, fim -> d_id)) {
+		if (Melder_equ (id, fim -> d_id.get())) {
 			index = i;
 			break;
 		}
@@ -193,11 +193,11 @@ integer FileInMemorySet_getIndexFromId (FileInMemorySet me, const char32 *id) {
 	return index;
 }
 
-integer FileInMemorySet_lookUp (FileInMemorySet me, const char32 *path) {
+integer FileInMemorySet_lookUp (FileInMemorySet me, conststring32 path) {
 	integer index = 0;
 	for (integer i = 1; i <= my size; i ++) {
 		FileInMemory fim = (FileInMemory) my at [i];
-		if (Melder_equ (path, fim -> d_path)) {
+		if (Melder_equ (path, fim -> d_path.get())) {
 			index = i;
 			break;
 		}
@@ -205,24 +205,24 @@ integer FileInMemorySet_lookUp (FileInMemorySet me, const char32 *path) {
 	return index;
 }
 
-integer FileInMemorySet_findNumberOfMatches_path (FileInMemorySet me, kMelder_string which, const char32 *criterion) {
+integer FileInMemorySet_findNumberOfMatches_path (FileInMemorySet me, kMelder_string which, conststring32 criterion) {
 	integer numberOfMatches = 0;
 	for (integer ifile = 1; ifile <= my size; ifile ++) {
 		FileInMemory fim = static_cast <FileInMemory> (my at [ifile]);
-		if (Melder_stringMatchesCriterion (fim -> d_path, which, criterion, true)) {
+		if (Melder_stringMatchesCriterion (fim -> d_path.get(), which, criterion, true)) {
 			numberOfMatches ++;
 		}
 	}
 	return numberOfMatches;
 }
 
-bool FileInMemorySet_hasDirectory (FileInMemorySet me, const char32 *name) {
+bool FileInMemorySet_hasDirectory (FileInMemorySet me, conststring32 name) {
 	bool match = false;
 	autoMelderString regex;
 	for (integer i = 1; i <= my size; i ++) {
 		FileInMemory fim = (FileInMemory) my at [i];
 		MelderString_append (& regex, U".*/", name, U"/.*"); 
-		if (Melder_stringMatchesCriterion (fim -> d_path, kMelder_string :: MATCH_REGEXP, regex.string, true)) {
+		if (Melder_stringMatchesCriterion (fim -> d_path.get(), kMelder_string :: MATCH_REGEXP, regex.string, true)) {
 			match = true;
 			break;
 		}
@@ -234,11 +234,11 @@ bool FileInMemorySet_hasDirectory (FileInMemorySet me, const char32 *name) {
 autoStrings FileInMemorySet_to_Strings_id (FileInMemorySet me) {
 	try {
 		autoStrings thee = Thing_new (Strings);
-		thy strings = NUMvector <char32 *> (1, my size);
+		thy strings = autostring32vector (my size);
 		thy numberOfStrings = 0;
 		for (integer ifile = 1; ifile <= my size; ifile ++) {
 			FileInMemory fim = (FileInMemory) my at [ifile];
-			thy strings [ifile] = Melder_dup_f (fim -> d_id);
+			thy strings [ifile] = Melder_dup_f (fim -> d_id.get());
 			thy numberOfStrings ++;
 		}
 		return thee;
@@ -247,7 +247,7 @@ autoStrings FileInMemorySet_to_Strings_id (FileInMemorySet me) {
 	}
 }
 
-char * FileInMemorySet_getCopyOfData (FileInMemorySet me, const char32 *id, integer *numberOfBytes) {
+char * FileInMemorySet_getCopyOfData (FileInMemorySet me, conststring32 id, integer *numberOfBytes) {
 	*numberOfBytes = 0;
 	integer index = FileInMemorySet_getIndexFromId (me, id);
 	if (index == 0) {
@@ -264,7 +264,7 @@ char * FileInMemorySet_getCopyOfData (FileInMemorySet me, const char32 *id, inte
 	return data;
 }
 
-const char * FileInMemorySet_getData (FileInMemorySet me, const char32 *id, integer *numberOfBytes) {
+const char * FileInMemorySet_getData (FileInMemorySet me, conststring32 id, integer *numberOfBytes) {
 	*numberOfBytes = 0;
 	integer index = FileInMemorySet_getIndexFromId (me, id);
 	if (index == 0) {
