@@ -98,7 +98,7 @@ void s_cat (char *lp, const char *rpp[], integer rnp[], integer *np, integer ll)
 
 /* compare two strings */
 integer s_cmp (const char *a0, const char *b0, integer la, integer lb) {
-	register unsigned char *a, *aend, *b, *bend;
+	unsigned char *a, *aend, *b, *bend;
 	a = (unsigned char *) a0;
 	b = (unsigned char *) b0;
 	aend = a + la;
@@ -139,8 +139,8 @@ integer s_cmp (const char *a0, const char *b0, integer la, integer lb) {
 	return (0);
 }
 
-void s_copy (register char *a, register char *b, integer la, integer lb) {
-	register char *aend, *bend;
+void s_copy (char *a, char *b, integer la, integer lb) {
+	char *aend, *bend;
 
 	aend = a + la;
 
