@@ -1,6 +1,6 @@
 /* AffineTransform.cpp
  *
- * Copyright (C) 1993-2017 David Weenink
+ * Copyright (C) 1993-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,46 +47,37 @@
 #include "oo_DESCRIPTION.h"
 #include "AffineTransform_def.h"
 
-void structAffineTransform :: v_transform (double **in, integer nrows, double **out) {
-	for (integer i = 1; i <= nrows; i ++) {
-		for (integer j = 1; j <= n; j ++) {
-			double tmp = 0;
-			for (integer k = 1; k <= n; k ++) {
-				tmp += in [i] [k] * r [k] [j];
-			}
-			out [i] [j] = tmp + t [j];
-		}
-	}
+void structAffineTransform :: v_transform (MAT out, MAT in) {
+	Melder_assert (in.nrow == out.nrow);
+	Melder_assert (in.ncol == out.ncol);
+	MATVUmul (out, in, r.get());
+	out += t;
 }
 
 autoAffineTransform structAffineTransform :: v_invert () {
 	autoAffineTransform thee = Data_copy (this);
 	double tolerance = 0.000001;
 
-	NUMpseudoInverse (r, n, n, thy r, tolerance);
-	for (integer i = 1; i <= n; i ++) {
-		thy t [i] = 0.0;
-		for (integer j = 1; j <= thy n; j ++) {
-			thy t [i] -= thy r [i] [j] * t [j];
-		}
-	}
+	thy r = MATpseudoInverse (r.get(), tolerance);
+	for (integer i = 1; i <= dimension; i ++)
+		thy t[i] = - NUMinner (thy r.row (i), t.get ());
 	return thee;
 }
 
 Thing_implement (AffineTransform, Daata, 0);
 
-void AffineTransform_init (AffineTransform me, integer n) {
-	Melder_require (n > 0, U"Dimensionality should be greater than zero.");
+void AffineTransform_init (AffineTransform me, integer dimension) {
+	Melder_require (dimension > 0, U"Dimensionality should be greater than zero.");
 	
-	my n = n;
-	my r = NUMmatrix<double> (1, n, 1, n);
-	my t = NUMvector<double> (1, n);
+	my dimension = dimension;
+	my r = newMATzero (dimension, dimension);
+	my t = newVECzero (dimension);
 }
 
-autoAffineTransform AffineTransform_create (integer n) {
+autoAffineTransform AffineTransform_create (integer dimension) {
 	try {
 		autoAffineTransform me = Thing_new (AffineTransform);
-		AffineTransform_init (me.get(), n);
+		AffineTransform_init (me.get(), dimension);
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"AffineTransform not created.");
@@ -99,9 +90,9 @@ autoAffineTransform AffineTransform_invert (AffineTransform me) {
 
 autoTableOfReal AffineTransform_extractMatrix (AffineTransform me) {
 	try {
-		autoTableOfReal thee = TableOfReal_create (my n, my n);
-		NUMmatrix_copyElements (my r, thy data, 1, my n, 1, my n);
-		for (integer i = 1; i <= my n; i ++) {
+		autoTableOfReal thee = TableOfReal_create (my dimension, my dimension);
+		thy data.all() <<= my r.all();
+		for (integer i = 1; i <= my dimension; i ++) {
 			char32 label [40];
 			Melder_sprint (label,40, i);
 			TableOfReal_setRowLabel (thee.get(), i, label);
@@ -115,10 +106,8 @@ autoTableOfReal AffineTransform_extractMatrix (AffineTransform me) {
 
 autoTableOfReal AffineTransform_extractTranslationVector (AffineTransform me) {
 	try {
-		autoTableOfReal thee = TableOfReal_create (1, my n);
-		for (integer i = 1; i <= my n; i ++) {
-			thy data [1] [i] = my t [i];
-		}
+		autoTableOfReal thee = TableOfReal_create (1, my dimension);
+		thy data.row (1) <<= my t.get();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": translation vector not extracted.");
