@@ -1,6 +1,6 @@
 /* CC.cpp
  *
- * Copyright (C) 1993-2012, 2014-2017 David Weenink
+ * Copyright (C) 1993-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,11 +52,8 @@ Thing_implement (CC, Sampled, 1);
 static integer CC_getMaximumNumberOfCoefficientsUsed (CC me) {
 	integer numberOfCoefficients = 0;
 	for (integer iframe = 1; iframe <= my nx; iframe ++) {
-		CC_Frame cf = (CC_Frame) & my frame [iframe];
-		integer numberOfCoefficients_iframe = cf -> numberOfCoefficients;
-		if (numberOfCoefficients_iframe > numberOfCoefficients) {
-			numberOfCoefficients = numberOfCoefficients_iframe;
-		}
+		CC_Frame cf = & my frame [iframe];
+		numberOfCoefficients = std::max (cf -> numberOfCoefficients, numberOfCoefficients);
 	}
 	return numberOfCoefficients;
 }
@@ -72,7 +69,7 @@ void structCC :: v_info () {
 }
 
 void CC_Frame_init (CC_Frame me, integer numberOfCoefficients) {
-	my c = NUMvector<double> (1, numberOfCoefficients);
+	my c = newVECzero (numberOfCoefficients);
 	my numberOfCoefficients = numberOfCoefficients;
 }
 
@@ -91,17 +88,14 @@ autoMatrix CC_to_Matrix (CC me) {
 		integer numberOfCoefficients = 0;
 		for (integer i = 1; i <= my nx; i ++) {
 			CC_Frame cf = & my frame [i];
-			if (cf -> numberOfCoefficients > numberOfCoefficients) {
+			if (cf -> numberOfCoefficients > numberOfCoefficients)
 				numberOfCoefficients = cf -> numberOfCoefficients;
-			}
 		}
 		autoMatrix thee = Matrix_create (my xmin, my xmax, my nx, my dx, my x1, 1.0, numberOfCoefficients, numberOfCoefficients, 1.0, 1.0);
 		
 		for (integer i = 1; i <= my nx; i ++) {
 			CC_Frame cf = & my frame [i];
-			for (integer j = 1; j <= cf -> numberOfCoefficients; j ++) {
-				thy z [j] [i] = cf -> c [j];
-			}
+			thy z.column (i) <<= cf -> c.get(); // not coefficients must equal zero!
 		}
 		return thee;
 	} catch (MelderError) {
@@ -126,44 +120,43 @@ void CC_drawC0 (CC me, Graphics g, double xmin, double xmax, double ymin, double
 	(void) garnish;
 
 	if (xmin >= xmax) {
-		xmin = my xmin; xmax = my xmax;
+		xmin = my xmin;
+		xmax = my xmax;
 	}
 
 	integer bframe, eframe;
 	(void) Sampled_getWindowSamples (me, xmin, xmax, & bframe, & eframe);
-	autoNUMvector <double> c (bframe, eframe);
-	for (integer i = bframe; i <= eframe; i ++) {
-		CC_Frame cf = & my frame [i];
+	integer numberOfSelected = eframe - bframe + 1;
+	autoVEC c = newVECraw (numberOfSelected);
+	for (integer i = 1; i <= numberOfSelected; i ++) {
+		CC_Frame cf = & my frame [bframe + i - 1];
 		c [i] = cf -> c0;
 	}
 	if (ymin >= ymax) {
-		NUMvector_extrema (c.peek(), bframe, eframe, &ymin, &ymax);
+		NUMextrema (c.get(), & ymin, & ymax);
 		if (ymax <= ymin) {
 			ymin -= 1.0;
 			ymax += 1.0;
 		}
-	} else {
-		NUMvector_clip (c.peek(), bframe, eframe, ymin, ymax);
-	}
+	} else
+		VECclip_inplace (c.get(), ymin, ymax);
+
 	Graphics_setInner (g);
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
-	Graphics_function (g, c.peek(), bframe, eframe, xmin, xmax);
+	Graphics_function (g, c.at, 1, numberOfSelected, xmin, xmax);
 	Graphics_unsetInner (g);
 }
 
-void CC_getNumberOfCoefficients_extrema (CC me, integer startframe, integer endframe, integer *p_min, integer *p_max) {
+void CC_getNumberOfCoefficients_extrema (CC me, integer startframe, integer endframe, integer *out_min, integer *out_max) {
 
 	Melder_assert (startframe <= endframe);
 
 	if (startframe == 0 && endframe == 0) {
-		startframe = 1; endframe = my nx;
-	}
-	if (startframe < 1) {
-		startframe = 1;
-	}
-	if (endframe > my nx) {
+		startframe = 1; 
 		endframe = my nx;
 	}
+	if (startframe < 1) startframe = 1;
+	if (endframe > my nx) endframe = my nx;
 
 	integer min = my maximumNumberOfCoefficients;
 	integer max = 0;
@@ -172,75 +165,53 @@ void CC_getNumberOfCoefficients_extrema (CC me, integer startframe, integer endf
 		CC_Frame f = & my frame [i];
 		integer nc = f -> numberOfCoefficients;
 
-		if (nc < min) {
-			min = nc;
-		} else if (nc > max) {
-			max = nc;
-		}
-	}
-	if (p_min) {
-		*p_min = min;
-	}
-	if (p_max) {
-		*p_max = max;
+		if (nc < min) min = nc;
+		else if (nc > max) max = nc;
 	}
+	if (out_min) *out_min = min;
+	if (out_max) *out_max = max;
 }
 
 integer CC_getMinimumNumberOfCoefficients (CC me, integer startframe, integer endframe) {
 	integer min, max;
-
 	CC_getNumberOfCoefficients_extrema (me, startframe, endframe, & min, & max);
-
 	return min;
 }
 
 integer CC_getMaximumNumberOfCoefficients (CC me, integer startframe, integer endframe) {
 	integer min, max;
-
 	CC_getNumberOfCoefficients_extrema (me, startframe, endframe, & min, & max);
-
 	return max;
 }
 
 integer CC_getNumberOfCoefficients (CC me, integer iframe) {
-	if (iframe < 1 || iframe > my nx) {
+	if (iframe < 1 || iframe > my nx)
 		return 0;
-	}
 	CC_Frame cf = & me -> frame[iframe];
 	return cf -> numberOfCoefficients;
 }
 
 
 double CC_getValueInFrame (CC me, integer iframe, integer index) {
-	if (iframe < 1 || iframe > my nx) {
+	if (iframe < 1 || iframe > my nx)
 		return undefined;
-	}
 	CC_Frame cf = & me -> frame [iframe];
-	return index > cf -> numberOfCoefficients ? undefined : cf -> c [index];
+	return ( index > cf -> numberOfCoefficients ? undefined : cf -> c [index] );
 }
 
 double CC_getValue (CC me, double t, integer index) {
 	integer iframe = Sampled_xToNearestIndex (me, t);
-	if (iframe < 1 || iframe > my nx) {
+	if (iframe < 1 || iframe > my nx)
 		return undefined;
-	}
 	CC_Frame cf = & me -> frame [iframe];
-	return index > cf -> numberOfCoefficients ? undefined : cf -> c [index];
+	return ( index > cf -> numberOfCoefficients ? undefined : cf -> c [index] );
 }
 
 double CC_getC0ValueInFrame (CC me, integer iframe) {
-	if (iframe < 1 || iframe > my nx) {
+	if (iframe < 1 || iframe > my nx)
 		return undefined;
-	}
 	CC_Frame cf = & me -> frame [iframe];
 	return cf -> c0;
 }
 
-#if 0
-double CC_getC0ValueAtTime (CC me, double t) {
-	integer iframe = Sampled_xToNearestIndex (me, t);
-	return CC_getC0ValueInFrame (me, iframe);
-}
-#endif
-
 /* End of file CC.cpp */
