@@ -2,7 +2,7 @@
 #define _CategoriesEditor_h_
 /* CategoriesEditor.h
  *
- * Copyright (C) 1993-2011, 2015 David Weenink
+ * Copyright (C) 1993-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,6 @@ Thing_define (CategoriesEditor, Editor) {
 		override;
 };
 
-autoCategoriesEditor CategoriesEditor_create (const char32 *title, Categories data);
+autoCategoriesEditor CategoriesEditor_create (conststring32 title, Categories data);
 
 #endif /* _CategoriesEditor_h_ */
