@@ -1,6 +1,6 @@
 /* Confusion.cpp
  *
- * Copyright (C) 1993-2017 David Weenink
+ * Copyright (C) 1993-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,11 +63,11 @@ autoConfusion Confusion_createFromStringses (Strings me, Strings thee) {
 		
 		autoConfusion him = Confusion_create (my numberOfStrings, thy numberOfStrings);
 		for (integer irow = 1; irow <= my numberOfStrings; irow ++) {
-			const char32 *label = my strings [irow];
+			conststring32 label = my strings [irow].get();
 			TableOfReal_setRowLabel (him.get(), irow, label);
 		}
 		for (integer icol = 1; icol <= thy numberOfStrings; icol ++) {
-			const char32 *label = thy strings [icol];
+			conststring32 label = thy strings [icol].get();
 			TableOfReal_setColumnLabel (him.get(), icol, label);
 		}
 		return him;
@@ -86,22 +86,23 @@ autoConfusion Confusion_create (integer numberOfStimuli, integer numberOfRespons
 	}
 }
 
-autoConfusion Confusion_createSimple (const char32 *labels) {
+autoConfusion Confusion_createSimple (conststring32 labels_string) {
 	try {
-		integer numberOfLabels = Melder_countTokens (labels);
-		Melder_require (numberOfLabels > 0, U"There should be at least one label.");
+		autostring32vector labels = newSTRVECtokenize (labels_string);
+		Melder_require (labels.size > 0, U"There should be at least one label.");
 		
-		autoConfusion me = Confusion_create (numberOfLabels, numberOfLabels);
+		autoConfusion me = Confusion_create (labels.size, labels.size);
 		integer ilabel = 1;
-		for (char32 *token = Melder_firstToken (labels); token != 0; token = Melder_nextToken ()) {
+		for (integer itoken = 1; itoken <= labels.size; itoken ++) {
+			conststring32 token = labels [itoken].get();
 			for (integer i = 1; i <= ilabel - 1; i ++) {
-				if (Melder_cmp (token, my rowLabels [i]) == 0) {
+				if (Melder_equ (token, my rowLabels [i].get())) {
 					Melder_throw (U"Label ", i, U" and ", ilabel, U" should not be equal.");
 				}
 			}
 			TableOfReal_setRowLabel (me.get(), ilabel, token);
 			TableOfReal_setColumnLabel (me.get(), ilabel, token);
-			ilabel++;
+			ilabel ++;
 		}
 		return me;
 	} catch (MelderError) {
@@ -119,15 +120,15 @@ autoConfusion Categories_to_Confusion (Categories me, Categories thee) {
 
 		for (integer i = 1; i <= ul1->size; i ++) {
 			SimpleString s = ul1->at [i];
-			TableOfReal_setRowLabel (him.get(), i, s -> string);
+			TableOfReal_setRowLabel (him.get(), i, s -> string.get());
 		}
 		for (integer i = 1; i <= ul2->size; i ++) {
 			SimpleString s = ul2->at [i];
-			TableOfReal_setColumnLabel (him.get(), i, s -> string);
+			TableOfReal_setColumnLabel (him.get(), i, s -> string.get());
 		}
 		for (integer i = 1; i <= my size; i ++) {
 			SimpleString myi = my at [i], thyi = thy at [i];
-			Confusion_increase (him.get(), myi -> string, thyi -> string);
+			Confusion_increase (him.get(), myi -> string.get(), thyi -> string.get());
 		}
 		return him;
 	} catch (MelderError) {
@@ -135,124 +136,60 @@ autoConfusion Categories_to_Confusion (Categories me, Categories thee) {
 	}
 }
 
-#define TINY 1.0e-30
-
-void Confusion_getEntropies (Confusion me, double *p_h, double *p_hx, double *p_hy, double *p_hygx, double *p_hxgy, double *p_uygx, double *p_uxgy, double *p_uxy) {
-	double h = 0.0, hx = 0.0, hy = 0.0, hxgy = 0.0, hygx = 0.0, uygx = 0.0, uxgy = 0.0, uxy = 0.0;
-
-	autoNUMvector<double> rowSum (1, my numberOfRows);
-	autoNUMvector<double> colSum (1, my numberOfColumns);
-
-	double sum = 0.0;
-	for (integer i = 1; i <= my numberOfRows; i ++) {
-		for (integer j = 1; j <= my numberOfColumns; j ++) {
-			rowSum [i] += my data [i] [j];
-			colSum [j] += my data [i] [j];
-			sum += my data [i] [j];
-		}
-	}
-	for (integer i = 1; i <= my numberOfRows; i ++) {
-		if (rowSum [i] > 0.0) {
-			hy -= rowSum [i] / sum * NUMlog2 (rowSum [i] / sum);
-		}
-	}
-	for (integer j = 1; j <= my numberOfColumns; j ++) {
-		if (colSum [j] > 0.0) {
-			hx -= colSum [j] / sum * NUMlog2 (colSum [j] / sum);
-		}
-	}
-	for (integer i = 1; i <= my numberOfRows; i ++) {
-		for (integer j = 1; j <= my numberOfColumns; j ++) {
-			if (my data [i] [j] > 0.0) {
-				h -= my data [i] [j] / sum * NUMlog2 (my data [i] [j] / sum);
-			}
-		}
-	}
-
-	hygx = h - hx;
-	hxgy = h - hy;
-	uygx = (hy - hygx) / (hy + TINY);
-	uxgy = (hx - hxgy) / (hx + TINY);
-	uxy = 2.0 * (hx + hy - h) / (hx + hy + TINY);
-	if (p_h) {
-		*p_h  = h;
-	}
-	if (p_hx) {
-		*p_hx  = hx;
-	}
-	if (p_hy) {
-		*p_hy  = hy;
-	}
-	if (p_hygx) {
-		*p_hygx  = hygx;
-	}
-	if (p_hxgy) {
-		*p_hxgy  = hxgy;
-	}
-	if (p_uygx) {
-		*p_uygx  = uygx;
-	}
-	if (p_uxgy) {
-		*p_uxgy  = uxgy;
-	}
-	if (p_uxy) {
-		*p_uxy  = uxy;
-	}
+void Confusion_getEntropies (Confusion me, double *out_h, double *out_hx, double *out_hy,
+	double *out_hygx, double *out_hxgy, double *out_uygx, double *out_uxgy, double *out_uxy)
+{
+	NUMgetEntropies (my data.get(), out_h, out_hx,
+			out_hy,	out_hygx, out_hxgy, out_uygx, out_uxgy, out_uxy);
 }
 
-void Confusion_increase (Confusion me, const char32 *stim, const char32 *resp) {
+void Confusion_increase (Confusion me, conststring32 stimulus, conststring32 response) {
 	try {
-		integer stimIndex = TableOfReal_rowLabelToIndex (me, stim);
-		Melder_require (stimIndex > 0, U"The stimulus name should be valid.");
+		integer stimulusIndex = TableOfReal_rowLabelToIndex (me, stimulus);
+		Melder_require (stimulusIndex > 0, U"The stimulus name should be valid.");
 		
-		integer respIndex = TableOfReal_columnLabelToIndex (me, resp);
-		Melder_require (respIndex > 0, U"The response name should be valid.");
+		integer responseIndex = TableOfReal_columnLabelToIndex (me, response);
+		Melder_require (responseIndex > 0, U"The response name should be valid.");
 
-		my data [stimIndex] [respIndex] += 1.0;
+		my data [stimulusIndex] [responseIndex] += 1.0;
 	} catch (MelderError) {
 		Melder_throw (me, U": not increased.");
 	}
 }
 
-double Confusion_getValue (Confusion me, const char32 *stim, const char32 *resp) {
-	integer stimIndex = TableOfReal_rowLabelToIndex (me, stim);
+double Confusion_getValue (Confusion me, conststring32 stimulus, conststring32 response) {
+	integer stimulusIndex = TableOfReal_rowLabelToIndex (me, stimulus);
 	
-	Melder_require (stimIndex > 0, U"The stimulus should be valid.");
+	Melder_require (stimulusIndex > 0, U"The stimulus name should be valid.");
 	
-	integer respIndex = TableOfReal_columnLabelToIndex (me, resp);
+	integer responseIndex = TableOfReal_columnLabelToIndex (me, response);
 	
-	Melder_require (respIndex > 0, U"The response should be valid.");
+	Melder_require (responseIndex > 0, U"The response name should be valid.");
 	
-	return my data [stimIndex] [respIndex];
+	return my data [stimulusIndex] [responseIndex];
 }
 
-void Confusion_getFractionCorrect (Confusion me, double *p_fraction, integer *p_numberOfCorrect) {
+void Confusion_getFractionCorrect (Confusion me, double *out_fraction, integer *out_numberOfCorrect) {
 	double fraction = undefined;
 	integer numberOfCorrect = -1;
 
 	double c = 0.0, ct = 0.0;
 	for (integer i = 1; i <= my numberOfRows; i ++) {
 		for (integer j = 1; j <= my numberOfColumns; j ++) {
-			if (! my rowLabels [i] || ! my columnLabels [j]) {
+			if (! my rowLabels [i] || ! my columnLabels [j])
 				return;
-			}
-			ct += my data[i][j];
-			if (str32cmp (my rowLabels [i], my columnLabels [j]) == 0) {
+			ct += my data [i] [j];
+			if (str32equ (my rowLabels [i].get(), my columnLabels [j].get()))
 				c += my data [i] [j];
-			}
 		}
 	}
-
-	if (ct != 0.0) {
+	if (ct != 0.0)
 		fraction = c / ct;
-	}
-	if (p_fraction) {
-		*p_fraction = fraction;
-	}
+	if (out_fraction)
+		*out_fraction = fraction;
 	numberOfCorrect = Melder_ifloor (c);
-	if (p_numberOfCorrect) {
-		*p_numberOfCorrect = numberOfCorrect;
-	}
+	if (out_numberOfCorrect)
+		*out_numberOfCorrect = numberOfCorrect;
 }
 
 /*************** Confusion_Matrix_draw ****************************************/
@@ -265,7 +202,8 @@ static autoPolygon Polygon_createPointer () {
 		double y [NPOINTS + 1] = { 0.0, 0.0, 0.0, 0.5, 1.0, 1.0, 0.0 };
 		autoPolygon me = Polygon_create (NPOINTS);
 		for (integer i = 1; i <= NPOINTS; i ++) {
-			my x [i] = x [i]; my y [i] = y [i];
+			my x [i] = x [i];
+			my y [i] = y [i];
 		}
 		return me;
 	} catch (MelderError) {
@@ -284,21 +222,15 @@ void Confusion_Matrix_draw (Confusion me, Matrix thee, Graphics g, integer index
 	}
 	Melder_require (thy ny == my numberOfRows, U"The number of stimuli should equal the number of rows in the matrix.");
 
-	if (xmax <= xmin) {
-		(void) Matrix_getWindowExtrema (thee, 1, 1, 1, thy ny, &xmin, &xmax);
-	}
-
-	if (xmax <= xmin) {
+	if (xmax <= xmin)
+		(void) Matrix_getWindowExtrema (thee, 1, 1, 1, thy ny, & xmin, & xmax);
+	if (xmax <= xmin)
 		return;
-	}
-
-	if (ymax <= ymin) {
-		(void) Matrix_getWindowExtrema (thee, 2, 2, 1, thy ny, &ymin, &ymax);
-	}
-
-	if (ymax <= ymin) {
+	if (ymax <= ymin)
+		(void) Matrix_getWindowExtrema (thee, 2, 2, 1, thy ny, & ymin, & ymax);
+	if (ymax <= ymin)
 		return;
-	}
+
 	double rmax = fabs (xmax - xmin) / 10.0;
 	double rmin = rmax / 10;
 
@@ -306,23 +238,21 @@ void Confusion_Matrix_draw (Confusion me, Matrix thee, Graphics g, integer index
 	Graphics_setWindow (g, xmin - rmax, xmax + rmax, ymin - rmax, ymax + rmax);
 	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
 	for (integer i = 1; i <= my numberOfRows; i ++) {
-		Graphics_text (g, thy z [i] [1], thy z [i] [2], my rowLabels[i]);
+		Graphics_text (g, thy z [i] [1], thy z [i] [2], my rowLabels [i].get());
 	}
 	for (integer i = ib; i <= ie; i ++) {
 		double xSum = 0.0;
 		for (integer j = 1; j <= my numberOfColumns; j ++) {
 			xSum += my data [i] [j];
 		}
-
 		if (xSum <= 0.0) {
 			continue;    /* no confusions */
 		}
-
 		double x1 = thy z [i] [1];
 		double y1 = thy z [i] [2];
 		double r = rmax * my data [i] [i] / xSum;
 
-		Graphics_circle (g, x1, y1, r > rmin ? r : rmin);
+		Graphics_circle (g, x1, y1, ( r > rmin ? r : rmin ));
 
 		for (integer j = 1; j <= my numberOfColumns; j ++) {
 			double x2 = thy z [j] [1], y2 = thy z [j] [2];
@@ -333,20 +263,22 @@ void Confusion_Matrix_draw (Confusion me, Matrix thee, Graphics g, integer index
 			if (perc == 0.0 || perc < lowerPercentage || j == i) {
 				continue;
 			}
-
-			xmin = x1; xmax = x2;
+			xmin = x1;
+			xmax = x2;
 			if (x2 < x1) {
-				xmin = x2; xmax = x1;
+				xmin = x2;
+				xmax = x1;
 			}
-			ymin = y1; xmax = y2;
+			ymin = y1;
+			xmax = y2;
 			if (y2 < y1) {
-				ymin = y2; ymax = y1;
+				ymin = y2;
+				ymax = y1;
 			}
 			autoPolygon p = Polygon_createPointer();
 			double xs = sqrt (dx * dx + dy * dy) - 2.2 * r;
-			if (xs < 0.0) {
+			if (xs < 0.0)
 				xs = 0.0;
-			}
 			double ys = perc * rmax / 100.0;
 			Polygon_scale (p.get(), xs, ys);
 			Polygon_translate (p.get(), x1, y1 - ys / 2);
@@ -355,9 +287,7 @@ void Confusion_Matrix_draw (Confusion me, Matrix thee, Graphics g, integer index
 			Polygon_drawInside (p.get(), g);
 		}
 	}
-
 	Graphics_unsetInner (g);
-
 	if (garnish) {
 		Graphics_drawInnerBox (g);
 		Graphics_marksBottom (g, 2, true, true, false);
@@ -391,46 +321,49 @@ autoMatrix Confusion_difference (Confusion me, Confusion thee) {
 }
 
 integer Confusion_getNumberOfEntries (Confusion me) {
-	double total = 0.0;
+	longdouble total = 0.0;
 	for (integer i = 1; i <= my numberOfRows; i ++) {
 		for (integer j = 1; j <= my numberOfColumns; j ++) {
 			total += my data [i] [j];
 		}
 	}
-	return Melder_ifloor (total);
+	return Melder_ifloor ((double) total);
 }
 
-static void create_index (char32 **s, integer sb, integer se, char32 **ref, integer rb, integer re, integer *index) {
-	for (integer i = sb; i <= se; i ++) {
+static autoINTVEC create_index (string32vector s, string32vector ref) {
+	autoINTVEC index = newINTVECraw (s.size);
+	for (integer i = 1; i <= s.size; i ++) {
 		integer indxj = 0;
-		for (integer j = rb; j <= re; j ++) {
+		for (integer j = 1; j <= ref.size; j ++) {
 			if (str32equ (s [i], ref [j])) {
-				indxj = j; break;
+				indxj = j;
+				break;
 			}
 		}
 		index [i] = indxj;
 	}
+	return index;
 }
 
-autoConfusion Confusion_condense (Confusion me, const char32 *search, const char32 *replace,
+autoConfusion Confusion_condense (Confusion me, conststring32 search, conststring32 replace,
 	integer maximumNumberOfReplaces, bool use_regexp) {
 	try {
 		integer nmatches, nstringmatches;
-		Melder_require (my rowLabels != nullptr && my columnLabels != nullptr, U"Both row and column labels should be present.");
+		Melder_require (my rowLabels && my columnLabels, U"Both row and column labels should be present.");
 		
-		autostring32vector rowLabels (strs_replace (my rowLabels, 1, my numberOfRows, search, replace,
-			maximumNumberOfReplaces, & nmatches, & nstringmatches, use_regexp), 1, my numberOfRows);
+		autostring32vector rowLabels = string32vector_searchAndReplace (my rowLabels.get(),
+			search, replace, maximumNumberOfReplaces, & nmatches, & nstringmatches, use_regexp);
 
-		autostring32vector columnLabels (strs_replace (my columnLabels, 1, my numberOfColumns,  search, replace,
-			 maximumNumberOfReplaces, & nmatches, & nstringmatches, use_regexp), 1, my numberOfColumns);
+		autostring32vector columnLabels = string32vector_searchAndReplace (my columnLabels.get(),
+			search, replace, maximumNumberOfReplaces, & nmatches, & nstringmatches, use_regexp);
 
 		autoStrings srow = Thing_new (Strings);
 		srow -> numberOfStrings = my numberOfRows;
-		srow -> strings = rowLabels.transfer();
+		srow -> strings = std::move (rowLabels);
 
 		autoStrings scol = Thing_new (Strings);
 		scol -> numberOfStrings = my numberOfColumns;
-		scol -> strings = columnLabels.transfer();
+		scol -> strings = std::move (columnLabels);
 
 		/* Find dimension of new Confusion */
 		autoDistributions dcol = Strings_to_Distributions (scol.get());
@@ -441,13 +374,11 @@ autoConfusion Confusion_condense (Confusion me, const char32 *search, const char
 
 		autoConfusion thee = Confusion_create (nstim, nresp);
 
-		NUMstrings_copyElements (drow -> rowLabels, thy rowLabels, 1, nstim);
-		NUMstrings_copyElements (dcol -> rowLabels, thy columnLabels, 1, nresp);
+		thy rowLabels.all() <<= drow -> rowLabels.all();
+		thy columnLabels.all() <<= dcol -> rowLabels.all();
 
-		autoNUMvector<integer> rowIndex (1, my numberOfRows);
-		create_index (srow -> strings, 1, my numberOfRows, drow -> rowLabels, 1, nstim, rowIndex.peek());
-		autoNUMvector<integer> columnIndex (1, my numberOfColumns);
-		create_index (scol -> strings, 1, my numberOfColumns, dcol -> rowLabels, 1, nresp, columnIndex.peek());
+		autoINTVEC rowIndex = create_index (srow -> strings.get(), drow -> rowLabels.get());
+		autoINTVEC columnIndex = create_index (scol -> strings.get(), dcol -> rowLabels.get());
 
 		for (integer i = 1; i <= my numberOfRows; i ++) {
 			for (integer j = 1; j <= my numberOfColumns; j ++) {
@@ -462,7 +393,7 @@ autoConfusion Confusion_condense (Confusion me, const char32 *search, const char
 
 autoConfusion TableOfReal_to_Confusion (TableOfReal me) {
 	try {
-		Melder_require (TableOfReal_checkPositive (me), U"Elements should not be negative.");
+		Melder_require (TableOfReal_checkNonNegativity (me), U"Elements should not be negative.");
 		
 		autoConfusion thee = Thing_new (Confusion);
 		my structTableOfReal :: v_copy (thee.get());
@@ -472,7 +403,7 @@ autoConfusion TableOfReal_to_Confusion (TableOfReal me) {
 	}
 }
 
-autoConfusion Confusion_group (Confusion me, const char32 *labels, const char32 *newLabel, integer newpos) {
+autoConfusion Confusion_group (Confusion me, conststring32 labels, conststring32 newLabel, integer newpos) {
 	try {
 		autoConfusion stim = Confusion_groupStimuli (me, labels, newLabel, newpos);
 		autoConfusion thee = Confusion_groupResponses (stim.get(), labels, newLabel, newpos);
@@ -482,18 +413,18 @@ autoConfusion Confusion_group (Confusion me, const char32 *labels, const char32
 	}
 }
 
-autoConfusion Confusion_groupStimuli (Confusion me, const char32 *labels, const char32 *newLabel, integer newpos) {
+autoConfusion Confusion_groupStimuli (Confusion me, conststring32 labels_string, conststring32 newLabel, integer newpos) {
 	try {
-		integer ncondense = Melder_countTokens (labels);
-		autoNUMvector<integer> irow (1, my numberOfRows);
+		autostring32vector labels = newSTRVECtokenize (labels_string);
+		integer ncondense = labels.size;
+		autoINTVEC irow = newINTVECraw (my numberOfRows);
 
-		for (integer i = 1; i <= my numberOfRows; i ++) {
+		for (integer i = 1; i <= my numberOfRows; i ++)
 			irow [i] = i;
-		}
-
-		for (char32 *token = Melder_firstToken (labels); token != nullptr; token = Melder_nextToken ()) {
+		for (integer itoken = 1; itoken <= labels.size; itoken ++) {
+			conststring32 token = labels [itoken].get();
 			for (integer i = 1; i <= my numberOfRows; i ++) {
-				if (Melder_equ (token, my rowLabels [i])) {
+				if (Melder_equ (token, my rowLabels [i].get())) {
 					irow [i] = 0;
 					break;
 				}
@@ -501,40 +432,33 @@ autoConfusion Confusion_groupStimuli (Confusion me, const char32 *labels, const
 		}
 		integer nfound = 0;
 		for (integer i = 1; i <= my numberOfRows; i ++) {
-			if (irow [i] == 0) {
+			if (irow [i] == 0)
 				nfound ++;
-			}
 		}
 		Melder_require (nfound > 0, U"The stimulus labels are invalid.");
 		
-		if (nfound != ncondense) {
+		if (nfound != ncondense)
 			Melder_warning (U"One or more of the given stimulus labels are suspect.");
-		}
 		integer newnstim = my numberOfRows - nfound + 1;
-		if (newpos < 1) {
+		if (newpos < 1)
 			newpos = 1;
-		}
-		if (newpos > newnstim) {
+		if (newpos > newnstim)
 			newpos = newnstim;
-		}
 		autoConfusion thee = Confusion_create (newnstim, my numberOfColumns);
-		NUMstrings_copyElements (my columnLabels, thy columnLabels, 1, my numberOfColumns);
-
+		thy columnLabels.all() <<= my columnLabels.all();
 		TableOfReal_setRowLabel (thee.get(), newpos, newLabel);
 		integer inewrow = 1;
 		for (integer i = 1; i <= my numberOfRows; i ++) {
 			integer rowpos = newpos;
 			if (irow [i] > 0) {
-				if (inewrow == newpos) {
-					inewrow++;
-				}
+				if (inewrow == newpos)
+					inewrow ++;
 				rowpos = inewrow;
-				inewrow++;
-				TableOfReal_setRowLabel (thee.get(), rowpos, my rowLabels[i]);
+				inewrow ++;
+				TableOfReal_setRowLabel (thee.get(), rowpos, my rowLabels [i].get());
 			}
-			for (integer j = 1; j <= my numberOfColumns; j ++) {
+			for (integer j = 1; j <= my numberOfColumns; j ++)
 				thy data [rowpos] [j] += my data [i] [j];
-			}
 		}
 		return thee;
 	} catch (MelderError) {
@@ -542,18 +466,18 @@ autoConfusion Confusion_groupStimuli (Confusion me, const char32 *labels, const
 	}
 }
 
-autoConfusion Confusion_groupResponses (Confusion me, const char32 *labels, const char32 *newLabel, integer newpos) {
+autoConfusion Confusion_groupResponses (Confusion me, conststring32 labels_string, conststring32 newLabel, integer newpos) {
 	try {
-		integer ncondense = Melder_countTokens (labels);
-		autoNUMvector<integer> icol (1, my numberOfColumns);
+		autostring32vector labels = newSTRVECtokenize (labels_string);
+		integer ncondense = labels.size;
+		autoINTVEC icol = newINTVECraw (my numberOfColumns);
 
-		for (integer i = 1; i <= my numberOfColumns; i ++) {
+		for (integer i = 1; i <= my numberOfColumns; i ++)
 			icol [i] = i;
-		}
-
-		for (char32 *token = Melder_firstToken (labels); token != 0; token = Melder_nextToken ()) {
+		for (integer itoken = 1; itoken <= labels.size; itoken ++) {
+			conststring32 token = labels [itoken].get();
 			for (integer i = 1; i <= my numberOfColumns; i ++) {
-				if (Melder_equ (token, my columnLabels [i])) {
+				if (Melder_equ (token, my columnLabels [i].get())) {
 					icol [i] = 0;
 					break;
 				}
@@ -561,40 +485,33 @@ autoConfusion Confusion_groupResponses (Confusion me, const char32 *labels, cons
 		}
 		integer nfound = 0;
 		for (integer i = 1; i <= my numberOfColumns; i ++) {
-			if (icol [i] == 0) {
+			if (icol [i] == 0)
 				nfound ++;
-			}
 		}
-		
 		Melder_require (nfound > 0, U"The response labels are invalid.");
 		
-		if (nfound != ncondense) {
+		if (nfound != ncondense)
 			Melder_warning (U"One or more of the given response labels are suspect.");
-		}
 		integer newnresp = my numberOfColumns - nfound + 1;
-		if (newpos < 1) {
+		if (newpos < 1)
 			newpos = 1;
-		}
-		if (newpos > newnresp) {
+		if (newpos > newnresp)
 			newpos = newnresp;
-		}
 		autoConfusion thee = Confusion_create (my numberOfRows, newnresp);
-		NUMstrings_copyElements (my rowLabels, thy rowLabels, 1, my numberOfRows);
+		thy rowLabels.all() <<= my rowLabels.all();
 		TableOfReal_setColumnLabel (thee.get(), newpos, newLabel);
 		integer inewcol = 1;
 		for (integer i = 1; i <= my numberOfColumns; i ++) {
 			integer colpos = newpos;
 			if (icol [i] > 0) {
-				if (inewcol == newpos) {
+				if (inewcol == newpos)
 					inewcol ++;
-				}
 				colpos = inewcol;
 				inewcol ++;
-				TableOfReal_setColumnLabel (thee.get(), colpos, my columnLabels [i]);
+				TableOfReal_setColumnLabel (thee.get(), colpos, my columnLabels [i].get());
 			}
-			for (integer j = 1; j <= my numberOfRows; j ++) {
+			for (integer j = 1; j <= my numberOfRows; j ++)
 				thy data [j] [colpos] += my data [j] [i];
-			}
 		}
 		return thee;
 	} catch (MelderError) {
@@ -606,29 +523,28 @@ autoTableOfReal Confusion_to_TableOfReal_marginals (Confusion me) {
 	try {
 		autoTableOfReal thee = TableOfReal_create (my numberOfRows + 1, my numberOfColumns + 1);
 
-		double total = 0.0;
+		longdouble total = 0.0;
 		for (integer i = 1; i <= my numberOfRows; i ++) {
-			double rowsum = 0.0;
+			longdouble rowSum = 0.0;
 			for (integer j = 1; j <= my numberOfColumns; j ++) {
 				thy data [i] [j] = my data [i] [j];
-				rowsum += my data [i] [j];
+				rowSum += my data [i] [j];
 			}
-			thy data [i] [my numberOfColumns + 1] = rowsum;
-			total += rowsum;
+			thy data [i] [my numberOfColumns + 1] = (double) rowSum;
+			total += rowSum;
 		}
 
-		thy data [my numberOfRows + 1] [my numberOfColumns + 1] = total;
+		thy data [my numberOfRows + 1] [my numberOfColumns + 1] = (double) total;
 
 		for (integer j = 1; j <= my numberOfColumns; j ++) {
-			double colsum = 0;
-			for (integer i = 1; i <= my numberOfRows; i ++) {
-				colsum += my data [i] [j];
-			}
-			thy data [my numberOfRows + 1] [j] = colsum;
+			longdouble columnSum = 0.0;
+			for (integer i = 1; i <= my numberOfRows; i ++)
+				columnSum += my data [i] [j];
+			thy data [my numberOfRows + 1] [j] = (double) columnSum;
 		}
 
-		NUMstrings_copyElements (my rowLabels, thy rowLabels, 1, my numberOfRows);
-		NUMstrings_copyElements (my columnLabels, thy columnLabels, 1, my numberOfColumns);
+		thy rowLabels.part (1, my numberOfRows) <<= my rowLabels.all();
+		thy columnLabels.part (1, my numberOfColumns) <<= my columnLabels.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": table with marginals not created.");
