@@ -130,36 +130,28 @@ static void DTW_findPath_special (DTW me, bool matchStart, bool matchEnd, int sl
 
 /* DTW_getXTime (DTW me, (DTW_getYTime (DTW me, double tx)) == tx */
 double DTW_getYTimeFromXTime (DTW me, double tx) {
-	// Catch cases where tier would give constant extrapolation
-	if (tx < my xmin) {
-		return my ymin - (my xmin - tx);
+	double time = tx; // Catch cases where tier would give constant extrapolation
+	if (tx > my xmin && tx < my xmax) {
+		DTW_Path_Query thee = & my pathQuery;
+		time = RealTier_getValueAtTime (thy yfromx.get(), tx);
 	}
-	if (tx > my xmax) {
-		return my ymax + (tx - my xmax);
-	}
-	DTW_Path_Query thee = & my pathQuery;
-	return RealTier_getValueAtTime(thy yfromx.get(), tx);
-
+	return time;
 }
 
 double DTW_getXTimeFromYTime (DTW me, double ty) {
-	// Catch cases where tier would give constant extrapolation
-	if (ty < my ymin) {
-		return my ymin - (my ymin - ty);
+	double time = ty; // Catch cases where tier would give constant extrapolation
+	if (ty > my ymin && ty < my ymax) {
+		DTW_Path_Query thee = & my pathQuery;
+		time = RealTier_getValueAtTime (thy xfromy.get(), ty);
 	}
-	if (ty > my ymax) {
-		return my ymax + (ty - my ymax);
-	}
-
-	DTW_Path_Query thee = & my pathQuery;
-	return RealTier_getValueAtTime(thy xfromy.get(), ty);
+	return time;
 }
 
 void DTW_Path_Query_init (DTW_Path_Query me, integer ny, integer nx) {
 	Melder_assert (ny > 0 && nx > 0);
 	my ny = ny;
 	my nx = nx;
-	my nxy = 2 * (ny > nx ? ny : nx) + 2; // maximum number of points
+	my nxy = 2 * std::max (ny, nx) + 2; // maximum number of points
 	my xfromy = Thing_new (RealTier);
 	my yfromx = Thing_new (RealTier);
 }
@@ -179,19 +171,24 @@ void DTW_Path_recode (DTW me) {
 			double x,y;
 		};
 
-		/* 1. Starting point always at origin */
+		// 1. Starting point always at origin
+		
 		integer nxymax = thy nx + thy ny + 2;
 		autoNUMvector<struct structPoint> xytimes (1, nxymax);
 		xytimes [1]. x = my xmin;
 		xytimes [1]. y = my ymin;
-		/* 2. next point lower left of first cell */
+		
+		// 2. next point lower left of first cell
+		
 		nsc_x = my path [1]. x;
 		ixp = nsc_x - 1;
 		xytimes [2]. x = my x1 + (nsc_x - 1 - 0.5) * my dx;
 		nsc_y = my path [1]. y;
 		iyp = nsc_y - 1;
 		xytimes [2]. y = my y1 + (nsc_y - 1 - 0.5) * my dy;
-		/* 3. follow all cells. implicit: my x1 - 0.5 * my dx > my xmin && my y1 - 0.5 * my dy > my ymin */
+		
+		// 3. follow all cells. implicit: my x1 - 0.5 * my dx > my xmin && my y1 - 0.5 * my dy > my ymin */
+		
 		nxy = 2;
 		for (integer j = 1; j <= my pathLength; j ++) {
 			integer index; // where are we in the new path?
@@ -201,7 +198,7 @@ void DTW_Path_recode (DTW me) {
 
 			if (iy == iyp) { // horizontal path?
 				xDirection = true;
-				if (yDirection) { // we came from vertical direction?
+				if (yDirection) {
 					// We came from a vertical direction so this is the second horizontal cell in a row.
 					// The statement after this "if" updates nsc_x to 2.
 					nsc_x = 1;
@@ -217,10 +214,8 @@ void DTW_Path_recode (DTW me) {
 					x = xright - nsc_x * my dx + nsc_x * my dx * f;
 					y = ytop - my dy + my dy * f;
 					index = nxy - 1;
-					if (nsc_x == 2) {
-						index = nxy;
-						nxy ++;
-					}
+					if (nsc_x == 2) index = nxy ++;
+
 					xytimes [index]. x = x;
 					xytimes [index]. y = y;
 				}
@@ -239,19 +234,14 @@ void DTW_Path_recode (DTW me) {
 					x = xright - my dx + my dx * f;
 					y = ytop - nsc_y * my dy + nsc_y * my dy * f;
 					index = nxy - 1;
-					if (nsc_y == 2) {
-						index = nxy;
-						nxy ++;
-					}
+					if (nsc_y == 2) index = nxy ++;
 					xytimes [index]. x = x;
 					xytimes [index]. y = y;
 				}
 				nd = 0;
 			} else if (ix == ixp + 1 && iy == iyp + 1) { // diagonal
 				nd ++;
-				if (nd == 1) {
-					nxy ++;
-				}
+				if (nd == 1) nxy ++;
 				nsc_x = nsc_y = 1;
 			} else {
 				Melder_throw (U"The path goes back in time.");
@@ -264,11 +254,10 @@ void DTW_Path_recode (DTW me) {
 		}
 
 		if (my xmax > xytimes [nxy].x || my ymax > xytimes [nxy].y) {
-			nxy ++;
-			xytimes [nxy].x = my xmax;
+			xytimes [++ nxy].x = my xmax;
 			xytimes [nxy].y = my ymax;
 		}
-		Melder_assert (nxy <= 2 * (my ny > my nx ? my ny : my nx) + 2);
+		Melder_assert (nxy <= 2 * std::max (my ny, my nx) + 2);
 		
 		thy nxy = nxy;
 		thy yfromx = RealTier_create (my xmin, my xmax);
@@ -277,145 +266,31 @@ void DTW_Path_recode (DTW me) {
 			RealTier_addPoint (thy yfromx.get(), xytimes [i]. x, xytimes [i]. y);
 			RealTier_addPoint (thy xfromy.get(), xytimes [i]. y, xytimes [i]. x);
 		}
-		//DTW_Path_makeIndex (me, DTW_X);
-		//DTW_Path_makeIndex (me, DTW_Y);
 	} catch (MelderError) {
 		Melder_throw (me, U": not recoded.");
 	}
 }
 
-#if 0
-void DTW_Path_recode (DTW me) {
-	DTW_Path_Query thee = & my pathQuery;
-	integer nxy;		// current number of elements in recoded path
-	integer nx = 1;	// current number of successive horizontal cells in the cells chain
-	integer ny = 1;	// current number of successive vertical cells in the cells chain
-	integer nd = 0;	// current number of successive diagonal cells in the cells chain
-	int isv = 0;	// previous segment in the original path was vertical
-	int ish = 0;	// previous segment in the original path was horizontal
-	integer ixp = 0, iyp = 0; // previous cell
-
-	/* Algorithm
-		1: get all the points in the path
-		2. get the x and y indices
-	*/
-
-	/* 1. Starting point always at origin */
-	thy xytimes [1]. x = my xmin;
-	thy xytimes [1]. y = my ymin;
-	nx = my path [1]. x;
-	ixp = nx - 1;
-	thy xytimes [2]. x = my x1 + (nx - 1 - 0.5) * my dx;
-	ny = my path [1]. y;
-	iyp = ny - 1;
-	thy xytimes [2]. y = my y1 + (ny - 1 - 0.5) * my dy;
-	// implicit: my x1 - 0.5 * my dx > my xmin && my y1 - 0.5 * my dy > my ymin
-	nxy = 2;
-	for (integer j = 1; j <= my pathLength; j ++) {
-		integer index; // where are we in the new path?
-		integer ix = my path [j]. x, iy = my path [j]. y;
-		double xright = my x1 + (ix - 1 + 0.5) * my dx;
-		double x, y, f, ytop = my y1 + (iy - 1 + 0.5) * my dy;
-
-		if (iy == iyp) { // horizontal path?
-			ish = 1;
-			if (isv) { // we came from vertical direction?
-				// We came from a vertical direction so this is the second horizontal cell in a row.
-				// The statement after this "if" updates nx to 2.
-				nx = 1; isv = 0;
-			}
-			nx ++;
-
-			if (ny > 1 || nd > 1) {
-				// Previous segment was diagonal or vertical: modify intersection
-				// The vh intersection (x,y) = (nx*dx, dy) * (ny-1)/(nx*ny-1)
-				// A diagonal segment has ny = 1.
-				f = (ny - 1.0) / (nx * ny - 1);
-				x = xright - nx * my dx + nx * my dx * f;
-				y = ytop - my dy + my dy * f;
-				index = nxy - 1;
-				if (nx == 2) {
-					index = nxy;
-					nxy ++;
-				}
-				thy xytimes [index]. x = x;
-				thy xytimes [index]. y = y;
-			}
-			nd = 0;
-		} else if (ix == ixp) { // vertical
-			isv = 1;
-			if (ish) {
-				ny = 1;
-				ish = 0;
-			}
-			ny ++;
-
-			if (nx > 1 || nd > 1) {
-				// The hv intersection (x,y) = (dx, dy*ny ) * (nx-1)/(nx*ny-1)
-				f = (nx - 1.0) / (nx * ny - 1);
-				x = xright - my dx + my dx * f;
-				y = ytop - ny * my dy + ny * my dy * f;
-				index = nxy - 1;
-				if (ny == 2) {
-					index = nxy;
-					nxy ++;
-				}
-				thy xytimes [index]. x = x;
-				thy xytimes [index]. y = y;
-			}
-			nd = 0;
-		} else if (ix == ixp + 1 && iy == iyp + 1) { // diagonal
-			nd ++;
-			if (nd == 1) {
-				nxy ++;
-			}
-			nx = ny = 1;
-		} else {
-			Melder_throw (U"The path goes back in time.");
-		}
-		// update
-		thy xytimes [nxy]. x = xright;
-		thy xytimes [nxy]. y = ytop;
-		ixp = ix;
-		iyp = iy;
-	}
-
-	if (my xmax > thy xytimes [nxy]. x || my ymax > thy xytimes [nxy]. y) {
-		nxy ++;
-		thy xytimes [nxy]. x = my xmax;
-		thy xytimes [nxy]. y = my ymax;
-	}
-	Melder_assert (nxy <= 2 * (my ny > my nx ? my ny : my nx) + 2);
-	thy nxy = nxy;
-
-	DTW_Path_makeIndex (me, DTW_X);
-	DTW_Path_makeIndex (me, DTW_Y);
-}
-#endif
-
 void DTW_pathRemoveRedundantNodes (DTW me) {
 	integer i = 1, skip = 0;
 
 	for (integer j = 2; j <= my pathLength; j ++) {
-		if ( (my path [j]. y == my path [i]. y) || my path [j]. x == my path [i]. x) {
+		if (my path [j]. y == my path [i]. y || my path [j]. x == my path [i]. x)
 			skip ++;
-		} else {
+		else {
 			/* if (j-1)^th was the last of a series: store it */
-			if (skip > 0) {
+			if (skip > 0)
 				my path [++ i] = my path [j - 1];
-			}
 			/* same check again */
 			skip = 0;
-			if ( (my path [j]. y == my path [i]. y) || my path [j]. x == my path [i]. x) {
+			if (my path [j]. y == my path [i]. y || my path [j]. x == my path [i]. x)
 				skip ++;
-			} else {
+			else 
 				my path [++ i] = my path [j];
-			}
 		}
 	}
-	if (skip > 0) {
+	if (skip > 0)
 		my path [++ i] = my path [my pathLength];
-	}
 	my pathLength = i;
 }
 
@@ -445,12 +320,9 @@ void DTW_setWeights (DTW me, double wx, double wy, double wd) {
 autoDTW DTW_swapAxes (DTW me) {
 	try {
 		autoDTW thee = DTW_create (my xmin, my xmax, my nx, my dx, my x1, my ymin, my ymax, my ny, my dy, my y1);
-
-		for (integer x = 1; x <= my nx; x ++) {
-			for (integer y = 1; y <= my ny; y ++) {
-				thy z [x] [y] = my z [y] [x];
-			}
-		}
+		
+		MATtranspose_preallocated (thy z.get(), my z.get());
+		
 		thy pathLength = my pathLength;
 		for (integer i = 1; i <= my pathLength; i ++) {
 			thy path [i]. x = my path [i]. y;
@@ -466,32 +338,27 @@ double DTW_getPathY (DTW me, double tx) {
 	// Assume linear behaviour outside domains
 	//  Other option: scale with x_domain/y_domain
 
-	if (tx < my xmin) {
+	if (tx < my xmin)
 		return my ymin - (my xmin - tx);
-	}
-	if (tx > my xmax) {
+	if (tx > my xmax)
 		return my ymax + (tx - my xmax);
-	}
+
 
 	// Find column in DTW matrix
 
 	integer ix = Melder_ifloor ((tx - my x1) / my dx) + 1;
-	if (ix < 1) {
-		ix = 1;
-	}
-	if (ix > my nx) {
-		ix = my nx;
-	}
+	if (ix < 1) ix = 1;
+	if (ix > my nx) ix = my nx;
 
 	// Find index in the path and the row number (iy)
 
 	integer i = ix + my path [1]. x - 1;
-	while (i <= my pathLength && my path [i]. x != ix) {
+	while (i <= my pathLength && my path [i]. x != ix)
 		i ++;
-	}
-	if (i > my pathLength) {
+
+	if (i > my pathLength)
 		return undefined;
-	}
+
 	integer iy = my path [i]. y; /* row */
 
 	/*
@@ -513,41 +380,38 @@ double DTW_getPathY (DTW me, double tx) {
 	// Horizontal path? Find left and right positions.
 
 	integer ileft = i - 1;
-	while (ileft >= 1 && my path [ileft]. y == iy) {
+	while (ileft >= 1 && my path [ileft]. y == iy)
 		ileft --;
-	}
+
 	ileft ++;
-	if (ileft == 1 && ix > 1 && my path [ileft]. y > 1) {
+	if (ileft == 1 && ix > 1 && my path [ileft]. y > 1)
 		ileft ++;
-	}
 
 	integer iright = i + 1;
-	while (iright <= my pathLength && my path [iright]. y == iy) {
+	while (iright <= my pathLength && my path [iright]. y == iy)
 		iright ++;
-	}
+
 	iright --;
-	if (iright == my pathLength && ix < my nx && my path [iright]. y < my ny) {
+	if (iright == my pathLength && ix < my nx && my path [iright]. y < my ny)
 		iright --;
-	}
 
 	integer nxx = iright - ileft + 1;
 
 	// Vertical path? Only if not already horizontal.
 
-	integer ibottom = i;
-	integer itop = i;
+	integer ibottom = i, itop = i;
 
 	if (nxx == 1) {
 		ibottom --;
-		while (ibottom >= 1 && my path [ibottom]. x == ix) {
+		while (ibottom >= 1 && my path [ibottom]. x == ix)
 			ibottom --;
-		}
+
 		ibottom ++;
 
 		itop ++;
-		while (itop <= 1 && my path [itop]. x == ix) {
+		while (itop <= 1 && my path [itop]. x == ix)
 			itop --;
-		}
+
 		itop ++;
 	}
 
@@ -601,8 +465,7 @@ integer DTW_getMaximumConsecutiveSteps (DTW me, int direction) {
 	return nglobal;
 }
 
-static void DTW_paintDistances_raw (DTW me, Graphics g, double xmin, double xmax, double ymin,
-                                    double ymax, double minimum, double maximum, bool garnish, bool inset) {
+static void DTW_paintDistances_raw (DTW me, Graphics g, double xmin, double xmax, double ymin, double ymax, double minimum, double maximum, bool garnish, bool inset) {
 	integer ixmin, ixmax, iymin, iymax;
 	if (xmax <= xmin) {
 		xmin = my xmin;
@@ -621,18 +484,18 @@ static void DTW_paintDistances_raw (DTW me, Graphics g, double xmin, double xmax
 		minimum -= 1.0;
 		maximum += 1.0;
 	}
-	if (xmin >= xmax || ymin >= ymax) {
+	if (xmin >= xmax || ymin >= ymax)
 		return;
-	}
-	if (inset) {
+	if (inset)
 		Graphics_setInner (g);
-	}
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
-	Graphics_cellArray (g, my z, ixmin, ixmax, Matrix_columnToX (me, ixmin - 0.5), Matrix_columnToX (me, ixmax + 0.5), iymin, iymax, Matrix_rowToY (me, iymin - 0.5), Matrix_rowToY (me, iymax + 0.5), minimum, maximum);
+	Graphics_cellArray (g, my z.part (iymin, iymax, ixmin, ixmax),
+			Matrix_columnToX (me, ixmin - 0.5), Matrix_columnToX (me, ixmax + 0.5),
+			Matrix_rowToY (me, iymin - 0.5), Matrix_rowToY (me, iymax + 0.5),
+			minimum, maximum);
 	Graphics_rectangle (g, xmin, xmax, ymin, ymax);
-	if (inset) {
+	if (inset)
 		Graphics_unsetInner (g);
-	}
 	if (garnish) {
 		Graphics_marksBottom (g, 2, true, true, false);
 		Graphics_marksLeft (g, 2, true, true, false);
@@ -645,9 +508,8 @@ void DTW_paintDistances (DTW me, Graphics g, double xmin, double xmax, double ym
 
 static double RealTier_getXAtIndex (RealTier me, integer point) {
 	double x = undefined;
-	if (point > 0 && point <= my points.size) {
+	if (point > 0 && point <= my points.size)
 		x = my points.at [point] -> number;
-	}
 	return x;
 }
 
@@ -655,15 +517,17 @@ static void DTW_drawPath_raw (DTW me, Graphics g, double xmin, double xmax, doub
 	DTW_Path_Query thee = & my pathQuery;
 
 	if (xmin >= xmax) {
-		xmin = my xmin; xmax = my xmax;
+		xmin = my xmin;
+		xmax = my xmax;
 	}
 	if (ymin >= ymax) {
-		ymin = my ymin; ymax = my ymax;
+		ymin = my ymin;
+		ymax = my ymax;
 	}
 
-	if (inset) {
+	if (inset)
 		Graphics_setInner (g);
-	}
+
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
 	double x1 = RealTier_getXAtIndex (thy yfromx.get(), 1);
 	double y1 = RealTier_getValueAtIndex (thy yfromx.get(), 1);
@@ -671,16 +535,16 @@ static void DTW_drawPath_raw (DTW me, Graphics g, double xmin, double xmax, doub
 		double x2 = RealTier_getXAtIndex (thy yfromx.get(), i);
 		double y2 = RealTier_getValueAtIndex (thy yfromx.get(), i);
 		double xc1, yc1, xc2, yc2;
-		if (NUMclipLineWithinRectangle (x1, y1, x2, y2, xmin, ymin, xmax, ymax, & xc1, & yc1, & xc2, & yc2)) {
+		if (NUMclipLineWithinRectangle (x1, y1, x2, y2, xmin, ymin, xmax, ymax, & xc1, & yc1, & xc2, & yc2))
 			Graphics_line (g, xc1, yc1, xc2, yc2);
-		}
+
 		x1 = x2; 
 		y1 = y2;
 	}
 
-	if (inset) {
+	if (inset)
 		Graphics_unsetInner (g);
-	}
+
 	if (garnish) {
 		Graphics_drawInnerBox (g);
 		Graphics_marksBottom (g, 2, true, true, false);
@@ -693,20 +557,22 @@ void DTW_drawPath (DTW me, Graphics g, double xmin, double xmax, double ymin, do
 }
 
 static void DTW_drawWarp_raw (DTW me, Graphics g, double xmin, double xmax, double ymin, double ymax, double t, bool garnish, bool inset, bool warpX) {
-	double tx = warpX ? t : DTW_getXTimeFromYTime (me, t);
-	double ty = warpX ? DTW_getYTimeFromXTime (me, t) : t;
+	double tx = ( warpX ? t : DTW_getXTimeFromYTime (me, t) );
+	double ty = ( warpX ? DTW_getYTimeFromXTime (me, t) : t );
 	int lineType = Graphics_inqLineType (g);
 
 	if (xmin >= xmax) {
-		xmin = my xmin; xmax = my xmax;
+		xmin = my xmin;
+		xmax = my xmax;
 	}
 	if (ymin >= ymax) {
-		ymin = my ymin; ymax = my ymax;
+		ymin = my ymin;
+		ymax = my ymax;
 	}
 
-	if (inset) {
+	if (inset)
 		Graphics_setInner (g);
-	}
+
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
 
 	Graphics_setLineType (g, Graphics_DOTTED);
@@ -719,15 +585,14 @@ static void DTW_drawWarp_raw (DTW me, Graphics g, double xmin, double xmax, doub
 
 	Graphics_setLineType (g, lineType);
 
-	if (inset) {
+	if (inset)
 		Graphics_unsetInner (g);
-	}
+
 
 	if (garnish) {
 		Graphics_markBottom (g, tx, true, true, false, nullptr);
-		if (ty <= ymax) {
+		if (ty <= ymax)
 			Graphics_markLeft (g, ty, true, true, false, nullptr);
-		}
 	}
 }
 
@@ -766,17 +631,16 @@ static void drawBox (Graphics g) {
 
 /*
   In a picture with a DTW and a left and bottom Sound, we want "width" of the vertical sound
-  and the "height" of the horizontal Sound t be equal.
+  and the "height" of the horizontal Sound to be equal.
   Given the horizontal fraction of the DTW-part, this routine returns the vertical part.
 */
 static double _DTW_Sounds_getPartY (Graphics g, double dtw_part_x) {
 	double x1NDC, x2NDC, y1NDC, y2NDC;
 	Graphics_inqViewport (g, & x1NDC, & x2NDC, & y1NDC, & y2NDC);
-	return 1.0 - ((1.0 - dtw_part_x) * (x2NDC - x1NDC)) / (y2NDC - y1NDC);
+	return 1.0 - (1.0 - dtw_part_x) * (x2NDC - x1NDC) / (y2NDC - y1NDC);
 }
 
-void DTW_Sounds_draw (DTW me, Sound y, Sound x, Graphics g, double xmin, double xmax, double ymin, double ymax, bool garnish)
-{
+void DTW_Sounds_draw (DTW me, Sound y, Sound x, Graphics g, double xmin, double xmax, double ymin, double ymax, bool garnish) {
 	DTW_Sounds_checkDomains (me, & y, & x, & xmin, & xmax, & ymin, & ymax);
 
 	Graphics_setInner (g);
@@ -797,18 +661,16 @@ void DTW_Sounds_draw (DTW me, Sound y, Sound x, Graphics g, double xmin, double
 
 	vp = Graphics_insetViewport (g, 0.0, 1.0 - dtw_part_x, 1.0 - dtw_part_y, 1.0);
 	Sound_draw_btlr (y, g, ymin, ymax, -1.0, 1.0, FROM_BOTTOM_TO_TOP, 0);
-	if (garnish) {
+	if (garnish)
 		drawBox (g);
-	}
 	Graphics_resetViewport (g, vp);
 
 	// Sound x
 
 	vp = Graphics_insetViewport (g, 1 - dtw_part_x, 1, 0, 1 - dtw_part_y);
 	Sound_draw_btlr (x, g, xmin, xmax, -1.0, 1.0, FROM_LEFT_TO_RIGHT, 0);
-	if (garnish) {
+	if (garnish)
 		drawBox (g);
-	}
 	Graphics_resetViewport (g, vp);
 
 
@@ -863,7 +725,7 @@ void DTW_Sounds_drawWarpX (DTW me, Sound yy, Sound xx, Graphics g, double xmin,
 autoMatrix DTW_to_Matrix_distances (DTW me) {
 	try {
 		autoMatrix thee = Matrix_create (my xmin, my xmax, my nx, my dx, my x1, my ymin, my ymax, my ny, my dy, my y1);
-		NUMmatrix_copyElements (my z, thy z, 1, my ny, 1, my nx);
+		thy z.all() <<= my z.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": distances not converted to Matrix.");
@@ -873,45 +735,36 @@ autoMatrix DTW_to_Matrix_distances (DTW me) {
 /* nog aanpassen, dl = sqrt (dx^2 + dy^2) */
 void DTW_drawDistancesAlongPath (DTW me, Graphics g, double xmin, double xmax, double dmin, double dmax, bool garnish) {
 	if (xmin >= xmax) {
-		xmin = my xmin; xmax = my xmax;
+		xmin = my xmin;
+		xmax = my xmax;
 	}
 	integer ixmax, ixmin;
-	if (! Matrix_getWindowSamplesX (me, xmin, xmax, & ixmin, & ixmax)) {
+	if (Matrix_getWindowSamplesX (me, xmin, xmax, & ixmin, & ixmax) == 0)
 		return;
-	}
 
 	integer ii = 1;
-	while (ii < my pathLength && my path [ii]. x < ixmin) {
+	while (ii < my pathLength && my path [ii]. x < ixmin)
 		ii ++;
-	}
 	ixmin = ii;
 
-	while (ii <= my pathLength && my path [ii]. x < ixmax) {
+	while (ii <= my pathLength && my path [ii]. x < ixmax)
 		ii ++;
-	}
 	ixmax = ii;
 
-	autoNUMvector<double> d (ixmin, ixmax);
+	long numberOfSelected = ixmax - ixmin + 1;
+	autoVEC d = newVECraw (numberOfSelected);
 
-	for (integer i = ixmin; i <= ixmax; i ++) {
-		d [i] = my z [my path [i]. y] [i];
-	}
+	for (integer i = ixmin; i <= ixmax; i ++)
+		d [i - ixmin + 1] = my z [my path [i]. y] [i];
 
-	if (dmin >= dmax) {
-		NUMvector_extrema (d.peek(), ixmin, ixmax, & dmin, & dmax);
-	} else {
-		for (integer i = ixmin; i <= ixmax; i ++) {
-			if (d [i] > dmax) {
-				d [i] = dmax;
-			} else if (d [i] < dmin) {
-				d [i] = dmin;
-			}
-		}
-	}
+	if (dmin >= dmax)
+		NUMextrema (d.get(), & dmin, & dmax);
+	else 
+		VECclip_inplace (d.get(), dmin, dmax);
 
 	Graphics_setInner (g);
 	Graphics_setWindow (g, xmin, xmax, dmin, dmax);
-	Graphics_function (g, d.peek(), ixmin, ixmax, xmin, xmax);
+	Graphics_function (g, d.at, 1, numberOfSelected, xmin, xmax);
 	Graphics_unsetInner (g);
 
 	if (garnish) {
@@ -941,9 +794,8 @@ autoDTW Matrices_to_DTW (Matrix me, Matrix thee, bool matchStart, bool matchEnd,
 				double dmax = 0.0, d = 0.0;
 				for (integer k = 1; k <= my ny; k ++) {
 					double dtmp = fabs (my z [k] [i] - thy z [k] [j]);
-					if (dtmp > dmax) {
+					if (dtmp > dmax)
 						dmax = dtmp;
-					}
 				}
 				if (dmax > 0) {
 					for (integer k = 1; k <= my ny; k ++) {
@@ -975,14 +827,12 @@ autoDTW Spectrograms_to_DTW (Spectrogram me, Spectrogram thee, bool matchStart,
 		// Take log10 for dB's (4e-10 scaling not necessary)
 
 		for (integer i = 1; i <= my ny; i ++) {
-			for (integer j = 1; j <= my nx; j ++) {
+			for (integer j = 1; j <= my nx; j ++)
 				m1 -> z [i] [j] = 10 * log10 (m1 -> z [i] [j]);
-			}
 		}
 		for (integer i = 1; i <= thy ny; i ++) {
-			for (integer j = 1; j <= thy nx; j ++) {
+			for (integer j = 1; j <= thy nx; j ++)
 				m2 -> z [i] [j] = 10 * log10 (m2 -> z [i] [j]);
-			}
 		}
 
 		autoDTW him = Matrices_to_DTW (m1.get(), m2.get(), matchStart, matchEnd, slope, metric);
@@ -994,13 +844,13 @@ autoDTW Spectrograms_to_DTW (Spectrogram me, Spectrogram thee, bool matchStart,
 
 static int Pitch_findFirstAndLastVoicedFrame (Pitch me, integer *first, integer *last) {
 	*first = 1;
-	while (*first <= my nx && ! Pitch_isVoiced_i (me, *first)) {
+	while (*first <= my nx && ! Pitch_isVoiced_i (me, *first))
 		(*first) ++;
-	}
+
 	*last = my nx;
-	while (*last >= *first && ! Pitch_isVoiced_i (me, *last)) {
+	while (*last >= *first && ! Pitch_isVoiced_i (me, *last))
 		(*last) --;
-	}
+
 	return *first <= my nx && *last >= 1;
 }
 
@@ -1020,11 +870,10 @@ autoDTW Pitches_to_DTW_sgc (Pitch me, Pitch thee, double vuv_costs, double time_
 			by making the other cell's distances very large.
 		*/
 		autoDTW him = DTW_create (my xmin, my xmax, my nx, my dx, my x1, thy xmin, thy xmax, thy nx, thy dx, thy x1);
-		autoNUMvector<double> pitchx (1, thy nx);
+		autoVEC pitchx = newVECraw (thy nx);
 		kPitch_unit unit = kPitch_unit::SEMITONES_100;
-		for (integer j = 1; j <= thy nx; j ++) {
+		for (integer j = 1; j <= thy nx; j ++)
 			pitchx [j] = Sampled_getValueAtSample (thee, j, Pitch_LEVEL_FREQUENCY, (int) unit);
-		}
 
 		for (integer i = 1; i <= my nx; i ++) {
 			double pitchy = Sampled_getValueAtSample (me, i, Pitch_LEVEL_FREQUENCY, (int) unit);
@@ -1034,9 +883,8 @@ autoDTW Pitches_to_DTW_sgc (Pitch me, Pitch thee, double vuv_costs, double time_
 				double dist_f = 0.0;   // based on pitch difference
 				double dist_t = fabs (t1 - t2);
 				if (isundef (pitchy)) {
-					if (isdefined (pitchx [j])) {
+					if (isdefined (pitchx [j]))
 						dist_f = vuv_costs;
-					}
 				} else if (isundef (pitchx [j])) {
 					dist_f = vuv_costs;
 				} else {
@@ -1058,11 +906,10 @@ autoDTW Pitches_to_DTW (Pitch me, Pitch thee, double vuv_costs, double time_weig
 		Melder_require (time_weight >= 0.0, U"Time costs weight should not be negative.");
 
 		autoDTW him = DTW_create (my xmin, my xmax, my nx, my dx, my x1, thy xmin, thy xmax, thy nx, thy dx, thy x1);
-		autoNUMvector<double> pitchx (1, thy nx);
+		autoVEC pitchx = newVECraw (thy nx);
 		kPitch_unit unit = kPitch_unit::SEMITONES_100;
-		for (integer j = 1; j <= thy nx; j ++) {
+		for (integer j = 1; j <= thy nx; j ++)
 			pitchx [j] = Sampled_getValueAtSample (thee, j, Pitch_LEVEL_FREQUENCY, (int) unit);
-		}
 
 		for (integer i = 1; i <= my nx; i ++) {
 			double pitchy = Sampled_getValueAtSample (me, i, Pitch_LEVEL_FREQUENCY, (int) unit);
@@ -1072,9 +919,8 @@ autoDTW Pitches_to_DTW (Pitch me, Pitch thee, double vuv_costs, double time_weig
 				double dist_f = 0; // based on pitch difference
 				double dist_t = fabs (t1 - t2);
 				if (isundef (pitchy)) {
-					if (isdefined (pitchx [j])) {
+					if (isdefined (pitchx [j]))
 						dist_f = vuv_costs;
-					}
 				} else if (isundef (pitchx [j])) {
 					dist_f = vuv_costs;
 				} else {
@@ -1101,12 +947,11 @@ void DTW_Matrix_replace (DTW me, Matrix thee) {
 			U"The X and Y domains of the matrix and the DTW must be equal.");
 		Melder_require (my nx == thy nx && my dx == thy dx && my ny == thy ny && my dy == thy dy,
 			U"The sampling of the matrix and the DTW should be equal.");
-		
 		double minimum, maximum;
 		Matrix_getWindowExtrema (me, 0, 0, 0, 0, & minimum, & maximum);
-		Melder_require (minimum >= 0.0, U"Distances should not be negative.");
-		
-		NUMmatrix_copyElements<double> (thy z, my z, 1, my ny, 1, my nx);
+		Melder_require (minimum >= 0.0,
+			U"Distances should not be negative.");
+		my z.get() <<= thy z.get();
 	} catch (MelderError) {
 		Melder_throw (me, U": distances not replaced.");
 	}
@@ -1143,10 +988,10 @@ static void DTW_checkSlopeConstraints (DTW me, double band, int slope) {
 		Melder_require (slope > 0 && slope < 5, U"Invalid slope constraint.");
 		Melder_require (! (dtw_slope == 0.0 && slope != 1), U"Band too wide.");
 
-        if (dtw_slope < 1.0) {
+        if (dtw_slope < 1.0)
             dtw_slope = 1.0 / dtw_slope;
-        }
-        Melder_require (dtw_slope <= slopes [slope], U"There is a conflict between the chosen slope constraint and the relative duration. The duration ratio of the integerest and the shortest object is ", dtw_slope, U". This implies that the largest slope in the constraint must have a value greater or equal to this ratio.");
+
+        Melder_require (dtw_slope <= slopes [slope], U"There is a conflict between the chosen slope constraint and the relative duration. The duration ratio of the longest and the shortest object is ", dtw_slope, U". This implies that the largest slope in the constraint must have a value greater or equal to this ratio.");
     } catch (MelderError) {
         Melder_throw (U"Slope constraints cannot be met.");
     }
@@ -1170,9 +1015,8 @@ static void DTW_Polygon_setUnreachableParts (DTW me, Polygon thee, integer **psi
             for (integer iy = iystart + 1; iy <= my ny; iy ++) {
                 double y = my y1 + (iy - 1) * my dy;
                 if (Polygon_getLocationOfPoint (thee, x, y, eps) == Polygon_OUTSIDE) {
-                    for (integer k = iy; k <= my ny; k ++) {
+                    for (integer k = iy; k <= my ny; k ++)
                         psi [k] [ix] =  DTW_UNREACHABLE;
-                    }
                     break;
                 }
             }
@@ -1185,9 +1029,8 @@ static void DTW_Polygon_setUnreachableParts (DTW me, Polygon thee, integer **psi
             for (integer iy = iystart - 1; iy >= 1; iy --) {
                 double y = my y1 + (iy - 1) * my dy;
                 if (Polygon_getLocationOfPoint (thee, x, y, eps) == Polygon_OUTSIDE) {
-                    for (integer k = iy; k >= 1; k --) {
+                    for (integer k = iy; k >= 1; k --)
                         psi [k] [ix] = DTW_UNREACHABLE;
-                    }
                     break;
                 }
             }
@@ -1337,11 +1180,10 @@ void DTW_Polygon_findPathInside (DTW me, Polygon thee, int localSlope, autoMatri
     try {
         double slopes [5] = { DTW_BIG, DTW_BIG, 3.0, 2.0, 1.5 };
         // if localSlope == 1 start of path is within 10% of minimum duration. Starts farther away
-        integer delta_xy = (my nx < my ny ? my nx : my ny) / 10; // if localSlope == 1 start within 10% of
+        integer delta_xy = std::min (my nx, my ny) / 10; // if localSlope == 1 start within 10% of
 
-        if (localSlope < 1 || localSlope > 4) {
-            Melder_throw (U"Local slope parameter is illegal.");
-        }
+		Melder_require (localSlope > 0 && localSlope < 5, 
+			U"Local slope parameter is illegal.");
 
         autoNUMmatrix<double> delta (-2, my ny, -2, my nx);
         autoNUMmatrix<integer> psi (-2, my ny, -2, my nx);
@@ -1377,9 +1219,9 @@ void DTW_Polygon_findPathInside (DTW me, Polygon thee, int localSlope, autoMatri
         }
         // Make begin part of first row reachable
         integer colto = delta_xy;
-        if (localSlope != 1) {
+        if (localSlope != 1) 
 			colto = Melder_ifloor (slopes [localSlope]) + 1;
-		}
+
         for (integer ix = 2; ix <= colto; ix ++) {
             if (localSlope != 1) {
                 delta [1] [ix] = delta [1] [ix -1] + my z [1] [ix];
@@ -1491,9 +1333,8 @@ void DTW_Polygon_findPathInside (DTW me, Polygon thee, int localSlope, autoMatri
                 psi [i] [j] = direction;
                 delta [i] [j] = gmin;
             }
-            if ((j % 10) == 2) {
+            if ((j % 10) == 2)
                 Melder_progress (0.999 * j / my nx, U"Calculate time warp: frame ", j, U" from ", my nx, U".");
-            }
         }
 
         // Find minimum at end of path and trace back.
@@ -1507,7 +1348,7 @@ void DTW_Polygon_findPathInside (DTW me, Polygon thee, int localSlope, autoMatri
                 minimum = delta [iy = i] [my nx];
             }
         }
-        
+
         integer pathIndex = my nx + my ny - 1;   // maximum path length
         my weightedDistance = minimum / (my nx + my ny);
         my path [pathIndex]. y = iy;
@@ -1534,9 +1375,8 @@ void DTW_Polygon_findPathInside (DTW me, Polygon thee, int localSlope, autoMatri
 
         my pathLength = my nx + my ny - 1 - pathIndex + 1;
         if (pathIndex > 1) {
-            for (integer j = 1; j <= my pathLength; j ++) {
+            for (integer j = 1; j <= my pathLength; j ++)
                 my path [j] = my path [pathIndex ++];
-            }
         }
 
         DTW_Path_recode (me);
@@ -1544,9 +1384,8 @@ void DTW_Polygon_findPathInside (DTW me, Polygon thee, int localSlope, autoMatri
             autoMatrix him = Matrix_create (my xmin, my xmax, my nx, my dx, my x1,
                 my ymin, my ymax, my ny, my dy, my y1);
             for (integer i = 1; i <= my ny; i ++) {
-                for (integer j = 1; j <= my nx; j ++) {
+                for (integer j = 1; j <= my nx; j ++)
                     his z [i] [j] = delta [i] [j];
-                }
             }
             *cumulativeDists = him.move();
         }
