@@ -2,7 +2,7 @@
 #define _Distance_h_
 /* Distance.h
  *
- * Copyright (C) 1993-2017 David Weenink
+ * Copyright (C) 1993-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "Configuration.h"
 #include "Proximity.h"
 #include "Graphics.h"
 
@@ -26,8 +27,19 @@ Thing_define (Distance, Proximity) {
 
 autoDistance Distance_create (integer numberOfPoints);
 
+autoDistance Configuration_to_Distance (Configuration me);
+
 void Distance_drawDendogram (Distance me, Graphics g, int method);
 
 double Distance_getMaximumDistance (Distance me);
 
+Collection_define (DistanceList, OrderedOf, Distance) {
+	ProximityList asProximityList () {
+		return reinterpret_cast<ProximityList> (this);
+	}
+	TableOfRealList asTableOfRealList () {
+		return reinterpret_cast<TableOfRealList> (this);
+	}
+};
+
 #endif /* _Distance_h_ */
