@@ -31,41 +31,34 @@ static autoEEG EEG_copyWithoutSound (EEG me) {
  		autoEEG thee = EEG_create (my xmin, my xmax);
 		thy numberOfChannels = my numberOfChannels;
 		thy textgrid = Data_copy (my textgrid.get());
-		autostring32vector channelNames (1, my numberOfChannels);
-		for (integer i = 1; i <= my numberOfChannels; i ++) {
-			channelNames [i] = Melder_dup (my channelNames [i]);
-		}
-		thy channelNames = channelNames.transfer();
+		thy channelNames = newSTRVECcopy (my channelNames.get());
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not copied.");
 	}
 }
 
-static integer *EEG_channelNames_to_channelNumbers (EEG me, char32 **channelNames, integer numberOfChannelNames) {
+static autoINTVEC EEG_channelNames_to_channelNumbers (EEG me, string32vector channelNames) {
 	try {
-		autoNUMvector<integer> channelNumbers (1, numberOfChannelNames);
-		for (integer i = 1; i <= numberOfChannelNames; i ++) {
-			for (integer j = 1; j <= my numberOfChannels; j ++) {
-				if (Melder_equ (channelNames [i], my channelNames [j])) {
+		autoINTVEC channelNumbers = newINTVECzero (channelNames.size);
+		for (integer i = 1; i <= channelNames.size; i ++) {
+			for (integer j = 1; j <= my numberOfChannels; j ++)
+				if (Melder_equ (channelNames [i], my channelNames [j].get()))
 					channelNumbers [i] = j;
-				}
-			}
-			if (channelNumbers [i] == 0) {
+			if (channelNumbers [i] == 0)
 				Melder_throw (U"Channel name \"", channelNames [i], U"\" not found.");
-			}
 		}
-		return channelNumbers.transfer();
+		return channelNumbers;
 	} catch (MelderError) {
 		Melder_throw (me, U": channelNames not found.");
 	}
 }
 
-static void EEG_setChannelNames_selected (EEG me, const char32 *precursor, integer *channelNumbers, integer numberOfChannels) {
+static void EEG_setChannelNames_selected (EEG me, conststring32 precursor, constINTVEC channelNumbers) {
 	autoMelderString name;
-	const char32 *zero = U"0";
-	for (integer i = 1; i <= numberOfChannels; i ++) {
-		MelderString_copy (&name, precursor);
+	conststring32 zero = U"0";
+	for (integer i = 1; i <= channelNumbers.size; i ++) {
+		MelderString_copy (& name, precursor);
 		if (my numberOfChannels > 100) {
 			if (i < 10) {
 				MelderString_append (& name, zero);
@@ -81,12 +74,13 @@ static void EEG_setChannelNames_selected (EEG me, const char32 *precursor, integ
 	}
 }
 
-autoCrossCorrelationTable EEG_to_CrossCorrelationTable (EEG me, double startTime, double endTime, double lagStep, const char32 *channelRanges)
+autoCrossCorrelationTable EEG_to_CrossCorrelationTable (EEG me, double startTime, double endTime, double lagStep, conststring32 channelRanges)
 {
 	try {
 		// autowindow
 		if (startTime == endTime) {
-			startTime = my xmin; endTime = my xmax;
+			startTime = my xmin;
+			endTime = my xmax;
 		}
 		// don't allow times outside domain
 		if (startTime < my xmin) {
@@ -96,14 +90,13 @@ autoCrossCorrelationTable EEG_to_CrossCorrelationTable (EEG me, double startTime
 			endTime = my xmax;
 		}
 		autoEEG thee = EEG_extractPart (me, startTime, endTime, true);
-		integer numberOfChannels;
-		autoNUMvector <integer> channels (NUMstring_getElementsOfRanges (channelRanges, thy numberOfChannels, & numberOfChannels, nullptr, U"channel", true), 1);
+		autoINTVEC channels = NUMstring_getElementsOfRanges (channelRanges, thy numberOfChannels, U"channel", true);
 		autoSound soundPart = Sound_copyChannelRanges (thy sound.get(), channelRanges);
 		autoCrossCorrelationTable him = Sound_to_CrossCorrelationTable (soundPart.get(), startTime, endTime, lagStep);
 		// assign channel names
-		for (integer i = 1; i <= numberOfChannels; i ++) {
+		for (integer i = 1; i <= channels.size; i ++) {
 			integer ichannel = channels [i];
-			char32 *label = my channelNames [ichannel];
+			conststring32 label = my channelNames [ichannel].get();
 			TableOfReal_setRowLabel (him.get(), i, label);
 			TableOfReal_setColumnLabel (him.get(), i, label);
 		}
@@ -113,7 +106,7 @@ autoCrossCorrelationTable EEG_to_CrossCorrelationTable (EEG me, double startTime
 	}
 }
 
-autoCovariance EEG_to_Covariance (EEG me, double startTime, double endTime, const char32 *channelRanges)
+autoCovariance EEG_to_Covariance (EEG me, double startTime, double endTime, conststring32 channelRanges)
 {
 	try {
 		double lagStep = 0.0;
@@ -127,12 +120,13 @@ autoCovariance EEG_to_Covariance (EEG me, double startTime, double endTime, cons
 }
 
 autoCrossCorrelationTableList EEG_to_CrossCorrelationTableList (EEG me,
-	double startTime, double endTime, integer numberOfCrossCorrelations, double lagStep, const char32 *channelRanges)
+	double startTime, double endTime, integer numberOfCrossCorrelations, double lagStep, conststring32 channelRanges)
 {
 	try {
 		// autowindow
 		if (startTime == endTime) {
-			startTime = my xmin; endTime = my xmax;
+			startTime = my xmin;
+			endTime = my xmax;
 		}
 		// don't allow times outside domain
 		if (startTime < my xmin) {
@@ -142,8 +136,7 @@ autoCrossCorrelationTableList EEG_to_CrossCorrelationTableList (EEG me,
 			endTime = my xmax;
 		}
 		autoEEG thee = EEG_extractPart (me, startTime, endTime, true);
-		integer numberOfChannels;
-		autoNUMvector <integer> channels (NUMstring_getElementsOfRanges (channelRanges, thy numberOfChannels, & numberOfChannels, nullptr, U"channel", true), 1);
+		autoINTVEC channels = NUMstring_getElementsOfRanges (channelRanges, thy numberOfChannels, U"channel", true);
 		autoSound soundPart = Sound_copyChannelRanges (thy sound.get(), channelRanges);
 		autoCrossCorrelationTableList him = Sound_to_CrossCorrelationTableList (soundPart.get(),
 			startTime, endTime, numberOfCrossCorrelations, lagStep);
@@ -153,7 +146,7 @@ autoCrossCorrelationTableList EEG_to_CrossCorrelationTableList (EEG me,
 	}
 }
 
-autoPCA EEG_to_PCA (EEG me, double startTime, double endTime, const char32 *channelRanges, int fromCorrelation) {
+autoPCA EEG_to_PCA (EEG me, double startTime, double endTime, conststring32 channelRanges, int fromCorrelation) {
 	try {
 		autoCovariance cov = EEG_to_Covariance (me, startTime, endTime, channelRanges);
 		autoPCA him;
@@ -171,20 +164,19 @@ autoPCA EEG_to_PCA (EEG me, double startTime, double endTime, const char32 *chan
 
 autoEEG EEG_PCA_to_EEG_whiten (EEG me, PCA thee, integer numberOfComponents) {
 	try {
-		if (numberOfComponents <= 0 || numberOfComponents > thy numberOfEigenvalues) {
+		if (numberOfComponents <= 0 || numberOfComponents > thy numberOfEigenvalues)
 			numberOfComponents = thy numberOfEigenvalues;
-		}
-		numberOfComponents = ( numberOfComponents > my numberOfChannels ? my numberOfChannels : numberOfComponents );
+		numberOfComponents = std::min (numberOfComponents, my numberOfChannels);
 
-		autoNUMvector <integer> channelNumbers (EEG_channelNames_to_channelNumbers (me, thy labels, thy dimension), 1);
+		Melder_assert (thy labels.size == thy dimension);
+		autoINTVEC channelNumbers = EEG_channelNames_to_channelNumbers (me, thy labels.get());
 
 		autoEEG him = Data_copy (me);
-		autoSound white = Sound_PCA_whitenSelectedChannels (my sound.get(), thee, numberOfComponents, channelNumbers.peek(), thy dimension);
-		for (integer i = 1; i <= thy dimension; i ++) {
-			integer ichannel = channelNumbers [i];
-			NUMvector_copyElements<double> (white -> z [i], his sound -> z [ichannel], 1, his sound -> nx);
-		}
-		EEG_setChannelNames_selected (him.get(), U"wh", channelNumbers.peek(), thy dimension);
+		autoSound white = Sound_PCA_whitenSelectedChannels (my sound.get(), thee, numberOfComponents, channelNumbers.get());
+		for (integer i = 1; i <= channelNumbers.size; i ++)
+			his sound -> z.row (channelNumbers [i]) <<= white -> z.row (i);
+
+		EEG_setChannelNames_selected (him.get(), U"wh", channelNumbers.get());
 		return him;
 	} catch(MelderError) {
 		Melder_throw (me, U": not whitened with ", thee);
@@ -193,33 +185,33 @@ autoEEG EEG_PCA_to_EEG_whiten (EEG me, PCA thee, integer numberOfComponents) {
 
 autoEEG EEG_PCA_to_EEG_principalComponents (EEG me, PCA thee, integer numberOfComponents) {
 	try {
-		if (numberOfComponents <= 0 || numberOfComponents > thy numberOfEigenvalues) {
+		if (numberOfComponents <= 0 || numberOfComponents > thy numberOfEigenvalues)
 			numberOfComponents = thy numberOfEigenvalues;
-		}
-		numberOfComponents = numberOfComponents > my numberOfChannels ? my numberOfChannels : numberOfComponents;
+		numberOfComponents = std::min (numberOfComponents, my numberOfChannels);
 
-		autoNUMvector <integer> channelNumbers (EEG_channelNames_to_channelNumbers (me, thy labels, thy dimension), 1);
+		Melder_assert (thy labels.size == thy dimension);
+		autoINTVEC channelNumbers = EEG_channelNames_to_channelNumbers (me, thy labels.get());
 		autoEEG him = Data_copy (me);
-		autoSound pc = Sound_PCA_to_Sound_pc_selectedChannels (my sound.get(), thee, numberOfComponents, channelNumbers.peek(), thy dimension);
-		for (integer i = 1; i <= thy dimension; i ++) {
-			integer ichannel = channelNumbers [i];
-			NUMvector_copyElements<double> (pc -> z [i], his sound -> z [ichannel], 1, his sound -> nx);
-		}
-		EEG_setChannelNames_selected (him.get(), U"pc", channelNumbers.peek(), thy dimension);
+		autoSound pc = Sound_PCA_to_Sound_pc_selectedChannels (my sound.get(), thee, numberOfComponents, channelNumbers.get());
+		for (integer i = 1; i <= channelNumbers.size; i ++)
+			his sound -> z.row (channelNumbers [i]) <<= pc -> z.row (i);
+		
+		EEG_setChannelNames_selected (him.get(), U"pc", channelNumbers.get());
 		return him;
 	} catch (MelderError) {
 		Melder_throw (me, U": not projected.");
 	}
 }
 
-void EEG_to_EEG_bss (EEG me, double startTime, double endTime, integer numberOfCrossCorrelations, double lagStep, const char32 *channelRanges,
+void EEG_to_EEG_bss (EEG me, double startTime, double endTime, integer numberOfCrossCorrelations, double lagStep, conststring32 channelRanges,
 	int whiteningMethod, int diagonalizerMethod, integer maxNumberOfIterations, double tol,
 	autoEEG *p_resultingEEG, autoMixingMatrix *p_resultingMixingMatrix)
 {
 	try {
 		// autowindow
 		if (startTime == endTime) {
-			startTime = my xmin; endTime = my xmax;
+			startTime = my xmin;
+			endTime = my xmax;
 		}
 		// don't allow times outside domain
 		if (startTime < my xmin) {
@@ -228,8 +220,7 @@ void EEG_to_EEG_bss (EEG me, double startTime, double endTime, integer numberOfC
 		if (endTime > my xmax) {
 			endTime = my xmax;
 		}
-		integer numberOfChannels;
-		autoNUMvector <integer> channelNumbers (NUMstring_getElementsOfRanges (channelRanges, my numberOfChannels, & numberOfChannels, nullptr, U"channel", true), 1);
+		autoINTVEC channelNumbers = NUMstring_getElementsOfRanges (channelRanges, my numberOfChannels, U"channel", true);
 		autoEEG thee = EEG_extractPart (me, startTime, endTime, true);
 		if (whiteningMethod != 0) {
 			bool fromCorrelation = ( whiteningMethod == 2 );
@@ -243,34 +234,33 @@ void EEG_to_EEG_bss (EEG me, double startTime, double endTime, integer numberOfC
 
 		autoEEG him = EEG_copyWithoutSound (me);
 		his sound = Sound_MixingMatrix_unmix (my sound.get(), mm.get());
-		EEG_setChannelNames_selected (him.get(), U"ic", channelNumbers.peek(), numberOfChannels);
+		EEG_setChannelNames_selected (him.get(), U"ic", channelNumbers.get());
 
 		// Calculate the cross-correlations between eye-channels and the ic's
 
-		*p_resultingEEG = thee.move();
-		*p_resultingMixingMatrix = mm.move();
+		if (p_resultingEEG) *p_resultingEEG = thee.move();
+		if (p_resultingMixingMatrix) *p_resultingMixingMatrix = mm.move();
 	} catch (MelderError) {
 		Melder_throw (me, U": no independent components determined.");
 	}
 }
 
-autoSound EEG_to_Sound_modulated (EEG me, double baseFrequency, double channelBandwidth, const char32 *channelRanges) {
+autoSound EEG_to_Sound_modulated (EEG me, double baseFrequency, double channelBandwidth, conststring32 channelRanges) {
 	try {
-		integer numberOfChannels;
-		autoNUMvector <integer> channelNumbers (NUMstring_getElementsOfRanges (channelRanges, my numberOfChannels, & numberOfChannels, nullptr, U"channel", true), 1);
+		autoINTVEC channelNumbers = NUMstring_getElementsOfRanges (channelRanges, my numberOfChannels, U"channel", true);
 		double maxFreq = baseFrequency + my numberOfChannels * channelBandwidth;
 		double samplingFrequency = 2.0 * maxFreq;
-		samplingFrequency = samplingFrequency < 44100.0 ? 44100.0 : samplingFrequency;
+		samplingFrequency = std::max (samplingFrequency, 44100.0);
 		autoSound thee = Sound_createSimple (1, my xmax - my xmin, samplingFrequency);
-		for (integer i = 1; i <= numberOfChannels; i ++) {
+		for (integer i = 1; i <= channelNumbers.size; i ++) {
 			integer ichannel = channelNumbers [i];
 			double fbase = baseFrequency;// + (ichannel - 1) * channelBandwidth;
 			autoSound si = Sound_extractChannel (my sound.get(), ichannel);
-			autoSpectrum spi = Sound_to_Spectrum (si.get(), 1);
+			autoSpectrum spi = Sound_to_Spectrum (si.get(), true);
 			Spectrum_passHannBand (spi.get(), 0.5, channelBandwidth - 0.5, 0.5);
 			autoSpectrum spi_shifted = Spectrum_shiftFrequencies (spi.get(), fbase, samplingFrequency / 2.0, 30);
 			autoSound resampled = Spectrum_to_Sound (spi_shifted.get());
-			integer nx = resampled -> nx < thy nx ? resampled -> nx : thy nx;
+			integer nx = std::min (resampled -> nx, thy nx);
 			for (integer j = 1; j <= nx; j ++) {
 				thy z [1] [j] += resampled -> z [1] [j];
 			}
@@ -285,7 +275,7 @@ autoSound EEG_to_Sound_modulated (EEG me, double baseFrequency, double channelBa
 autoSound EEG_to_Sound_frequencyShifted (EEG me, integer channel, double frequencyShift, double samplingFrequency, double maxAmp) {
 	try {
 		autoSound si = Sound_extractChannel (my sound.get(), channel);
-		autoSpectrum spi = Sound_to_Spectrum (si.get(), 1);
+		autoSpectrum spi = Sound_to_Spectrum (si.get(), true);
 		autoSpectrum spi_shifted = Spectrum_shiftFrequencies (spi.get(), frequencyShift, samplingFrequency / 2.0, 30);
 		autoSound thee = Spectrum_to_Sound (spi_shifted.get());
 		if (maxAmp > 0) {
