@@ -68,32 +68,33 @@ autoWarpingPath WarpingPath_create (integer length) {
 }
 
 static void WarpingPath_reset (WarpingPath me) {
-	for (integer i = 1; i <= my _capacity; i ++) {
-		my path [i].x = my path [i].y = 0;
-	}
+	for (integer i = 1; i <= my _capacity; i ++)
+		my path [i]. x = my path [i]. y = 0;
 	my pathLength = my _capacity;
 }
 
-static void WarpingPath_getPath (WarpingPath me, short **psi, integer iy, integer ix) { // psi [0..nrows-1] [0..ncols-1]
+static void WarpingPath_getPath (WarpingPath me, constINTMAT const& psi, integer iy, integer ix) { // psi [1..nrows] [1..ncols]
 	integer index = my pathLength;
-	my path [index].x = ix;
-	my path [index].y = iy;
-	while (!(ix == 0 && iy == 0)) {
-		if (psi [iy] [ix] == WARPING_fromLeft) {
+	my path [index]. x = ix;
+	my path [index]. y = iy;
+	while (! (ix == 0 && iy == 0)) {
+		Melder_assert (ix >= 0 && iy >= 0);
+		if (psi [1+iy] [1+ix] == WARPING_fromLeft) {
 			ix --;
-		} else if (psi [iy] [ix] == WARPING_fromBelow) {
+		} else if (psi [1+iy] [1+ix] == WARPING_fromBelow) {
 			iy --;
 		} else { // WARPING_fromDiag
-			ix --; iy --;
+			ix --;
+			iy --;
 		}
-		my path [-- index].x = ix;
-		my path [index].y = iy;
+		my path [-- index]. x = ix;
+		my path [index]. y = iy;
 	}
 	if (index > 1) {
 		integer k = 1;
 		for (integer i = index; i <= my pathLength; i ++) {
 			my path [k ++] = my path [i];
-			my path [i].x = my path [i].y = 0;
+			my path [i]. x = my path [i]. y = 0;
 		}
 		my pathLength = k - 1;
 	}
@@ -101,7 +102,8 @@ static void WarpingPath_getPath (WarpingPath me, short **psi, integer iy, intege
 
 static void WarpingPath_shiftPathByOne (WarpingPath me) {
 	for (integer i = 1; i <= my pathLength; i ++) {
-		(my path [i].x) ++; (my path [i].y) ++;
+		my path [i]. x ++;
+		my path [i]. y ++;
 	}
 }
 
@@ -109,54 +111,48 @@ integer WarpingPath_getColumnsFromRowIndex (WarpingPath me, integer iy, integer
 	integer ix1 = 0, ix2 = 0, numberOfColumns = 0;
 	if (iy > 0) {
 		for (integer i = 1; i <= my pathLength; i ++) {
-			if (my path [i].y < iy) {
+			if (my path [i]. y < iy) {
 				continue;
-			} else if (my path [i].y == iy) {
+			} else if (my path [i]. y == iy) {
 				if (ix1 == 0) {
-					ix1 = my path [i].x;
+					ix1 = my path [i]. x;
 				}
-				ix2 = my path [i].x;
+				ix2 = my path [i]. x;
 			} else {
 				break;
 			}
 		}
 		numberOfColumns = ix2 - ix1 + 1;
 	}
-	if (p_ix1) {
+	if (p_ix1)
 		*p_ix1 = ix1;
-	}
-	if (p_ix2) {
+	if (p_ix2)
 		*p_ix2 = ix2;
-	}
 	return numberOfColumns;
 }
 
 integer WarpingPath_getRowsFromColumnIndex (WarpingPath me, integer ix, integer *p_iy1, integer *p_iy2) {
-	if (ix <= 0) {
+	if (ix <= 0)
 		return 0;
-	}
 	integer iy1 = 0, iy2 = 0, numberOfRows = 0;
 	if (ix > 0) {
 		for (integer i = 1; i <= my pathLength; i ++) {
-			if (my path [i].x < ix) {
+			if (my path [i]. x < ix) {
 				continue;
-			} else if (my path [i].x == ix) {
-				if (iy1 == 0) {
-					iy1 = my path [i].y;
-				}
-				iy2 = my path [i].y;
+			} else if (my path [i]. x == ix) {
+				if (iy1 == 0)
+					iy1 = my path [i]. y;
+				iy2 = my path [i]. y;
 			} else {
 				break;
 			}
 		}
 		numberOfRows = iy2 - iy1 + 1;
 	}
-	if (p_iy1) {
+	if (p_iy1)
 		*p_iy1 = iy1;
-	}
-	if (p_iy2) {
+	if (p_iy2)
 		*p_iy2 = iy2;
-	}
 	return numberOfRows;
 }
 
@@ -168,15 +164,15 @@ void structEditCostsTable :: v_info () {
 	MelderInfo_writeLine (U"Source:", numberOfColumns - 2, U" symbols.");
 }
 
-bool structEditCostsTable :: v_matchTargetSymbol (const char32 *targetSymbol, const char32 *symbol) {
+bool structEditCostsTable :: v_matchTargetSymbol (conststring32 targetSymbol, conststring32 symbol) {
 	return Melder_equ (targetSymbol, symbol);
 }
 
-bool structEditCostsTable :: v_matchSourceSymbol (const char32 *sourceSymbol, const char32 *symbol) {
+bool structEditCostsTable :: v_matchSourceSymbol (conststring32 sourceSymbol, conststring32 symbol) {
 	return Melder_equ (sourceSymbol, symbol);
 }
 
-bool structEditCostsTable :: v_matchTargetWithSourceSymbol (const char32 *targetSymbol, const char32 *sourceSymbol) {
+bool structEditCostsTable :: v_matchTargetWithSourceSymbol (conststring32 targetSymbol, conststring32 sourceSymbol) {
 	return Melder_equ (targetSymbol, sourceSymbol);
 }
 
@@ -210,36 +206,34 @@ void EditCostsTable_setDefaultCosts (EditCostsTable me, double insertionCosts, d
 	my data [my numberOfRows - 1] [my numberOfColumns] = insertionCosts;
 }
 
-integer EditCostsTable_getTargetIndex (EditCostsTable me, const char32 *symbol) {
-	for (integer i = 1; i <= my numberOfRows - 2; i ++) {
-		if (my v_matchTargetSymbol (my rowLabels [i], symbol)) {
+integer EditCostsTable_getTargetIndex (EditCostsTable me, conststring32 symbol) {
+	for (integer i = 1; i <= my numberOfRows - 2; i ++)
+		if (my v_matchTargetSymbol (my rowLabels [i].get(), symbol))
 			return i;
-		}
-	}
 	return 0;
 }
 
-integer EditCostsTable_getSourceIndex (EditCostsTable me, const char32 *symbol) {
-	for (integer j = 1; j <= my numberOfColumns - 2; j ++) {
-		if (my v_matchSourceSymbol (my columnLabels [j], symbol)) {
+integer EditCostsTable_getSourceIndex (EditCostsTable me, conststring32 symbol) {
+	for (integer j = 1; j <= my numberOfColumns - 2; j ++)
+		if (my v_matchSourceSymbol (my columnLabels [j].get(), symbol))
 			return j;
-		}
-	}
 	return 0;
 }
 
-void EditCostsTable_setInsertionCosts (EditCostsTable me, char32 *targets, double cost) {
-	for (char32 *token = Melder_firstToken (targets); token != 0; token = Melder_nextToken ()) {
-		integer irow = EditCostsTable_getTargetIndex (me, token);
-		irow = irow > 0 ? irow : my numberOfRows - 1; // nomatch condition to penultimate row
+void EditCostsTable_setInsertionCosts (EditCostsTable me, conststring32 targets_string, double cost) {
+	autostring32vector targets = newSTRVECtokenize (targets_string);
+	for (integer itarget = 1; itarget <= targets.size; itarget ++) {
+		integer irow = EditCostsTable_getTargetIndex (me, targets [itarget].get());
+		irow = ( irow > 0 ? irow : my numberOfRows - 1 );   // nomatch condition to penultimate row
 		my data [irow] [my numberOfColumns] = cost;
 	}
 }
 
-void EditCostsTable_setDeletionCosts (EditCostsTable me, char32 *sources, double cost) {
-	for (char32 *token = Melder_firstToken (sources); token != 0; token = Melder_nextToken ()) {
-		integer icol = EditCostsTable_getSourceIndex (me, token);
-		icol = icol > 0 ? icol : my numberOfColumns - 1; // nomatch condition to penultimate column
+void EditCostsTable_setDeletionCosts (EditCostsTable me, conststring32 sources_string, double cost) {
+	autostring32vector sources = newSTRVECtokenize (sources_string);
+	for (integer isource = 1; isource <= sources.size; isource ++) {
+		integer icol = EditCostsTable_getSourceIndex (me, sources [isource].get());
+		icol = ( icol > 0 ? icol : my numberOfColumns - 1 );   // nomatch condition to penultimate column
 		my data [my numberOfRows] [icol] = cost;
 	}
 }
@@ -252,72 +246,69 @@ void EditCostsTable_setOthersCosts (EditCostsTable me, double insertionCost, dou
 }
 
 double EditCostsTable_getOthersCost (EditCostsTable me, int costType) {
-	return costType == 1 ? my data [my numberOfRows - 1] [my numberOfColumns] : //insertion
-		costType == 2 ? my data [my numberOfRows] [my numberOfColumns - 1] : // deletion
-		costType == 3 ? my data [my numberOfRows] [my numberOfColumns] : // equality
-		 my data [my numberOfRows - 1] [my numberOfColumns -1]; // inequality
+	return ( costType == 1 ? my data [my numberOfRows - 1] [my numberOfColumns] :   //insertion
+		( costType == 2 ? my data [my numberOfRows] [my numberOfColumns - 1] :   // deletion
+		( costType == 3 ? my data [my numberOfRows] [my numberOfColumns] :   // equality
+		my data [my numberOfRows - 1] [my numberOfColumns - 1] ) ) );   // inequality
 }
 
-void EditCostsTable_setSubstitutionCosts (EditCostsTable me, char32 *targets, char32 *sources, double cost) {
+void EditCostsTable_setSubstitutionCosts (EditCostsTable me, conststring32 targets_string, conststring32 sources_string, double cost) {
 	try {
+		autostring32vector targets = newSTRVECtokenize (targets_string);
+		autostring32vector sources = newSTRVECtokenize (sources_string);
 		autoNUMvector<integer> targetIndex (1, my numberOfRows);
 		autoNUMvector<integer> sourceIndex (1, my numberOfRows);
 		integer numberOfTargetSymbols = 0;
-		for (char32 *token = Melder_firstToken (targets); token != 0; token = Melder_nextToken ()) {
-			integer index = EditCostsTable_getTargetIndex (me, token);
-			if (index > 0) {
-				targetIndex [ ++numberOfTargetSymbols] = index;
-			}
-		}
-		if (numberOfTargetSymbols == 0) {
-			targetIndex [ ++numberOfTargetSymbols] = my numberOfRows - 1;
+		for (integer itarget = 1; itarget <= targets.size; itarget ++) {
+			integer index = EditCostsTable_getTargetIndex (me, targets [itarget].get());
+			if (index > 0)
+				targetIndex [++ numberOfTargetSymbols] = index;
 		}
+		if (numberOfTargetSymbols == 0)
+			targetIndex [++ numberOfTargetSymbols] = my numberOfRows - 1;
 		integer numberOfSourceSymbols = 0;
-		for (char32 *token = Melder_firstToken (sources); token != 0; token = Melder_nextToken ()) {
-			integer index = EditCostsTable_getSourceIndex (me, token);
-			if (index > 0) {
-				sourceIndex [ ++numberOfSourceSymbols] = index;
-			}
-		}
-		if (numberOfSourceSymbols == 0) {
-			sourceIndex [ ++numberOfSourceSymbols] = my numberOfColumns - 1;
+		for (integer isource = 1; isource <= sources.size; isource ++) {
+			integer index = EditCostsTable_getSourceIndex (me, sources [isource].get());
+			if (index > 0)
+				sourceIndex [++ numberOfSourceSymbols] = index;
 		}
+		if (numberOfSourceSymbols == 0)
+			sourceIndex [++ numberOfSourceSymbols] = my numberOfColumns - 1;
 		for (integer i = 1; i <= numberOfTargetSymbols; i ++) {
 			integer irow = targetIndex [i];
-			for (integer j = 1; j <= numberOfSourceSymbols; j ++) {
+			for (integer j = 1; j <= numberOfSourceSymbols; j ++)
 				my data [irow] [sourceIndex [j]] = cost;
-			}
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": substitution costs not set.");
 	}
 }
 
-double EditCostsTable_getInsertionCost (EditCostsTable me, const char32 *symbol) {
+double EditCostsTable_getInsertionCost (EditCostsTable me, conststring32 symbol) {
 	integer irow = EditCostsTable_getTargetIndex (me, symbol);
-	irow = irow == 0 ? my numberOfRows - 1 : irow; // others is penultimate row
+	irow = ( irow == 0 ? my numberOfRows - 1 : irow );   // others is penultimate row
 	return my data [irow] [my numberOfColumns];
 }
 
-double EditCostsTable_getDeletionCost (EditCostsTable me, const char32 *sourceSymbol) {
+double EditCostsTable_getDeletionCost (EditCostsTable me, conststring32 sourceSymbol) {
 	integer icol = EditCostsTable_getSourceIndex (me, sourceSymbol);
-	icol = icol == 0 ? my numberOfColumns - 1 : icol; // others is penultimate column
+	icol = ( icol == 0 ? my numberOfColumns - 1 : icol );   // others is penultimate column
 	return my data [my numberOfRows] [icol];
 }
 
-double EditCostsTable_getSubstitutionCost (EditCostsTable me, const char32 *symbol, const char32 *replacement) {
+double EditCostsTable_getSubstitutionCost (EditCostsTable me, conststring32 symbol, conststring32 replacement) {
 	integer irow = EditCostsTable_getTargetIndex (me, symbol);
 	integer icol = EditCostsTable_getSourceIndex (me, replacement);
 	if (irow == 0 && icol == 0) { // nomatch
 		irow = my numberOfRows;
 		icol = my numberOfColumns;
 		if (my v_matchTargetWithSourceSymbol (symbol, replacement)) {
-			 --irow;
-			 --icol;
+			 -- irow;
+			 -- icol;
 		}
 	} else {
-		irow = irow == 0 ? my numberOfRows - 1 : irow;
-		icol = icol == 0 ? my numberOfColumns - 1 : icol;
+		irow = ( irow == 0 ? my numberOfRows - 1 : irow );
+		icol = ( icol == 0 ? my numberOfColumns - 1 : icol );
 	}
 	return my data [irow] [icol];
 }
@@ -325,15 +316,10 @@ double EditCostsTable_getSubstitutionCost (EditCostsTable me, const char32 *symb
 autoTableOfReal EditCostsTable_to_TableOfReal (EditCostsTable me) {
 	try {
 		autoTableOfReal thee = TableOfReal_create (my numberOfRows, my numberOfColumns);
-		for (integer j = 1; j <= my numberOfColumns; j ++) {
-			thy columnLabels [j] = Melder_dup (my columnLabels [j]);
-		}
-		for (integer i = 1; i <= my numberOfRows; i ++) {
-			thy rowLabels [i] = Melder_dup (my rowLabels [i]);
-		}
-		NUMmatrix_copyElements<double> (my data, thy data, 1, my numberOfRows, 1, my numberOfColumns);
+		thy columnLabels.all() <<= my columnLabels.all();
+		thy rowLabels.all() <<= my rowLabels.all();
+		thy data.all() <<= my data.all();
 		return thee;
-
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to TableOfReal.");
 	}
@@ -353,13 +339,11 @@ autoEditDistanceTable EditDistanceTable_create (Strings target, Strings source)
 		integer numberOfSourceSymbols = source -> numberOfStrings, numberOfTargetSymbols = target -> numberOfStrings;
 		TableOfReal_init (me.get(), numberOfTargetSymbols + 1, numberOfSourceSymbols + 1);
 		TableOfReal_setColumnLabel (me.get(), 1, U"");
-		for (integer j = 1; j <= numberOfSourceSymbols; j ++) {
-			my columnLabels [j + 1] = Melder_dup (source -> strings [j]);
-		}
+		for (integer j = 1; j <= numberOfSourceSymbols; j ++)
+			my columnLabels [j + 1] = Melder_dup (source -> strings [j].get());
 		TableOfReal_setRowLabel (me.get(), 1, U"");
-		for (integer i = 1; i <= numberOfTargetSymbols; i ++) {
-			my rowLabels [i + 1] = Melder_dup (target -> strings [i]);
-		}
+		for (integer i = 1; i <= numberOfTargetSymbols; i ++)
+			my rowLabels [i + 1] = Melder_dup (target -> strings [i].get());
 		my warpingPath = WarpingPath_create (numberOfTargetSymbols + numberOfSourceSymbols + 1);
 		my editCostsTable = EditCostsTable_createDefault ();
 		EditDistanceTable_findPath (me.get(), 0);
@@ -377,7 +361,7 @@ void EditDistanceTable_setEditCosts (EditDistanceTable me, EditCostsTable thee)
 	}
 }
 
-autoEditDistanceTable EditDistanceTable_createFromCharacterStrings (const char32 *chars1, const char32 *chars2) {
+autoEditDistanceTable EditDistanceTable_createFromCharacterStrings (const char32 chars1 [], const char32 chars2 []) {
 	try {
 		autoStrings s1 = Strings_createAsCharacters (chars1);
 		autoStrings s2 = Strings_createAsCharacters (chars2);
@@ -424,7 +408,7 @@ static void print4 (char *buffer, double value, int iformat, int width, int prec
 		else
 			snprintf (buffer, 40, "%.7g", value);
 	} else {
-		snprintf (formatString, 40, "%%%d.%d%c", width, precision, iformat == 1 ? 'f' : iformat == 2 ? 'e' : 'g');
+		snprintf (formatString, 40, "%%%d.%d%c", width, precision, ( iformat == 1 ? 'f' : ( iformat == 2 ? 'e' : 'g' ) ));
 		snprintf (buffer, 40, formatString, value);
 	}
 }
@@ -435,7 +419,7 @@ static double getMaxRowLabelWidth (TableOfReal me, Graphics graphics, integer ro
 	fixRows (me, & rowmin, & rowmax);
 	for (integer irow = rowmin; irow <= rowmax; irow ++) {
 		if (my rowLabels [irow] && my rowLabels [irow] [0]) {
-			double textWidth = Graphics_textWidth_ps (graphics, my rowLabels [irow], true);   /* SILIPA is bigger than XIPA */
+			double textWidth = Graphics_textWidth_ps (graphics, my rowLabels [irow].get(), true);   /* SILIPA is bigger than XIPA */
 			if (textWidth > maxWidth) {
 				maxWidth = textWidth;
 			}
@@ -445,7 +429,7 @@ static double getMaxRowLabelWidth (TableOfReal me, Graphics graphics, integer ro
 }
 
 static double getLeftMargin (Graphics graphics) {
-	return Graphics_dxMMtoWC (graphics, 1);
+	return Graphics_dxMMtoWC (graphics, 1.0);
 }
 
 static double getLineSpacing (Graphics graphics) {
@@ -455,21 +439,20 @@ static double getLineSpacing (Graphics graphics) {
 void EditDistanceTable_draw (EditDistanceTable me, Graphics graphics, int iformat, int precision, double angle) {
 	integer rowmin = 1, rowmax = my numberOfRows;
 	Graphics_setInner (graphics);
-	Graphics_setWindow (graphics, 0.5, my numberOfColumns + 0.5, 0, 1);
+	Graphics_setWindow (graphics, 0.5, my numberOfColumns + 0.5, 0.0, 1.0);
 	double leftMargin = getLeftMargin (graphics);   // not earlier!
 	double lineSpacing = getLineSpacing (graphics);   // not earlier!
 	double maxTextWidth = getMaxRowLabelWidth (me, graphics, rowmin, rowmax);
-	double y = 1 + 0.1 * lineSpacing;
-	autoNUMmatrix<bool> onPath (1, my numberOfRows, 1, my numberOfColumns);
+	double y = 1.0 + 0.1 * lineSpacing;
+	autoBOOLMAT onPath = newBOOLMATzero (my numberOfRows, my numberOfColumns);
 	for (integer i = 1; i <= my warpingPath -> pathLength; i ++) {
 		structPairOfInteger poi = my warpingPath -> path [i];
 		onPath [poi.y] [poi.x] = true;
 	}
-
 	for (integer irow = my numberOfRows; irow > 0; irow --) {
 		Graphics_setTextAlignment (graphics, Graphics_RIGHT, Graphics_HALF);
 		if (my rowLabels && my rowLabels [irow] && my rowLabels [irow] [0])
-			Graphics_text (graphics, 0.5 - leftMargin, y, my rowLabels [irow]);
+			Graphics_text (graphics, 0.5 - leftMargin, y, my rowLabels [irow].get());
 		Graphics_setTextAlignment (graphics, Graphics_CENTRE, Graphics_HALF);
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			char text [40];
@@ -477,7 +460,7 @@ void EditDistanceTable_draw (EditDistanceTable me, Graphics graphics, int iforma
 			Graphics_setBold (graphics, onPath [irow] [icol]);
 			Graphics_text (graphics, icol, y, Melder_peek8to32 (text));
 			if (onPath [irow] [icol]) {
-				Graphics_rectangle (graphics, icol-0.5, icol+0.5, y - 0.5*lineSpacing, y + 0.5*lineSpacing);
+				Graphics_rectangle (graphics, icol - 0.5, icol + 0.5, y - 0.5 * lineSpacing, y + 0.5 * lineSpacing);
 			}
 		}
 		y -= lineSpacing;
@@ -485,31 +468,31 @@ void EditDistanceTable_draw (EditDistanceTable me, Graphics graphics, int iforma
 	}
 
 	double left = 0.5;
-	if (maxTextWidth > 0.0) left -= maxTextWidth + 2 * leftMargin;
+	if (maxTextWidth > 0.0) left -= maxTextWidth + 2.0 * leftMargin;
 	Graphics_line (graphics, left, y, my numberOfColumns + 0.5, y);
 
 	Graphics_setTextRotation (graphics, angle);
-	if (angle < 0) {
-		y -= 0.3*lineSpacing;
+	if (angle < 0.0) {
+		y -= 0.3 * lineSpacing;
 		Graphics_setTextAlignment (graphics, Graphics_LEFT, Graphics_HALF);
-	} else if (angle > 0) {
+	} else if (angle > 0.0) {
 		Graphics_setTextAlignment (graphics, Graphics_RIGHT, Graphics_HALF);
-		y -= 0.3*lineSpacing;
+		y -= 0.3 * lineSpacing;
 	} else {
 		Graphics_setTextAlignment (graphics, Graphics_CENTRE, Graphics_TOP);
 	}
 	for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 		if (my columnLabels && my columnLabels [icol] && my columnLabels [icol] [0])
-			Graphics_text (graphics, icol, y, my columnLabels [icol]);
+			Graphics_text (graphics, icol, y, my columnLabels [icol].get());
 	}
-	Graphics_setTextRotation (graphics, 0);
+	Graphics_setTextRotation (graphics, 0.0);
 	y -= lineSpacing;
 	Graphics_line (graphics, 0.5, y, 0.5, 1 + 0.5 * lineSpacing);
 	Graphics_unsetInner (graphics);
 }
 
 void EditDistanceTable_drawEditOperations (EditDistanceTable me, Graphics graphics) {
-	const char32 *oinsertion = U"i", *insertion = U"*", *odeletion = U"d", *deletion = U"*", *osubstitution = U"s", *oequal = U"";
+	conststring32 oinsertion = U"i", insertion = U"*", odeletion = U"d", deletion = U"*", osubstitution = U"s", oequal = U"";
 	Graphics_setWindow (graphics, 0.5, my warpingPath -> pathLength - 0.5, 0, 1); // pathLength-1 symbols
 	double lineSpacing = getLineSpacing (graphics);
 	double ytarget = 1 - lineSpacing, ysource = ytarget - 2 * lineSpacing, yoper = ysource - lineSpacing;
@@ -518,17 +501,17 @@ void EditDistanceTable_drawEditOperations (EditDistanceTable me, Graphics graphi
 		structPairOfInteger p = my warpingPath -> path [i], p1 = my warpingPath -> path [i - 1];
 		double x = i - 1;
 		if (p.x == p1.x) { // insertion
-			Graphics_text (graphics, x, ytarget, my rowLabels [p.y]);
+			Graphics_text (graphics, x, ytarget, my rowLabels [p.y].get());
 			Graphics_text (graphics, x, ysource, deletion);
 			Graphics_text (graphics, x, yoper, oinsertion);
 		} else if (p.y == p1.y) { // deletion
 			Graphics_text (graphics, x, ytarget, insertion);
-			Graphics_text (graphics, x, ysource, my columnLabels [p.x]);
+			Graphics_text (graphics, x, ysource, my columnLabels [p.x].get());
 			Graphics_text (graphics, x, yoper, odeletion);
 		} else { // substitution ?
-			Graphics_text (graphics, x, ytarget, my rowLabels [p.y]);
-			Graphics_text (graphics, x, ysource, my columnLabels [p.x]);
-			Graphics_text (graphics, x, yoper, (Melder_equ (my rowLabels [p.y], my columnLabels [p.x]) ? oequal : osubstitution));
+			Graphics_text (graphics, x, ytarget, my rowLabels [p.y].get());
+			Graphics_text (graphics, x, ysource, my columnLabels [p.x].get());
+			Graphics_text (graphics, x, yoper, ( Melder_equ (my rowLabels [p.y].get(), my columnLabels [p.x].get()) ? oequal : osubstitution ));
 		}
 		Graphics_line (graphics, x, ysource + lineSpacing, x, ytarget - 0.1 * lineSpacing);
 	}
@@ -536,74 +519,69 @@ void EditDistanceTable_drawEditOperations (EditDistanceTable me, Graphics graphi
 
 void EditDistanceTable_setDefaultCosts (EditDistanceTable me, double insertionCosts, double deletionCosts, double substitutionCosts) {
 	EditCostsTable_setDefaultCosts (my editCostsTable.get(), insertionCosts, deletionCosts, substitutionCosts);
-	EditDistanceTable_findPath (me, 0);
+	EditDistanceTable_findPath (me, nullptr);
 }
 
 autoTableOfReal EditDistanceTable_to_TableOfReal_directions (EditDistanceTable me) {
-		autoTableOfReal tor;
-		EditDistanceTable_findPath (me, &tor);
-		return tor;
+	autoTableOfReal tor;
+	EditDistanceTable_findPath (me, & tor);
+	return tor;
 }
 
-void EditDistanceTable_findPath (EditDistanceTable me, autoTableOfReal *directions) {
+void EditDistanceTable_findPath (EditDistanceTable me, autoTableOfReal *out_directions) {
 	try {
 		/* What do we have to do to source to get target?
-		 * Origin [0] [0] is at bottom-left corner
+		 * Origin [1] [1] is at bottom-left corner
 		 * Target vertical, source horizontal
 		 * Going in the vertical direction is a deletion, horizontal is insertion, diagonal is substitution
 		 */
 		integer numberOfSources = my numberOfColumns - 1, numberOfTargets = my numberOfRows - 1;
-		autoNUMmatrix<short> psi (0, numberOfTargets, 0, numberOfSources);
-		autoNUMmatrix<double> delta (0, numberOfTargets, 0, numberOfSources);
+		autoINTMAT psi = newINTMATzero (my numberOfRows, my numberOfColumns);
+		autoMAT delta = newMATzero (my numberOfRows, my numberOfColumns);
 
-		for (integer j = 1; j <= numberOfSources; j ++) {
-			delta [0] [j] = delta [0] [j - 1] + EditCostsTable_getDeletionCost (my editCostsTable.get(), my columnLabels [j+1]);
-			psi [0] [j] = WARPING_fromLeft;
+		for (integer icol = 2; icol <= my numberOfColumns; icol ++) {
+			delta [1] [icol] = delta [1] [icol - 1] + EditCostsTable_getDeletionCost (my editCostsTable.get(), my columnLabels [icol].get());
+			psi [1] [icol] = WARPING_fromLeft;
 		}
-		for (integer i = 1; i <= numberOfTargets; i ++) {
-			delta [i] [0] = delta [i - 1] [0] + EditCostsTable_getInsertionCost (my editCostsTable.get(), my rowLabels [i+1]);
-			psi [i] [0] = WARPING_fromBelow;
+		for (integer irow = 2; irow <= my numberOfRows; irow ++) {
+			delta [irow] [1] = delta [irow - 1] [1] + EditCostsTable_getInsertionCost (my editCostsTable.get(), my rowLabels [irow].get());
+			psi [irow] [1] = WARPING_fromBelow;
 		}
-		for (integer j = 1; j <= numberOfSources; j ++) {
-			for (integer i = 1; i <= numberOfTargets; i ++) {
+		for (integer icol = 2; icol <= my numberOfColumns; icol ++) {
+			for (integer irow = 2; irow <= my numberOfRows; irow ++) {
 				// the substitution, deletion and insertion costs.
-				double left = delta [i] [j - 1] + EditCostsTable_getInsertionCost (my editCostsTable.get(), my rowLabels [i+1]);
-				double bottom = delta [i - 1] [j] + EditCostsTable_getDeletionCost (my editCostsTable.get(), my columnLabels [j+1]);
-				double mindist = delta [i - 1] [j - 1] + EditCostsTable_getSubstitutionCost (my editCostsTable.get(), my rowLabels [i+1], my columnLabels [j+1]); // diag
-				psi [i] [j] = WARPING_fromDiag;
+				double left = delta [irow] [icol - 1] +
+						EditCostsTable_getInsertionCost (my editCostsTable.get(), my rowLabels [irow].get());
+				double bottom = delta [irow - 1] [icol] +
+						EditCostsTable_getDeletionCost (my editCostsTable.get(), my columnLabels [icol].get());
+				double mindist = delta [irow - 1] [icol - 1] +
+						EditCostsTable_getSubstitutionCost (my editCostsTable.get(), my rowLabels [irow].get(), my columnLabels [icol].get()); // diag
+				psi [irow] [icol] = WARPING_fromDiag;
 				if (bottom < mindist) {
 					mindist = bottom;
-					psi [i] [j] = WARPING_fromBelow;
+					psi [irow] [icol] = WARPING_fromBelow;
 				}
 				if (left < mindist) {
 					mindist = left;
-					psi [i] [j] = WARPING_fromLeft;
+					psi [irow] [icol] = WARPING_fromLeft;
 				}
-				delta [i] [j] = mindist;
+				delta [irow] [icol] = mindist;
 			}
 		}
 		// find minimum distance in last column
 		integer iy = numberOfTargets, ix = numberOfSources;
-
 		WarpingPath_reset (my warpingPath.get());
-
-		WarpingPath_getPath (my warpingPath.get(), psi.peek(), iy, ix);
-
+		WarpingPath_getPath (my warpingPath.get(), psi.get(), iy, ix);
 		WarpingPath_shiftPathByOne (my warpingPath.get());
 
-		for (integer i = 0; i <= numberOfTargets; i ++) {
-			for (integer j = 0; j <= numberOfSources; j ++) {
-				my data [i + 1] [j + 1] = delta [i] [j];
-			}
-		}
-		if (directions != 0) {
+		my data.all() <<= delta.all();
+
+		if (out_directions) {
 			autoTableOfReal him = EditDistanceTable_to_TableOfReal (me);
-			for (integer i = 0; i <= numberOfTargets; i ++) {
-				for (integer j = 0; j <= numberOfSources; j ++) {
-					his data [i + 1] [j + 1] = psi [i] [j];
-				}
-			}
-			*directions = him.move();
+			for (integer irow = 1; irow <= my numberOfRows; irow ++)
+				for (integer icol = 1; icol <= my numberOfColumns; icol ++)
+					his data [irow] [icol] = double (psi [irow] [icol]);
+			*out_directions = him.move();
 		}
 	} catch (MelderError) {
 		Melder_throw (me, U": minimum path not found.");
@@ -613,13 +591,9 @@ void EditDistanceTable_findPath (EditDistanceTable me, autoTableOfReal *directio
 autoTableOfReal EditDistanceTable_to_TableOfReal (EditDistanceTable me) {
 	try {
 		autoTableOfReal thee = TableOfReal_create (my numberOfRows, my numberOfColumns);
-		for (integer j = 1; j <= my numberOfColumns; j ++) {
-			thy columnLabels [j] = Melder_dup (my columnLabels [j]);
-		}
-		for (integer i = 1; i <= my numberOfRows; i ++) {
-			thy rowLabels [i] = Melder_dup (my rowLabels [i]);
-		}
-		NUMmatrix_copyElements<double> (my data, thy data, 1, my numberOfRows, 1, my numberOfColumns);
+		thy columnLabels.all() <<= my columnLabels.all();
+		thy rowLabels.all() <<= my rowLabels.all();
+		thy data.all() <<= my data.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": no TableOfReal created.");
