@@ -4,7 +4,7 @@
  *
  * Multi Dimensional Scaling
  *
- * Copyright (C) 1993-2011, 2015-2016 David Weenink, 2015,2017 Paul Boersma
+ * Copyright (C) 1993-2018 David Weenink, 2015,2017,2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,14 +36,6 @@
 #define MDS_MATRIXCONDITIONAL 1
 #define MDS_ROWCONDITIONAL 2
 
-/* analysis level */
-#define MDS_ABSOLUTE 0
-#define MDS_RATIO	1
-#define MDS_INTERVAL 2
-#define MDS_SPLINE 3
-#define MDS_ORDINAL 4
-#define MDS_NOMINAL 5
-
 /* normalization */
 #define CONFIGURATION_COLUMNS 1
 #define CONFIGURATION_MATRIX 2
@@ -65,6 +57,7 @@
 #include "Minimizers.h"
 #include "Confusion.h"
 #include "ContingencyTable.h"
+#include "MDSVec.h"
 #include "TableOfReal_extensions.h"
 #include "Proximity.h"
 #include "Distance.h"
@@ -91,35 +84,8 @@ integer Salience_correctNegatives (Salience me);
 
 void Salience_draw (Salience me, Graphics g, int xdimension, int ydimension, bool garnish);
 
-/************************** class MDSVec ******************************/
-
-Thing_define (MDSVec, Daata) {
-	integer nProximities, nPoints;
-	double *proximity;
-	integer *iPoint, *jPoint;
-
-	void v_destroy () noexcept
-		override;
-};
-
-autoMDSVec MDSVec_create (integer nObjects);
-
-/************** class MDSVecs *********************************/
-
-Collection_define (MDSVecList, OrderedOf, MDSVec) {
-};
-
 autoConfiguration ContingencyTable_to_Configuration_ca (ContingencyTable me, integer numberOfDimensions, int scaling);
 
-/********************* class ProximityList *******************************/
-
-Collection_define (ProximityList, OrderedOf, Proximity) {
-	TableOfRealList asTableOfRealList () {
-		return reinterpret_cast<TableOfRealList> (this);
-	}
-};
-
-
 #pragma mark - class ConfusionList
 
 Collection_define (ConfusionList, OrderedOf, Confusion) {
@@ -133,14 +99,6 @@ autoConfusion ConfusionList_sum (ConfusionList me);
 
 #pragma mark - class DistanceList
 
-Collection_define (DistanceList, OrderedOf, Distance) {
-	ProximityList asProximityList () {
-		return reinterpret_cast<ProximityList> (this);
-	}
-	TableOfRealList asTableOfRealList () {
-		return reinterpret_cast<TableOfRealList> (this);
-	}
-};
 
 
 #pragma mark - class ScalarProduct
@@ -162,17 +120,6 @@ Collection_define (ScalarProductList, OrderedOf, ScalarProduct) {
 
 #pragma mark - class Dissimilarity
 
-Thing_define (Dissimilarity, Proximity) {
-};
-
-autoDissimilarity Dissimilarity_create (integer numberOfPoints);
-
-double Dissimilarity_getAdditiveConstant (Dissimilarity me);
-/*
-	Get the best estimate for the additive constant:
-		"distance = dissimilarity + constant"
-	F. Cailliez (1983), The analytical solution of the additive constant problem, Psychometrika 48, 305-308.
-*/
 
 
 #pragma mark - class Transformator
@@ -194,10 +141,9 @@ autoDistance Transformator_transform (Transformator me, MDSVec vec, Distance dis
 
 Thing_define (ISplineTransformator, Transformator) {
 	integer numberOfInteriorKnots, order, numberOfParameters;
-	double **m, *b, *knot;
+	autoMAT m;
+	autoVEC b, knot;
 
-	void v_destroy () noexcept
-		override;
 	autoDistance v_transform (MDSVec vec, Distance dist, Weight w)
 		override;
 };
@@ -225,24 +171,8 @@ autoMonotoneTransformator MonotoneTransformator_create (integer numberPoints);
 void MonotoneTransformator_setTiesProcessing (MonotoneTransformator, int tiesHandling);
 
 
-/*************** class DissimilyList ****************************/
-
-Collection_define (DissimilarityList, OrderedOf, Dissimilarity) {
-	ProximityList asProximityList () {
-		return reinterpret_cast<ProximityList> (this);
-	}
-	TableOfRealList asTableOfRealList () {
-		return reinterpret_cast<TableOfRealList> (this);
-	}
-};
-
-
 /**************** class Similarity *****************************/
 
-Thing_define (Similarity, Proximity) {
-};
-
-autoSimilarity Similarity_create (integer numberOfPoints);
 
 
 /************** KRUSKAL *********************************************/
@@ -255,11 +185,8 @@ Thing_define (Kruskal, Thing) {
 	autoProximityList proximities;
 	int stress_formula;
 	autoMDSVec vec;
-	double **dx;
+	autoMAT dx;
 	autoMinimizer minimizer;
-
-	void v_destroy () noexcept
-		override;
 };
 
 autoKruskal Kruskal_create (integer numberOfpoints, integer numberOfDimensions);
@@ -278,7 +205,7 @@ double Distance_Weight_congruenceCoefficient (Distance x, Distance y, Weight w);
 	Congruence coefficient B&G page 350.
 */
 
-void Distance_Weight_rawStressComponents (Distance fit, Distance conf, Weight weight, double *eta_fit, double *eta_conf, double *rho);
+void Distance_Weight_rawStressComponents (Distance fit, Distance conf, Weight weight, double *out_eta_fit, double *out_eta_conf, double *out_rho);
 /*
 	Computes
 		eta_fit = sum (i<j,i=1..n; w[i][j] * dfit[i][j]^2)
@@ -289,55 +216,91 @@ void Distance_Weight_rawStressComponents (Distance fit, Distance conf, Weight we
 		conf[i][j] = distance between x[i] and x[j] (in the configuration)
 */
 
-double Dissimilarity_Configuration_Transformator_Weight_stress (Dissimilarity d, Configuration c, Transformator t, Weight w, int stressMeasure);
+double Dissimilarity_Configuration_Transformator_Weight_stress (Dissimilarity d, Configuration c, Transformator t, Weight w,
+	int stressMeasure);
 
-double Dissimilarity_Configuration_Weight_absolute_stress (Dissimilarity d, Configuration c, Weight w, int stressMeasure);
+double Dissimilarity_Configuration_Weight_absolute_stress (Dissimilarity d, Configuration c, Weight w,
+	int stressMeasure);
 
-double Dissimilarity_Configuration_Weight_ratio_stress (Dissimilarity d, Configuration c, Weight w, int stressMeasure);
+double Dissimilarity_Configuration_Weight_ratio_stress (Dissimilarity d, Configuration c, Weight w,
+	int stressMeasure);
 
-double Dissimilarity_Configuration_Weight_interval_stress (Dissimilarity d, Configuration c, Weight w, int stressMeasure);
+double Dissimilarity_Configuration_Weight_interval_stress (Dissimilarity d, Configuration c, Weight w,
+	int stressMeasure);
 
-double Dissimilarity_Configuration_Weight_monotone_stress (Dissimilarity d, Configuration c, Weight w, int tiesHandling, int stressMeasure);
+double Dissimilarity_Configuration_Weight_monotone_stress (Dissimilarity d, Configuration c, Weight w,
+	int tiesHandling, int stressMeasure);
 
-double Dissimilarity_Configuration_Weight_ispline_stress (Dissimilarity d, Configuration c, Weight w, integer numberOfInteriorKnots, integer order, int stressMeasure);
+double Dissimilarity_Configuration_Weight_ispline_stress (Dissimilarity d, Configuration c, Weight w,
+	integer numberOfInteriorKnots, integer order, int stressMeasure);
 
 void Distance_Weight_smacofNormalize (Distance d, Weight w);
 
-autoConfiguration Dissimilarity_Configuration_Weight_Transformator_smacof (Dissimilarity me, Configuration conf, Weight weight, Transformator t, double tolerance, integer numberOfIterations, bool showProgress, double *stress);
+autoConfiguration Dissimilarity_Configuration_Weight_Transformator_smacof (Dissimilarity me, Configuration conf, Weight weight, Transformator t,
+	double tolerance, integer numberOfIterations, bool showProgress, double *out_stress);
 
-autoConfiguration Dissimilarity_Configuration_Weight_Transformator_multiSmacof (Dissimilarity me, Configuration conf, Weight w, Transformator t, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Configuration_Weight_Transformator_multiSmacof (Dissimilarity me, Configuration conf, Weight w, Transformator t,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
-autoConfiguration Dissimilarity_Configuration_Weight_absolute_mds (Dissimilarity dis, Configuration cstart, Weight w, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Configuration_Weight_absolute_mds (Dissimilarity dis, Configuration cstart, Weight w,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
-autoConfiguration Dissimilarity_Configuration_Weight_ratio_mds (Dissimilarity dis, Configuration cstart, Weight w, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Configuration_Weight_ratio_mds (Dissimilarity dis, Configuration cstart, Weight w,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
-autoConfiguration Dissimilarity_Configuration_Weight_interval_mds (Dissimilarity dis, Configuration cstart, Weight w, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Configuration_Weight_interval_mds (Dissimilarity dis, Configuration cstart, Weight w,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
-autoConfiguration Dissimilarity_Configuration_Weight_monotone_mds (Dissimilarity dis, Configuration cstart, Weight w, int tiesHandling, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Configuration_Weight_monotone_mds (Dissimilarity dis, Configuration cstart, Weight w, int tiesHandling,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
-autoConfiguration Dissimilarity_Configuration_Weight_ispline_mds (Dissimilarity me, Configuration cstart, Weight w, integer numberOfInteriorKnots, integer order, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Configuration_Weight_ispline_mds (Dissimilarity me, Configuration cstart, Weight w,
+	integer numberOfInteriorKnots, integer order,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress
+);
 
 autoConfiguration Dissimilarity_Weight_absolute_mds (Dissimilarity me, Weight w, integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
-autoConfiguration Dissimilarity_Weight_ratio_mds (Dissimilarity dis, Weight w, integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Weight_ratio_mds (Dissimilarity dis, Weight w,
+	integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
+autoConfiguration Dissimilarity_Weight_interval_mds (Dissimilarity dis, Weight w,
+	integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
 
-autoConfiguration Dissimilarity_Weight_interval_mds (Dissimilarity dis, Weight w, integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
-
-autoConfiguration Dissimilarity_Weight_monotone_mds(Dissimilarity me, Weight w, integer numberOfDimensions, int tiesHandling, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+autoConfiguration Dissimilarity_Weight_monotone_mds(Dissimilarity me, Weight w,
+	integer numberOfDimensions, int tiesHandling,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress
+);
 
 autoConfiguration Dissimilarity_Weight_ispline_mds (Dissimilarity me, Weight weight, integer numberOfDimensions,
-	integer numberOfInteriorKnots, integer order, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress);
+	integer numberOfInteriorKnots, integer order,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress
+);
 
-void Dissimilarity_Configuration_Weight_drawAbsoluteRegression (Dissimilarity d, Configuration c, Weight w, Graphics g, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Dissimilarity_Configuration_Weight_drawAbsoluteRegression (Dissimilarity d, Configuration c, Weight w, Graphics g,
+	double xmin, double xmax, double ymin, double ymax,
+	double size_mm, conststring32 mark, bool garnish
+);
 
-void Dissimilarity_Configuration_Weight_drawRatioRegression (Dissimilarity d, Configuration c, Weight w, Graphics g, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Dissimilarity_Configuration_Weight_drawRatioRegression (Dissimilarity d, Configuration c, Weight w, Graphics g,
+	double xmin, double xmax, double ymin, double ymax,
+	double size_mm, conststring32 mark, bool garnish
+);
 
-void Dissimilarity_Configuration_Weight_drawIntervalRegression (Dissimilarity d, Configuration c, Weight w, Graphics g, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Dissimilarity_Configuration_Weight_drawIntervalRegression (Dissimilarity d, Configuration c, Weight w, Graphics g,
+	double xmin, double xmax, double ymin, double ymax,
+	double size_mm, conststring32 mark, bool garnish
+);
 
-void Dissimilarity_Configuration_Weight_drawMonotoneRegression (Dissimilarity d, Configuration c, Weight w, Graphics g, int tiesHandling, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Dissimilarity_Configuration_Weight_drawMonotoneRegression (Dissimilarity d, Configuration c, Weight w, Graphics g,
+	int tiesHandling, double xmin, double xmax, double ymin, double ymax,
+	double size_mm, conststring32 mark, bool garnish
+);
 
-void Dissimilarity_Configuration_Weight_drawISplineRegression (Dissimilarity d, Configuration c, Weight w, Graphics g, integer numberOfInternalKnots, integer order, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Dissimilarity_Configuration_Weight_drawISplineRegression (Dissimilarity d, Configuration c, Weight w, Graphics g,
+	integer numberOfInternalKnots, integer order, double xmin, double xmax, double ymin, double ymax,
+	double size_mm, conststring32 mark, bool garnish
+);
 
 autoDistance Dissimilarity_Configuration_Transformator_Weight_transform (Dissimilarity d, Configuration c, Transformator t, Weight w);
 
@@ -349,7 +312,10 @@ autoScalarProduct Distance_to_ScalarProduct (Distance me, bool normalize);
 
 /************** DISTANCE & PROXIMITY ********************************/
 
-void Proximity_Distance_drawScatterDiagram (Proximity me, Distance thee, Graphics g, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Proximity_Distance_drawScatterDiagram (Proximity me, Distance thee, Graphics g,
+	double xmin, double xmax, double ymin, double ymax,
+	double size_mm, conststring32 mark, bool garnish
+);
 
 
 /********** COVARIANCE & CONFIGURATION ***************************/
@@ -369,26 +335,34 @@ autoConfiguration Correlation_to_Configuration (Correlation me, integer numberOf
 autoDistance Configuration_to_Distance (Configuration me);
 /* Calculates distances between the points */
 
-void Distance_Configuration_drawScatterDiagram (Distance me, Configuration him, Graphics g, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Distance_Configuration_drawScatterDiagram (Distance me, Configuration him, Graphics g, double xmin, double xmax, double ymin, double ymax, double size_mm, conststring32 mark, bool garnish);
 
-autoConfiguration Distance_to_Configuration_torsca (Distance me, int numberOfDimensions);
+autoConfiguration Distance_to_Configuration_torsca (Distance me, integer numberOfDimensions);
 
 
 /************** DISSIMILARITY & CONFIGURATION ************************/
 
-autoConfiguration Dissimilarity_to_Configuration_kruskal (Dissimilarity me, integer numberOfDimensions, integer metric, int tiesHandling, int stress_formula, double tolerance, integer numberOfIterations, integer numberOfRepetitions);
+autoConfiguration Dissimilarity_to_Configuration_kruskal (Dissimilarity me,
+	integer numberOfDimensions, integer metric, int tiesHandling, int stress_formula,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions
+);
 
-autoConfiguration Dissimilarity_Configuration_kruskal (Dissimilarity me, Configuration him, int tiesHandling, int stress_formula, double tolerance, integer numberOfIterations, integer numberOfRepetitions);
+autoConfiguration Dissimilarity_Configuration_kruskal (Dissimilarity me, Configuration him,
+	int tiesHandling, int stress_formula,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions
+);
 
 double Dissimilarity_Configuration_getStress (Dissimilarity me, Configuration him, int tiesHandling, int stress_formula);
 
-void Dissimilarity_Configuration_drawShepardDiagram (Dissimilarity me, Configuration him, Graphics g, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Dissimilarity_Configuration_drawShepardDiagram (Dissimilarity me, Configuration him, Graphics g,
+	double xmin, double xmax, double ymin, double ymax, double size_mm, conststring32 mark, bool garnish);
 
 autoDistance Dissimilarity_Configuration_monotoneRegression (Dissimilarity dis, Configuration conf, int tiesHandling);
 
 autoDistanceList DissimilarityList_Configuration_monotoneRegression (DissimilarityList dissims, Configuration conf, int tiesHandling);
 
-void Dissimilarity_Configuration_drawMonotoneRegression	(Dissimilarity me, Configuration him, Graphics g, int tiesHandling, double xmin, double xmax, double ymin, double ymax, double size_mm, const char32 *mark, bool garnish);
+void Dissimilarity_Configuration_drawMonotoneRegression	(Dissimilarity me, Configuration him, Graphics g, int tiesHandling,
+	double xmin, double xmax, double ymin, double ymax, double size_mm, conststring32 mark, bool garnish);
 
 
 /************** SIMILARITY & TABLESOFREAL ********************************/
@@ -403,8 +377,6 @@ autoSimilarity ConfigurationList_to_Similarity_cc (ConfigurationList me, Weight
 autoDistance Dissimilarity_to_Distance (Dissimilarity me, int scale);
 /* with optional scaling with "additive constant" */
 
-autoDissimilarity Distance_to_Dissimilarity (Distance me);
-
 autoDistance Dissimilarity_Distance_monotoneRegression (Dissimilarity me, Distance thee, int tiesHandling);
 
 
@@ -413,13 +385,6 @@ autoDistance Dissimilarity_Distance_monotoneRegression (Dissimilarity me, Distan
 autoDissimilarity Confusion_to_Dissimilarity_pdf (Confusion me, double minimumConfusionLevel);
 
 
-/************** DISSIMILARITY & MDSVEC ***************************************/
-
-autoMDSVec Dissimilarity_to_MDSVec (Dissimilarity me);
-
-autoMDSVecList DissimilarityList_to_MDSVecList (DissimilarityList me);
-
-
 /************** DISSIMILARITY & SIMILARITY ***********************************/
 
 autoDissimilarity Similarity_to_Dissimilarity (Similarity me, double maximumDissimilarity);
@@ -439,12 +404,12 @@ autoSimilarity Confusion_to_Similarity (Confusion me, bool normalize, int symmet
 
 autoDissimilarityList DistanceList_to_DissimilarityList (DistanceList me);
 
-autoDistanceList DissimilarityList_to_DistanceList (DissimilarityList me, int scale);
 
 
 /************** DistanceList & Configuration ************************************/
 
-void DistanceList_to_Configuration_ytl (DistanceList me, int numberOfDimensions, int normalizeScalarProducts, autoConfiguration *out1, autoSalience *out2);
+void DistanceList_to_Configuration_ytl (DistanceList me,
+	int numberOfDimensions, int normalizeScalarProducts, autoConfiguration *out1, autoSalience *out2);
 /*
 	F.W. Young, Y. Takane & R. Lewyckyj (1978), Three notes on ALSCAL,
 	Psychometrika 43, 433-435.
@@ -466,47 +431,55 @@ autoDistanceList MDSVecList_Distance_monotoneRegression (MDSVecList me, Distance
 
 void ScalarProduct_Configuration_getVariances (ScalarProduct me, Configuration thee, double *varianceExplained, double *varianceTotal);
 
-void ScalarProductList_Configuration_Salience_vaf (ScalarProductList me, Configuration thee, Salience him, double *vaf);
+void ScalarProductList_Configuration_Salience_vaf (ScalarProductList me, Configuration thee, Salience him, double *out_varianceAccountedFor);
 
 autoScalarProductList DistanceList_to_ScalarProductList (DistanceList me, bool normalize);
 
 void ScalarProductList_to_Configuration_ytl (ScalarProductList me, int numberOfDimensions, autoConfiguration *out1, autoSalience *out2);
 
-void ScalarProductList_Configuration_Salience_indscal (ScalarProductList sp, Configuration conf, Salience weights, double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2, double *vaf);
+void ScalarProductList_Configuration_Salience_indscal (ScalarProductList sp, Configuration conf, Salience weights,
+	double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2, double *out_varianceAccountedFor);
 
 
 /************** INDSCAL & ....... ***********************************/
 
-void DissimilarityList_indscal (DissimilarityList me, integer numberOfDimensions, int tiesHandlingMethod, bool normalizeScalarProducts, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress, autoConfiguration *out1, autoSalience *out2);
+void DissimilarityList_indscal (DissimilarityList me, integer numberOfDimensions, int tiesHandlingMethod, bool normalizeScalarProducts,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress, autoConfiguration *out1, autoSalience *out2);
 
-void DistanceList_to_Configuration_indscal (DistanceList me, integer numberOfDimensions, bool normalizeScalarProducts, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress, autoConfiguration *out1, autoSalience *out2);
+void DistanceList_to_Configuration_indscal (DistanceList me, integer numberOfDimensions, bool normalizeScalarProducts,
+	double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress, autoConfiguration *out1, autoSalience *out2);
 
-void DissimilarityList_Configuration_indscal (DissimilarityList me, Configuration conf, int tiesHandlingMethod, bool normalizeScalarProducts, double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2);
+void DissimilarityList_Configuration_indscal (DissimilarityList me, Configuration conf, int tiesHandlingMethod, bool normalizeScalarProducts,
+	double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2);
 
-void DistanceList_Configuration_indscal (DistanceList dists, Configuration conf, bool normalizeScalarProducts, double tolerance, integer numberOfIterations,
-	bool showProgress, autoConfiguration *out1, autoSalience *out2);
+void DistanceList_Configuration_indscal (DistanceList dists, Configuration conf, bool normalizeScalarProducts,
+	double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2);
 
-void DissimilarityList_Configuration_Salience_indscal (DissimilarityList dissims, Configuration conf, Salience w, int tiesHandlingMethod,
-	bool normalizeScalarProducts, double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2, double *vaf);
+void DissimilarityList_Configuration_Salience_indscal (DissimilarityList dissims, Configuration conf, Salience w,
+	int tiesHandlingMethod, bool normalizeScalarProducts,
+	double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2, double *out_varianceAccountedFor);
 
 autoDistanceList MDSVecList_Configuration_Salience_monotoneRegression (MDSVecList vecs, Configuration conf, Salience weights, int tiesHandlingMethod);
 
-void DistanceList_Configuration_Salience_indscal (DistanceList dists, Configuration conf, Salience weights, bool normalizeScalarProducts, double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2, double *vaf);
+void DistanceList_Configuration_Salience_indscal (DistanceList dists, Configuration conf, Salience weights, bool normalizeScalarProducts,
+	double tolerance, integer numberOfIterations, bool showProgress, autoConfiguration *out1, autoSalience *out2, double *out_varianceAccountedFor);
 
-void DistanceList_Configuration_Salience_vaf (DistanceList me, Configuration thee, Salience him, bool normalizeScalarProducts, double *vaf);
+void DistanceList_Configuration_Salience_vaf (DistanceList me, Configuration thee, Salience him, bool normalizeScalarProducts, double *out_varianceAccountedFor);
 
 void DissimilarityList_Configuration_Salience_vaf (DissimilarityList me, Configuration thee,
-	Salience him, int tiesHandlingMethod, bool normalizeScalarProducts, double *vaf);
+	Salience him, int tiesHandlingMethod, bool normalizeScalarProducts, double *out_varianceAccountedFor);
 
-void DistanceList_Configuration_vaf (DistanceList me, Configuration thee, bool normalizeScalarProducts, double *vaf);
+void DistanceList_Configuration_vaf (DistanceList me, Configuration thee, bool normalizeScalarProducts, double *out_varianceAccountedFor);
 
-void DissimilarityList_Configuration_vaf (DissimilarityList me, Configuration thee, int tiesHandlingMethod, bool normalizeScalarProducts, double *vaf);
+void DissimilarityList_Configuration_vaf (DissimilarityList me, Configuration thee,
+	int tiesHandlingMethod, bool normalizeScalarProducts, double *out_varianceAccountedFor);
 
 autoSalience ScalarProductList_Configuration_to_Salience (ScalarProductList me, Configuration him);
 
 autoSalience DistanceList_Configuration_to_Salience (DistanceList me, Configuration him, bool normalizeScalarProducts);
 
-autoSalience DissimilarityList_Configuration_to_Salience (DissimilarityList me, Configuration him, int tiesHandling, bool normalizeScalarProducts);
+autoSalience DissimilarityList_Configuration_to_Salience (DissimilarityList me, Configuration him,
+	int tiesHandling, bool normalizeScalarProducts);
 
 
 /********* Conversions from & to TableOfReal *****************************/
