@@ -1,6 +1,6 @@
 /* Minimizers.cpp
  *
- * Copyright (C) 2001-2013,2015-2016 David Weenink
+ * Copyright (C) 2001-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "melder.h"
 #include "NUM2.h"
 #include "Graphics.h"
 #include "Minimizers.h"
@@ -37,16 +38,10 @@
 
 Thing_implement (Minimizer, Thing, 0);
 
-void structMinimizer :: v_destroy () noexcept {
-	NUMvector_free<double> (p, 1);
-	NUMvector_free<double> (history, 1);
-	Minimizer_Parent :: v_destroy ();
-}
-
 static void classMinimizer_afterHook (Minimizer me, Thing /* boss */) {
-	if (my success || ! my gmonitor) {
+	if (my success || ! my gmonitor)
 		return;
-	}
+
 	if (my start == 1) {
 		Minimizer_drawHistory (me, my gmonitor, 0, my maxNumOfIterations, 0.0, 1.1 * my history [1], 1);
 		Graphics_textTop (my gmonitor, false, Melder_cat (U"Dimension of search space: ", my nParameters));
@@ -62,11 +57,12 @@ static void classMinimizer_afterHook (Minimizer me, Thing /* boss */) {
 
 void Minimizer_init (Minimizer me, integer nParameters, Daata object) {
 	my nParameters = nParameters;
-	my p = NUMvector <double> (1, nParameters);
+	my p = newVECzero (nParameters);
 	my object = object;
 	my minimum = 1e308;
 	my afterHook = classMinimizer_afterHook;
-	Minimizer_reset (me, nullptr);   // added 27/11/97
+	autoVEC my_p; 
+	Minimizer_reset (me, my_p.get());   // do weights initialization if my_p.size == 0
 }
 
 static void monitor_off (Minimizer me) {
@@ -81,32 +77,24 @@ void Minimizer_minimize (Minimizer me, integer maxNumOfIterations, double tolera
 	try {
 
 		my tolerance = tolerance;
-		if (maxNumOfIterations <= 0) {
-			return;
-		}
+		if (maxNumOfIterations <= 0) return;
 
 		if (my iteration + maxNumOfIterations > my maxNumOfIterations) {
 			my maxNumOfIterations += maxNumOfIterations;
-			if (my history) {   // clumsy because vector must have been allocated  before one can append
-				NUMvector_append <double> (& my history, 1, & my maxNumOfIterations);
-			} else {
-				my history = NUMvector<double> (1, my maxNumOfIterations);
-			}
+			my history.resize (my maxNumOfIterations);
 		}
-		if (monitor) {
+		if (monitor) 
 			my gmonitor = (Graphics) Melder_monitor (0.0, U"Starting...");
-		}
+
 		my start = 1;   // for my after()
 		my v_minimize ();
-		if (monitor) {
+		if (monitor)
 			monitor_off (me);
-		}
 		if (my success) Melder_casual (U"Minimizer_minimize:", U" minimum ", my minimum, U" reached \nafter ", my iteration,
 			U" iterations and ", my funcCalls, U" function calls.");
 	} catch (MelderError) {
-		if (monitor) {
+		if (monitor)
 			monitor_off (me);   // temporarily until better monitor facilities
-		}
 		Melder_clearError();   // memory error in history mechanism is not fatal
 	}
 }
@@ -115,57 +103,45 @@ void Minimizer_minimizeManyTimes (Minimizer me, integer numberOfTimes, integer m
 	double fopt = my minimum;
 	int monitorSingle = numberOfTimes == 1;
 
-	autoNUMvector<double> popt (NUMvector_copy<double> (my p, 1, my nParameters), 1);
+	autoVEC popt = newVECraw (my nParameters);
+	autoVEC p; 
+	popt.get () <<= my p.get();
 
-	if (! monitorSingle) {
+	if (! monitorSingle)
 		Melder_progress (0.0, U"Minimize many times");
-	}
+
 	/* on first iteration start with current parameters 27/11/97 */
-	for (integer i = 1; i <= numberOfTimes; i ++) {
+	for (integer iter = 1; iter <= numberOfTimes; iter ++) {
 		Minimizer_minimize (me, maxIterationsPerTime, tolerance, monitorSingle);
-		Melder_casual (U"Current ", i, U": minimum = ", my minimum);
+		Melder_casual (U"Current ", iter, U": minimum = ", my minimum);
 		if (my minimum < fopt) {
-			NUMvector_copyElements (my p, popt.peek(), 1, my nParameters);
+			my p.get () <<= popt.get();
 			fopt = my minimum;
 		}
-		Minimizer_reset (me, nullptr);
+		Minimizer_reset (me, p.get()); // do initialization if p.size == 0
 		if (! monitorSingle) {
 			try {
-				Melder_progress ((double) i / numberOfTimes, i, U" from ", numberOfTimes);
+				Melder_progress ((double) iter / numberOfTimes, iter, U" from ", numberOfTimes);
 			} catch (MelderError) {
 				Melder_clearError ();   // interrupted, no error
 				break;
 			}
 		}
 	}
-	if (! monitorSingle) {
+	if (! monitorSingle)
 		Melder_progress (1.0);
-	}
-	Minimizer_reset (me, popt.peek());
+	Minimizer_reset (me, popt.get());
 }
 
-#if 0
-void Minimizer_setAfterEachIteration (Minimizer me, int (*afterHook) (Minimizer me, Thing afterBoss), Thing afterBoss);
-/* set the procedure that is executed after each iteration. */
-void Minimizer_setAfterEachIteration (Minimizer me, void (*afterHook) (Minimizer me, Thing afterBoss), Thing afterBoss) {
-	my afterHook = afterHook;
-	my afterBoss = afterBoss;
-}
-#endif
-
-void Minimizer_reset (Minimizer me, const double guess []) {
-	if (guess) {
-		for (integer i = 1; i <= my nParameters; i ++) {
-			my p [i] = guess [i];
-		}
-	} else {
-		for (integer i = 1; i <= my nParameters; i ++) {
+void Minimizer_reset (Minimizer me, constVEC guess) {
+	Melder_assert (guess.size == 0 || guess.size >= my nParameters);
+	if (guess.size > 0)
+		my p.get() <<= guess;
+	else
+		for (integer i = 1; i <= my nParameters; i ++)
 			my p [i] = NUMrandomUniform (-1.0, 1.0);
-		}
-	}
 
-	NUMvector_free<double> (my history, 1);
-	my history = nullptr;
+	my history.resize (0);
 	my maxNumOfIterations = my funcCalls = my iteration = 0;
 	my success = false;
 	my minimum = 1.0e38;
@@ -173,29 +149,26 @@ void Minimizer_reset (Minimizer me, const double guess []) {
 }
 
 void Minimizer_drawHistory (Minimizer me, Graphics g, integer iFrom, integer iTo, double hmin, double hmax, int garnish) {
-	if (! my history) {
+	if (my history.size == 0)
 		return;
-	}
+
 	if (iTo <= iFrom) {
-		iFrom = 1; iTo = my iteration;
+		iFrom = 1;
+		iTo = my iteration;
 	}
 	integer itmin = iFrom, itmax = iTo;
-	if (itmin < 1) {
-		itmin = 1;
-	}
-	if (itmax > my iteration) {
-		itmax = my iteration;
-	}
-	if (hmax <= hmin) {
-		NUMvector_extrema (my history, itmin, itmax, & hmin, & hmax);
-	}
+	if (itmin < 1) itmin = 1;
+	if (itmax > my iteration) itmax = my iteration;
+	if (hmax <= hmin)
+		NUMextrema (my history.get(), itmin, itmax, & hmin, & hmax);
+
 	if (hmax <= hmin) {
 		hmin -= 0.5 * fabs (hmin);
 		hmax += 0.5 * fabs (hmax);
 	}
 	Graphics_setInner (g);
 	Graphics_setWindow (g, iFrom, iTo, hmin, hmax);
-	Graphics_function (g, my history, itmin, itmax, itmin, itmax);
+	Graphics_function (g, my history.at, itmin, itmax, itmin, itmax);
 	Graphics_unsetInner (g);
 	if (garnish) {
 		Graphics_drawInnerBox (g);
@@ -214,17 +187,17 @@ double Minimizer_getMinimum (Minimizer me) {
 Thing_implement	(SteepestDescentMinimizer, Minimizer, 0);
 
 void structSteepestDescentMinimizer :: v_minimize () {
-	autoNUMvector<double> dp (1, nParameters);
-	autoNUMvector<double> dpp (1, nParameters);
-	double fret = func (object, p);
+	autoVEC dp = newVECraw (nParameters);
+	autoVEC dpp = newVECraw (nParameters);
+	double fret = func (object, p.get());
 	while (iteration < maxNumOfIterations) {
-		dfunc (object, p, dp.peek());
+		dfunc (object, p.get(), dp.get());
 		for (integer i = 1; i <= nParameters; i ++) {
 			dpp [i] = - eta * dp [i] + momentum * dpp [i];
 			p [i] += dpp [i];
 		}
-		history [ ++iteration] = minimum = func (object, p);
-		success = ( 2.0 * fabs (fret - minimum) < tolerance * (fabs (fret) + fabs (minimum)) );
+		history [++ iteration] = minimum = func (object, p.get());
+		success = 2.0 * fabs (fret - minimum) < tolerance * (fabs (fret) + fabs (minimum));
 		if (our afterHook) {
 			try {
 				our afterHook (this, our afterBoss);
@@ -234,14 +207,12 @@ void structSteepestDescentMinimizer :: v_minimize () {
 				break;
 			}
 		}
-		if (success) {
-			break;
-		}
+		if (success) break;
 		fret = minimum;
 	}
 }
 
-autoSteepestDescentMinimizer SteepestDescentMinimizer_create (integer nParameters, Daata object, double (*func) (Daata object, const double p []), void (*dfunc) (Daata object, const double p [], double dp [])) {
+autoSteepestDescentMinimizer SteepestDescentMinimizer_create (integer nParameters, Daata object, double (*func) (Daata object, VEC p), void (*dfunc) (Daata object, VEC p, VEC)) {
 	try {
 		autoSteepestDescentMinimizer me = Thing_new (SteepestDescentMinimizer);
 		Minimizer_init (me.get(), nParameters, object);
@@ -260,7 +231,7 @@ Thing_implement (VDSmagtMinimizer, Minimizer, 0);
 void structVDSmagtMinimizer :: v_minimize () {
 	int decrease_direction_found = 1;
 	int l_iteration = 1;   // yes, we can iterate in steps, therefore local and global counter
-	double rtemp, rtemp2;
+	longdouble rtemp, rtemp2;
 
 	// df is estimate of function reduction obtainable during line search
 	// restart = 2 => line search in direction of steepest descent
@@ -270,31 +241,30 @@ void structVDSmagtMinimizer :: v_minimize () {
 	//    OK; must restart
 
 	if (restart_flag) {
-		minimum = func (object, p);
-		dfunc (object, p, dp);
+		minimum = func (object, p.get());
+		dfunc (object, p.get(), dp.get());
 		df = minimum;
 		restart = 2;
 		one_up = flag = 0;
 		gcg0 = gopt_sq = 0.0;
 	}
 	restart_flag = true;
-	while ( ++ this -> iteration <= maxNumOfIterations) {
+	while (++ this -> iteration <= maxNumOfIterations) {
 		if (flag & 1) {
 			if (one_up) {
 				decrease_direction_found = 0;
 				this -> iteration --;
 				break;
-			} else {
+			} else
 				one_up = 1;
-			}
-		} else {
+		} else
 			one_up = 0;
-		}
-		if (flag & 2) {
+
+		if (flag & 2)
 			restart = 2; /* flag & 1 ??? */
-		} else if (fabs ((double) gcg0) > 0.2 * gopt_sq) {
+		else if (fabs ((double) gcg0) > 0.2 * gopt_sq)
 			restart = 1;
-		}
+
 		if (restart == 0) {
 			rtemp = rtemp2 = 0.0;
 			for (integer i = 1; i <= nParameters; i ++) {
@@ -302,18 +272,15 @@ void structVDSmagtMinimizer :: v_minimize () {
 				rtemp2 += gc [i] * srst [i];
 			}
 			gamma = rtemp / gamma_in;
-			if (fabs (beta * gropt - gamma * rtemp2) > 0.2 * gopt_sq) {
+			if (fabs (beta * gropt - gamma * rtemp2) > 0.2 * gopt_sq)
 				restart = 1;
-			} else {
-				for (integer i = 1; i <= nParameters; i ++) {
+			else
+				for (integer i = 1; i <= nParameters; i ++)
 					s [i] = beta * s [i] + gamma * srst [i] - gc [i];
-				}
-			}
 		}
 		if (restart == 2) {
-			for (integer i = 1; i <= nParameters; i ++) {
+			for (integer i = 1; i <= nParameters; i ++)
 				s [i] = - dp [i];
-			}
 			restart = 1;
 		} else if (restart == 1) {
 			gamma_in = gropt - gr0;
@@ -336,9 +303,8 @@ void structVDSmagtMinimizer :: v_minimize () {
 			g0 [i] = dp [i];
 		}
 		gr0 = gropt = rtemp;
-		if (l_iteration == 1) {
+		if (l_iteration == 1)
 			alphamin = fabs (df / gropt);
-		}
 		if (gr0 > 0) {
 			flag = 1;
 			restart = 2;
@@ -353,32 +319,28 @@ void structVDSmagtMinimizer :: v_minimize () {
 		alplim = -1;
 		again = -1;
 		rtemp = fabs (df / gropt);
-		dalpha = alphamin < rtemp ? alphamin : rtemp;
+		dalpha = std::min (alphamin, (double) rtemp);
 		alphamin = 0;
 		do {
 			do {
 				if (lineSearch_iteration) {
-					if (! (fch == 0)) {
+					if (! (fch == 0))
 						gr2s += (temp + temp) / dalpha;
-					}
 
-					if (alplim < -0.5) {
+					if (alplim < -0.5)
 						dalpha = 9.0 * alphamin;
-					} else {
+					else 
 						dalpha = 0.5 * (alplim - alphamin);
-					}
 
 					grs = gropt + dalpha * gr2s;
-					if (gropt * grs < 0) {
+					if (gropt * grs < 0)
 						dalpha *= gropt / (gropt - grs);
-					}
 				}
 				alpha = alphamin + dalpha;
-				for (integer i = 1; i <= nParameters; i ++) {
+				for (integer i = 1; i <= nParameters; i ++)
 					pc [i] = p [i] + dalpha * s [i];
-				}
-				fc = func (object, pc);
-				dfunc (object, pc, gc);
+				fc = func (object, pc.get());
+				dfunc (object, pc.get(), gc.get());
 				l_iteration ++;
 				lineSearch_iteration ++;
 				gsq = grc = 0.0;
@@ -390,14 +352,12 @@ void structVDSmagtMinimizer :: v_minimize () {
 				gr2s = (grc - gropt) / dalpha;
 				temp = (fch + fch) / dalpha - grc - gropt;
 				if ((fc < minimum) || ((fc == minimum) && (grc / gropt > -1))) {
-					double *tmp;
 					gopt_sq = gsq;
 					history [this -> iteration] = minimum = fc;
-					tmp = p; p = pc; pc = tmp;
-					tmp = dp; dp = gc; gc = tmp;
-					if (grc * gropt <= 0) {
+					std::swap (p.at, pc.at);
+					std::swap (dp.at, gc.at);
+					if (grc * gropt <= 0)
 						alplim = alphamin;
-					}
 					alphamin = alpha;
 					gropt = grc;
 					dalpha = - dalpha;
@@ -411,12 +371,8 @@ void structVDSmagtMinimizer :: v_minimize () {
 							break;
 						}
 					}
-					if (success) {
-						return;
-					}
-					if (fabs (gropt / gr0) < lineSearchGradient) {
-						break;
-					}
+					if (success) return;
+					if (fabs (gropt / gr0) < lineSearchGradient) break;
 				} else {
 					alplim = alpha;
 				}
@@ -432,55 +388,36 @@ void structVDSmagtMinimizer :: v_minimize () {
 			gcg0 = rtemp;
 			if (fabs (gropt - gr0) > tolerance) {
 				beta = (gopt_sq - gcg0) / (gropt - gr0);
-				if (fabs (beta * gropt) < 0.2 * gopt_sq) {
-					break;
-				}
+				if (fabs (beta * gropt) < 0.2 * gopt_sq) break;
 			}
 			again ++;
-			if (again > 0) {
-				flag += 2;
-			}
+			if (again > 0) flag += 2;
 		} while (flag < 1);
 
-		if (f0 <= minimum) {
-			flag += 1;
-		}
+		if (f0 <= minimum) flag += 1;
 		df = gr0 * alphamin;
 	}
-	if (this -> iteration > maxNumOfIterations) {
+	if (this -> iteration > maxNumOfIterations)
 		this -> iteration = maxNumOfIterations;
-	}
-	if (decrease_direction_found) {
+	if (decrease_direction_found)
 		restart_flag = false;
-	}
-}
-
-void structVDSmagtMinimizer :: v_destroy () noexcept {
-	NUMvector_free<double> (dp, 1);
-	NUMvector_free<double> (pc, 1);
-	NUMvector_free<double> (gc, 1);
-	NUMvector_free<double> (g0, 1);
-	NUMvector_free<double> (s, 1);
-	NUMvector_free<double> (srst, 1);
-	NUMvector_free<double> (grst, 1);
-	VDSmagtMinimizer_Parent :: v_destroy ();
 }
 
 void structVDSmagtMinimizer :: v_reset () {
 	restart_flag = true;
 }
 
-autoVDSmagtMinimizer VDSmagtMinimizer_create (integer nParameters, Daata object, double (*func) (Daata object, const double x []), void (*dfunc) (Daata object, const double x [], double dx [])) {
+autoVDSmagtMinimizer VDSmagtMinimizer_create (integer nParameters, Daata object, double (*func) (Daata object, VEC x), void (*dfunc) (Daata object, VEC x, VEC dx)) {
 	try {
 		autoVDSmagtMinimizer me = Thing_new (VDSmagtMinimizer);
 		Minimizer_init (me.get(), nParameters, object);
-		my dp = NUMvector<double> (1, nParameters);
-		my pc = NUMvector<double> (1, nParameters);
-		my gc = NUMvector<double> (1, nParameters);
-		my g0 = NUMvector<double> (1, nParameters);
-		my s = NUMvector<double> (1, nParameters);
-		my srst = NUMvector<double> (1, nParameters);
-		my grst = NUMvector<double> (1, nParameters);
+		my dp = newVECzero (nParameters);
+		my pc = newVECzero (nParameters);
+		my gc = newVECzero (nParameters);
+		my g0 = newVECzero (nParameters);
+		my s = newVECzero (nParameters);
+		my srst = newVECzero (nParameters);
+		my grst = newVECzero (nParameters);
 		my func = func;
 		my dfunc = dfunc;
 		my lineSearchGradient = 0.9;
@@ -493,18 +430,12 @@ autoVDSmagtMinimizer VDSmagtMinimizer_create (integer nParameters, Daata object,
 
 /************ class LineMinimizer *******************************/
 
-void structLineMinimizer :: v_destroy () noexcept {
-	NUMvector_free (ptry, 1);
-	NUMvector_free (direction, 1);
-	LineMinimizer_Parent :: v_destroy ();
-}
-
 Thing_implement (LineMinimizer, Minimizer, 0);
 
-void LineMinimizer_init (LineMinimizer me, integer nParameters, Daata object, double (*func) (Daata, const double [])) {
+void LineMinimizer_init (LineMinimizer me, integer nParameters, Daata object, double (*func) (Daata, VEC p)) {
 	Minimizer_init (me, nParameters, object);
-	my direction = NUMvector<double> (1, nParameters);
-	my ptry = NUMvector<double> (1, nParameters);
+	my direction = newVECzero (nParameters);
+	my ptry = newVECzero (nParameters);
 	my func = func;
 	my maxLineStep = 100;
 }
