@@ -2,7 +2,7 @@
 #define _Polynomial_h_
 /* Polynomial.h
  *
- * Copyright (C) 1993-2017 David Weenink
+ * Copyright (C) 1993-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@
 
 void FunctionTerms_init (FunctionTerms me, double xmin, double xmax, integer numberOfCoefficients);
 
-void FunctionTerms_initFromString (FunctionTerms me, double xmin, double xmax, const char32 *s, bool allowTrailingZeros);
+void FunctionTerms_initFromString (FunctionTerms me, double xmin, double xmax, conststring32 s, bool allowTrailingZeros);
 
 autoFunctionTerms FunctionTerms_create (double xmin, double xmax, integer numberOfCoefficients);
 
@@ -53,7 +53,7 @@ double FunctionTerms_evaluate (FunctionTerms me, double x);
 
 void FunctionTerms_evaluate_z (FunctionTerms me, dcomplex *z, dcomplex *p);
 
-void FunctionTerms_evaluateTerms (FunctionTerms me, double x, double terms[]);
+void FunctionTerms_evaluateTerms (FunctionTerms me, double x, VEC terms);
 
 void FunctionTerms_getExtrema (FunctionTerms me, double x1, double x2, double *xmin, double *ymin, double *xmax, double *ymax);
 
@@ -88,14 +88,14 @@ Thing_define (Polynomial, FunctionTerms) {
 	public:
 		virtual double v_evaluate (double x);
 		virtual dcomplex v_evaluate_z (dcomplex z);
-		virtual void v_evaluateTerms (double x, double terms[]);
+		virtual void v_evaluateTerms (double x, VEC terms);
 		virtual void v_getExtrema (double x1, double x2, double *xmin, double *ymin, double *xmax, double *ymax);
 		//virtual integer v_getDegree ();   David, is het OK dat deze niet overschreven wordt? Ja
 };
 
 autoPolynomial Polynomial_create (double xmin, double xmax, integer degree);
 
-autoPolynomial Polynomial_createFromString (double xmin, double xmax, const char32 *s);
+autoPolynomial Polynomial_createFromString (double xmin, double xmax, conststring32 s);
 
 void Polynomial_scaleCoefficients_monic (Polynomial me);
 /* Make coefficent of leading term 1.0 */
@@ -111,11 +111,11 @@ dcomplex Polynomial_evaluate_z (Polynomial me, dcomplex z);
  * Precondition : my numberOfCoeffcients >= 3+2*numberOfOmegas
  * 	Polynomial is uses as a "buffer". We define it one and reuse it 
  */
-void Polynomial_initFromProductOfSecondOrderTerms (Polynomial me, double *a, integer numberOfTerms);
-autoPolynomial Polynomial_createFromProductOfSecondOrderTermsString (double xmin, double xmax, const char32 *s);
+void Polynomial_initFromProductOfSecondOrderTerms (Polynomial me, constVEC a);
+autoPolynomial Polynomial_createFromProductOfSecondOrderTermsString (double xmin, double xmax, conststring32 s);
 
-void Polynomial_initFromRealRoots (Polynomial me, double *roots, integer numberOfRoots);
-autoPolynomial Polynomial_createFromRealRootsString (double xmin, double xmax, const char32 *s);
+void Polynomial_initFromRealRoots (Polynomial me, constVEC roots);
+autoPolynomial Polynomial_createFromRealRootsString (double xmin, double xmax, conststring32 s);
 
 double Polynomial_getArea (Polynomial me, double xmin, double xmax);
 
@@ -129,8 +129,8 @@ double Polynomial_evaluate (Polynomial me, double x);
 
 void Polynomial_evaluateWithDerivative (Polynomial me, double x, double *fx, double *dfx);
 
-void Polynomial_evaluateDerivatives (Polynomial me, double x, double *derivatives /*[0.. numberOfDerivatives]*/, integer numberOfDerivatives);
-/* derivatives[0] = Polynomial_evaluate (me, x); */
+autoVEC Polynomial_evaluateDerivatives (Polynomial me, double x, long numberOfDerivatives);
+/* result[1] is function value at x, result[2..numberOfDerivatives+1] are derivatives at x. */
 
 void Polynomial_evaluateTerms (Polynomial me, double x, double terms[]);
 
@@ -146,7 +146,7 @@ void Polynomial_multiply_secondOrderFactor (Polynomial me, double factor);
  * Postcondition: my numberOfCoefficients = old_numberOfCoefficients + 2
  */
 
-void Polynomials_divide (Polynomial me, Polynomial thee, autoPolynomial *q, autoPolynomial *r);
+void Polynomials_divide (Polynomial me, Polynomial thee, autoPolynomial *out_q, autoPolynomial *out_r);
 
 void Polynomial_divide_firstOrderFactor (Polynomial me, double factor, double *p_remainder); // P(x)/(x-a)
 /* Functions: calculate coefficients of new polynomial P(x)/(x-a)
@@ -165,13 +165,13 @@ Thing_define (LegendreSeries, FunctionTerms) {
 	// overridden methods:
 	public:
 		virtual double v_evaluate (double x);
-		virtual void v_evaluateTerms (double x, double terms[]);
+		virtual void v_evaluateTerms (double x, VEC terms);
 		virtual void v_getExtrema (double x1, double x2, double *xmin, double *ymin, double *xmax, double *ymax);
 };
 
 autoLegendreSeries LegendreSeries_create (double xmin, double xmax, integer numberOfPolynomials);
 
-autoLegendreSeries LegendreSeries_createFromString (double xmin, double xmax, const char32 *s);
+autoLegendreSeries LegendreSeries_createFromString (double xmin, double xmax, conststring32 s);
 
 autoLegendreSeries LegendreSeries_getDerivative (LegendreSeries me);
 
@@ -194,7 +194,7 @@ autoRoots Polynomial_to_Roots_ev (Polynomial me);
 integer Roots_getNumberOfRoots (Roots me);
 
 void Roots_draw (Roots me, Graphics g, double rmin, double rmax, double imin, double imax,
-	const char32 *symbol, int fontSize, int garnish);
+	conststring32 symbol, int fontSize, bool garnish);
 
 dcomplex Roots_getRoot (Roots me, integer index);
 
@@ -234,13 +234,13 @@ Thing_define (ChebyshevSeries, FunctionTerms) {
 	// overridden methods:
 	public:
 		virtual double v_evaluate (double x);
-		virtual void v_evaluateTerms (double x, double terms[]);
+		virtual void v_evaluateTerms (double x, VEC terms);
 		virtual void v_getExtrema (double x1, double x2, double *xmin, double *ymin, double *xmax, double *ymax);
 };
 
 autoChebyshevSeries ChebyshevSeries_create (double xmin, double xmax, integer numberOfPolynomials);
 
-autoChebyshevSeries ChebyshevSeries_createFromString (double xmin, double xmax, const char32 *s);
+autoChebyshevSeries ChebyshevSeries_createFromString (double xmin, double xmax, conststring32 s);
 
 autoPolynomial ChebyshevSeries_to_Polynomial (ChebyshevSeries me);
 
@@ -257,28 +257,28 @@ Thing_define (MSpline, Spline) {
 	// overridden methods:
 	public:
 		virtual double v_evaluate (double x);
-		virtual void v_evaluateTerms (double x, double terms[]);
+		virtual void v_evaluateTerms (double x, VEC terms);
 };
 
 autoMSpline MSpline_create (double xmin, double xmax, integer degree, integer numberOfInteriorKnots);
 
-autoMSpline MSpline_createFromStrings (double xmin, double xmax, integer degree, const char32 *coef, const char32 *interiorKnots);
+autoMSpline MSpline_createFromStrings (double xmin, double xmax, integer degree, conststring32 coef, conststring32 interiorKnots);
 
 Thing_define (ISpline, Spline) {
 	// overridden methods:
 	public:
 		virtual double v_evaluate (double x);
-		virtual void v_evaluateTerms (double x, double terms[]);
+		virtual void v_evaluateTerms (double x, VEC terms);
 		virtual integer v_getOrder ();
 };
 
 autoISpline ISpline_create (double xmin, double xmax, integer degree, integer numberOfInteriorKnots);
 
-autoISpline ISpline_createFromStrings (double xmin, double xmax, integer degree, const char32 *coef, const char32 *interiorKnots);
+autoISpline ISpline_createFromStrings (double xmin, double xmax, integer degree, conststring32 coef, conststring32 interiorKnots);
 
 /****************** fit **********************************************/
 
-void FunctionTerms_RealTier_fit (FunctionTerms me, RealTier thee, int freezeCoefficients[], double tol, int ic, autoCovariance *c);
+void FunctionTerms_RealTier_fit (FunctionTerms me, RealTier thee, INTVEC freezeCoefficients, double tol, int ic, autoCovariance *c);
 
 autoPolynomial RealTier_to_Polynomial (RealTier me, integer degree, double tol, int ic, autoCovariance *cvm);
 
