@@ -2,7 +2,7 @@
 #define _SSCP_h_
 /* SSCP.h
  *
- * Copyright (C) 1993-2017 David Weenink
+ * Copyright (C) 1993-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +47,9 @@ void SSCP_init (SSCP me, integer dimension, integer storage);
 
 autoSSCP SSCP_create (integer dimension);
 
-void SSCP_drawTwoDimensionalEllipse_inside (SSCP me, Graphics g, double scale, const char32 *label, int fontSize);
+void SSCP_reset (SSCP me);
+
+void SSCP_drawTwoDimensionalEllipse_inside (SSCP me, Graphics g, double scale, conststring32 label, int fontSize);
 
 double SSCP_getEllipseScalefactor (SSCP me, double scale, bool confidence);
 
@@ -75,6 +77,7 @@ double SSCP_getConcentrationEllipseArea(SSCP me, double scale, bool confidence,
 double SSCP_getFractionVariation (SSCP me, integer from, integer to);
 
 autoSSCP TableOfReal_to_SSCP (TableOfReal me, integer rowb, integer rowe, integer colb, integer cole);
+autoSSCP TableOfReal_to_SSCP_rowWeights (TableOfReal me, integer rowb, integer rowe, integer colb, integer cole, integer weightColumnNumber);
 
 autoTableOfReal SSCP_TableOfReal_extractDistanceQuantileRange (SSCP me, TableOfReal thee, double qlow, double qhigh);
 
@@ -103,7 +106,7 @@ autoTableOfReal SSCP_extractCentroid (SSCP me);
 autoTableOfReal Covariance_to_TableOfReal_randomSampling (Covariance me, integer numberOfData);
 /* Generate a table with data based on the covariance matrix */
 
-void Covariance_PCA_generateOneVector (Covariance me, PCA thee, double *vec, double *buf);
+void Covariance_PCA_generateOneVector_inline (Covariance me, PCA thee, VEC vec, VEC buf);
 /*
 	A convenience function to avoid the calculation of the PCA each time we want to generate a random vector
 	The PCA must be the result of a previous SSCP_to_PCA call !
@@ -128,7 +131,7 @@ autoCCA SSCP_to_CCA (SSCP me, integer ny);
 
 autoCovariance Covariance_create (integer dimension);
 
-autoCovariance Covariance_createSimple (char32 *covars, char32 *centroid, integer numberOfObservations);
+autoCovariance Covariance_createSimple (conststring32 covars, conststring32 centroid, integer numberOfObservations);
 
 autoCovariance Covariance_create_reduceStorage (integer dimension, integer storage);
 /*
@@ -143,7 +146,7 @@ autoCovariance Covariance_create_reduceStorage (integer dimension, integer stora
 
 autoCorrelation Correlation_create (integer dimension);
 
-autoCorrelation Correlation_createSimple (char32 *s_correlations, char32 *s_centroid, integer numberOfObservations);
+autoCorrelation Correlation_createSimple (conststring32 s_correlations, conststring32 s_centroid, integer numberOfObservations);
 
 autoTableOfReal Correlation_confidenceIntervals (Correlation me, double confidenceLevel, integer numberOfTests, int method);
 /*
@@ -159,39 +162,37 @@ autoTableOfReal Correlation_confidenceIntervals (Correlation me, double confiden
 */
 
 /* Precondition ||vector|| = 1 */
-void Covariance_getMarginalDensityParameters (Covariance me, double *vector, double *mu, double *stdev);
+void Covariance_getMarginalDensityParameters (Covariance me, constVEC vector, double *p_mu, double *p_stdev);
 
-double Covariance_getMarginalProbabilityAtPosition (Covariance me, double vector[], double x);
+double Covariance_getMarginalProbabilityAtPosition (Covariance me, constVEC vector, double x);
 
-double Covariance_getProbabilityAtPosition_string (Covariance me, char32 *xpos);
+double Covariance_getProbabilityAtPosition_string (Covariance me, conststring32 xpos);
 
-double Covariance_getProbabilityAtPosition (Covariance me, double x[]);
+double Covariance_getProbabilityAtPosition (Covariance me, constVEC x);
 /* evaluate the pdf(x,mu,Sigma) at x */
 
 autoCovariance SSCP_to_Covariance (SSCP me, integer numberOfConstraints);
 
 autoSSCP Covariance_to_SSCP (Covariance me);
 
-void SSCP_getDiagonality_bartlett (SSCP me, integer numberOfContraints, double *chisq, double *prob, double *df);
+void SSCP_getDiagonality_bartlett (SSCP me, integer numberOfContraints, double *out_chisq, double *out_prob, double *out_df);
 
-void Correlation_testDiagonality_bartlett (Correlation me, integer numberOfContraints, double *chisq, double *prob, double *df);
+void Correlation_testDiagonality_bartlett (Correlation me, integer numberOfContraints, double *out_chisq, double *out_prob, double *out_df);
 /* Test if a Correlation matrix is diagonal, Morrison pp. 116-118 */
 
 autoCorrelation SSCP_to_Correlation (SSCP me);
 
-void Covariance_difference (Covariance me, Covariance thee, double *prob, double *chisq, double *df);
+void Covariance_difference (Covariance me, Covariance thee, double *out_prob, double *out_chisq, double *out_df);
 
-void Covariance_getSignificanceOfOneMean (Covariance me, integer index, double mu,	double *probability, double *t, double *df);
+void Covariance_getSignificanceOfOneMean (Covariance me, integer index, double mu,	double *out_probability, double *out_t, double *out_df);
 
-void Covariance_getSignificanceOfMeansDifference (Covariance me, integer index1, integer index2, double mu, int paired, int equalVariances,
-	double *probability, double *t, double *ndf);
+void Covariance_getSignificanceOfMeansDifference (Covariance me, integer index1, integer index2, double mu, int paired, int equalVariances,	double *out_probability, double *out_t, double *out_ndf);
 
-void Covariance_getSignificanceOfOneVariance (Covariance me, integer index, double sigmasq, double *probability, double *chisq, double *ndf);
+void Covariance_getSignificanceOfOneVariance (Covariance me, integer index, double sigmasq, double *out_probability, double *out_chisq, double *out_ndf);
 
-void Covariance_getSignificanceOfVariancesRatio (Covariance me, integer index1, integer index2, double ratio, double *probability,
-	double *f, double *df);
+void Covariance_getSignificanceOfVariancesRatio (Covariance me, integer index1, integer index2, double ratio, double *out_probability, double *out_f, double *out_df);
 
-double Covariances_getMultivariateCentroidDifference (Covariance me, Covariance thee, int equalCovariances, double *prob, double *fisher, double *df1, double *df2);
+double Covariances_getMultivariateCentroidDifference (Covariance me, Covariance thee, int equalCovariances, double *out_prob, double *out_fisher, double *out_df1, double *out_df2);
 /* Are the centroids of me and thee different?
 	Assumption: the two covariances are equal. (we pool me and thee, dimension p).
 	Two sample test of Morrison (1990), page 141:
@@ -200,7 +201,7 @@ double Covariances_getMultivariateCentroidDifference (Covariance me, Covariance
 	f has Fisher distribution with p and n1+n2-p-1 degrees of freedom.
 */
 
-void Covariances_equality (CovarianceList me, int method, double *prob, double *chisq, double *df);
+void Covariances_equality (CovarianceList me, int method, double *out_prob, double *out_chisq, double *out_df);
 /*
 	Equality of covariance.
 	method = 1 : Bartlett (Morrison, 1990)
@@ -208,26 +209,27 @@ void Covariances_equality (CovarianceList me, int method, double *prob, double *
 */
 
 autoCovariance CovarianceList_to_Covariance_pool (CovarianceList me);
+autoCovariance CovarianceList_to_Covariance_between (CovarianceList me);
+autoCovariance CovarianceList_to_Covariance_within (CovarianceList me);
 
-autoSSCPList TableOfReal_to_SSCPList_byLabel (TableOfReal me);
 
 autoSSCP SSCPList_to_SSCP_sum (SSCPList me);
 /* Sum the sscp's and weigh each means with it's numberOfObservations. */
 
 autoSSCP SSCPList_to_SSCP_pool (SSCPList me);
 
-void SSCPList_getHomegeneityOfCovariances_box (SSCPList me, double *probability, double *chisq, double *df);
+void SSCPList_getHomegeneityOfCovariances_box (SSCPList me, double *out_probability, double *out_chisq, double *out_df);
 
-autoSSCP SSCP_toTwoDimensions (SSCP me, double *v1, double *v2);
+autoSSCP SSCP_toTwoDimensions (SSCP me, constVEC v1, constVEC v2);
 
-autoSSCPList SSCPList_toTwoDimensions (SSCPList me, double *v1, double *v2);
+autoSSCPList SSCPList_toTwoDimensions (SSCPList me, constVEC v1, constVEC v2);
 
 autoSSCPList SSCPList_extractTwoDimensions (SSCPList me, integer d1, integer d2);
 
 /* For inheritors */
 
 void SSCPList_drawConcentrationEllipses (SSCPList me, Graphics g, double scale,
-	bool confidence, const char32 *label, integer d1, integer d2, double xmin, double xmax,
+	bool confidence, conststring32 label, integer d1, integer d2, double xmin, double xmax,
 	double ymin, double ymax, int fontSize, bool garnish);
 
 void SSCPList_getEllipsesBoundingBoxCoordinates (SSCPList me, double scale, bool confidence,
@@ -249,7 +251,7 @@ void SSCP_expand (SSCP me);
 void SSCP_unExpand (SSCP me);
 /* Use only if the memory is really needed! */
 
-void SSCP_expandLowerCholesky (SSCP me); // create lower square root of covariance matrix
+void SSCP_expandLowerCholeskyInverse (SSCP me); // create lower square root of covariance matrix
 
 void SSCP_unExpandLowerCholesky (SSCP me);
 
