@@ -43,16 +43,15 @@ autoSound Sound_MixingMatrix_mixPart (Sound me, MixingMatrix thee, double fromTi
 			U"The number of inputs in the MixingMatrix and the number of channels in the Sound should be equal.");
 		
 		if (fromTime == toTime) { 
-			fromTime = my xmin; toTime = my xmax; 
+			fromTime = my xmin;
+			toTime = my xmax; 
 		}
 	
 		// Determine index range. We use all the real or virtual samples that fit within [fromTime..toTime].
 
 		integer ix1 = 1 + Melder_iceiling ((fromTime - my x1) / my dx);
 		integer ix2 = 1 + Melder_ifloor ((toTime - my x1) / my dx);
-		if (ix2 < ix1) {
-			Melder_throw (U"Mixed Sound would contain no samples.");
-		}
+		Melder_require (ix1 <= ix2, U"Mixed Sound would contain no samples.");
 
 		autoSound him = Sound_create (thy numberOfRows, fromTime, toTime, ix2 - ix1 + 1, my dx, my x1 + (ix1 - 1) * my dx);
 		/*
@@ -68,33 +67,33 @@ autoSound Sound_MixingMatrix_mixPart (Sound me, MixingMatrix thee, double fromTi
 		*		ix1=-3, ix2=8 [1,8] -> [5,12] 
 		* (2) copy from [1,   nx] to [2-ix1, 1 - ix1 +  nx]
 		* 		ix1=-3, ix2=13 [1,12] -> [5,16] 
-		* (3) copy from [ix1,ix2] to [1    , ix2  -ix1 + 1]
+		* (3) copy from [ix1,ix2] to [1    , ix2 - ix1 + 1]
 		* 		ix1=4, ix2=10 [4,10] -> [1,7]
-		* (4) copy from [ix1, nx] to [1    , nx -ix1 + 1]
+		* (4) copy from [ix1, nx] to [1    , nx - ix1 + 1]
 		* 		ix1=4, ix2=21 [4,12] -> [1,9]
 		*/
 		if (! (toTime < my xmin || fromTime > my xmax)) {
+			integer off1 = 0, off2 = 0;
+			integer to_i1 = 1, to_i2 = his nx;
+			if (ix1 < 1) { // (1) + (2)
+				off1 = 1 - ix1;
+				to_i1 = 1 - ix1;
+				to_i2 = to_i1 + my nx; // (2)
+				if (ix2 < my nx) // (1)
+					to_i2 = 1 - ix1 + ix2;
+			} else { // (3) + (4)
+				off2 = ix1 - 1;
+				to_i2 = to_i1 + ix2 - ix1; // (3)
+				if (ix2 > my nx) // (4)
+					to_i2 = his nx;
+			}
 			for (integer i = 1; i <= thy numberOfRows; i ++) {
 				for (integer ichan = 1; ichan <= my ny; ichan ++) {
 					double mixingCoeffient = thy data [i] [ichan];
 					if (mixingCoeffient != 0.0) {
-						double *from = my z [ichan], *to = his z [i];
-						integer to_i1 = 1, to_i2 = his nx;
-						if (ix1 < 1) { // (1) + (2)
-							to = his z [i] + 1 - ix1;
-							to_i1 = 1 - ix1; to_i2 = to_i1 + my nx; // (2)
-							if (ix2 < my nx) { // (1)
-								to_i2 = 1 - ix1 + ix2;
-							}
-						} else { // (3) + (4)
-							from = my z [ichan] + ix1 - 1; 
-							to_i2 = to_i1 + ix2 - ix1; // (3)
-							if (ix2 > my nx) { // (4)
-								to_i2 = his nx;
-							}
-						}
 						for (integer j = 1; j <= to_i2 - to_i1 + 1; j ++) {
-							to [j] += mixingCoeffient * from [j];
+							//to [j] += mixingCoeffient * from [j];
+							his z [i] [j + off1] += mixingCoeffient * my z [ichan] [j + off2];
 						}
 					}
 				}
@@ -111,16 +110,14 @@ autoSound Sound_MixingMatrix_unmix (Sound me, MixingMatrix thee) {
 		Melder_require (my ny == thy numberOfColumns,
 			U"The number of inputs in the MixingMatrix and the number of channels in the Sound should be equal.");
 
-		autoNUMmatrix<double> minv (1, thy numberOfColumns, 1, thy numberOfRows);
-		NUMpseudoInverse (thy data, thy numberOfRows, thy numberOfColumns, minv.peek(), 0);
+		autoMAT minv = MATpseudoInverse (thy data.get(), 0.0);
 		autoSound him = Sound_create (thy numberOfColumns, my xmin, my xmax, my nx, my dx, my x1);
-		for (integer i = 1; i <= thy numberOfColumns; i ++) {
-			for (integer j = 1; j <= my nx; j ++) {
+		for (integer irow = 1; irow <= thy numberOfColumns; irow ++) {
+			for (integer icol = 1; icol <= my nx; icol ++) {
 				longdouble s = 0.0;
-				for (integer k = 1; k <= my ny; k ++) {
-					s += minv [i] [k] * my z [k] [j];
-				}
-				his z [i] [j] = (double) s;
+				for (integer k = 1; k <= my ny; k ++)
+					s += minv [irow] [k] * my z [k] [icol];
+				his z [irow] [icol] = (double) s;
 			}
 		}
 		return him;
