@@ -2,7 +2,7 @@
 #define _Sound_and_PCA_h_
 /* Sound_and_PCA.h
  *
- * Copyright (C) 2012-2017 David Weenink
+ * Copyright (C) 2012-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,11 +23,11 @@
 
 autoPCA Sound_to_PCA_channels (Sound me, double startTime, double endTime);
 
-autoSound Sound_PCA_to_Sound_pc_selectedChannels (Sound me, PCA thee, integer numberOfComponents, integer *channels, integer numberOfChannels);
+autoSound Sound_PCA_to_Sound_pc_selectedChannels (Sound me, PCA thee, integer numberOfComponents, constINTVEC channels);
 
 autoSound Sound_PCA_principalComponents (Sound me, PCA thee, integer numberOfComponents);
 
-autoSound Sound_PCA_whitenSelectedChannels (Sound me, PCA thee, integer numberOfComponents, integer *channels, integer numberOfChannels);
+autoSound Sound_PCA_whitenSelectedChannels (Sound me, PCA thee, integer numberOfComponents, constINTVEC channels);
 
 autoSound Sound_PCA_whitenChannels (Sound me, PCA thee, integer numberOfComponents);
 
