@@ -1,6 +1,6 @@
 /* SpeechSynthesizer.cpp
  *
-//  * Copyright (C) 2011-2017 David Weenink
+//  * Copyright (C) 2011-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -90,115 +90,118 @@ void EspeakVoice_setDefaults (EspeakVoice me) {
 	(void) me;
 }
 
-void EspeakVoice_initFromEspeakVoice (EspeakVoice me, voice_t *voice) {
-	my v_name = Melder_dup (Melder_peek8to32 (voice -> v_name));
+void EspeakVoice_initFromEspeakVoice (EspeakVoice me, voice_t *voicet) {
+	my v_name = Melder_dup (Melder_peek8to32 (voicet -> v_name));
 
-	my phoneme_tab_ix = voice -> phoneme_tab_ix;
-	my pitch_base = voice -> pitch_base;
-	my pitch_range = voice -> pitch_range;
+	my phoneme_tab_ix = voicet -> phoneme_tab_ix;
+	my pitch_base = voicet -> pitch_base;
+	my pitch_range = voicet -> pitch_range;
 
-	my speedf1 = voice -> speedf1;
-	my speedf2 = voice -> speedf2;
-	my speedf3 = voice -> speedf3;
+	my speedf1 = voicet -> speedf1;
+	my speedf2 = voicet -> speedf2;
+	my speedf3 = voicet -> speedf3;
 
-	my speed_percent = voice -> speed_percent;
-	my flutter = voice -> flutter;
-	my roughness = voice -> roughness;
-	my echo_delay = voice -> echo_delay;
-	my echo_amp = voice -> echo_amp;
-	my n_harmonic_peaks = voice -> n_harmonic_peaks;
-	my peak_shape = voice -> peak_shape;
-	my voicing = voice -> voicing;
-	my formant_factor = voice -> formant_factor;
-	my consonant_amp = voice -> consonant_amp;
-	my consonant_ampv = voice -> consonant_ampv;
-	my samplerate = voice -> samplerate;
+	my speed_percent = voicet -> speed_percent;
+	my flutter = voicet -> flutter;
+	my roughness = voicet -> roughness;
+	my echo_delay = voicet -> echo_delay;
+	my echo_amp = voicet -> echo_amp;
+	my n_harmonic_peaks = voicet -> n_harmonic_peaks;
+	my peak_shape = voicet -> peak_shape;
+	my voicing = voicet -> voicing;
+	my formant_factor = voicet -> formant_factor;
+	my consonant_amp = voicet -> consonant_amp;
+	my consonant_ampv = voicet -> consonant_ampv;
+	my samplerate = voicet -> samplerate;
 	my numberOfKlattParameters = 8;
 	for (integer i = 1; i <= my numberOfKlattParameters; i ++) {
-		my klattv [i] = voice -> klattv [i - 1];
+		my klattv [i] = voicet -> klattv [i - 1];
 	}
 	for (integer i = 1; i <= my numberOfFormants; i ++) {
-		my freq [i] = voice -> freq [i - 1];
-		my height [i] = voice -> height [i - 1];
-		my width [i] = voice -> width [i - 1];
-		my freqadd [i] = voice -> freqadd [i - 1];
-		my freq2 [i] = voice -> freq2 [i - 1];
-		my height2 [i] = voice -> height2 [i - 1];
-		my width2 [i] = voice -> width2 [i - 1];
-		my breath [i] = voice -> breath [i - 1];
-		my breathw [i] = voice -> breathw [i - 1];
+		my freq [i] = voicet -> freq [i - 1];
+		my height [i] = voicet -> height [i - 1];
+		my width [i] = voicet -> width [i - 1];
+		my freqadd [i] = voicet -> freqadd [i - 1];
+		my freq2 [i] = voicet -> freq2 [i - 1];
+		my height2 [i] = voicet -> height2 [i - 1];
+		my width2 [i] = voicet -> width2 [i - 1];
+		my breath [i] = voicet -> breath [i - 1];
+		my breathw [i] = voicet -> breathw [i - 1];
 	}
 	my numberOfToneAdjusts = 1000;
 	for (integer i = 1; i <= my numberOfToneAdjusts; i ++) {
-		my tone_adjust [i] = voice -> tone_adjust [i - 1];
+		my tone_adjust [i] = voicet -> tone_adjust [i - 1];
 	}
 }
 
-void EspeakVoice_into_voice (EspeakVoice me, voice_t *voice) {
+void EspeakVoice_into_voice (EspeakVoice me, voice_t *voicet) {
 
 	if (my v_name) {
-		strncpy (voice -> v_name, Melder_peek32to8 (my v_name), 40);
+		strncpy (voicet -> v_name, Melder_peek32to8 (my v_name.get()), 40);
 	}
 	if (my language_name) {
-		strncpy (voice -> language_name, Melder_peek32to8 (my language_name), 20);
+		strncpy (voicet -> language_name, Melder_peek32to8 (my language_name.get()), 20);
 	}
-	voice -> phoneme_tab_ix = my phoneme_tab_ix;
-	voice -> pitch_base = my pitch_base;
-	voice -> pitch_range = my pitch_range;
+	voicet -> phoneme_tab_ix = my phoneme_tab_ix;
+	voicet -> pitch_base = my pitch_base;
+	voicet -> pitch_range = my pitch_range;
 
-	voice -> speedf1 = my speedf1;
-	voice -> speedf2 = my speedf2;
-	voice -> speedf3 = my speedf3;
+	voicet -> speedf1 = my speedf1;
+	voicet -> speedf2 = my speedf2;
+	voicet -> speedf3 = my speedf3;
 
-	voice -> speed_percent = my speed_percent;
-	voice -> flutter = my flutter;
-	voice -> roughness = my roughness;
-	voice -> echo_delay = my echo_delay;
-	voice -> echo_amp = my echo_amp;
-	voice -> n_harmonic_peaks = my n_harmonic_peaks;
-	voice -> peak_shape = my peak_shape;
-	voice -> voicing = my voicing;
-	voice -> formant_factor = my formant_factor;
-	voice -> consonant_amp = my consonant_amp;
-	voice -> consonant_ampv = my consonant_ampv;
-	voice -> samplerate = my samplerate;
+	voicet -> speed_percent = my speed_percent;
+	voicet -> flutter = my flutter;
+	voicet -> roughness = my roughness;
+	voicet -> echo_delay = my echo_delay;
+	voicet -> echo_amp = my echo_amp;
+	voicet -> n_harmonic_peaks = my n_harmonic_peaks;
+	voicet -> peak_shape = my peak_shape;
+	voicet -> voicing = my voicing;
+	voicet -> formant_factor = my formant_factor;
+	voicet -> consonant_amp = my consonant_amp;
+	voicet -> consonant_ampv = my consonant_ampv;
+	voicet -> samplerate = my samplerate;
 	for (integer i = 1; i <= my numberOfKlattParameters; i ++) {
-		voice -> klattv [i - 1] = my klattv [i];
+		voicet -> klattv [i - 1] = my klattv [i];
 	}
 	for (integer i = 1; i <= my numberOfFormants; i ++) {
-		voice -> freq [i - 1] = my freq [i];
-		voice -> height [i - 1] = my height [i];
-		voice -> width [i - 1] = my width [i];
-		voice -> freqadd [i - 1] = my freqadd [i];
-		voice -> freq2 [i - 1] = my freq2 [i];
-		voice -> height2 [i - 1] = my height2 [i];
-		voice -> width2 [i - 1] = my width2 [i];
-		voice -> breath [i - 1] = my breath [i];
-		voice -> breathw [i - 1] = my breathw [i];
+		voicet -> freq [i - 1] = my freq [i];
+		voicet -> height [i - 1] = my height [i];
+		voicet -> width [i - 1] = my width [i];
+		voicet -> freqadd [i - 1] = my freqadd [i];
+		voicet -> freq2 [i - 1] = my freq2 [i];
+		voicet -> height2 [i - 1] = my height2 [i];
+		voicet -> width2 [i - 1] = my width2 [i];
+		voicet -> breath [i - 1] = my breath [i];
+		voicet -> breathw [i - 1] = my breathw [i];
 	}
 	for (integer i = 1; i <= my numberOfToneAdjusts; i ++) {
-		voice -> tone_adjust [i - 1] = voice -> tone_adjust [i];
+		voicet -> tone_adjust [i - 1] = voicet -> tone_adjust [i];
 	}
 }
 
 Thing_implement (SpeechSynthesizer, Daata, 1);
 
 void structSpeechSynthesizer :: v_info () {
-	SpeechSynthesizer_Parent :: v_info ();
-	MelderInfo_writeLine (U"Synthesizer version: espeak-ng ", d_synthesizerVersion);
-	MelderInfo_writeLine (U"Language: ", d_languageName);
-	MelderInfo_writeLine (U"Voice: ", d_voiceName);
-	MelderInfo_writeLine (U"Phoneme set: ", d_phonemeSet);
-	MelderInfo_writeLine (U"Input text format: ", (d_inputTextFormat == SpeechSynthesizer_INPUT_TEXTONLY ? U"text only" :
+	our SpeechSynthesizer_Parent :: v_info ();
+	MelderInfo_writeLine (U"Synthesizer version: espeak-ng ", our d_synthesizerVersion.get());
+	MelderInfo_writeLine (U"Language: ", our d_languageName.get());
+	MelderInfo_writeLine (U"Voice: ", our d_voiceName.get());
+	MelderInfo_writeLine (U"Phoneme set: ", our d_phonemeSet.get());
+	MelderInfo_writeLine (U"Input text format: ", (our d_inputTextFormat == SpeechSynthesizer_INPUT_TEXTONLY ? U"text only" :
 		d_inputTextFormat == SpeechSynthesizer_INPUT_PHONEMESONLY ? U"phonemes only" : U"tagged text"));
-	MelderInfo_writeLine (U"Input phoneme coding: ", (d_inputPhonemeCoding == SpeechSynthesizer_PHONEMECODINGS_KIRSHENBAUM ? U"Kirshenbaum" : U"???"));
-	MelderInfo_writeLine (U"Sampling frequency: ", d_samplingFrequency, U" Hz");
-	MelderInfo_writeLine (U"Word gap: ", d_wordgap, U" s");
-	MelderInfo_writeLine (U"Pitch multiplier: ", d_pitchAdjustment, U" (0.5-2.0)");
-	MelderInfo_writeLine (U"Pitch range multiplier: ", d_pitchRange, U" (0.0-2.0)");
-	MelderInfo_writeLine (U"Speaking rate: ", d_wordsPerMinute, U" words per minute", (d_estimateSpeechRate ? U" (but estimated from speech if possible)" : U" (fixed)"));
-
-	MelderInfo_writeLine (U"Output phoneme coding: ", (d_inputPhonemeCoding == SpeechSynthesizer_PHONEMECODINGS_KIRSHENBAUM ? U"Kirshenbaum" : d_inputPhonemeCoding == SpeechSynthesizer_PHONEMECODINGS_IPA ? U"IPA" : U"???"));
+	MelderInfo_writeLine (U"Input phoneme coding: ", (our d_inputPhonemeCoding == SpeechSynthesizer_PHONEMECODINGS_KIRSHENBAUM ? U"Kirshenbaum" : U"???"));
+	MelderInfo_writeLine (U"Sampling frequency: ", our d_samplingFrequency, U" Hz");
+	MelderInfo_writeLine (U"Word gap: ", our d_wordgap, U" s");
+	MelderInfo_writeLine (U"Pitch multiplier: ", our d_pitchAdjustment, U" (0.5-2.0)");
+	MelderInfo_writeLine (U"Pitch range multiplier: ", our d_pitchRange, U" (0.0-2.0)");
+	MelderInfo_writeLine (U"Speaking rate: ", our d_wordsPerMinute, U" words per minute",
+		our d_estimateSpeechRate ? U" (but estimated from speech if possible)" : U" (fixed)");
+	MelderInfo_writeLine (U"Output phoneme coding: ",
+		our d_inputPhonemeCoding == SpeechSynthesizer_PHONEMECODINGS_KIRSHENBAUM ? U"Kirshenbaum" :
+		our d_inputPhonemeCoding == SpeechSynthesizer_PHONEMECODINGS_IPA ? U"IPA" : U"???"
+	);
 }
 
 static void NUMvector_extendNumberOfElements (integer elementSize, void **v, integer lo, integer *hi, integer extraDemand)
@@ -286,11 +289,11 @@ static int synthCallback (short *wav, int numsamples, espeak_EVENT *events)
 	return 0;
 }
 
-const char32 *SpeechSynthesizer_getLanguageCode (SpeechSynthesizer me) {
+conststring32 SpeechSynthesizer_getLanguageCode (SpeechSynthesizer me) {
 	try {
-		integer irow = Table_searchColumn (espeakdata_languages_propertiesTable.get(), 2, my d_languageName);
+		integer irow = Table_searchColumn (espeakdata_languages_propertiesTable.get(), 2, my d_languageName.get());
 		if (irow == 0) {
-			Melder_throw (U"Cannot find language \"", my d_languageName, U"\".");
+			Melder_throw (U"Cannot find language \"", my d_languageName.get(), U"\".");
 		}
 		return Table_getStringValue_Assert (espeakdata_languages_propertiesTable.get(), irow, 1);
 	} catch (MelderError) {
@@ -298,11 +301,11 @@ const char32 *SpeechSynthesizer_getLanguageCode (SpeechSynthesizer me) {
 	}
 }
 
-const char32 *SpeechSynthesizer_getPhonemeCode (SpeechSynthesizer me) {
+conststring32 SpeechSynthesizer_getPhonemeCode (SpeechSynthesizer me) {
 	try {
-		integer irow = Table_searchColumn (espeakdata_languages_propertiesTable.get(), 2, my d_phonemeSet);
+		integer irow = Table_searchColumn (espeakdata_languages_propertiesTable.get(), 2, my d_phonemeSet.get());
 		if (irow == 0) {
-			Melder_throw (U"Cannot find phoneme set \"", my d_phonemeSet, U"\".");
+			Melder_throw (U"Cannot find phoneme set \"", my d_phonemeSet.get(), U"\".");
 		}
 		return Table_getStringValue_Assert (espeakdata_languages_propertiesTable.get(), irow, 1);
 	} catch (MelderError) {
@@ -310,11 +313,11 @@ const char32 *SpeechSynthesizer_getPhonemeCode (SpeechSynthesizer me) {
 	}
 }
 
-const char32 *SpeechSynthesizer_getVoiceCode (SpeechSynthesizer me) {
+conststring32 SpeechSynthesizer_getVoiceCode (SpeechSynthesizer me) {
 	try {
-		integer irow = Table_searchColumn (espeakdata_voices_propertiesTable.get(), 2, my d_voiceName);
+		integer irow = Table_searchColumn (espeakdata_voices_propertiesTable.get(), 2, my d_voiceName.get());
 		if (irow == 0) {
-			Melder_throw (U": Cannot find voice variant \"", my d_voiceName, U"\".");
+			Melder_throw (U": Cannot find voice variant \"", my d_voiceName.get(), U"\".");
 		}
 		return Table_getStringValue_Assert (espeakdata_voices_propertiesTable.get(), irow, 1);
 	} catch (MelderError) {
@@ -322,10 +325,10 @@ const char32 *SpeechSynthesizer_getVoiceCode (SpeechSynthesizer me) {
 	}
 }
 
-autoSpeechSynthesizer SpeechSynthesizer_create (const char32 *languageName, const char32 *voiceName) {
+autoSpeechSynthesizer SpeechSynthesizer_create (conststring32 languageName, conststring32 voiceName) {
 	try {
 		autoSpeechSynthesizer me = Thing_new (SpeechSynthesizer);
-		my d_synthesizerVersion = Melder_dup(ESPEAK_NG_VERSION);
+		my d_synthesizerVersion = Melder_dup (ESPEAK_NG_VERSION);
 		my d_languageName = Melder_dup (languageName);
 		(void) SpeechSynthesizer_getLanguageCode (me.get());  // existence check
 		my d_voiceName = Melder_dup (voiceName);
@@ -365,7 +368,7 @@ void SpeechSynthesizer_setSpeechOutputSettings (SpeechSynthesizer me, double sam
 	my d_outputPhonemeCoding = outputPhonemeCoding;
 }
 
-void SpeechSynthesizer_playText (SpeechSynthesizer me, const char32 *text) {
+void SpeechSynthesizer_playText (SpeechSynthesizer me, conststring32 text) {
 	autoSound thee = SpeechSynthesizer_to_Sound (me, text, nullptr, nullptr);
 	Sound_play (thee.get(), nullptr, nullptr);
 }
@@ -385,7 +388,7 @@ static autoSound buffer_to_Sound (int *wav, integer numberOfSamples, double samp
 	}
 }
 
-static void IntervalTier_addBoundaryUnsorted (IntervalTier me, integer iinterval, double time, const char32 *newLabel, bool isNewleftLabel) {
+static void IntervalTier_addBoundaryUnsorted (IntervalTier me, integer iinterval, double time, conststring32 newLabel, bool isNewleftLabel) {
 	if (time <= my xmin || time >= my xmax) {
 		Melder_throw (U"Time is outside interval domains.");
 	}
@@ -408,7 +411,7 @@ static void Table_setEventTypeString (Table me) {
 	try {
 		for (integer i = 1; i <= my rows.size; i ++) {
 			int type = Table_getNumericValue_Assert (me, i, 2);
-			const char32 *label = U"0";
+			conststring32 label = U"0";
 			if (type == espeakEVENT_WORD) {
 				label = U"word";
 			} else if (type == espeakEVENT_SENTENCE) {
@@ -444,11 +447,11 @@ static void IntervalTier_mergeSpecialIntervals (IntervalTier me) {
 	integer intervalIndex = my intervals.size;
 	TextInterval right = my intervals.at [intervalIndex];
 	integer labelLength_right = TextInterval_labelLength (right);
-	bool isEmptyInterval_right = labelLength_right == 0 || (labelLength_right == 1 && Melder_equ (right -> text, U"\001"));
+	bool isEmptyInterval_right = labelLength_right == 0 || (labelLength_right == 1 && Melder_equ (right -> text.get(), U"\001"));
 	while (intervalIndex > 1) {
 		TextInterval left = my intervals.at [intervalIndex - 1];
 		integer labelLength_left = TextInterval_labelLength (left);
-		bool isEmptyInterval_left = labelLength_left == 0 || (labelLength_left == 1 && Melder_equ (left -> text, U"\001"));
+		bool isEmptyInterval_left = labelLength_left == 0 || (labelLength_left == 1 && Melder_equ (left -> text.get(), U"\001"));
 		if (isEmptyInterval_right && isEmptyInterval_left) {
 			// remove right interval and empty left interval
 			left -> xmax = right -> xmax;
@@ -534,7 +537,7 @@ static void IntervalTier_removeVeryShortIntervals (IntervalTier me) {
 	}
 }
 
-static autoTextGrid Table_to_TextGrid (Table me, const char32 *text, double xmin, double xmax) {
+static autoTextGrid Table_to_TextGrid (Table me, conststring32 text, double xmin, double xmax) {
 	//Table_createWithColumnNames (0, L"time type type-t t-pos length a-pos sample id uniq");
 	try {
 		integer length, textLength = str32len (text);
@@ -609,7 +612,7 @@ static autoTextGrid Table_to_TextGrid (Table me, const char32 *text, double xmin
 				wordEnd = true;
 				p1w = pos;
 			} else if (type == espeakEVENT_PHONEME) {
-				const char32 *id = Table_getStringValue_Assert (me, i, idColumnIndex);
+				conststring32 id = Table_getStringValue_Assert (me, i, idColumnIndex);
 				if (time > time_phon_p) {
 					// Insert new boudary and label interval with the id
 					// TODO: Translate the id to the correct notation
@@ -644,7 +647,7 @@ static autoTextGrid Table_to_TextGrid (Table me, const char32 *text, double xmin
 	}
 }
 
-static void espeakdata_SetVoiceByName (const char32 *languageName, const char32 *voiceName)
+static void espeakdata_SetVoiceByName (conststring32 languageName, conststring32 voiceName)
 {
 	espeak_VOICE voice_selector;
 
@@ -658,7 +661,7 @@ static void espeakdata_SetVoiceByName (const char32 *languageName, const char32
 	}
 }
 
-autoSound SpeechSynthesizer_to_Sound (SpeechSynthesizer me, const char32 *text, autoTextGrid *tg, autoTable *events) {
+autoSound SpeechSynthesizer_to_Sound (SpeechSynthesizer me, conststring32 text, autoTextGrid *tg, autoTable *events) {
 	try {
 		espeak_ng_InitializePath (nullptr); // PATH_ESPEAK_DATA
 		espeak_ng_ERROR_CONTEXT context = { 0 };
@@ -685,12 +688,12 @@ autoSound SpeechSynthesizer_to_Sound (SpeechSynthesizer me, const char32 *text,
 			pitchRange_0_99 = my d_pitchRange * 49.5,
 			where 0 <= my d_pitchRange <= 2
 		*/
-		int pitchAdjustment_0_99 = (int) ((49.5 / log10(2.0)) * log10 (my d_pitchAdjustment) + 49.5);
+		int pitchAdjustment_0_99 = (int) ((49.5 / log10(2.0)) * log10 (my d_pitchAdjustment) + 49.5);   // rounded towards zero
 		espeak_ng_SetParameter (espeakPITCH, pitchAdjustment_0_99, 0);
-		int pitchRange_0_99 = (int) (my d_pitchRange * 49.5);
+		int pitchRange_0_99 = (int) (my d_pitchRange * 49.5);   // rounded towards zero
 		espeak_ng_SetParameter (espeakRANGE, pitchRange_0_99, 0);
-		const char32 *languageCode = SpeechSynthesizer_getLanguageCode (me);
-		const char32 *voiceCode = SpeechSynthesizer_getVoiceCode (me);
+		conststring32 languageCode = SpeechSynthesizer_getLanguageCode (me);
+		conststring32 voiceCode = SpeechSynthesizer_getVoiceCode (me);
 		
 		espeak_ng_SetVoiceByName(Melder_peek32to8 (Melder_cat (languageCode, U"+", voiceCode)));
 		int wordgap_10ms = my d_wordgap * 100; // espeak wordgap is in units of 10 ms
@@ -700,9 +703,9 @@ autoSound SpeechSynthesizer_to_Sound (SpeechSynthesizer me, const char32 *text,
 		
 		status =  espeak_ng_InitializeOutput (ENOUTPUT_MODE_SYNCHRONOUS, 2048, nullptr);
 		espeak_SetSynthCallback (synthCallback);
-		if (! Melder_equ (my d_phonemeSet, my d_languageName)) {
-			const char32 *phonemeCode = SpeechSynthesizer_getPhonemeCode (me);
-			int index_phon_table_list = LookupPhonemeTable (Melder_32to8 (phonemeCode));
+		if (! Melder_equ (my d_phonemeSet.get(), my d_languageName.get())) {
+			conststring32 phonemeCode = SpeechSynthesizer_getPhonemeCode (me);
+			int index_phon_table_list = LookupPhonemeTable (Melder_peek32to8 (phonemeCode));
 			if (index_phon_table_list > 0) {
 				voice -> phoneme_tab_ix = index_phon_table_list;
 				DoVoiceChange(voice);
@@ -712,7 +715,7 @@ autoSound SpeechSynthesizer_to_Sound (SpeechSynthesizer me, const char32 *text,
 		my d_events = Table_createWithColumnNames (0, U"time type type-t t-pos length a-pos sample id uniq");
 
 		#ifdef _WIN32
-                wchar_t *textW = Melder_peek32toW (text);
+                conststringW textW = Melder_peek32toW (text);
                 espeak_ng_Synthesize (textW, wcslen (textW) + 1, 0, POS_CHARACTER, 0, synth_flags, nullptr, me);
 		#else
                 espeak_ng_Synthesize (text, str32len (text) + 1, 0, POS_CHARACTER, 0, synth_flags, nullptr, me);
