@@ -29,17 +29,27 @@
 #include "NUMmachar.h"
 
 // prototypes
-static void IntervalTier_splitInterval (IntervalTier me, double time, const char32 *leftLabel, integer interval, double precision);
-static autoIntervalTier IntervalTier_IntervalTier_cutPartsMatchingLabel (IntervalTier me, IntervalTier thee, const char32 *label, double precision);
-static autoIntervalTier IntervalTiers_patch_noBoundaries (IntervalTier me, IntervalTier thee, const char32 *patchLabel, double precision);
+static void IntervalTier_splitInterval (IntervalTier me, double time, conststring32 leftLabel, integer interval, double precision);
+static autoIntervalTier IntervalTier_IntervalTier_cutPartsMatchingLabel (IntervalTier me, IntervalTier thee, conststring32 label, double precision);
+static autoIntervalTier IntervalTiers_patch_noBoundaries (IntervalTier me, IntervalTier thee, conststring32 patchLabel, double precision);
 static autoTable IntervalTiers_to_Table_textAlignmentment (IntervalTier target, IntervalTier source, EditCostsTable costs);
 
+
+static void IntervalTier_checkRange (IntervalTier me, integer startInterval, integer endinterval) {
+	Melder_require (startInterval <= endinterval, 
+		U"The interval range end number should not be smaller than the interval range start number.");
+	Melder_require (startInterval > 0, 
+		U"The specified interval range start number is ", startInterval, U", but should be at least 1.");
+	Melder_require (endinterval <= my intervals.size,
+		U"The specified interval range end number (", endinterval, U") exceeds the number of intervals (", my intervals.size, U") in this tier.");
+}
+
 autoSound SpeechSynthesizer_TextInterval_to_Sound (SpeechSynthesizer me, TextInterval thee, autoTextGrid *p_tg)
 {
 	try {
 		Melder_require (thy text && thy text[0] != U'\0', U"TextInterval should not be empty.");
 		
-		autoSound him = SpeechSynthesizer_to_Sound (me, thy text, p_tg, nullptr);
+		autoSound him = SpeechSynthesizer_to_Sound (me, thy text.get(), p_tg, nullptr);
 		return him;
 	} catch (MelderError) {
 		Melder_throw (U"Sound not created from TextInterval.");
@@ -61,7 +71,7 @@ autoSound SpeechSynthesizer_TextGrid_to_Sound (SpeechSynthesizer me, TextGrid th
 }
 
 #if 0
-static double TextGrid_getStartTimeOfFirstOccurence (TextGrid thee, integer tierNumber, const char32 *label) {
+static double TextGrid_getStartTimeOfFirstOccurence (TextGrid thee, integer tierNumber, conststring32 label) {
 	TextGrid_checkSpecifiedTierNumberWithinRange (thee, tierNumber);
 	IntervalTier intervalTier = (IntervalTier) thy tiers->at [tierNumber];
 	if (intervalTier -> classInfo != classIntervalTier) {
@@ -78,7 +88,7 @@ static double TextGrid_getStartTimeOfFirstOccurence (TextGrid thee, integer tier
 	return start;
 }
 
-static double TextGrid_getEndTimeOfLastOccurence (TextGrid thee, integer tierNumber, const char32 *label) {
+static double TextGrid_getEndTimeOfLastOccurence (TextGrid thee, integer tierNumber, conststring32 label) {
 	TextGrid_checkSpecifiedTierNumberWithinRange (thee, tierNumber);
 	IntervalTier intervalTier = (IntervalTier) thy tiers->at [tierNumber];
 	if (intervalTier -> classInfo != classIntervalTier) {
@@ -96,12 +106,12 @@ static double TextGrid_getEndTimeOfLastOccurence (TextGrid thee, integer tierNum
 }
 #endif
 
-static void IntervalTier_getLabelInfo (IntervalTier me, const char32 *label, double *labelDurations, integer *numberOfOccurences) {
+static void IntervalTier_getLabelInfo (IntervalTier me, conststring32 label, double *labelDurations, integer *numberOfOccurences) {
     *labelDurations = 0;
     *numberOfOccurences = 0;
     for (integer i = 1; i <= my intervals.size; i ++) {
         TextInterval ti = my intervals.at [i];
-        if (Melder_equ (ti -> text, label)) {
+        if (Melder_equ (ti -> text.get(), label)) {
             *labelDurations += ti -> xmax - ti -> xmin;
             (*numberOfOccurences)++;
         }
@@ -109,7 +119,7 @@ static void IntervalTier_getLabelInfo (IntervalTier me, const char32 *label, dou
 }
 
 #define TIMES_ARE_CLOSE(x,y) (fabs((x)-(y)) < precision)
-void IntervalTier_splitInterval (IntervalTier me, double time, const char32 *leftLabel, integer interval, double precision) {
+void IntervalTier_splitInterval (IntervalTier me, double time, conststring32 leftLabel, integer interval, double precision) {
     try {
 		Melder_assert (interval > 0);
         TextInterval ti = nullptr;
@@ -134,7 +144,7 @@ void IntervalTier_splitInterval (IntervalTier me, double time, const char32 *lef
 
 }
 
-static autoTextTier TextTier_IntervalTier_cutPartsMatchingLabel (TextTier me, IntervalTier thee, const char32 *label, double precision) {
+static autoTextTier TextTier_IntervalTier_cutPartsMatchingLabel (TextTier me, IntervalTier thee, conststring32 label, double precision) {
     try {
         if (my xmin != thy xmin || my xmax != thy xmax) {
             Melder_throw (U"Domains should be equal.");
@@ -144,7 +154,7 @@ static autoTextTier TextTier_IntervalTier_cutPartsMatchingLabel (TextTier me, In
         autoTextTier him = TextTier_create (0.0, my xmax - my xmin);
         for (integer j = 1; j <= thy intervals.size; j ++) {
             TextInterval cut = thy intervals.at [j];
-            if (Melder_equ (cut -> text, label)) {
+            if (Melder_equ (cut -> text.get(), label)) {
                 timeCut += cut -> xmax - cut -> xmin;
             } else {
                  while (myIndex <= my points.size) {
@@ -155,7 +165,7 @@ static autoTextTier TextTier_IntervalTier_cutPartsMatchingLabel (TextTier me, In
                     } else if (tp -> number < cut -> xmax + precision) {
                         // point is in (no)cut
                         double time = tp -> number - my xmin - timeCut;
-                        TextTier_addPoint (him.get(), time, tp -> mark);
+                        TextTier_addPoint (him.get(), time, tp -> mark.get());
                         myIndex++;
                     } else {
                         break;
@@ -171,7 +181,7 @@ static autoTextTier TextTier_IntervalTier_cutPartsMatchingLabel (TextTier me, In
 }
 
 // Cut parts from me marked by labels in thee
-autoIntervalTier IntervalTier_IntervalTier_cutPartsMatchingLabel (IntervalTier me, IntervalTier thee, const char32 *label, double precision) {
+autoIntervalTier IntervalTier_IntervalTier_cutPartsMatchingLabel (IntervalTier me, IntervalTier thee, conststring32 label, double precision) {
     try {
         if (my xmin != thy xmin || my xmax != thy xmax) {
             Melder_throw (U"Domains should be equal.");
@@ -184,7 +194,7 @@ autoIntervalTier IntervalTier_IntervalTier_cutPartsMatchingLabel (IntervalTier m
         integer myInterval = 1;
         for (integer j = 1; j <= thy intervals.size; j ++) {
             TextInterval cut = thy intervals.at [j];
-            if (Melder_equ (cut -> text, label)) { // trim
+            if (Melder_equ (cut -> text.get(), label)) { // trim
                 while (myInterval <= my intervals.size) {
                     TextInterval ti = my intervals.at [myInterval];
                     if (ti -> xmin > cut -> xmin - precision && ti -> xmax < cut -> xmax + precision) {
@@ -226,11 +236,11 @@ autoIntervalTier IntervalTier_IntervalTier_cutPartsMatchingLabel (IntervalTier m
             TextInterval ti = my intervals.at [i];
             time += durations[i];
             if (fabs (time - totalDuration) > precision) {
-                IntervalTier_splitInterval (him.get(), time, ti -> text, hisInterval, precision);
+                IntervalTier_splitInterval (him.get(), time, ti -> text.get(), hisInterval, precision);
                 hisInterval++;
             } else { // last interval
                 TextInterval histi = his intervals.at [hisInterval];
-                TextInterval_setText (histi, ti -> text);
+                TextInterval_setText (histi, ti -> text.get());
             }
         }
         return him;
@@ -239,7 +249,7 @@ autoIntervalTier IntervalTier_IntervalTier_cutPartsMatchingLabel (IntervalTier m
     }
 }
 
-autoTextGrid TextGrid_IntervalTier_cutPartsMatchingLabel (TextGrid me, IntervalTier thee, const char32 *label, double precision) {
+autoTextGrid TextGrid_IntervalTier_cutPartsMatchingLabel (TextGrid me, IntervalTier thee, conststring32 label, double precision) {
     try {
         if (my xmin != thy xmin || my xmax != thy xmax) {
             Melder_throw (U"Domains should be equal.");
@@ -247,7 +257,7 @@ autoTextGrid TextGrid_IntervalTier_cutPartsMatchingLabel (TextGrid me, IntervalT
         double cutDurations = 0;
         for (integer i = 1; i <= thy intervals.size; i ++) {
             TextInterval cut = thy intervals.at [i];
-            if (Melder_equ (cut -> text, label)) {
+            if (Melder_equ (cut -> text.get(), label)) {
                 cutDurations += cut -> xmax - cut -> xmin;
             }
         }
@@ -273,7 +283,7 @@ autoTextGrid TextGrid_IntervalTier_cutPartsMatchingLabel (TextGrid me, IntervalT
 
 // Patch thy intervals that match patchLabel into my intervals
 // The resulting IntervalTier has thy xmin as starting time and thy xmax as end time
-autoIntervalTier IntervalTiers_patch_noBoundaries (IntervalTier me, IntervalTier thee, const char32 *patchLabel, double precision) {
+autoIntervalTier IntervalTiers_patch_noBoundaries (IntervalTier me, IntervalTier thee, conststring32 patchLabel, double precision) {
     try {
 		autoNUMvector <double> durations ((integer) 0, my intervals.size + 1);
 		for (integer i = 1; i <= my intervals.size; i ++) {
@@ -284,7 +294,7 @@ autoIntervalTier IntervalTiers_patch_noBoundaries (IntervalTier me, IntervalTier
 		double xShift = thy xmin - my xmin;
         for (integer j = 1; j <= thy intervals.size; j ++) {
             TextInterval patch = thy intervals.at [j];
-            if (Melder_equ (patch -> text, patchLabel)) {
+            if (Melder_equ (patch -> text.get(), patchLabel)) {
 				if (j == 1) {
 					xShift += durations[0] = patch -> xmax - patch -> xmin;
 				} else if (j == thy intervals.size) {
@@ -324,7 +334,7 @@ autoIntervalTier IntervalTiers_patch_noBoundaries (IntervalTier me, IntervalTier
 		for (integer i = 1; i <= my intervals.size; i ++) {
 			TextInterval ti = my intervals.at [i];
 			time += durations [i];
-			IntervalTier_splitInterval (him.get(), time, ti -> text, hisInterval, precision);
+			IntervalTier_splitInterval (him.get(), time, ti -> text.get(), hisInterval, precision);
 			hisInterval++;
 		}
 		if (durations [my intervals.size + 1] > 0) {
@@ -338,7 +348,7 @@ autoIntervalTier IntervalTiers_patch_noBoundaries (IntervalTier me, IntervalTier
 }
 
 #if 0
-static autoIntervalTier IntervalTiers_patch (IntervalTier me, IntervalTier thee, const char32 *patchLabel, double precision) {
+static autoIntervalTier IntervalTiers_patch (IntervalTier me, IntervalTier thee, conststring32 patchLabel, double precision) {
     try {
         autoIntervalTier him = IntervalTier_create (thy xmin, thy xmax);
         integer myInterval = 1, hisInterval = 1;
@@ -393,20 +403,20 @@ static autoIntervalTier IntervalTiers_patch (IntervalTier me, IntervalTier thee,
 }
 #endif
 
-static autoTextTier TextTier_IntervalTier_patch (TextTier me, IntervalTier thee, const char32 *patchLabel, double precision) {
+static autoTextTier TextTier_IntervalTier_patch (TextTier me, IntervalTier thee, conststring32 patchLabel, double precision) {
     try {
         integer myIndex = 1;
         autoTextTier him = TextTier_create (thy xmin, thy xmax);
         double xShift = thy xmin - my xmin;
         for (integer i = 1; i <= thy intervals.size; i ++) {
             TextInterval ti = thy intervals.at [i];
-            if (Melder_equ (ti -> text, patchLabel)) {
+            if (Melder_equ (ti -> text.get(), patchLabel)) {
                 if (i > 1) {
                     while (myIndex <= my points.size) {
                         TextPoint tp = my points.at [myIndex];
                         double time = tp -> number + xShift;
                         if (time < ti -> xmin + precision) {
-                            autoTextPoint newPoint = TextPoint_create (time, tp -> mark);
+                            autoTextPoint newPoint = TextPoint_create (time, tp -> mark.get());
                             his points. addItem_move (newPoint.move());
                         } else {
                             break;
@@ -420,7 +430,7 @@ static autoTextTier TextTier_IntervalTier_patch (TextTier me, IntervalTier thee,
                     TextPoint tp = my points.at [myIndex];
                     double time = tp -> number + xShift;
                     if (time < ti -> xmin + precision) {
-                        autoTextPoint newPoint = TextPoint_create (time, tp -> mark);
+                        autoTextPoint newPoint = TextPoint_create (time, tp -> mark.get());
                         his points. addItem_move (newPoint.move());
                     }
                     myIndex++;
@@ -433,7 +443,7 @@ static autoTextTier TextTier_IntervalTier_patch (TextTier me, IntervalTier thee,
     }
 }
 
-autoTextGrid TextGrid_IntervalTier_patch (TextGrid me, IntervalTier thee, const char32 *patchLabel, double precision) {
+autoTextGrid TextGrid_IntervalTier_patch (TextGrid me, IntervalTier thee, conststring32 patchLabel, double precision) {
     try {
         double patchDurations;
         integer numberOfPatches;
@@ -467,74 +477,66 @@ autoTextGrid SpeechSynthesizer_Sound_TextInterval_align (SpeechSynthesizer me, S
 		Melder_require (fabs (1.0 / thy dx - my d_samplingFrequency) < 1e-9, 
 			U"The sampling frequencies of the SpeechSynthesizer and the Sound should be equal.");
 
-		integer numberOfTokens = Melder_countTokens (his text);
+		autostring32vector tokens = newSTRVECtokenize (his text.get());
+		integer numberOfTokens = tokens.size;
 		Melder_require (numberOfTokens > 0, U"The interval should have text.");
 		
-		// Remove silent intervals from start and end of sounds
+		/*
+			Remove silent intervals from start and end of sounds because
+			1. it will improve the word rate guess
+			2. it will improve the DTW matching.
+		*/
+		
 		double minPitch = 200.0, timeStep = 0.005, precision = thy dx;
-		double t1_thee, t2_thee;
-		autoSound s_thee = Sound_trimSilencesAtStartAndEnd (thee, 0.0, minPitch, timeStep, silenceThreshold, minSilenceDuration, minSoundingDuration, & t1_thee, & t2_thee);
-		double s_thee_duration = s_thee -> xmax - s_thee -> xmin;
-		bool hasSilence_thee = fabs (t1_thee - thy xmin) > precision || fabs (t2_thee - thy xmax) > precision;
+		double startTimeOfSounding, endTimeOfSounding;
+		autoSound soundTrimmed = Sound_trimSilencesAtStartAndEnd (thee, 0.0, minPitch, timeStep, silenceThreshold, minSilenceDuration, minSoundingDuration, & startTimeOfSounding, & endTimeOfSounding);
+		double duration_soundTrimmed = soundTrimmed -> xmax - soundTrimmed -> xmin;
+		bool hasSilence_sound = fabs (startTimeOfSounding - thy xmin) > precision || fabs (endTimeOfSounding - thy xmax) > precision;
 
 		if (my d_estimateSpeechRate) {
 			// estimate speaking rate with the number of words per minute from the text
-			double wordsPerMinute_rawTokens = 60.0 * numberOfTokens / s_thee_duration;
+			double wordsPerMinute_rawTokens = 60.0 * numberOfTokens / duration_soundTrimmed;
 			// compensation for long words: 5 characters / word
-			double wordsPerMinute_rawText = 60.0 * (str32len (his text) / 5.0) / s_thee_duration;
+			double wordsPerMinute_rawText = 60.0 * (str32len (his text.get()) / 5.0) / duration_soundTrimmed;
 			my d_wordsPerMinute = Melder_ifloor (0.5 * (wordsPerMinute_rawTokens + wordsPerMinute_rawText));
 		}
-		autoTextGrid tg2;
-		autoSound synth = SpeechSynthesizer_TextInterval_to_Sound (me, him, & tg2);
-		autoTextGrid silentTextGrid;
+		
+		autoTextGrid textgrid_synth, textgrid_synth_sounding;
+		autoSound synth = SpeechSynthesizer_TextInterval_to_Sound (me, him, & textgrid_synth);
+
 		/*
-		 * For the synthesizer the silence threshold has to be < -30 dB, otherwise fricatives will not
-		 * be found as sounding! This is ok since silences are almost at zero amplitudes for synthesized sounds.
-		 * We also have to decrease the minimum silence and minimum sounding duration to catch, for example,
-		 * the final plosive "t" from the synthesized sound "text"
-		 *
+			For the synthesizer the silence threshold has to be < -30 dB, otherwise fricatives will not
+			be found as sounding! This is ok since silences are almost at zero amplitudes for synthesized sounds.
+			We also have to decrease the minimum silence and minimum sounding duration to catch, for example,
+			the final plosive "t" from the synthesized sound "text".
 		 */
-		double synth_silenceThreshold = -40.0, synth_minSilenceDuration = 0.05, synth_minSoundingDuration = 0.05;
-		double t1_synth, t2_synth;
-		autoSound synth_trimmed = Sound_trimSilencesAtStartAndEnd (synth.get(), 0.0, minPitch, timeStep,
-			synth_silenceThreshold, synth_minSilenceDuration, synth_minSoundingDuration, & t1_synth, & t2_synth);
-		double synth_trimmed_duration = synth_trimmed -> xmax - synth_trimmed -> xmin;
-		bool hasSilence_synth = fabs (t1_synth - synth -> xmin) > precision || fabs (t2_synth - synth -> xmax) > precision;
-		if (hasSilence_synth) {
-			silentTextGrid = TextGrid_extractPart (tg2.get(), t1_synth, t2_synth, true);
-		}
-		double analysisWidth = 0.02, dt = 0.005, band = 0.0;
-		// compare the durations of the two sounds to get an indication of the slope constraint of the DTW
-		double slope = s_thee_duration / synth_trimmed_duration;
+		double silenceThreshold_synth = -40.0, minSilenceDuration_synth = 0.05, minSoundingDuration_synth = 0.05;
+		double startTimeOfSounding_synth, endTimeOfSounding_synth;
+		autoSound synthTrimmed = Sound_trimSilencesAtStartAndEnd (synth.get(), 0.0, minPitch, timeStep, silenceThreshold_synth,
+			minSilenceDuration_synth, minSoundingDuration_synth, & startTimeOfSounding_synth, & endTimeOfSounding_synth);
+		double synthTrimmed_duration = synthTrimmed -> xmax - synthTrimmed -> xmin;
+		bool hasSilence_synth = fabs (startTimeOfSounding_synth - synth -> xmin) > precision || 
+								fabs (endTimeOfSounding_synth - synth -> xmax) > precision;
+
+		if (hasSilence_synth) textgrid_synth_sounding = 
+			TextGrid_extractPart (textgrid_synth.get(), startTimeOfSounding_synth, endTimeOfSounding_synth, true);
+
+		// compare the durations of the two sounds to get an indication of the slope constraint needed for the DTW
+		
+		double slope = duration_soundTrimmed / synthTrimmed_duration;
 		slope = (slope > 1.0 ? slope : 1.0 / slope);
-        int constraint = (slope < 1.5 ? 4 : slope < 2.0 ? 3 : slope < 3.0 ? 2 : 1);
-		//autoMFCC m1 = Sound_to_MFCC ((hasSilence_thee ? s_thee.get() : thee),
-		//	numberOfCoefficients, analysisWidth, dt, f1_mel, fmax_mel, df_mel);
-		//autoMFCC m2 = Sound_to_MFCC ((hasSilence_synth ? synth_trimmed.get() : synth.get()),
-		//	numberOfCoefficients, analysisWidth, dt, f1_mel, fmax_mel, df_mel);
-		//double wc = 1, wle = 0, wr = 0, wer = 0, dtr = 0;
-		//int matchStart = 1, matchEnd = 1, constraint = 4; // no 1/3 1/2 2/3
-		//autoDTW dtw = CCs_to_DTW (m1.get(), m2.get(), wc, wle, wr, wer, dtr, matchStart, matchEnd, constraint);
-        autoDTW dtw = Sounds_to_DTW ((hasSilence_thee ? s_thee.get() : thee), (hasSilence_synth ? synth_trimmed.get() : synth.get()),
-			analysisWidth, dt, band, constraint);
-		autoTextGrid result = DTW_TextGrid_to_TextGrid (dtw.get(), (hasSilence_synth ? silentTextGrid.get() : tg2.get()), precision);
-		if (hasSilence_thee) {
-			if (t1_thee > thy xmin) {
+        int constraint = (slope < 1.5 ? 4 : slope < 2.0 ? 3 : slope < 3.0 ? 2 : 1); // TODO enums
+		
+		double analysisWidth = 0.02, dt = 0.005, band = 0.0;
+        autoDTW dtw = Sounds_to_DTW ((hasSilence_sound ? soundTrimmed.get() : thee), 
+			(hasSilence_synth ? synthTrimmed.get() : synth.get()), analysisWidth, dt, band, constraint);
+		
+		autoTextGrid result = DTW_TextGrid_to_TextGrid (dtw.get(), (hasSilence_synth ? textgrid_synth_sounding.get() : textgrid_synth.get()), precision);
+		if (hasSilence_sound) {
+			if (startTimeOfSounding > thy xmin)
 				TextGrid_setEarlierStartTime (result.get(), thy xmin, U"", U"");
-			}
-			if (result -> xmax > thy xmax) { // one sample or so out of sync
-				result -> xmax = thy xmax;
-				for (integer itier = 1; itier <= 4; itier	++) {
-					IntervalTier tier = result -> intervalTier_cast (itier);
-					tier -> xmax = thy xmax;
-					TextInterval textInterval = tier -> intervals.at [tier -> intervals.size];
-					textInterval -> xmax = thy xmax;
-				}
-			} else {	
-				if (t2_thee < thy xmax + thy dx) {
+			if (endTimeOfSounding < thy xmax || result -> xmax < thy xmax)
 					TextGrid_setLaterEndTime (result.get(), thy xmax, U"", U"");
-				}
-			}
 		}
 		return result;
 	} catch (MelderError) {
@@ -563,7 +565,7 @@ static autoTextGrid SpeechSynthesizer_Sound_TextInterval_align2 (SpeechSynthesiz
 		Melder_require (fabs (1.0 / thy dx - my d_samplingFrequency) < 1e-9, 
 			U"The sampling frequencies of the SpeechSynthesizer and the Sound should be equal.");
 
-        const char32 *trimLabel = U"trim";
+        conststring32 trimLabel = U"trim";
         // 1. trim the silences of the sound
         /*
          * For the synthesizer the silence threshold has to be < -30 dB, otherwise fricatives will not
@@ -607,15 +609,16 @@ static autoTextGrid SpeechSynthesizer_Sound_TextInterval_align2 (SpeechSynthesiz
 
 autoTextGrid SpeechSynthesizer_Sound_IntervalTier_align (SpeechSynthesizer me, Sound thee, IntervalTier him, integer istart, integer iend, double silenceThreshold, double minSilenceDuration, double minSoundingDuration) {
     try {
-		Melder_require (istart > 0 && istart <= iend && iend <= his intervals.size,
-			U"Not a valid interval range.");
+		IntervalTier_checkRange (him, istart, iend);
+        TextInterval tib = his intervals.at [istart];
+        TextInterval tie = his intervals.at [iend];
+		Melder_require (tib -> xmin >= thy xmin && tie -> xmax <= thy xmax, 
+			U"The chosen interval(s) must lie within the sound.");
         OrderedOf<structTextGrid> textgrids;
-        TextInterval tb = his intervals.at [istart];
-        TextInterval te = his intervals.at [iend];
-        autoTextGrid result = TextGrid_create (tb -> xmin, te -> xmax, U"sentence clause word phoneme", U"");
+        autoTextGrid result = TextGrid_create (tib -> xmin, tie -> xmax, U"sentence clause word phoneme", U"");
         for (integer iint = istart; iint <= iend; iint ++) {
             TextInterval ti = his intervals.at [iint];
-            if (ti -> text && str32len (ti -> text) > 0) {
+            if (ti -> text && ti -> text [0] != U'\0') {
                 autoSound sound = Sound_extractPart (thee, ti -> xmin, ti -> xmax,  kSound_windowShape::RECTANGULAR, 1, true);
                 autoTextGrid grid = SpeechSynthesizer_Sound_TextInterval_align (me, sound.get(), ti, silenceThreshold, minSilenceDuration, minSoundingDuration);
                 textgrids. addItem_move (grid.move());
@@ -632,16 +635,14 @@ autoTextGrid SpeechSynthesizer_Sound_IntervalTier_align (SpeechSynthesizer me, S
 
 static autoTextGrid SpeechSynthesizer_Sound_IntervalTier_align2 (SpeechSynthesizer me, Sound thee, IntervalTier him, integer istart, integer iend, double silenceThreshold, double minSilenceDuration, double minSoundingDuration, double trimDuration) {
     try {
-		Melder_require (istart > 0 && istart <= iend && iend <= his intervals.size,
-			U"Not a valid interval range.");
-
-        OrderedOf<structTextGrid> textgrids;
+		IntervalTier_checkRange (him, istart, iend);
         TextInterval tb = his intervals.at [istart];
         TextInterval te = his intervals.at [iend];
         autoTextGrid result = TextGrid_create (tb -> xmin, te -> xmax, U"sentence clause word phoneme", U"");
+        OrderedOf<structTextGrid> textgrids;
         for (integer iint = istart; iint <= iend; iint ++) {
             TextInterval ti = his intervals.at [iint];
-            if (ti -> text && str32len (ti -> text) > 0) {
+            if (ti -> text && ti -> text [0] != U'\0') {
                 autoSound sound = Sound_extractPart (thee, ti -> xmin, ti -> xmax,  kSound_windowShape::RECTANGULAR, 1, true);
                 autoTextGrid grid = SpeechSynthesizer_Sound_TextInterval_align2 (me, sound.get(), ti, silenceThreshold, minSilenceDuration, minSoundingDuration, trimDuration);
                 textgrids. addItem_move (grid.move());
@@ -657,7 +658,9 @@ static autoTextGrid SpeechSynthesizer_Sound_IntervalTier_align2 (SpeechSynthesiz
 }
 
 autoTextGrid SpeechSynthesizer_Sound_TextGrid_align (SpeechSynthesizer me, Sound thee, TextGrid him, integer tierNumber, integer istart, integer iend, double silenceThreshold, double minSilenceDuration, double minSoundingDuration) {
-	try {//TODO: check not empty tier
+	try {
+		Melder_require (thy xmin == his xmin && thy xmax == his xmax, 
+			U"The domains of the Sound and the TextGrid must be equal.");
 		IntervalTier tier = TextGrid_checkSpecifiedTierIsIntervalTier (him, tierNumber);
 		autoTextGrid grid = SpeechSynthesizer_Sound_IntervalTier_align (me, thee, tier, istart, iend, silenceThreshold, minSilenceDuration, minSoundingDuration);
 		return grid;
@@ -680,11 +683,11 @@ autoTextGrid SpeechSynthesizer_Sound_TextGrid_align2 (SpeechSynthesizer me, Soun
 static autoStrings IntervalTier_to_Strings_withOriginData (IntervalTier me, integer *from) {
 	try {
 		autoStrings thee = Thing_new (Strings);
-		thy strings = NUMvector<char32 *> (1, my intervals.size);
+		thy strings = autostring32vector (my intervals.size);
 		for (integer i = 1; i <= my intervals.size; i ++) {
 			TextInterval ti = my intervals.at [i];
 			if (ti -> text && ti -> text [0] != U'\0') {
-				thy strings [++ thy numberOfStrings] = Melder_dup (ti -> text);
+				thy strings [++ thy numberOfStrings] = Melder_dup (ti -> text.get());
 				if (from) {
 					from [thy numberOfStrings] = i;
 				}
@@ -716,20 +719,20 @@ autoTable IntervalTiers_to_Table_textAlignmentment (IntervalTier target, Interva
 			structPairOfInteger p1 = edit -> warpingPath -> path[i - 1];
 			double targetStart = undefined, targetEnd = undefined;
 			double sourceStart = undefined, sourceEnd = undefined;
-			const char32 * targetText = U"", *sourceText = U"";
+			conststring32 targetText = U"", sourceText = U"";
 			integer targetInterval = p.y > 1 ? targetOrigin[p.y - 1] : 0;
 			integer sourceInterval = p.x > 1 ? sourceOrigin[p.x - 1] : 0;
 			if (targetInterval > 0) {
 				TextInterval ti = target -> intervals.at [targetInterval];
 				targetStart = ti -> xmin;
 				targetEnd =  ti -> xmax;
-				targetText = ti -> text;
+				targetText = ti -> text.get();
 			}
 			if (sourceInterval > 0) {
 				TextInterval ti = source -> intervals.at [sourceInterval];
 				sourceStart = ti -> xmin;
 				sourceEnd =  ti -> xmax;
-				sourceText = ti -> text;
+				sourceText = ti -> text.get();
 			}
 			integer irow = i - 1;
 			if (p.y == p1.y) { // deletion
@@ -770,7 +773,7 @@ autoTable IntervalTiers_to_Table_textAlignmentment (IntervalTier target, Interva
 	}
 }
 
-autoTable TextGrids_to_Table_textAlignmentment (TextGrid target, integer ttier, TextGrid source, integer stier, EditCostsTable costs) {
+autoTable TextGrids_to_Table_textAlignment (TextGrid target, integer ttier, TextGrid source, integer stier, EditCostsTable costs) {
 	try {
 		IntervalTier targetTier = TextGrid_checkSpecifiedTierIsIntervalTier (target, ttier);
 		IntervalTier sourceTier = TextGrid_checkSpecifiedTierIsIntervalTier (source, stier);
