@@ -1,6 +1,6 @@
 /* VowelEditor.cpp
  *
- * Copyright (C) 2008-2017 David Weenink, 2015,2017 Paul Boersma
+ * Copyright (C) 2008-2019 David Weenink, 2015,2017,2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,9 +91,6 @@ Thing_implement (VowelEditor, Editor, 0);
 #define BUFFER_SIZE_SEC 4
 #define SAMPLING_FREQUENCY 44100
 
-#define STATUSINFO_STARTINTR0 U"Start (F1,F2,F0) = ("
-#define STATUSINFO_ENDINTR0 U"End (F1,F2,F0) = ("
-#define STATUSINFO_ENDING U")"
 #define MICROSECPRECISION(x) (round((x)*1000000)/1000000)
 
 // To prevent the generation of inaudible short Sounds we set a minimum duration
@@ -187,21 +184,19 @@ static autoVowel Vowel_create (double duration) {
 static autoVowel Vowel_create_twoFormantSchwa (double duration) {
 	try {
 		autoVowel me = Vowel_create (duration);
-		autoFormantPoint fp = FormantPoint_create (0.0);
-		fp -> formant [0] = 500.0;
-		fp -> bandwidth [0] = 50.0;
-		fp -> formant [1] = 1500.0;
-		fp -> bandwidth [1] = 150.0;
-		fp -> numberOfFormants = 2;
+		autoFormantPoint fp = FormantPoint_create (0.0, 2);
+		fp -> formant [1] = 500.0;
+		fp -> bandwidth [1] = 50.0;
+		fp -> formant [2] = 1500.0;
+		fp -> bandwidth [2] = 150.0;
 		my ft -> points. addItem_move (fp.move());
 		RealTier_addPoint (my pt.get(), 0.0, 140.0);
 
-		fp = FormantPoint_create (duration);
-		fp -> formant [0] = 500.0;
-		fp -> bandwidth [0] = 50.0;
-		fp -> formant [1] = 1500.0;
-		fp -> bandwidth [1] = 150.0;
-		fp -> numberOfFormants = 2;
+		fp = FormantPoint_create (duration, 2);
+		fp -> formant [1] = 500.0;
+		fp -> bandwidth [1] = 50.0;
+		fp -> formant [2] = 1500.0;
+		fp -> bandwidth [2] = 150.0;
 		my ft -> points. addItem_move (fp.move());
 		RealTier_addPoint (my pt.get(), duration, 140.0);
 		return me;
@@ -229,8 +224,8 @@ static autoFormantGrid FormantTier_to_FormantGrid (FormantTier me) {
 			FormantPoint fp = my points.at [ipoint];
 			double t = fp -> number;
 			for (integer iformant = 1; iformant <= fp -> numberOfFormants; iformant ++) {
-				FormantGrid_addFormantPoint (thee.get(), iformant, t, fp -> formant [iformant - 1]);
-				FormantGrid_addBandwidthPoint (thee.get(), iformant, t, fp -> bandwidth [iformant - 1]);
+				FormantGrid_addFormantPoint (thee.get(), iformant, t, fp -> formant [iformant]);
+				FormantGrid_addBandwidthPoint (thee.get(), iformant, t, fp -> bandwidth [iformant]);
 			}
 		}
 		return thee;
@@ -251,20 +246,19 @@ static void VowelEditor_getF1F2FromXY (VowelEditor me, double x, double y, doubl
 }
 
 #define REPRESENTNUMBER(x,i) (isundef (x) ? U" undef" : Melder_pad (6, Melder_fixed (x, 1)))
-static void appendF1F2F0 (MelderString *statusInfo, const char32 *intro, double f1, double f2, double f0, const char32 *ending) {
+static void appendF1F2F0 (MelderString *statusInfo, conststring32 intro, double f1, double f2, double f0, conststring32 ending) {
 	MelderString_append (statusInfo, intro, REPRESENTNUMBER (f1, 1), U", ", REPRESENTNUMBER (f2, 2), U", ", REPRESENTNUMBER (f0, 3), ending);
 }
 
 static double getRealFromTextWidget (GuiText me) {
 	double value = undefined;
-	char32 *dirty = GuiText_getString (me);
+	autostring32 dirty = GuiText_getString (me);
 	try {
-		Interpreter_numericExpression (nullptr, dirty, & value);
+		Interpreter_numericExpression (nullptr, dirty.get(), & value);
 	} catch (MelderError) {
 		Melder_clearError ();
 		value = undefined;
 	}
-	Melder_free (dirty);
 	return value;
 }
 
@@ -284,15 +278,12 @@ static void checkF1F2 (VowelEditor me, double *f1, double *f2) {
 }
 
 static void checkF0 (structVowelEditor_F0 *f0p, double *f0) {
-	if (isundef (*f0)) {
+	if (isundef (*f0))
 		*f0 = f0p -> start;
-	}
-	if (*f0 > f0p -> maximum) {
+	if (*f0 > f0p -> maximum)
 		*f0 = f0p -> maximum;
-	}
-	if (*f0 < f0p -> minimum) {
+	if (*f0 < f0p -> minimum)
 		*f0 = f0p -> minimum;
-	}
 }
 
 static void	checkXY (double *x, double *y) {
@@ -325,37 +316,32 @@ static void VowelEditor_updateF0Info (VowelEditor me) {
 	GuiText_setString (my f0TextField, Melder_double (f0));
 	my f0.start = f0;
 	double slopeOctPerSec = getRealFromTextWidget (my f0SlopeTextField);
-	if (isundef (slopeOctPerSec)) {
+	if (isundef (slopeOctPerSec))
 		slopeOctPerSec = f0default.slopeOctPerSec;
-	}
 	my f0.slopeOctPerSec = slopeOctPerSec;
 	GuiText_setString (my f0SlopeTextField, Melder_double (my f0.slopeOctPerSec));
 }
 
 static void VowelEditor_updateExtendDuration (VowelEditor me) {
 	double extend = getRealFromTextWidget (my extendTextField);
-	if (isundef (extend) || extend <= MINIMUM_SOUND_DURATION || extend > my maximumDuration) {
+	if (isundef (extend) || extend <= MINIMUM_SOUND_DURATION || extend > my maximumDuration)
 		extend = MINIMUM_SOUND_DURATION;
-	}
 	GuiText_setString (my extendTextField, Melder_double (extend));
 	my extendDuration = prefs.extendDuration = extend;
 }
 
 static double VowelEditor_updateDurationInfo (VowelEditor me) {
 	double duration = getRealFromTextWidget (my durationTextField);
-	if (isundef (duration) || duration < MINIMUM_SOUND_DURATION) {
+	if (isundef (duration) || duration < MINIMUM_SOUND_DURATION)
 		duration = MINIMUM_SOUND_DURATION;
-	}
 	GuiText_setString (my durationTextField, Melder_double (MICROSECPRECISION (duration)));
 	return duration;
 }
 
 static void Sound_fadeIn (Sound me, double duration, bool fromFirstNonZeroSample) {
 	integer istart = 1, numberOfSamples = Melder_ifloor (duration / my dx);   // ppgb: waarom afronden naar beneden?
-
-	if (numberOfSamples < 2) {
+	if (numberOfSamples < 2)
 		return;
-	}
 	if (fromFirstNonZeroSample) {
 		// If the first part of the sound is very low level we put sample values to zero and
 		// start windowing from the position where the amplitude is above the minimum level.
@@ -369,13 +355,11 @@ static void Sound_fadeIn (Sound me, double duration, bool fromFirstNonZeroSample
 			istart ++;
 		}
 	}
-	if (numberOfSamples > my nx - istart + 1) {
+	if (numberOfSamples > my nx - istart + 1)
 		numberOfSamples = my nx - istart + 1;
-	}
 
 	for (integer i = 1; i <= numberOfSamples; i ++) {
 		double phase = NUMpi * (i - 1) / (numberOfSamples - 1);
-
 		my z [1] [istart + i - 1] *= 0.5 * (1.0 - cos (phase));
 	}
 }
@@ -383,17 +367,14 @@ static void Sound_fadeIn (Sound me, double duration, bool fromFirstNonZeroSample
 static void Sound_fadeOut (Sound me, double duration) {
 	integer istart, numberOfSamples = Melder_ifloor (duration / my dx);
 
-	if (numberOfSamples < 2) {
+	if (numberOfSamples < 2)
 		return;
-	}
-	if (numberOfSamples > my nx) {
+	if (numberOfSamples > my nx)
 		numberOfSamples = my nx;
-	}
 	istart = my nx - numberOfSamples;
 	// only one channel
 	for (integer i = 1; i <= numberOfSamples; i ++) {
 		double phase = NUMpi * (i - 1) / (numberOfSamples - 1);
-
 		my z [1] [istart + i] *= 0.5 * (1.0 + cos (phase));
 	}
 }
@@ -432,40 +413,35 @@ static void VowelEditor_shiftF1F2 (VowelEditor me, double f1_st, double f2_st) {
 	FormantTier ft = my vowel -> ft.get();
 	for (integer i = 1; i <= ft -> points.size; i ++) {
 		FormantPoint fp = ft -> points.at [i];
-		double f1 = fp -> formant [0], f2 = fp -> formant [1];
+		double f1 = fp -> formant [1], f2 = fp -> formant [2];
 
 		f1 *= pow (2, f1_st / 12.0);
-		if (f1 < my f1min) {
+		if (f1 < my f1min)
 			f1 = my f1min;
-		}
-		if (f1 > my f1max) {
+		if (f1 > my f1max)
 			f1 = my f1max;
-		}
-		fp -> formant [0] = f1;
-		fp -> bandwidth [0] = f1 / 10.0;
+		fp -> formant [1] = f1;
+		fp -> bandwidth [1] = f1 / 10.0;
 
 		f2 *= pow (2, f2_st / 12.0);
-		if (f2 < my f2min) {
+		if (f2 < my f2min)
 			f2 = my f2min;
-		}
-		if (f2 > my f2max) {
+		if (f2 > my f2max)
 			f2 = my f2max;
-		}
-		fp -> formant [1] = f2;
-		fp -> bandwidth [1] = f2 / 10.0;
+		fp -> formant [2] = f2;
+		fp -> bandwidth [2] = f2 / 10.0;
 		double f3, b3, f4, b4;
 		VowelEditor_getF3F4 (me, f1, f2, & f3, & b3, & f4, & b4);
-		fp -> formant [2] = f3;
-		fp -> bandwidth [2] = b3;
-		fp -> formant [3] = f4;
-		fp -> bandwidth [3] = b4;
+		fp -> formant [3] = f3;
+		fp -> bandwidth [3] = b3;
+		fp -> formant [4] = f4;
+		fp -> bandwidth [4] = b4;
 	}
 }
 
 static void FormantTier_newDuration (FormantTier me, double newDuration) {
 	if (newDuration != my xmax) {
 		double multiplier = newDuration / my xmax;
-
 		for (integer i = 1; i <= my points.size; i ++) {
 			FormantPoint fp = my points.at [i];
 			fp -> number *= multiplier;
@@ -539,13 +515,13 @@ static void FormantTier_drawF1F2Trajectory (FormantTier me, Graphics g, double f
 	if ( (my xmax - my xmin) < 0.005) {
 		Graphics_setColour (g, Graphics_RED);
 	}
-	x1p = x1 = GETX (fp -> formant [1]);
-	y1p = y1 = GETY (fp -> formant [0]);
+	x1p = x1 = GETX (fp -> formant [2]);
+	y1p = y1 = GETY (fp -> formant [1]);
 	t1 = fp -> number;
 	for (it = 2; it <= nfp; it ++) {
 		fp = my points.at [it];
-		double x2 = GETX (fp -> formant [1]);
-		double y2 = GETY (fp -> formant [0]);
+		double x2 = GETX (fp -> formant [2]);
+		double y2 = GETY (fp -> formant [1]);
 		double t2 = fp -> number;
 		Graphics_setLineWidth (g, 3);
 		if (x1 == x2 && y1 == y2) {
@@ -582,8 +558,11 @@ static void FormantTier_drawF1F2Trajectory (FormantTier me, Graphics g, double f
 			}
 			imark ++;
 		}
-		x1p = x1; y1p = y1;
-		x1 = x2; y1 = y2; t1 = t2;
+		x1p = x1;
+		y1p = y1;
+		x1 = x2;
+		y1 = y2;
+		t1 = t2;
 	}
 	// Arrow at end
 	{
@@ -591,17 +570,16 @@ static void FormantTier_drawF1F2Trajectory (FormantTier me, Graphics g, double f
 		double size = 10.0 * arrowSize * Graphics_getResolution (g) / 75.0 / width, size2 = size * size;
 		Graphics_setArrowSize (g, arrowSize);
 		it = 1;
-		while (it <= (nfp - 1)) {
+		while (it <= nfp - 1) {
 			fp = my points.at [nfp - it];
-			double dx = GETX (fpn -> formant [1]) - GETX (fp -> formant [1]);
-			double dy = GETY (fpn -> formant [0]) - GETY (fp -> formant [0]);
+			double dx = GETX (fpn -> formant [2]) - GETX (fp -> formant [2]);
+			double dy = GETY (fpn -> formant [1]) - GETY (fp -> formant [1]);
 			double d2 = dx * dx + dy * dy;
-			if (d2 > size2) {
+			if (d2 > size2)
 				break;
-			}
 			it ++;
 		}
-		Graphics_arrow (g, GETX (fp -> formant [1]), GETY (fp -> formant [0]), GETX (fpn -> formant [1]), GETY (fpn -> formant [0]));
+		Graphics_arrow (g, GETX (fp -> formant [2]), GETY (fp -> formant [1]), GETX (fpn -> formant [2]), GETY (fpn -> formant [1]));
 		Graphics_setArrowSize (g, gas);
 	}
 	Graphics_unsetInner (g);
@@ -649,8 +627,8 @@ static void Table_addColumn_size (Table me, int size) {
 
 static void VowelEditor_setMarks (VowelEditor me, int marksDataset, int speakerType, int fontSize) {
 	autoTable te;
-	const char32 *Type [4] = { U"", U"m", U"w", U"c" };
-	const char32 *Sex [3] = { U"", U"m", U"f"};
+	conststring32 Type [4] = { U"", U"m", U"w", U"c" };
+	conststring32 Sex [3] = { U"", U"m", U"f"};
 	if (marksDataset == 1) {   // American-English
 		autoTable thee = Table_create_petersonBarney1952 ();
 		te = Table_extractRowsWhereColumn_string (thee.get(), 1, kMelder_string::EQUAL_TO, Type [speakerType]);
@@ -683,15 +661,13 @@ static void VowelEditor_createTableFromVowelMarksInPreferences (VowelEditor me)
 		autoTable newMarks = Table_createWithColumnNames (0, U"Vowel F1 F2 Size");
 		integer nmarksFound = 0;
 		for (integer i = 1; i <= numberOfRows; i ++) {
-			autoMelderTokens rowi (prefs.mark [i - 1]);
-			integer numberOfTokens = rowi.count();
-			if (numberOfTokens < 4) { // we are done
+			autostring32vector rowi = newSTRVECtokenize (prefs.mark [i - 1]);
+			integer numberOfTokens = rowi.size;
+			if (numberOfTokens < 4)
 				break;
-			}
 			Table_appendRow (newMarks.get());
-			for (integer j = 1; j <= 4; j ++) {
-				Table_setStringValue (newMarks.get(), i, j, rowi [j]);
-			}
+			for (integer j = 1; j <= 4; j ++)
+				Table_setStringValue (newMarks.get(), i, j, rowi [j].get());
 			nmarksFound ++;
 		}
 		if (nmarksFound == 0) {
@@ -735,18 +711,14 @@ static void VowelEditor_setF3F4 (VowelEditor me, double f3, double b3, double f4
 	double xmin = my f2min, xmax = my f2max, dx = my f2max - my f2min, x1 = dx / 2.0;
 	double dy = my f1max - my f1min, y1 = dy / 2;
 
-	if (! my f3) {
+	if (! my f3)
 		my f3 = Matrix_create (xmin, xmax, 1, dx, x1, my f1min, my f1max, 1, dy, y1);
-	}
-	if (! my b3) {
+	if (! my b3)
 		my b3 = Matrix_create (xmin, xmax, 1, dx, x1, my f1min, my f1max, 1, dy, y1);
-	}
-	if (! my f4) {
+	if (! my f4)
 		my f4 = Matrix_create (xmin, xmax, 1, dx, x1, my f1min, my f1max, 1, dy, y1);
-	}
-	if (! my b4) {
+	if (! my b4)
 		my b4 = Matrix_create (xmin, xmax, 1, dx, x1, my f1min, my f1max, 1, dy, y1);
-	}
 	my f3 -> z [1] [1] = f3;
 	my b3 -> z [1] [1] = b3;
 	my f4 -> z [1] [1] = f4;
@@ -772,15 +744,14 @@ static void VowelEditor_drawBackground (VowelEditor me, Graphics g) {
 		integer col_f2 = Table_getColumnIndexFromColumnLabel (my marks.get(), U"F2");
 		integer col_fs = Table_findColumnIndexFromColumnLabel (my marks.get(), U"Size");
 		for (integer i = 1; i <= my marks -> rows.size; i ++) {
-			const char32 *label = Table_getStringValue_Assert (my marks.get(), i, col_vowel);
+			conststring32 label = Table_getStringValue_Assert (my marks.get(), i, col_vowel);
 			f1 = Table_getNumericValue_Assert (my marks.get(), i, col_f1);
 			f2 = Table_getNumericValue_Assert (my marks.get(), i, col_f2);
 			if (f1 >= my f1min && f1 <= my f1max && f2 >= my f2min && f2 <= my f2max) {
 				VowelEditor_getXYFromF1F2 (me, f1, f2, & x1, & y1);
 				int size = prefs.marksFontSize;
-				if (col_fs != 0) {
+				if (col_fs != 0)
 					size = Melder_ifloor (Table_getNumericValue_Assert (my marks.get(), i, col_fs));
-				}
 				Graphics_setFontSize (g, size);
 				Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
 				Graphics_text (g, x1, y1, label);
@@ -1007,8 +978,8 @@ static void menu_cb_showVowelMarksFromTableFile (VowelEditor me, EDITOR_ARGS_FOR
 }
 
 static void menu_cb_setF0 (VowelEditor me, EDITOR_ARGS_FORM) {
-	EDITOR_FORM (U"Set F0", nullptr);
-		POSITIVE (startF0, U"Start F0 (Hz)", U"150.0")
+	EDITOR_FORM (U"Set f0", nullptr);
+		POSITIVE (startF0, U"Start f0 (Hz)", U"150.0")
 		REAL (slope, U"Slope (oct/s)", U"0.0")
 	EDITOR_OK
 	EDITOR_DO
@@ -1038,18 +1009,17 @@ static void menu_cb_reverseTrajectory (VowelEditor me, EDITOR_ARGS_DIRECT) {
 }
 
 static void VowelEditor_Vowel_addData (VowelEditor me, Vowel thee, double time, double f1, double f2, double f0) {
-	autoFormantPoint fp = FormantPoint_create (time);
+	autoFormantPoint fp = FormantPoint_create (time, 4);
 	double f3, b3, f4, b4;
-	fp -> formant [0] = f1;
-	fp -> bandwidth [0] = f1 / 10.0;
-	fp -> formant [1] = f2;
-	fp -> bandwidth [1] = f2 / 10.0;
+	fp -> formant [1] = f1;
+	fp -> bandwidth [1] = f1 / 10.0;
+	fp -> formant [2] = f2;
+	fp -> bandwidth [2] = f2 / 10.0;
 	VowelEditor_getF3F4 (me, f1, f2, & f3, & b3, & f4, & b4);
-	fp -> formant [2] = f3;
-	fp -> bandwidth [2] = b3;
-	fp -> formant [3] = f4;
-	fp -> bandwidth [3] = b4;
-	fp -> numberOfFormants = 4;
+	fp -> formant [3] = f3;
+	fp -> bandwidth [3] = b3;
+	fp -> formant [4] = f4;
+	fp -> bandwidth [4] = b4;
 
 	thy ft -> points. addItem_move (fp.move());
 	RealTier_addPoint (thy pt.get(), time, f0);
@@ -1162,18 +1132,17 @@ static void gui_drawingarea_cb_expose (VowelEditor me, GuiDrawingArea_ExposeEven
 	FormantTier ft = my vowel -> ft.get();
 	Melder_assert (ft);
 	static MelderString statusInfo { };
-	if (! my graphics) {
+	if (! my graphics)
 		return;   // could be the case in the very beginning
-	}
 	Graphics_clearWs (my graphics.get());
 
-	appendF1F2F0 (& statusInfo, STATUSINFO_STARTINTR0, FormantTier_getValueAtTime (ft, 1, ts),
-	              FormantTier_getValueAtTime (ft, 2, ts), getF0 (& my f0, ts), STATUSINFO_ENDING);
+	appendF1F2F0 (& statusInfo, U"Start (F1,F2,f0) = (", FormantTier_getValueAtTime (ft, 1, ts),
+		FormantTier_getValueAtTime (ft, 2, ts), getF0 (& my f0, ts), U")");
 	GuiLabel_setText (my startInfo, statusInfo.string);
 	MelderString_empty (& statusInfo);
 
-	appendF1F2F0 (& statusInfo, STATUSINFO_ENDINTR0, FormantTier_getValueAtTime (ft, 1, te),
-	              FormantTier_getValueAtTime (ft, 2, te), getF0 (& my f0, te), STATUSINFO_ENDING);
+	appendF1F2F0 (& statusInfo, U"End (F1,F2,f0) = (", FormantTier_getValueAtTime (ft, 1, te),
+		FormantTier_getValueAtTime (ft, 2, te), getF0 (& my f0, te), U")");
 	GuiLabel_setText (my endInfo, statusInfo.string);
 	MelderString_empty (& statusInfo);
 
@@ -1195,9 +1164,8 @@ static void gui_drawingarea_cb_expose (VowelEditor me, GuiDrawingArea_ExposeEven
 
 static void gui_drawingarea_cb_resize (VowelEditor me, GuiDrawingArea_ResizeEvent event) {
 	Melder_assert (me);
-	if (! my graphics) {
+	if (! my graphics)
 		return;   // could be the case in the very beginning
-	}
 	Graphics_setWsViewport (my graphics.get(), 0.0, event -> width, 0.0, event -> height);
 	my width = event -> width;
 	my height = event -> height;
@@ -1218,21 +1186,20 @@ static void VowelEditor_Vowel_updateTiers (VowelEditor me, Vowel thee, double ti
 		thy pt -> xmax = time;
 	}
 	double f0 = getF0 (& my f0, time), f1, f2;
-	autoFormantPoint point = FormantPoint_create (time);
+	autoFormantPoint point = FormantPoint_create (time, 4);
 
 	VowelEditor_getF1F2FromXY (me, x, y, & f1, & f2);
 	double f3, b3, f4, b4;
 	VowelEditor_getF3F4 (me, f1, f2, & f3, & b3, & f4, & b4);
 
-	point -> formant [0] = f1;
-	point -> bandwidth [0] = f1 / 10.0;
-	point -> formant [1] = f2;
-	point -> bandwidth [1] = f2 / 10.0;
-	point -> formant [2] = f3;
-	point -> bandwidth [2] = b3;
-	point -> formant [3] = f4;
-	point -> bandwidth [3] = b4;
-	point -> numberOfFormants = 4;
+	point -> formant [1] = f1;
+	point -> bandwidth [1] = f1 / 10.0;
+	point -> formant [2] = f2;
+	point -> bandwidth [2] = f2 / 10.0;
+	point -> formant [3] = f3;
+	point -> bandwidth [3] = b3;
+	point -> formant [4] = f4;
+	point -> bandwidth [4] = b4;
 	thy ft -> points. addItem_move (point.move());
 	RealTier_addPoint (thy pt.get(), time, f0);
 }
@@ -1309,9 +1276,8 @@ static void gui_drawingarea_cb_click (VowelEditor me, GuiDrawingArea_ClickEvent
 end:
 	Graphics_unsetInner (my graphics.get());
 
-	if (! my shiftKeyPressed) {
+	if (! my shiftKeyPressed)
 		my vowel = athee.move();
-	}
 	Melder_assert (! athee);
 	gui_button_cb_play (me, & gb_event);
 }
@@ -1344,7 +1310,7 @@ void structVowelEditor :: v_createMenus () {
 	Editor_addCommand (this, U"File", U"Draw trajectory...", 0, menu_cb_drawTrajectory);
 	Editor_addCommand (this, U"File", U"-- scripting --", 0, nullptr);
 	Editor_addCommand (this, U"Edit", U"-- f0 --", 0, nullptr);
-	Editor_addCommand (this, U"Edit", U"Set F0...", 0, menu_cb_setF0);
+	Editor_addCommand (this, U"Edit", U"Set f0...", 0, menu_cb_setF0);
 	Editor_addCommand (this, U"Edit", U"Set F3 & F4...", 0, menu_cb_setF3F4);
 	Editor_addCommand (this, U"Edit", U"-- trajectory commands --", 0, nullptr);
 	Editor_addCommand (this, U"Edit", U"Reverse trajectory", 0, menu_cb_reverseTrajectory);
@@ -1369,7 +1335,7 @@ void structVowelEditor :: v_createHelpMenuItems (EditorMenu menu) {
 
 void structVowelEditor :: v_createChildren ()
 {
-	const int button_width = 90, text_width = 95, status_info_width = 290;
+	const int button_width = 90, text_width = 95, status_info_width = 330;
 	int top, bottom, bottom_widgets_top, bottom_widgets_bottom, bottom_widgets_halfway;
 
 	// Three buttons on a row: Play, Reverse, Publish
@@ -1380,8 +1346,8 @@ void structVowelEditor :: v_createChildren ()
 	reverseButton = GuiButton_createShown (our windowForm, left, right, top, bottom, U"Reverse", gui_button_cb_reverse, this, 0);
 	left = right + 10; right = left + button_width;
 	publishButton = GuiButton_createShown (our windowForm, left, right, top, bottom, U"Publish", gui_button_cb_publish, this, 0);
-	// Four Text widgets with the label on top: Duration, Extend, F0, Slope
-	// Make the F0 slope button 10 wider to accomodate the text
+	// Four Text widgets with the label on top: Duration, Extend, f0, Slope
+	// Make the f0 slope button 10 wider to accomodate the text
 	// We wil not use a callback from a Text widget. It will get called multiple times during the editing
 	// of the text. Better to have all editing done and then query the widget for its value!
 	left = right + 10; right = left + text_width; bottom_widgets_halfway = bottom = (top + bottom) / 2; top = bottom_widgets_top;
@@ -1395,21 +1361,25 @@ void structVowelEditor :: v_createChildren ()
 	extendTextField = GuiText_createShown (our windowForm, left, right, top, bottom, 0);
 
 	left = right + 10; right = left + text_width; top = bottom_widgets_top; bottom = bottom_widgets_halfway;
-	GuiLabel_createShown (our windowForm, left, right, top, bottom, U"Start F0 (Hz):", 0);
+	GuiLabel_createShown (our windowForm, left, right, top, bottom, U"Start f0 (Hz):", 0);
 	top = bottom; bottom = bottom_widgets_bottom;
 	f0TextField = GuiText_createShown (our windowForm, left, right, top, bottom, 0);
 
 	left = right + 10; right = left + text_width + 10; top = bottom_widgets_top; bottom = bottom_widgets_halfway;
-	GuiLabel_createShown (our windowForm, left, right, top, bottom, U"F0 slope (oct/s):", 0);
+	GuiLabel_createShown (our windowForm, left, right, top, bottom, U"Slope f0 (oct/s):", 0);
 	top = bottom; bottom = bottom_widgets_bottom;
 	f0SlopeTextField = GuiText_createShown (our windowForm, left, right, top, bottom, 0);
 
 	// The status startInfo and endInfo widget at the bottom:
 
-	bottom = - (STATUS_INFO - Gui_LABEL_HEIGHT) / 2; top = bottom - Gui_LABEL_HEIGHT; left = MARGIN_LEFT; right = left + status_info_width;
+	bottom = - (STATUS_INFO - Gui_LABEL_HEIGHT) / 2;
+	top = bottom - Gui_LABEL_HEIGHT;
+	left = MARGIN_LEFT;
+	right = left + status_info_width;
 	startInfo = GuiLabel_createShown (our windowForm, left, right, top, bottom, U"", 0);
 
-	left = right; right = left + status_info_width;
+	left = right;
+	right = left + status_info_width;
 	endInfo = GuiLabel_createShown (our windowForm, left, right, top, bottom, U"", 0);
 
 	/***** Create drawing area. *****/
@@ -1423,7 +1393,7 @@ void structVowelEditor :: v_createChildren ()
 	height = GuiControl_getHeight (drawingArea);
 }
 
-autoVowelEditor VowelEditor_create (const char32 *title, Daata data) {
+autoVowelEditor VowelEditor_create (conststring32 title, Daata data) {
 	try {
 		trace (U"enter");
 		autoVowelEditor me = Thing_new (VowelEditor);
