@@ -1,6 +1,6 @@
 /* praat_MDS_init.cpp
  *
- * Copyright (C) 1992-2012, 2015-2016 David Weenink
+ * Copyright (C) 1992-2018 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,57 +40,22 @@
 #include "TableOfReal_extensions.h"
 #include "Configuration_and_Procrustes.h"
 #include "Configuration_AffineTransform.h"
+#include "Proximity_and_Distance.h"
 #include "Confusion.h"
 #include "Formula.h"
 
 void praat_TableOfReal_init2 (ClassInfo klas);
 
-static const char32 *QUERY_BUTTON   = U"Query -";
-static const char32 *DRAW_BUTTON    = U"Draw -";
-static const char32 *ANALYSE_BUTTON = U"Analyse -";
-static const char32 *CONFIGURATION_BUTTON = U"To Configuration -";
+static const conststring32 QUERY_BUTTON   = U"Query -";
+static const conststring32 DRAW_BUTTON    = U"Draw -";
+static const conststring32 ANALYSE_BUTTON = U"Analyse -";
+static const conststring32 CONFIGURATION_BUTTON = U"To Configuration -";
 
 /* Tests */
 
-/*
-	Sort row 1 ascending and store in row 3
-	Sort row 1 and move row 2 along and store in rows 4 and 5 respectively
-	Make an index for row 1 and store in row 6
-*/
-static void TabelOfReal_testSorting (TableOfReal me) {
-	try {
-		integer rowtoindex = 1;
-		
-		Melder_require (my numberOfRows >= 6, U"TabelOfReal_sort2: we want at least 6 rows!!");
-		
-		autoNUMvector <integer> index (1, my numberOfColumns);
-		// Copy 1->3 and sort 3 inplace
-		NUMvector_copyElements (my data [1], my data [3], 1, my numberOfColumns);
-		NUMsort_d (my numberOfColumns, my data [3]);
-
-		// Copy 1->4 and 2->5, sort 4+5 in parallel
-		NUMvector_copyElements (my data [1], my data [4], 1, my numberOfColumns);
-		NUMvector_copyElements (my data [2], my data [5], 1, my numberOfColumns);
-		NUMsort2 (my numberOfColumns, my data [4], my data [5]);
-
-		NUMindexx (my data [rowtoindex], my numberOfColumns, index.peek());
-		for (integer i = 1; i <= my numberOfColumns; i ++) {
-			my data [6] [i] = index [i];
-		}
-	} catch (MelderError) {
-		Melder_throw (me, U": sorting test not ok.");
-	}
-}
-
 #undef iam
 #define iam iam_LOOP
 
-DIRECT(MODIFY_TabelOfReal_testSorting) {
-	MODIFY_EACH (TableOfReal)
-		TabelOfReal_testSorting (me);
-	MODIFY_EACH_END
-}
-
 /************************* examples ***************************************/
 
 FORM (NEW1_Dissimilarity_createLetterRExample, U"Create letter R example", U"Create letter R example...") {
@@ -161,23 +126,19 @@ DIRECT (HELP_AffineTransform_help) {
 DIRECT (NEW_AffineTransform_invert) {
 	CONVERT_EACH (AffineTransform)
 		autoAffineTransform result = AffineTransform_invert (me);
-	CONVERT_EACH_END (my name, U"_inv")
+	CONVERT_EACH_END (my name.get(), U"_inv")
 }
 
 FORM (REAL_AffineTransform_getTransformationElement, U"AffineTransform: Get transformation element", U"Procrustes") {
-	NATURAL (row, U"Row number", U"1")
-	NATURAL (column, U"Column number", U"1")
+	NATURAL (irow, U"Row number", U"1")
+	NATURAL (icol, U"Column number", U"1")
 	OK
 DO
 	NUMBER_ONE (AffineTransform)
-		if (row > my n) {
-			Melder_throw (U"Row number must not exceed number of rows.");
-		}
-		if (column > my n) {
-			Melder_throw (U"Column number must not exceed number of columns.");
-		}
-		double result = my r [row] [column];
-	NUMBER_ONE_END (U"")
+		Melder_require (irow <= my dimension, U"Row number should not exceed the dimension of the transform.");
+		Melder_require (icol <= my dimension, U"Column number should not exceed the dimension of the transform.");
+		double result = my r [irow] [icol];
+	NUMBER_ONE_END (U" r [", irow, U"] [", icol, U"]")
 }
 
 FORM (REAL_AffineTransform_getTranslationElement, U"AffineTransform: Get translation element", U"Procrustes") {
@@ -185,9 +146,7 @@ FORM (REAL_AffineTransform_getTranslationElement, U"AffineTransform: Get transla
 	OK
 DO
 	NUMBER_ONE (AffineTransform)
-		if (index > my n) {
-			Melder_throw (U"Index must not exceed number of elements.");
-		}
+		Melder_require (index <= my dimension, U"Index should not exceed the dimension of the transform.");
 		double result = my t [index];
 	NUMBER_ONE_END (U"")
 }
@@ -195,13 +154,13 @@ DO
 DIRECT (NEW_AffineTransform_extractMatrix) {
 	CONVERT_EACH (AffineTransform)
 		autoTableOfReal result = AffineTransform_extractMatrix (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_AffineTransform_extractTranslationVector) {
 	CONVERT_EACH (AffineTransform)
 		autoTableOfReal result = AffineTransform_extractTranslationVector (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 /***************** Configuration ***************************************/
@@ -334,7 +293,7 @@ DO
 DIRECT (NEW_Configuration_to_Distance) {
 	CONVERT_EACH (Configuration)
 		autoDistance result = Configuration_to_Distance (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 FORM (NEW_Configuration_varimax, U"Configuration: To Configuration (varimax)", U"Configuration: To Configuration (varimax)...") {
@@ -346,7 +305,7 @@ FORM (NEW_Configuration_varimax, U"Configuration: To Configuration (varimax)", U
 DO
 	CONVERT_EACH (Configuration)
 		autoConfiguration result = Configuration_varimax (me, normalizeRows, useQuartimax, maximumNumberOfIterations, tolerance);
-	CONVERT_EACH_END (my name, U"_varimax")
+	CONVERT_EACH_END (my name.get(), U"_varimax")
 }
 
 DIRECT (NEW1_Configurations_to_Similarity_cc) {
@@ -361,7 +320,7 @@ FORM (NEW1_Configurations_to_Procrustes, U"Configuration & Configuration: To Pro
 DO
 	CONVERT_COUPLE (Configuration)
 		autoProcrustes result = Configurations_to_Procrustes (me, you, useOrthogonalTransform);
-	CONVERT_COUPLE_END (your name, U"_to_", my name)
+	CONVERT_COUPLE_END (your name.get(), U"_to_", my name.get())
 }
 
 FORM (NEW1_Configurations_to_AffineTransform_congruence, U"Configurations: To AffineTransform (congruence)", U"Configurations: To AffineTransform (congruence)...") {
@@ -371,7 +330,7 @@ FORM (NEW1_Configurations_to_AffineTransform_congruence, U"Configurations: To Af
 DO
 	CONVERT_COUPLE (Configuration)
 		autoAffineTransform result = Configurations_to_AffineTransform_congruence (me, you, maximumNumberOfIterations, tolerance);
-	CONVERT_COUPLE_END (your name, U"_to_", my name)
+	CONVERT_COUPLE_END (your name.get(), U"_to_", my name.get())
 }
 
 DIRECT (NEW1_Configuration_Weight_to_Similarity_cc) {
@@ -383,13 +342,13 @@ DIRECT (NEW1_Configuration_Weight_to_Similarity_cc) {
 DIRECT (NEW1_Configuration_AffineTransform_to_Configuration) {
 	CONVERT_TWO (Configuration, AffineTransform)
 		autoConfiguration result = Configuration_AffineTransform_to_Configuration (me, you);
-	CONVERT_TWO_END (my name, U"_", your name)
+	CONVERT_TWO_END (my name.get(), U"_", your name.get())
 }
 
 DIRECT (NEW1_Configuration_Procrustes_to_Configuration) {
 	CONVERT_TWO (Configuration, Procrustes)
 		autoConfiguration result = Configuration_AffineTransform_to_Configuration (me, you);
-	CONVERT_TWO_END (my name, U"_", your name)
+	CONVERT_TWO_END (my name.get(), U"_", your name.get())
 }
 
 /*************** Confusion *********************************/
@@ -400,7 +359,7 @@ FORM (NEW_Confusion_to_Dissimilarity_pdf, U"Confusion: To Dissimilarity (pdf)",
 DO
 	CONVERT_EACH (Confusion)
 		autoDissimilarity result = Confusion_to_Dissimilarity_pdf (me, minimumConfusionLevel);
-	CONVERT_EACH_END (my name, U"_pdf")
+	CONVERT_EACH_END (my name.get(), U"_pdf")
 }
 
 FORM (NEW_Confusion_to_Similarity, U"Confusion: To Similarity", U"Confusion: To Similarity...") {
@@ -413,7 +372,7 @@ FORM (NEW_Confusion_to_Similarity, U"Confusion: To Similarity", U"Confusion: To
 DO
 	CONVERT_EACH (Confusion)
 		autoSimilarity result = Confusion_to_Similarity (me, normalize, symmetrizeMethod);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW1_Confusions_sum) {
@@ -425,7 +384,7 @@ DIRECT (NEW1_Confusions_sum) {
 DIRECT (NEW_Confusion_to_ContingencyTable) {
 	CONVERT_EACH (Confusion)
 		autoContingencyTable result = Confusion_to_ContingencyTable (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 /*************** ContingencyTable *********************************/
@@ -441,7 +400,7 @@ FORM (NEW_ContingencyTable_to_Configuration_ca, U"ContingencyTable: To Configura
 DO
 	CONVERT_EACH (ContingencyTable)
 		autoConfiguration result = ContingencyTable_to_Configuration_ca (me, numberOfDimensions, scalingType);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (REAL_ContingencyTable_chisqProbability) {
@@ -470,7 +429,7 @@ FORM (NEW_Correlation_to_Configuration, U"Correlation: To Configuration", nullpt
 DO
 	CONVERT_EACH (Correlation)
 		autoConfiguration result = Correlation_to_Configuration (me, numberOfDimensions);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 
@@ -486,7 +445,7 @@ FORM (NEW_Similarity_to_Dissimilarity, U"Similarity: To Dissimilarity", U"Simila
 DO
 	CONVERT_EACH (Similarity)
 		autoDissimilarity result = Similarity_to_Dissimilarity (me, maximumDissimilarity);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 /**************** Dissimilarity ***************************************/
@@ -538,7 +497,7 @@ FORM (NEW1_Dissimilarity_Configuration_kruskal, U"Dissimilarity & Configuration:
 DO
 	CONVERT_TWO (Dissimilarity, Configuration)
 		autoConfiguration result = Dissimilarity_Configuration_kruskal (me, you, tiesHandling, stressFormula, tolerance, maximumNumberOfIterations, numberOfRepetitions);
-	CONVERT_TWO_END (my name, U"_kruskal")
+	CONVERT_TWO_END (my name.get(), U"_kruskal")
 }
 
 FORM (NEW1_Dissimilarity_Configuration_absolute_mds, U"Dissimilarity & Configuration: To Configuration (absolute mds)", U"Dissimilarity & Configuration: To Configuration (absolute mds)...") {
@@ -548,7 +507,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Configuration)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_absolute_mds (me, you, nullptr, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_absolute");
+	CONVERT_TWO_END (my name.get(), U"_absolute");
 }
 
 FORM (NEW1_Dissimilarity_Configuration_ratio_mds, U"Dissimilarity & Configuration: To Configuration (ratio mds)", U"Dissimilarity & Configuration: To Configuration (ratio mds)...") {
@@ -558,7 +517,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Configuration)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_ratio_mds (me, you, nullptr, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_ratio");
+	CONVERT_TWO_END (my name.get(), U"_ratio");
 }
 
 FORM (NEW1_Dissimilarity_Configuration_interval_mds, U"Dissimilarity & Configuration: To Configuration (interval mds)", U"Dissimilarity & Configuration: To Configuration (interval mds)...") {
@@ -568,7 +527,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Configuration)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_interval_mds (me, you, nullptr, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_interval");
+	CONVERT_TWO_END (my name.get(), U"_interval");
 }
 
 FORM (NEW1_Dissimilarity_Configuration_monotone_mds, U"Dissimilarity & Configuration: To Configuration (monotone mds)", U"Dissimilarity & Configuration: To Configuration (monotone mds)...") {
@@ -581,7 +540,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Configuration)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_monotone_mds (me, you, nullptr, tiesHandling, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_monotone");
+	CONVERT_TWO_END (my name.get(), U"_monotone");
 }
 
 FORM (NEW1_Dissimilarity_Configuration_ispline_mds, U"Dissimilarity & Configuration: To Configuration (i-spline mds)", U"Dissimilarity & Configuration: To Configuration (i-spline mds)...") {
@@ -594,7 +553,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Configuration)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_ispline_mds (me, you, nullptr, numberOfInteriorKnots, order, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_ispline");
+	CONVERT_TWO_END (my name.get(), U"_ispline");
 }
 
 FORM (NEW1_Dissimilarity_Configuration_Weight_absolute_mds, U"Dissimilarity & Configuration & Weight: To Configuration (absolute mds)", U"Dissimilarity & Configuration & Weight: To Configuration...") {
@@ -604,7 +563,7 @@ DO
 	CONVERT_THREE (Dissimilarity, Configuration, Weight)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_absolute_mds (me, you, him, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_THREE_END (my name, U"_w_absolute")
+	CONVERT_THREE_END (my name.get(), U"_w_absolute")
 }
 
 FORM (NEW1_Dissimilarity_Configuration_Weight_ratio_mds, U"Dissimilarity & Configuration & Weight: To Configuration (ratio mds)", U"Dissimilarity & Configuration & Weight: To Configuration...") {
@@ -614,7 +573,7 @@ DO
 	CONVERT_THREE (Dissimilarity, Configuration, Weight)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_ratio_mds (me, you, him, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_THREE_END (my name, U"_w_ratio");
+	CONVERT_THREE_END (my name.get(), U"_w_ratio");
 }
 
 FORM (NEW1_Dissimilarity_Configuration_Weight_interval_mds, U"Dissimilarity & Configuration & Weight: To Configuration (interval mds)", U"Dissimilarity & Configuration & Weight: To Configuration...") {
@@ -624,7 +583,7 @@ DO
 	CONVERT_THREE (Dissimilarity, Configuration, Weight)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_interval_mds (me, you, him, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_THREE_END (my name, U"_w_interval")
+	CONVERT_THREE_END (my name.get(), U"_w_interval")
 }
 
 FORM (NEW1_Dissimilarity_Configuration_Weight_monotone_mds, U"Dissimilarity & Configuration & Weight: To Configuration (monotone mds)", U"Dissimilarity & Configuration & Weight: To Configuration...") {
@@ -637,7 +596,7 @@ DO
 	CONVERT_THREE (Dissimilarity, Configuration, Weight)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_monotone_mds (me, you, him, tiesHandling, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_THREE_END (my name, U"_sw_monotone")
+	CONVERT_THREE_END (my name.get(), U"_sw_monotone")
 }
 
 FORM (NEW1_Dissimilarity_Configuration_Weight_ispline_mds,  U"Dissimilarity & Configuration & Weight: To Configuration (i-spline mds)", U"Dissimilarity & Configuration & Weight: To Configuration...") {
@@ -650,7 +609,7 @@ DO
 	CONVERT_THREE (Dissimilarity, Configuration, Weight)
 		bool showProgress = true;
 		autoConfiguration result = Dissimilarity_Configuration_Weight_ispline_mds (me, you, him, numberOfInteriorKnots, order, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_THREE_END (my name, U"_sw_ispline");
+	CONVERT_THREE_END (my name.get(), U"_sw_ispline");
 }
 
 FORM (REAL_Dissimilarity_Configuration_getStress, U"Dissimilarity & Configuration: Get stress",  U"Dissimilarity & Configuration: get stress") {
@@ -664,7 +623,7 @@ FORM (REAL_Dissimilarity_Configuration_getStress, U"Dissimilarity & Configuratio
 DO
 	NUMBER_TWO (Dissimilarity, Configuration)
 		double result = Dissimilarity_Configuration_getStress (me, you, tiesHandling, stressFormula);
-	NUMBER_TWO_END (U"(stress)")
+	NUMBER_TWO_END (U" (stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_absolute_stress, U"Dissimilarity & Configuration: Get stress (absolute mds)", U"Dissimilarity & Configuration: Get stress (absolute mds)...") {
@@ -673,7 +632,7 @@ FORM (REAL_Dissimilarity_Configuration_absolute_stress, U"Dissimilarity & Config
 DO
 	NUMBER_TWO (Dissimilarity, Configuration)
 		double result = Dissimilarity_Configuration_Weight_absolute_stress (me, you, nullptr,stressMeasure);
-	NUMBER_TWO_END (U"(absolute mds stress)")
+	NUMBER_TWO_END (U" (absolute mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_ratio_stress, U"Dissimilarity & Configuration: Get stress (ratio mds)", U"Dissimilarity & Configuration: Get stress (ratio mds)...") {
@@ -682,7 +641,7 @@ FORM (REAL_Dissimilarity_Configuration_ratio_stress, U"Dissimilarity & Configura
 DO
 	NUMBER_TWO (Dissimilarity, Configuration)
 		double result = Dissimilarity_Configuration_Weight_ratio_stress (me, you, nullptr, stressMeasure);
-	NUMBER_TWO_END (U"(ratio mds stress)")
+	NUMBER_TWO_END (U" (ratio mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_interval_stress, U"Dissimilarity & Configuration: Get stress (interval mds)", U"Dissimilarity & Configuration: Get stress (interval mds)...") {
@@ -691,7 +650,7 @@ FORM (REAL_Dissimilarity_Configuration_interval_stress, U"Dissimilarity & Config
 DO
 	NUMBER_TWO (Dissimilarity, Configuration)
 		double result = Dissimilarity_Configuration_Weight_interval_stress (me, you, nullptr, stressMeasure);
-	NUMBER_TWO_END (U"(interval mds stress)")
+	NUMBER_TWO_END (U" (interval mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_monotone_stress, U"Dissimilarity & Configuration: Get stress (monotone mds)", U"Dissimilarity & Configuration: Get stress (monotone mds)...") {
@@ -703,7 +662,7 @@ FORM (REAL_Dissimilarity_Configuration_monotone_stress, U"Dissimilarity & Config
 DO
 	NUMBER_TWO (Dissimilarity, Configuration)
 		double result = Dissimilarity_Configuration_Weight_monotone_stress (me, you, nullptr, tiesHandling,stressMeasure);
-	NUMBER_TWO_END (U"(monotone mds stress)")
+	NUMBER_TWO_END (U" (monotone mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_ispline_stress, U"Dissimilarity & Configuration: Get stress (i-spline mds)", U"Dissimilarity & Configuration: Get stress (i-spline mds)...") {
@@ -714,7 +673,7 @@ FORM (REAL_Dissimilarity_Configuration_ispline_stress, U"Dissimilarity & Configu
 DO
 	NUMBER_TWO (Dissimilarity, Configuration)
 		double result = Dissimilarity_Configuration_Weight_ispline_stress (me, you, nullptr, numberOfInteriorKnots, order, stressMeasure);
-	NUMBER_TWO_END (U"(i-spline mds stress)")
+	NUMBER_TWO_END (U" (i-spline mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_Weight_absolute_stress, U"Dissimilarity & Configuration & Weight: Get stress (absolute mds)", U"Dissimilarity & Configuration & Weight: Get stress (absolute mds)...") {
@@ -723,7 +682,7 @@ FORM (REAL_Dissimilarity_Configuration_Weight_absolute_stress, U"Dissimilarity &
 DO
 	NUMBER_THREE (Dissimilarity, Configuration, Weight)
 		double result = Dissimilarity_Configuration_Weight_absolute_stress (me, you, him, stressMeasure);
-	NUMBER_THREE_END (U"(absolute mds stress)")
+	NUMBER_THREE_END (U" (absolute mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_Weight_ratio_stress, U"Dissimilarity & Configuration & Weight: Get stress (ratio mds)", U"Dissimilarity & Configuration & Weight: Get stress (ratio mds)...") {
@@ -732,7 +691,7 @@ FORM (REAL_Dissimilarity_Configuration_Weight_ratio_stress, U"Dissimilarity & Co
 DO
 	NUMBER_THREE (Dissimilarity, Configuration, Weight)
 		double result = Dissimilarity_Configuration_Weight_ratio_stress (me, you, him, stressMeasure);
-	NUMBER_THREE_END (U"(ratio mds stress)")
+	NUMBER_THREE_END (U" (ratio mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_Weight_interval_stress, U"Dissimilarity & Configuration & Weight: Get stress (interval mds)", U"Dissimilarity & Configuration & Weight: Get stress (interval mds)...") {
@@ -741,7 +700,7 @@ FORM (REAL_Dissimilarity_Configuration_Weight_interval_stress, U"Dissimilarity &
 DO
 	NUMBER_THREE (Dissimilarity, Configuration, Weight)
 		double result = Dissimilarity_Configuration_Weight_interval_stress (me, you, him, stressMeasure);
-	NUMBER_THREE_END (U"(interval mds stress)")
+	NUMBER_THREE_END (U" (interval mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_Weight_monotone_stress, U"Dissimilarity & Configuration & Weight: Get stress (monotone mds)", U"Dissimilarity & Configuration & Weight: Get stress (monotone mds)...") {
@@ -753,7 +712,7 @@ FORM (REAL_Dissimilarity_Configuration_Weight_monotone_stress, U"Dissimilarity &
 DO
 	NUMBER_THREE (Dissimilarity, Configuration, Weight)
 		double result = Dissimilarity_Configuration_Weight_monotone_stress (me, you, him, tiesHandling, stressMeasure);
-	NUMBER_THREE_END (U"(monotone mds stress)")
+	NUMBER_THREE_END (U" (monotone mds stress)")
 }
 
 FORM (REAL_Dissimilarity_Configuration_Weight_ispline_stress, U"Dissimilarity & Configuration & Weight: Get stress (i-spline mds)", U"Dissimilarity & Configuration & Weight: Get stress (i-spline mds)...") {
@@ -764,7 +723,7 @@ FORM (REAL_Dissimilarity_Configuration_Weight_ispline_stress, U"Dissimilarity &
 DO
 	NUMBER_THREE (Dissimilarity, Configuration, Weight)
 		double result = Dissimilarity_Configuration_Weight_ispline_stress (me, you, him, numberOfInteriorKnots, order, stressMeasure);
-	NUMBER_THREE_END (U"(i-spline mds stress)")
+	NUMBER_THREE_END (U" (i-spline mds stress)")
 }
 
 FORM (GRAPHICS_Dissimilarity_Configuration_drawShepardDiagram, U"Dissimilarity & Configuration: Draw Shepard diagram", U"Dissimilarity & Configuration: Draw Shepard diagram...") {
@@ -835,7 +794,7 @@ FORM (NEW_Dissimilarity_to_Configuration_kruskal, U"Dissimilarity: To Configurat
 DO
 	CONVERT_EACH (Dissimilarity)
 		autoConfiguration result = Dissimilarity_to_Configuration_kruskal (me, numberOfDimensions, distanceMetric, tiesHandling, stressFormula, tolerance, maximumNumberOfIterations, numberOfRepetitions);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 FORM (NEW_Dissimilarity_to_Configuration_absolute_mds, U"Dissimilarity: To Configuration (absolute mds)", U"Dissimilarity: To Configuration (absolute mds)...") {
@@ -847,7 +806,7 @@ DO
 	CONVERT_EACH (Dissimilarity)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_absolute_mds (me, nullptr, numberOfDimensions, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress); 
-	CONVERT_EACH_END (my name, U"_absolute")
+	CONVERT_EACH_END (my name.get(), U"_absolute")
 }
 
 FORM (NEW_Dissimilarity_to_Configuration_ratio_mds, U"Dissimilarity: To Configuration (ratio mds)", U"Dissimilarity: To Configuration (ratio mds)...") {
@@ -859,7 +818,7 @@ DO
 	CONVERT_EACH (Dissimilarity)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_ratio_mds (me, nullptr, numberOfDimensions, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_EACH_END (my name, U"_ratio")
+	CONVERT_EACH_END (my name.get(), U"_ratio")
 }
 
 FORM (NEW_Dissimilarity_to_Configuration_interval_mds, U"Dissimilarity: To Configuration (interval mds)", U"Dissimilarity: To Configuration (interval mds)...") {
@@ -871,7 +830,7 @@ DO
 	CONVERT_EACH (Dissimilarity)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_interval_mds (me, nullptr, numberOfDimensions, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_EACH_END (my name, U"_interval")
+	CONVERT_EACH_END (my name.get(), U"_interval")
 }
 
 FORM (NEW_Dissimilarity_to_Configuration_monotone_mds, U"Dissimilarity: To Configuration (monotone mds)", U"Dissimilarity: To Configuration (monotone mds)...") {
@@ -886,7 +845,7 @@ DO
 	CONVERT_EACH (Dissimilarity)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_monotone_mds (me, nullptr, numberOfDimensions, tiesHandling, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_EACH_END (my name, U"_monotone");
+	CONVERT_EACH_END (my name.get(), U"_monotone");
 }
 
 FORM (NEW_Dissimilarity_to_Configuration_ispline_mds, U"Dissimilarity: To Configuration (i-spline mds)", U"Dissimilarity: To Configuration (i-spline mds)...") {
@@ -904,7 +863,7 @@ DO
 	CONVERT_EACH (Dissimilarity)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_ispline_mds (me, nullptr, numberOfDimensions, numberOfInteriorKnots, order, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_EACH_END (my name, U"_ispline");
+	CONVERT_EACH_END (my name.get(), U"_ispline");
 }
 
 FORM (NEW1_Dissimilarity_Weight_ispline_mds, U"Dissimilarity & Weight: To Configuration (i-spline mds)", U"Dissimilarity & Weight: To Configuration (i-spline mds)...") {
@@ -922,7 +881,7 @@ DO
 			Melder_throw (U"Order-zero spline must at least have 1 interior knot.");
 		}
 		autoConfiguration result = Dissimilarity_Weight_ispline_mds (me, you, numberOfDimensions, numberOfInteriorKnots, order, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_ispline")
+	CONVERT_TWO_END (my name.get(), U"_ispline")
 }
 
 FORM (NEW1_Dissimilarity_Weight_absolute_mds, U"Dissimilarity & Weight: To Configuration (absolute mds)", U"Dissimilarity & Weight: To Configuration (absolute mds)...") {
@@ -934,7 +893,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Weight)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_absolute_mds (me, you, numberOfDimensions, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_absolute")
+	CONVERT_TWO_END (my name.get(), U"_absolute")
 }
 
 FORM (NEW1_Dissimilarity_Weight_ratio_mds, U"Dissimilarity & Weight: To Configuration (ratio mds)", U"Dissimilarity & Weight: To Configuration (ratio mds)...") {
@@ -946,7 +905,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Weight)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_ratio_mds (me, you, numberOfDimensions, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_absolute")
+	CONVERT_TWO_END (my name.get(), U"_absolute")
 }
 
 FORM (NEW1_Dissimilarity_Weight_interval_mds, U"Dissimilarity & Weight: To Configuration (interval mds)", U"Dissimilarity & Weight: To Configuration (interval mds)...") {
@@ -958,7 +917,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Weight)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_interval_mds (me, you, numberOfDimensions, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_absolute")
+	CONVERT_TWO_END (my name.get(), U"_absolute")
 }
 
 FORM (NEW1_Dissimilarity_Weight_monotone_mds, U"Dissimilarity & Weight: To Configuration (monotone mds)", U"Dissimilarity & Weight: To Configuration (monotone mds)...") {
@@ -973,7 +932,7 @@ DO
 	CONVERT_TWO (Dissimilarity, Weight)
 		int showProgress = 1;
 		autoConfiguration result = Dissimilarity_Weight_monotone_mds (me, you, numberOfDimensions, tiesHandling, tolerance, maximumNumberOfIterations, numberOfRepetitions, showProgress);
-	CONVERT_TWO_END (my name, U"_monotone")
+	CONVERT_TWO_END (my name.get(), U"_monotone")
 }
 
 FORM (NEW_Dissimilarity_to_Distance, U"Dissimilarity: To Distance", U"Dissimilarity: To Distance...") {
@@ -981,14 +940,20 @@ FORM (NEW_Dissimilarity_to_Distance, U"Dissimilarity: To Distance", U"Dissimilar
 	OK
 DO
 	CONVERT_EACH (Dissimilarity)
-		autoDistance result = Dissimilarity_to_Distance (me, scale);
-	CONVERT_EACH_END (my name)
+		autoDistance result = Dissimilarity_to_Distance (me, scale ? kMDS_AnalysisScale::Ordinal : kMDS_AnalysisScale::Absolute);
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_Dissimilarity_to_Weight) {
 	CONVERT_EACH (Dissimilarity)
 		autoWeight result = Dissimilarity_to_Weight (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
+}
+
+DIRECT (NEW_Dissimilarity_to_MDSVec) {
+	CONVERT_EACH (Dissimilarity)
+		autoMDSVec result = Dissimilarity_to_MDSVec (me);
+	CONVERT_EACH_END (my name.get())
 }
 
 /************************* Distance(s) ***************************************/
@@ -999,13 +964,13 @@ FORM (NEW_Distance_to_ScalarProduct, U"Distance: To ScalarProduct", U"Distance:
 DO
 	CONVERT_EACH (Distance)
 		autoScalarProduct result = Distance_to_ScalarProduct (me, scaleSumOfSquares);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_Distance_to_Dissimilarity) {
 	CONVERT_EACH (Distance)
 		autoDissimilarity result = Distance_to_Dissimilarity (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 FORM (NEWMANY_old_Distances_to_Configuration_indscal, U"Distance: To Configuration (indscal)", U"Distance: To Configuration (indscal)...") {
@@ -1137,6 +1102,7 @@ DO
 	FIND_LIST (Distance)
 		autoConfiguration configurationResult;
 		autoSalience salienceResult;
+		Melder_require (list.size > 1, U"There should me more than one Distance selected.");
 		DistanceList_to_Configuration_ytl ((DistanceList) & list, numberOfDimensions, normalizeScalarProducts, & configurationResult, & salienceResult);
 		praat_new (configurationResult.move(), U"ytl");
 		if (wantSalienceObject) {
@@ -1145,6 +1111,15 @@ DO
 	END
 }
 
+FORM (NEW_Distance_to_Configuration_torsca, U"Distance: To Configuration (torsca)", U"") {
+	NATURAL (numberOfDimensions, U"Number of dimensions", U"2")
+	OK
+DO
+	CONVERT_EACH (Distance)
+		autoConfiguration result = Distance_to_Configuration_torsca (me, numberOfDimensions);
+	CONVERT_EACH_END (my name.get(), U"_torsca")
+}
+
 FORM (NEW1_Dissimilarity_Distance_monotoneRegression, U"Dissimilarity & Distance: Monotone regression", nullptr) {
 	RADIO (tiesHandling, U"Handling of ties", 1)
 		RADIOBUTTON (U"Primary approach")
@@ -1153,7 +1128,7 @@ FORM (NEW1_Dissimilarity_Distance_monotoneRegression, U"Dissimilarity & Distance
 DO
 	CONVERT_TWO (Dissimilarity, Distance)
 		autoDistance result = Dissimilarity_Distance_monotoneRegression (me, you, tiesHandling);
-	CONVERT_TWO_END (my name)
+	CONVERT_TWO_END (my name.get())
 }
 
 FORM (GRAPHICS_Distance_Dissimilarity_drawShepardDiagram, U"Distance & Dissimilarity: Draw Shepard diagram", nullptr) {
@@ -1198,7 +1173,7 @@ FORM (NEW_Covariance_to_Configuration, U"Covariance: To Configuration", nullptr)
 DO
 	CONVERT_EACH (Covariance)
 		autoConfiguration result = Covariance_to_Configuration (me, numberOfDimensions);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 /********* Procrustes ***************************/
@@ -1218,49 +1193,49 @@ DIRECT (REAL_Procrustes_getScale) {
 DIRECT (NEW_TableOfReal_to_Dissimilarity) {
 	CONVERT_EACH (TableOfReal)
 		autoDissimilarity result = TableOfReal_to_Dissimilarity (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_TableOfReal_to_Similarity) {
 	CONVERT_EACH (TableOfReal)
 		autoSimilarity result = TableOfReal_to_Similarity (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_TableOfReal_to_Distance) {
 	CONVERT_EACH (TableOfReal)
 		autoTableOfReal result = TableOfReal_to_Distance (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_TableOfReal_to_Salience) {
 	CONVERT_EACH (TableOfReal)
 		autoSalience result = TableOfReal_to_Salience (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_TableOfReal_to_Weight) {
 	CONVERT_EACH (TableOfReal)
 		autoWeight result = TableOfReal_to_Weight (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_TableOfReal_to_ScalarProduct) {
 	CONVERT_EACH (TableOfReal)
 		autoScalarProduct result = TableOfReal_to_ScalarProduct (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_TableOfReal_to_Configuration) {
 	CONVERT_EACH (TableOfReal)
 		autoConfiguration result = TableOfReal_to_Configuration (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 DIRECT (NEW_TableOfReal_to_ContingencyTable) {
 	CONVERT_EACH (TableOfReal)
 		autoContingencyTable result = TableOfReal_to_ContingencyTable (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 /********************** TableOfReal ***************************************/
@@ -1331,7 +1306,7 @@ DIRECT (MODIFY_TableOfReal_standardizeColumns) {
 DIRECT (NEW_TableOfReal_to_Confusion) {
 	CONVERT_EACH (TableOfReal)
 		autoConfusion result = TableOfReal_to_Confusion (me);
-	CONVERT_EACH_END (my name)
+	CONVERT_EACH_END (my name.get())
 }
 
 static void praat_AffineTransform_init (ClassInfo klas) {
@@ -1351,12 +1326,11 @@ void praat_TableOfReal_extras (ClassInfo klas) {
 	praat_addAction1 (klas, 0, U"Normalize table...", U"Normalize columns...", 1, MODIFY_TableOfReal_normalizeTable);
 	praat_addAction1 (klas, 0, U"Standardize rows", U"Normalize table...", 1, MODIFY_TableOfReal_standardizeRows);
 	praat_addAction1 (klas, 0, U"Standardize columns", U"Standardize rows", 1, MODIFY_TableOfReal_standardizeColumns);
-	praat_addAction1 (klas, 0, U"Test sorting...", U"Standardize columns", praat_DEPTH_1 + praat_HIDDEN + praat_NO_API, MODIFY_TabelOfReal_testSorting);
 }
 
 void praat_uvafon_MDS_init ();
 void praat_uvafon_MDS_init () {
-	Thing_recognizeClassesByName (classAffineTransform, classProcrustes, classContingencyTable, classDissimilarity,
+	Thing_recognizeClassesByName (classAffineTransform, classProcrustes, classContingencyTable, classDissimilarity, classMDSVec,
 		classSimilarity, classConfiguration, classDistance, classSalience, classScalarProduct, classWeight, nullptr);
 	Thing_recognizeClassByOtherName (classProcrustes, U"Procrustus");
 
@@ -1435,6 +1409,7 @@ void praat_uvafon_MDS_init () {
 	praat_addAction1 (classDissimilarity, 1, U"To Configuration (kruskal)...", nullptr, 1, NEW_Dissimilarity_to_Configuration_kruskal);
 	praat_addAction1 (classDissimilarity, 0, U"To Distance...", nullptr, 0, NEW_Dissimilarity_to_Distance);
 	praat_addAction1 (classDissimilarity, 0, U"To Weight", nullptr, 0, NEW_Dissimilarity_to_Weight);
+	praat_addAction1 (classDissimilarity, 0, U"To MDSVec", nullptr, praat_HIDDEN, NEW_Dissimilarity_to_MDSVec);
 
 
 	praat_addAction1 (classCovariance, 0, U"To Configuration...", nullptr, 0, NEW_Covariance_to_Configuration);
@@ -1447,6 +1422,7 @@ void praat_uvafon_MDS_init () {
 	praat_addAction1 (classDistance, 0, U"To Configuration (indscal)...", nullptr, 1, NEWMANY_Distances_to_Configuration_indscal);
 	praat_addAction1 (classDistance, 0, U"-- linear scaling --", nullptr, 1, nullptr);
 	praat_addAction1 (classDistance, 0, U"To Configuration (ytl)...", nullptr, 1, NEWMANY_Distances_to_Configuration_ytl);
+	praat_addAction1 (classDistance, 0, U"To Configuration (torsca)...", nullptr, 1, NEW_Distance_to_Configuration_torsca);
 	praat_addAction1 (classDistance, 0, U"To Dissimilarity", nullptr, 0, NEW_Distance_to_Dissimilarity);
 	praat_addAction1 (classDistance, 0, U"To ScalarProduct...", nullptr, 0, NEW_Distance_to_ScalarProduct);
 
