@@ -23,7 +23,7 @@
 #include "config.h"
 
 #include <math.h>
-#include <stdbool.h>
+//#include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
