@@ -31,7 +31,7 @@ autoAmplitudeTier AmplitudeTier_create (double tmin, double tmax) {
 }
 
 void AmplitudeTier_draw (AmplitudeTier me, Graphics g, double tmin, double tmax,
-	double ymin, double ymax, const char32 *method, bool garnish)
+	double ymin, double ymax, conststring32 method, bool garnish)
 {
 	RealTier_draw (me, g, tmin, tmax, ymin, ymax, garnish, method, U"Sound pressure (Pa)");
 }
@@ -348,9 +348,8 @@ autoSound AmplitudeTier_to_Sound (AmplitudeTier me, double samplingFrequency, in
 		double dt = 1.0 / samplingFrequency;
 		double tmid = (my xmin + my xmax) / 2;
 		double t1 = tmid - 0.5 * (sound_nt - 1) * dt;
-		double *sound;
 		autoSound you = Sound_create (1, my xmin, my xmax, sound_nt, dt, t1);
-		sound = your z [1];
+		VEC sound = your z.row (1);
 		for (integer it = 1; it <= my points.size; it ++) {
 			RealPoint point = my points.at [it];
 			double t = point -> number, amplitude = point -> value, angle, halfampsinangle;
