@@ -1,6 +1,6 @@
 /* Cochleagram.cpp
  *
- * Copyright (C) 1992-2011,2015,2016,2017 Paul Boersma
+ * Copyright (C) 1992-2005,2007,2008,2011,2012,2015-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,9 +50,9 @@ void Cochleagram_paint (Cochleagram me, Graphics g, double tmin, double tmax, bo
 				}
 		Graphics_setInner (g);
 		Graphics_setWindow (g, tmin, tmax, 0.0, my ny * my dy);
-		Graphics_grey (g, copy -> z,
-			itmin, itmax, Matrix_columnToX (me, itmin), Matrix_columnToX (me, itmax),
-			1, my ny, 0.5 * my dy, (my ny - 0.5) * my dy,
+		Graphics_grey (g, copy -> z.part (1, my ny, itmin, itmax),
+			Matrix_columnToX (me, itmin), Matrix_columnToX (me, itmax),
+			0.5 * my dy, (my ny - 0.5) * my dy,
 			12, border);
 		Graphics_unsetInner (g);
 		if (garnish) {
@@ -71,13 +71,13 @@ double Cochleagram_difference (Cochleagram me, Cochleagram thee, double tmin, do
 	try {
 		if (my nx != thy nx || my dx != thy dx || my x1 != thy x1)
 			Melder_throw (U"Unequal time samplings.");
-		if (my ny != thy ny)
-			Melder_throw (U"Unequal numbers of frequencies.");
+		Melder_require (my ny == thy ny,
+			U"Unequal numbers of frequencies.");
 		if (tmax <= tmin) { tmin = my xmin; tmax = my xmax; }
 		integer itmin, itmax;
 		integer nt = Matrix_getWindowSamplesX (me, tmin, tmax, & itmin, & itmax);
-		if (nt == 0)
-			Melder_throw (U"Window too short.");
+		Melder_require (nt > 0,
+			U"Window too short.");
 		longdouble diff = 0.0;
 		for (integer itime = itmin; itime <= itmax; itime ++) {
 			for (integer ifreq = 1; ifreq <= my ny; ifreq ++) {
@@ -95,7 +95,7 @@ double Cochleagram_difference (Cochleagram me, Cochleagram thee, double tmin, do
 autoCochleagram Matrix_to_Cochleagram (Matrix me) {
 	try {
 		autoCochleagram thee = Cochleagram_create (my xmin, my xmax, my nx, my dx, my x1, my dy, my ny);
-		NUMmatrix_copyElements (my z, thy z, 1, my ny, 1, my nx);
+		thy z.all() <<= my z.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Cochleagram.");
@@ -105,7 +105,7 @@ autoCochleagram Matrix_to_Cochleagram (Matrix me) {
 autoMatrix Cochleagram_to_Matrix (Cochleagram me) {
 	try {
 		autoMatrix thee = Matrix_create (my xmin, my xmax, my nx, my dx, my x1, my ymin, my ymax, my ny, my dy, my y1);
-		NUMmatrix_copyElements (my z, thy z, 1, my ny, 1, my nx);
+		thy z.all() <<= my z.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Matrix.");
