@@ -33,7 +33,7 @@ Thing_define (DurationTier, RealTier) {
 autoDurationTier DurationTier_create (double tmin, double tmax);
 
 void DurationTier_draw (DurationTier me, Graphics g, double tmin, double tmax,
-	double ymin, double ymax, const char32 *method, int garnish);
+	double ymin, double ymax, conststring32 method, int garnish);
 
 autoDurationTier PointProcess_upto_DurationTier (PointProcess me);
 
