@@ -1,6 +1,6 @@
 /* FormantTier_def.h
  *
- * Copyright (C) 1992-2002,2015,2017 Paul Boersma
+ * Copyright (C) 1992-2002,2015,2017,2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 oo_DEFINE_CLASS (FormantPoint, AnyPoint)
 
 	oo_INT16 (numberOfFormants)
-	oo_DOUBLE_ARRAY (formant, 10, numberOfFormants)
-	oo_DOUBLE_ARRAY (bandwidth, 10, numberOfFormants)
+	oo_VEC (formant, numberOfFormants)
+	oo_VEC (bandwidth, numberOfFormants)
 
 oo_END_CLASS (FormantPoint)
 #undef ooSTRUCT
