@@ -1,6 +1,6 @@
 /* Formant_def.h
  *
- * Copyright (C) 1992-2002,2015 Paul Boersma
+ * Copyright (C) 1992-2005,2008,2011,2015-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,13 @@
 oo_DEFINE_STRUCT (Formant_Formant)
 
 	#if oo_READING
-		if (formatVersion <= 1) {
+		oo_VERSION_UNTIL (2)
 			oo_FLOAT (frequency)
 			oo_FLOAT (bandwidth)
-		} else {
+		oo_VERSION_ELSE
 			oo_DOUBLE (frequency)
 			oo_DOUBLE (bandwidth)
-		}
+		oo_VERSION_END
 	#else
 		oo_DOUBLE (frequency)
 		oo_DOUBLE (bandwidth)
