@@ -159,7 +159,7 @@ void Manipulation_playPart (Manipulation me, double tmin, double tmax, int metho
 				Melder_throw (U"Cannot synthesize overlap-add without a sound.");
 			autoSound part = Data_copy (my sound.get());
 			integer imin = Sampled_xToLowIndex (part.get(), tmin), imax = Sampled_xToHighIndex (part.get(), tmax);
-			double *amp = part -> z [1];
+			VEC amp = part -> z.row (1);
 			for (integer i = 1; i <= imin; i ++) amp [i] = 0.0;
 			for (integer i = imax; i <= part -> nx; i ++) amp [i] = 0.0;
 			autoSound saved = my sound.move();
@@ -167,7 +167,7 @@ void Manipulation_playPart (Manipulation me, double tmin, double tmax, int metho
 			try {
 				autoSound played = Manipulation_to_Sound (me, Manipulation_OVERLAPADD);
 				my sound = saved.move();
-				amp = played -> z [1];
+				amp = played -> z.row (1);
 				for (imin = 1; imin <= played -> nx; imin ++)
 					if (amp [imin] != 0.0) break;
 				for (imax = played -> nx; imax >= 1; imax --)
@@ -266,14 +266,14 @@ static void copyFlat (Sound me, double tmin, double tmax, Sound thee, double tmi
 	if (iminTarget < 1) iminTarget = 1;
 	trace (tmin, U" ", tmax, U" ", tminTarget, U" ", imin, U" ", imax, U" ", iminTarget);
 	Melder_assert (iminTarget + imax - imin <= thy nx);
-	NUMvector_copyElements (my z [1] + imin, thy z [1] + iminTarget, 0, imax - imin);
+	NUMvector_copyElements (& my z [1] [0] + imin, & thy z [1] [0] + iminTarget, 0, imax - imin);
 }
 
 autoSound Sound_Point_Point_to_Sound (Sound me, PointProcess source, PointProcess target, double maxT) {
 	try {
 		autoSound thee = Sound_create (1, my xmin, my xmax, my nx, my dx, my x1);
 		if (source -> nt < 2 || target -> nt < 2) {   // almost completely voiceless?
-			NUMvector_copyElements (my z [1], thy z [1], 1, my nx);
+			NUMvector_copyElements (& my z [1] [0], & thy z [1] [0], 1, my nx);
 			return thee;
 		}
 		for (integer i = 1; i <= target -> nt; i ++) {
@@ -549,7 +549,7 @@ static autoSound synthesize_pulses_pitch_hum (Manipulation me) {
 void Sound_Formant_Intensity_filter (Sound me, FormantTier formantTier, IntensityTier intensity) {
 	Sound_FormantTier_filter_inplace (me, formantTier);
 	if (intensity) Sound_IntensityTier_multiply_inplace (me, intensity);
-	NUMdeemphasize_f (my z [1], my nx, my dx, 50.0);
+	NUMdeemphasize_f (my z.row (1), my dx, 50.0);
 	Vector_scale (me, 0.99);
 }
 
@@ -608,7 +608,7 @@ static autoSound synthesize_pulses_lpc (Manipulation me) {
 		train -> dx = my lpc -> samplingPeriod;   // to be exact
 		Sound_PointProcess_fillVoiceless (train.get(), my pulses.get());
 		autoSound result = LPC_Sound_filter (my lpc.get(), train.get(), true);
-		NUMdeemphasize_f (result -> z [1], result -> nx, result -> dx, 50.0);
+		NUMdeemphasize_f (result -> z.row (1), result -> dx, 50.0);
 		Vector_scale (result.get(), 0.99);
 		return result;
 	} catch (MelderError) {
@@ -630,7 +630,7 @@ static autoSound synthesize_pitch_lpc (Manipulation me) {
 		train -> dx = my lpc -> samplingPeriod;   // to be exact
 		Sound_PointProcess_fillVoiceless (train.get(), my pulses.get());
 		autoSound result = LPC_Sound_filter (my lpc.get(), train.get(), true);
-		NUMdeemphasize_f (result -> z [1], result -> nx, result -> dx, 50.0);
+		NUMdeemphasize_f (result -> z.row (1), result -> dx, 50.0);
 		Vector_scale (result.get(), 0.99);
 		return result;
 	} catch (MelderError) {
