@@ -57,7 +57,7 @@ Thing_define (ManipulationEditor, FunctionEditor) {
 	#include "ManipulationEditor_prefs.h"
 };
 
-autoManipulationEditor ManipulationEditor_create (const char32 *title, Manipulation ana);
+autoManipulationEditor ManipulationEditor_create (conststring32 title, Manipulation ana);
 
 /* End of file ManipulationEditor.h */
 #endif
