@@ -2,7 +2,7 @@
 #define _PointProcess_h_
 /* PointProcess.h
  *
- * Copyright (C) 1992-2011,2015,2016,2017 Paul Boersma
+ * Copyright (C) 1992-2005,2007,2011,2015-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ integer PointProcess_getHighIndex (PointProcess me, double t);
 integer PointProcess_getNearestIndex (PointProcess me, double t);
 integer PointProcess_getWindowPoints (PointProcess me, double tmin, double tmax, integer *p_imin, integer *p_imax);
 void PointProcess_addPoint (PointProcess me, double t);
-void PointProcess_addPoints (PointProcess me, numvec times);
+void PointProcess_addPoints (PointProcess me, constVEC times);
 integer PointProcess_findPoint (PointProcess me, double t);
 void PointProcess_removePoint (PointProcess me, integer index);
 void PointProcess_removePointNear (PointProcess me, double t);
