@@ -1,6 +1,6 @@
 /* Praat_tests.cpp
  *
- * Copyright (C) 2001-2012,2015,2016,2017 Paul Boersma
+ * Copyright (C) 2001-2007,2009,2011-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,25 +37,31 @@
 #include <string>
 
 static void testAutoData (autoDaata data) {
-	fprintf (stderr, "testAutoData: %p %p\n", data.get(), data -> name);
+	fprintf (stderr, "testAutoData: %p %p\n", data.get(), data -> name.get());
 }
 static void testAutoDataRef (autoDaata& data) {
-	fprintf (stderr, "testAutoDataRef: %p %p\n", data.get(), data -> name);
+	fprintf (stderr, "testAutoDataRef: %p %p\n", data.get(), data -> name.get());
 }
 static void testData (Daata data) {
-	fprintf (stderr, "testData: %p %s\n", data, Melder_peek32to8 (data -> name));
+	fprintf (stderr, "testData: %p %s\n", data, Melder_peek32to8 (data -> name.get()));
 }
 static autoDaata newAutoData () {
 	autoDaata data (Thing_new (Daata));
 	return data;
 }
-static int length (const char32 *s) {
+static int length (conststring32 s) {
 	int result = str32len (s);
 	Melder_free (s);
 	return result;
 }
 
-int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, char32 *arg4) {
+static autoMAT constantHH (integer nrow, integer ncol, double value) {
+	autoMAT result = newMATraw (nrow, ncol);
+	result.all() <<= value;
+	return result;
+}
+
+int Praat_tests (kPraatTests itest, conststring32 arg1, conststring32 arg2, conststring32 arg3, conststring32 arg4) {
 	int64 n = Melder_atoi (arg1);
 	double t = 0.0;
 	(void) arg1;
@@ -77,16 +83,14 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 		} break;
 		case kPraatTests::TIME_SORT: {
 			integer size = Melder_atoi (arg2);
-			double *array = NUMvector <double> (1, size);
+			autoVEC array = newVECraw (size);
 			Melder_stopwatch ();
 			for (int64 iteration = 1; iteration <= n; iteration ++) {
-				for (int64 i = 1; i <= size; i ++) {
+				for (int64 i = 1; i <= size; i ++)
 					array [i] = NUMrandomFraction ();
-				}
-				NUMsort_d (size, array);
+				VECsort_inplace (array.get());
 			}
 			t = Melder_stopwatch () / (size * log2 (size));
-			NUMvector_free (array, 1);
 		} break;
 		case kPraatTests::TIME_INTEGER: {
 			int64 sum = 0;
@@ -103,7 +107,7 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 			MelderInfo_writeLine (sum);
 		} break;
 		case kPraatTests::TIME_FLOAT_TO_UNSIGNED_BUILTIN: {
-			uint64_t sum = 0;
+			uint64 sum = 0;
 			double fn = n;
 			for (double fi = 1.0; fi <= fn; fi += 1.0)
 				sum += (uint32) fi;
@@ -111,7 +115,7 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 			MelderInfo_writeLine (sum);
 		} break;
 		case kPraatTests::TIME_FLOAT_TO_UNSIGNED_EXTERN: {
-			uint64_t sum = 0;
+			uint64 sum = 0;
 			double fn = n;
 			for (double fi = 1.0; fi <= fn; fi += 1.0)
 				sum += (uint32) ((int32) (fi - 2147483648.0) + 2147483647L + 1);
@@ -290,44 +294,34 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 			double x = 0.0;
 			for (int64 i = 1; i <= n; i ++) {
 				x += (double) i;
-				isAllDefined &= ((* (uint64_t *) & x) & 0x7FF0000000000000) != 0x7FF0000000000000;
+				isAllDefined &= ((* (uint64 *) & x) & 0x7FF0'0000'0000'0000) != 0x7FF0'0000'0000'0000;
 			}
 			t = Melder_stopwatch ();   // 0.90 ns
 			MelderInfo_writeLine (isAllDefined, U" ", x);
 		} break;
 		case kPraatTests::TIME_INNER: {
-			int size = Melder_atoi (arg2);
-			autonumvec x { size, kTensorInitializationType::RAW }, y { size, kTensorInitializationType::RAW };
-			for (int64 i = 1; i <= size; i ++) {
-				x [i] = NUMrandomGauss (0.0, 1.0);
-				y [i] = NUMrandomGauss (0.0, 1.0);
-			}
+			integer size = Melder_atoi (arg2);
+			autoVEC x = newVECrandomGauss (size, 0.0, 1.0);
+			autoVEC y = newVECrandomGauss (size, 0.0, 1.0);
 			double z = 0.0;
-			for (int64 i = 1; i <= n; i ++) {
-				z += inner_scalar (x.get(), y.get());
-			}
+			for (int64 i = 1; i <= n; i ++)
+				z += NUMinner (x.get(), y.get());
 			t = Melder_stopwatch () / size;   // 0.43 ns per multiplication-addition pair
 			MelderInfo_writeLine (z);
 		} break;
 		case kPraatTests::TIME_OUTER_NUMMAT: {
-			int nrow = 100, ncol = 100;
-			numvec x { NUMvector<double> (1, nrow), nrow }, y { NUMvector<double> (1, ncol), ncol };
-			for (int64 i = 1; i <= nrow; i ++)
-				x.at [i] = NUMrandomGauss (0.0, 1.0);
-			for (int64 i = 1; i <= ncol; i ++)
-				y.at [i] = NUMrandomGauss (0.0, 1.0);
-			for (int64 i = 1; i <= n; i ++) {
-				const autonummat mat = outer_nummat (x, y);
-			}
+			integer nrow = 100, ncol = 100;
+			autoVEC x = newVECrandomGauss (nrow, 0.0, 1.0);
+			autoVEC y = newVECrandomGauss (ncol, 0.0, 1.0);
+			for (int64 i = 1; i <= n; i ++)
+				const autoMAT mat = newMATouter (x.get(), y.get());
 			t = Melder_stopwatch () / nrow / ncol;   // 0.29 ns, i.e. less than one clock cycle per cell
-			NUMvector_free (x.at, 1);
-			NUMvector_free (y.at, 1);
 		} break;
 		case kPraatTests::CHECK_INVFISHERQ: {
 			MelderInfo_writeLine (NUMinvFisherQ (0.003, 1, 100000));
 		} break;
 		case kPraatTests::TIME_AUTOSTRING: {
-			const char32 *strings [6] = { U"ghdg", U"jhd", U"hkfjjd", U"fhfj", U"jhksfd", U"hfjs" };
+			conststring32 strings [6] = { U"ghdg", U"jhd", U"hkfjjd", U"fhfj", U"jhksfd", U"hfjs" };
 			int64 sumOfLengths = 0;
 			for (int64 i = 1; i <= n; i ++) {
 				int istring = i % 6;
@@ -338,11 +332,11 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 			MelderInfo_writeLine (sumOfLengths);
 		} break;
 		case kPraatTests::TIME_CHAR32: {
-			const char32 *strings [6] = { U"ghdg", U"jhd", U"hkfjjd", U"fhfj", U"jhksfd", U"hfjs" };
+			conststring32 strings [6] = { U"ghdg", U"jhd", U"hkfjjd", U"fhfj", U"jhksfd", U"hfjs" };
 			int64 sumOfLengths = 0;
 			for (int64 i = 1; i <= n; i ++) {
 				int istring = i % 6;
-				char32 *s = Melder_dup (strings [istring]);
+				char32 *s = Melder_dup (strings [istring]).transfer();
 				sumOfLengths += length (s);
 			}
 			t = Melder_stopwatch ();   // 72 ns
@@ -350,12 +344,10 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 		} break;
 		case kPraatTests::TIME_SUM: {
 			integer size = Melder_atoi (arg2);
-			autonumvec x { size, kTensorInitializationType::RAW };
-			for (integer i = 1; i <= size; i ++)
-				x [i] = NUMrandomGauss (0.0, 1.0);
+			autoVEC x = newVECrandomGauss (size, 0.0, 1.0);
 			double z = 0.0;
 			for (int64 i = 1; i <= n; i ++) {
-				double sum = sum_scalar (x.get());
+				double sum = NUMsum (x.get());
 				z += sum;
 			}
 			t = Melder_stopwatch () / size;   // for size == 100: 0.31 ns
@@ -363,12 +355,10 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 		} break;
 		case kPraatTests::TIME_MEAN: {
 			integer size = Melder_atoi (arg2);
-			autonumvec x { size, kTensorInitializationType::RAW };
-			for (integer i = 1; i <= size; i ++)
-				x [i] = NUMrandomGauss (0.0, 1.0);
+			autoVEC x = newVECrandomGauss (size, 0.0, 1.0);
 			double z = 0.0;
 			for (int64 i = 1; i <= n; i ++) {
-				double sum = mean_scalar (x.get());
+				double sum = NUMmean (x.get());
 				z += sum;
 			}
 			t = Melder_stopwatch () / size;   // for size == 100: 0.34 ns
@@ -376,12 +366,10 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 		} break;
 		case kPraatTests::TIME_STDEV: {
 			integer size = 10000;
-			autonumvec x { size, kTensorInitializationType::RAW };
-			for (integer i = 1; i <= size; i ++)
-				x [i] = NUMrandomGauss (0.0, 1.0);
+			autoVEC x = newVECrandomGauss (size, 0.0, 1.0);
 			double z = 0.0;
 			for (int64 i = 1; i <= n; i ++) {
-				double stdev = stdev_scalar (x.get());
+				double stdev = NUMstdev (x.get());
 				z += stdev;
 			}
 			t = Melder_stopwatch () / size;
@@ -390,25 +378,25 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 		case kPraatTests::TIME_ALLOC: {
 			integer size = Melder_atoi (arg2);
 			for (int64 iteration = 1; iteration <= n; iteration ++) {
-				autonumvec result (size, kTensorInitializationType::RAW);
+				autoVEC result = newVECraw (size);
+				for (integer i = 1; i <= size; i ++)
+					result [i] = 0.0;
 			}
-			t = Melder_stopwatch () / size;
+			t = Melder_stopwatch () / size;   // 10^0..7: 70/6.9/1.08 / 0.074/0.0074/0.0091 / 0.51/0.00026 ns
 		} break;
 		case kPraatTests::TIME_ALLOC0: {
 			integer size = Melder_atoi (arg2);
-			for (int64 iteration = 1; iteration <= n; iteration ++) {
-				autonumvec result (size, kTensorInitializationType::RAW);
-			}
-			t = Melder_stopwatch () / size;
+			for (int64 iteration = 1; iteration <= n; iteration ++)
+				autoVEC result = newVECzero (size);
+			t = Melder_stopwatch () / size;   // 10^0..7: 76/7.7/1.23 / 0.165/0.24/0.25 / 1.30/1.63 ns
 		} break;
 		case kPraatTests::TIME_ZERO: {
 			integer size = Melder_atoi (arg2);
-			autonumvec result { size, kTensorInitializationType::RAW };
+			autoVEC result { size, kTensorInitializationType::RAW };
 			double z = 0.0;
 			for (int64 iteration = 1; iteration <= n; iteration ++) {
-				for (integer i = 1; i <= size; i ++) {
+				for (integer i = 1; i <= size; i ++)
 					result [i] = (double) i;
-				}
 				z += result [size - 1];
 			}
 			t = Melder_stopwatch () / size;
@@ -416,13 +404,13 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 		} break;
 		case kPraatTests::TIME_MALLOC: {
 			integer size = Melder_atoi (arg2);
+			double value = Melder_atof (arg3);
 			double z = 0.0;
 			for (int64 iteration = 1; iteration <= n; iteration ++) {
 				double *result = (double *) malloc (sizeof (double) * (size_t) size);
-				for (integer i = 0; i < size; i ++) {
-					result [i] = (double) i;
-				}
-				z += result [size - 1];
+				for (integer i = 0; i < size; i ++)
+					result [i] = value;
+				z += result [size / 2];
 				free (result);
 			}
 			t = Melder_stopwatch () / size;
@@ -431,17 +419,73 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 		case kPraatTests::TIME_CALLOC: {
 			integer size = Melder_atoi (arg2);
 			double z = 0.0;
-			for (int64 iteration = 1; iteration <= n; iteration ++) {
+			for (integer iteration = 1; iteration <= n; iteration ++) {
 				double *result = (double *) calloc (sizeof (double), (size_t) size);
-				for (integer i = 0; i < size; i ++) {
-					result [i] = (double) i;
-				}
-				z += result [size - 1];
+				z += result [size / 2];
 				free (result);
 			}
 			t = Melder_stopwatch () / size;
 			MelderInfo_writeLine (z);
 		} break;
+		case kPraatTests::TIME_ADD: {
+			integer size = Melder_atoi (arg2);
+			autoMAT result = newMATrandomGauss (size, size, 0.0, 1.0);
+			Melder_stopwatch ();
+			for (integer iteration = 1; iteration <= n; iteration ++)
+				result.all() <<= 5.0;
+			t = Melder_stopwatch () / size / size;   // 10^0..4: 2.7/0.16/0.24 / 0.38/0.98
+			double sum = NUMsum (result.get());
+			MelderInfo_writeLine (sum);
+		} break;
+		case kPraatTests::TIME_SIN: {
+			integer size = Melder_atoi (arg2);
+			autoMAT result = newMATrandomGauss (size, size, 0.0, 1.0);
+			Melder_stopwatch ();
+			for (integer iteration = 1; iteration <= n; iteration ++)
+				MATsin_inplace (result.get());
+			t = Melder_stopwatch () / size / size;   // 10^0..4: 18/5.3/5.2 / 5.3/12
+			double sum = NUMsum (result.get());
+			MelderInfo_writeLine (sum);
+		} break;
+		case kPraatTests::TIME_VECADD: {
+			integer size = Melder_atoi (arg2);
+			autoVEC x = newVECrandomGauss (size, 0.0, 1.0);
+			autoVEC y = newVECrandomGauss (size, 0.0, 1.0);
+			autoVEC result = newVECraw (size);
+			Melder_stopwatch ();
+			for (integer iteration = 1; iteration <= n; iteration ++)
+				//VECVUadd (result.all(), x.all(), y.all());
+				result.all() <<= x.all() + y.all();
+			t = Melder_stopwatch () / size;
+			double sum = NUMsum (result.get());
+			MelderInfo_writeLine (sum);
+		} break;
+		case kPraatTests::TIME_MATMUL: {
+			const integer size1 = Melder_atoi (arg2);
+			integer size2 = Melder_atoi (arg3);
+			integer size3 = Melder_atoi (arg4);
+			if (size2 == 0 || size3 == 0) size3 = size2 = size1;
+			//autoMAT const x = newMATrandomGauss (size1, size2, 0.0, 1.0);
+			//autoMAT const y = newMATrandomGauss (size2, size3, 0.0, 1.0);
+			autoMAT x = constantHH (size1, size2, 10.0);
+			autoMAT y = constantHH (size2, size3, 3.0);
+			autoMAT const result = newMATraw (size1, size3);
+			//MAT resultget = result.get();
+			//constMAT xget = x.get(), yget = y.get();
+			MATVU const result_all = result.all();
+			constMATVU const x_all = x.all();
+			constMATVU const y_all = y.all();
+			Melder_stopwatch ();
+			for (integer iteration = 1; iteration <= n; iteration ++)
+				MATVUmul_forceMetal_ (result_all, x_all, y_all);
+			const integer numberOfComputations = size1 * size2 * size3 * 2;
+			t = Melder_stopwatch () / numberOfComputations;
+			const double sum = NUMsum (result.get());
+			const integer numberOfStores = size1 * size2 + size2 * size3 + size1 * size3 + 10000;
+			MelderInfo_writeLine (double (numberOfComputations) / double (numberOfStores), U" computations per store");
+			MelderInfo_writeLine (sum, U" should be ", size1 * size2 * size3 * 30.0);
+			//Melder_require (NUMequal (result.get(), constantHH (size, size, size * 30.0).get()), U"...");
+		} break;
 		case kPraatTests::THING_AUTO: {
 			int numberOfThingsBefore = theTotalNumberOfThings;
 			{
@@ -499,8 +543,9 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 				//data11 = nullptr;   // disabled implicit assignment of pointer to autopointer
 				fprintf (stderr, "21\n");
 			}
-			int numberOfThingsAfter = theTotalNumberOfThings;
-			fprintf (stderr, "Number of things: before %d, after %d\n", numberOfThingsBefore, numberOfThingsAfter);
+			integer numberOfThingsAfter = theTotalNumberOfThings;
+			fprintf (stderr, "Number of things: before %ld, after %ld\n",
+					(long_not_integer) numberOfThingsBefore, (long_not_integer) numberOfThingsAfter);
 			#if 0
 				MelderCallback<void,structDaata>::FunctionType f;
 				typedef void (*DataFunc) (Daata);
@@ -521,31 +566,83 @@ int Praat_tests (kPraatTests itest, char32 *arg1, char32 *arg2, char32 *arg3, ch
 				autoMelderAsynchronous y = x.move();   // defined move constructor
 				//x = y;   // deleted copy assignment
 				x = y.move();   // defined move assignment
-				autonumvec a;
-				autonumvec b = a.move();
-				const autonumvec c;
-				const autonumvec d { };
+				autoVEC a;
+				autoVEC b = a.move();
+				const autoVEC c;
+				const autoVEC d { };
+				#if 0
 				double *e;
-				const autonumvec f { e, 10 };
-				const autonumvec g { 100, kTensorInitializationType::ZERO };
-				//return f;   // call to deleted constructor
-				numvec h;
-				autonumvec j;
-				numvec *ph = & h;
-				autonumvec *pj = & j;
+				const autoVEC f { e, 10 };
+				#endif
+				{
+					autoVEC g { 100, kTensorInitializationType::ZERO };
+					g [1] = 3.0;
+					VEC gg = g.get();
+					gg [2] = 4.0;
+					constVEC ggg = g.get();
+					//ggg [3] = 5.0;   // should be refused by the compiler
+					const VEC gggg = g.get();
+					//gggg [3] = 6.0;   // should be refused by the compiler
+					//return f;   // call to deleted constructor
+					//gggg.reset();   // should be refused by the compiler
+					//ggg.reset();   // should be refused by the compiler
+					//gg.reset();
+				}
+				{
+					double x [1+2], *px = & x [0];
+					const double *cpx = px;
+					VEC vx { px, 2 };
+					constVEC cvx { px, 2 };
+					const VEC c_vx { px, 2 };
+					double a = c_vx [1];
+					const double b = c_vx [2];
+					const double y = 0.0, *py = & y;
+					//VEC vy { py, 0 };   // should be refused by the compiler
+					constVEC cvy { py, 2 };
+					//const VEC c_vy = VEC (py, 2);
+					const VEC c_vy = (const VEC) VEC (const_cast<double *> (py), 2);
+					double c = c_vy [1];
+					const double d = c_vy [2];
+					//VEC c_vy2 = VEC (py, 2);
+				}
+
+				VEC h;
+				autoVEC j;
+				//VEC jh = j;
+				//VEC zero = newVECzero (10);   // should be ruled out
+				//constVEC zero = newVECzero (10);   // should be ruled out
+				//j = h;   // up assignment standardly correctly ruled out
+				//h = j;   // down assignment was explicitly ruled out as well
+				//h = VEC (j);
+				VEC & jref = j;   // (in)correctly? accepted
+				VEC *ph = & h;
+				autoVEC *pj = & j;
 				ph = pj;   // (in)correctly? accepted
 				//pj = ph;   // correctly ruled out
 				#endif
 				autoSound sound = Sound_create (1, 0.0, 1.0, 10000, 0.0001, 0.0);
 				sound = Sound_create (1, 0.0, 1.0, 10000, 0.0001, 0.00005);
 				Melder_casual (U"hello ", sound -> dx);
+				autostring32vector v;
+				mutablestring32 *pm = v.peek2();
+				const mutablestring32 *pcm = v.peek2();
+				//conststring32 *pc = v.peek2();
+				const conststring32 *pcc = v.peek2();
+				{
+					vector<double> aa, bb;
+					vector<const double> aac, bbc;
+					//aa = aac;
+					aa = bb;
+					aac = bbc;
+					bbc.at = bb.at;
+				}
 			}
 		} break;
 		case kPraatTests::FILEINMEMORYMANAGER_IO: {
 			test_FileInMemoryManager_io ();
 		} break;
 	}
-	MelderInfo_writeLine (Melder_single (t / n * 1e9), U" nanoseconds");
+	MelderInfo_writeLine (Melder_single (n / t * 1e-9), U" Gflop/s");
 	MelderInfo_close ();
 	return 1;
 }
@@ -616,4 +713,45 @@ autoPitch_ structSound_::toPitch () {   // this requires the prior definition of
 
 #endif
 
+/*
+	An attempt to not have VEC and constVEC, but VEC and const VEC instead.
+*/
+
+class Vec {
+public:
+	double *at;
+	integer size;
+	const double *const_propagate_at () const { return at; }
+	double *const_propagate_at () { return at; }
+public:
+	Vec (double *initialAt, integer initialSize) : at (initialAt), size (initialSize) { }
+	double& operator[] (integer index) { return at [index]; }   // selected for Vec (1)
+	const double& operator[] (integer index) const { return at [index]; }   // selected for const Vec (2)
+	//Vec (Vec& other) : at (other.const_propagate_at()), size (other.size) { };   // can assign Vec to Vec (3)
+	//Vec (Vec&& other) : at (other.at), size (other.size) { };   // can assign Vec to Vec (3)
+	Vec (Vec& other) : at (other.at), size (other.size) { };   // can assign Vec to Vec (3)
+	Vec (const Vec& other) = delete;   // cannot assign const Vec to Vec (4)
+		/* unfortunately, this also precludes initializing a *const* Vec from a const Vec */
+	//Vec (const Vec& other) const = default;   // attempt to copy a const Vec to a const Vec, but constructors cannot be const
+	//const Vec (const Vec& other) = default;   // attempt to copy a const Vec to a const Vec, but constructors cannot have a return type
+};
+
+static Vec copy (Vec x) {
+	return x;
+}
+
+/*static void tryVec () {
+	Vec x = Vec (nullptr, 0);
+	x [1] = 3.0;
+	double a = x [2];
+	const Vec cx = Vec (nullptr, 0);
+	//cx [1] = 3.0;   // should be refused by compiler, because operator[] returns a const value that cannot be assigned to (2)
+	a = cx [2];   // should be allowed by compiler, because not an assignment (2)
+	const Vec cy = x;   // should be allowed (3)
+	//Vec y = cx;   // should be refused (4)
+	const Vec cz = copy (x);
+	//cx.at [1] = 3.0;
+	////const Vec ca = cy;   // should be allowed
+}*/
+
 /* End of file Praat_tests.cpp */
