@@ -1,6 +1,6 @@
 /* Sound_PointProcess.cpp
  *
- * Copyright (C) 2010-2011,2015 Paul Boersma
+ * Copyright (C) 2010-2011,2015,2017 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,23 +27,23 @@ autoSound Sound_PointProcess_to_SoundEnsemble_correlate (Sound me, PointProcess
 	try {
 		if (my ny > 1)
 			Melder_throw (U"Sound has to be mono.");
-		long numberOfPoints = thy nt;
+		integer numberOfPoints = thy nt;
 		double hisDuration = toLag - fromLag;
-		long numberOfSamples = (long) floor (hisDuration / my dx) + 1;
+		integer numberOfSamples = Melder_ifloor (hisDuration / my dx) + 1;
 		if (numberOfSamples < 1)
 			Melder_throw (U"Time window too short.");
 		double midTime = 0.5 * (fromLag + toLag);
 		double hisPhysicalDuration = numberOfSamples * my dx;
 		double firstTime = midTime - 0.5 * hisPhysicalDuration + 0.5 * my dx;   // distribute the samples evenly over the time domain
 		autoSound him = Sound_create (numberOfPoints, fromLag, toLag, numberOfSamples, my dx, firstTime);
-		for (long ipoint = 1; ipoint <= numberOfPoints; ipoint ++) {
+		for (integer ipoint = 1; ipoint <= numberOfPoints; ipoint ++) {
 			double myTimeOfPoint = thy t [ipoint];
 			double hisTimeOfPoint = 0.0;
-			double mySample = 1 + (myTimeOfPoint - my x1) / my dx;
-			double hisSample = 1 + (hisTimeOfPoint - his x1) / my dx;
-			long sampleDifference = lround (mySample - hisSample);
-			for (long isample = 1; isample <= numberOfSamples; isample ++) {
-				long jsample = isample + sampleDifference;
+			double mySample = 1.0 + (myTimeOfPoint - my x1) / my dx;
+			double hisSample = 1.0 + (hisTimeOfPoint - his x1) / my dx;
+			integer sampleDifference = Melder_iround_tieDown (mySample - hisSample);
+			for (integer isample = 1; isample <= numberOfSamples; isample ++) {
+				integer jsample = isample + sampleDifference;
 				his z [ipoint] [isample] = jsample < 1 || jsample > my nx ? 0.0 : my z [1] [jsample];
 			}
 		}
