@@ -1,6 +1,6 @@
 /* Spectrogram.cpp
  *
- * Copyright (C) 1992-2012,2015,2016,2017 Paul Boersma
+ * Copyright (C) 1992-2008,2011,2012,2015-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ void Spectrogram_paintInside (Spectrogram me, Graphics g, double tmin, double tm
 	if (fmax <= fmin) { fmin = my ymin; fmax = my ymax; }
 	integer itmin, itmax, ifmin, ifmax;
 	if (! Matrix_getWindowSamplesX (me, tmin - 0.49999 * my dx, tmax + 0.49999 * my dx, & itmin, & itmax) ||
-		 ! Matrix_getWindowSamplesY (me, fmin - 0.49999 * my dy, fmax + 0.49999 * my dy, & ifmin, & ifmax))
+			! Matrix_getWindowSamplesY (me, fmin - 0.49999 * my dy, fmax + 0.49999 * my dy, & ifmin, & ifmax))
 		return;
 	Graphics_setWindow (g, tmin, tmax, fmin, fmax);
 	autoNUMvector <double> preemphasisFactor (ifmin, ifmax);
@@ -86,14 +86,13 @@ void Spectrogram_paintInside (Spectrogram me, Graphics g, double tmin, double tm
 		for (integer ifreq = ifmin; ifreq <= ifmax; ifreq ++)
 			my z [ifreq] [itime] += dynamicFactor [itime];
 	}
-	Graphics_image (g, my z,
-		itmin, itmax,
+	Graphics_image (g, my z.part (ifmin, ifmax, itmin, itmax),
 		Matrix_columnToX (me, itmin - 0.5),
 		Matrix_columnToX (me, itmax + 0.5),
-		ifmin, ifmax,
 		Matrix_rowToY (me, ifmin - 0.5),
 		Matrix_rowToY (me, ifmax + 0.5),
-		maximum - dynamic, maximum);
+		maximum - dynamic, maximum
+	);
 	for (integer ifreq = ifmin; ifreq <= ifmax; ifreq ++)
 		for (integer itime = itmin; itime <= itmax; itime ++) {
 			double value = 4.0e-10 * exp ((my z [ifreq] [itime] - dynamicFactor [itime]
@@ -105,7 +104,7 @@ void Spectrogram_paintInside (Spectrogram me, Graphics g, double tmin, double tm
 void Spectrogram_paint (Spectrogram me, Graphics g,
 	double tmin, double tmax, double fmin, double fmax, double maximum, int autoscaling,
 	double dynamic, double preemphasis, double dynamicCompression,
-	int garnish)
+	bool garnish)
 {
 	Graphics_setInner (g);
 	Spectrogram_paintInside (me, g, tmin, tmax, fmin, fmax, maximum, autoscaling, dynamic, preemphasis, dynamicCompression);
@@ -122,7 +121,7 @@ void Spectrogram_paint (Spectrogram me, Graphics g,
 autoSpectrogram Matrix_to_Spectrogram (Matrix me) {
 	try {
 		autoSpectrogram thee = Spectrogram_create (my xmin, my xmax, my nx, my dx, my x1, my ymin, my ymax, my ny, my dy, my y1);
-		NUMmatrix_copyElements (my z, thy z, 1, my ny, 1, my nx);
+		thy z.all() <<= my z.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Spectrogram.");
@@ -132,7 +131,7 @@ autoSpectrogram Matrix_to_Spectrogram (Matrix me) {
 autoMatrix Spectrogram_to_Matrix (Spectrogram me) {
 	try {
 		autoMatrix thee = Matrix_create (my xmin, my xmax, my nx, my dx, my x1, my ymin, my ymax, my ny, my dy, my y1);
-		NUMmatrix_copyElements (my z, thy z, 1, my ny, 1, my nx);
+		thy z.all() <<= my z.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to Matrix.");
