@@ -1,6 +1,6 @@
 /* TimeSoundEditor.cpp
  *
- * Copyright (C) 1992-2012,2013,2014,2015,2016,2017 Paul Boersma
+ * Copyright (C) 1992-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include "NUM2.h"
 #include "TimeSoundEditor.h"
 #include "EditorM.h"
-#include "UnicodeData.h"
+#include "../kar/UnicodeData.h"
 
 #include "enums_getText.h"
 #include "TimeSoundEditor_enums.h"
@@ -40,14 +40,14 @@ Thing_implement (TimeSoundEditor, FunctionEditor, 0);
 void structTimeSoundEditor :: v_destroy () noexcept {
 	if (our d_ownSound)
 		forget (our d_sound.data);
-	NUMvector_free (d_sound.muteChannels, 1);
+	NUMvector_free (our d_sound.muteChannels, 1);
 	TimeSoundEditor_Parent :: v_destroy ();
 }
 
 void structTimeSoundEditor :: v_info () {
 	TimeSoundEditor_Parent :: v_info ();
 	/* Sound flags: */
-	MelderInfo_writeLine (U"Sound scaling strategy: ", kTimeSoundEditor_scalingStrategy_getText (p_sound_scalingStrategy));
+	MelderInfo_writeLine (U"Sound scaling strategy: ", kTimeSoundEditor_scalingStrategy_getText (our p_sound_scalingStrategy));
 }
 
 /***** FILE MENU *****/
@@ -118,7 +118,8 @@ static void menu_cb_DrawSelectedSound (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 			Melder_throw (U"There is no sound to draw.");
 		autoSound publish = my d_longSound.data ?
 			LongSound_extractPart (my d_longSound.data, my startSelection, my endSelection, my pref_picture_preserveTimes ()) :
-			Sound_extractPart (my d_sound.data, my startSelection, my endSelection, kSound_windowShape::RECTANGULAR, 1.0, my pref_picture_preserveTimes ());
+			Sound_extractPart (my d_sound.data, my startSelection, my endSelection,
+				kSound_windowShape::RECTANGULAR, 1.0, my pref_picture_preserveTimes ());
 		Editor_openPraatPicture (me);
 		Sound_draw (publish.get(), my pictureGraphics, 0.0, 0.0, my pref_picture_bottom (), my pref_picture_top (),
 			my pref_picture_garnish (), U"Curve");
@@ -149,7 +150,7 @@ static void menu_cb_ExtractSelectedSound_preserveTimes (TimeSoundEditor me, EDIT
 static void menu_cb_ExtractSelectedSound_windowed (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Extract selected sound (windowed)", nullptr)
 		WORD (name, U"Name", U"slice")
-		OPTIONMENU_ENUM (windowShape, U"Window shape", kSound_windowShape, my default_extract_windowShape ())
+		OPTIONMENU_ENUM (kSound_windowShape, windowShape, U"Window shape", my default_extract_windowShape ())
 		POSITIVE (relativeWidth, U"Relative width", my default_extract_relativeWidth ())
 		BOOLEAN (preserveTimes, U"Preserve times", my default_extract_preserveTimes ())
 	EDITOR_OK
@@ -202,12 +203,13 @@ static void do_write (TimeSoundEditor me, MelderFile file, int format, int numbe
 		if (numberOfSamples) {
 			autoSound save = Sound_create (sound -> ny, 0.0, numberOfSamples * sound -> dx, numberOfSamples, sound -> dx, 0.5 * sound -> dx);
 			integer offset = first - 1;
-			if (first < 1) first = 1;
-			if (last > sound -> nx) last = sound -> nx;
+			if (first < 1)
+				first = 1;
+			if (last > sound -> nx)
+				last = sound -> nx;
 			for (integer channel = 1; channel <= sound -> ny; channel ++) {
-				for (integer i = first; i <= last; i ++) {
+				for (integer i = first; i <= last; i ++)
 					save -> z [channel] [i - offset] = sound -> z [channel] [i];
-				}
 			}
 			Sound_saveAsAudioFile (save.get(), file, format, numberOfBitsPerSamplePoint);
 		}
@@ -216,7 +218,7 @@ static void do_write (TimeSoundEditor me, MelderFile file, int format, int numbe
 
 static void menu_cb_WriteWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as WAV file", nullptr)
-		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".wav");
+		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name.get() : my d_sound.data -> name.get(), U".wav");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_WAV, 16);
 	EDITOR_END
@@ -225,7 +227,7 @@ static void menu_cb_WriteWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 static void menu_cb_SaveAs24BitWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as 24-bit WAV file", nullptr)
 		Melder_assert (! my d_longSound.data && my d_sound.data);
-		Melder_sprint (defaultName,300, my d_sound.data -> name, U".wav");
+		Melder_sprint (defaultName,300, my d_sound.data -> name.get(), U".wav");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_WAV, 24);
 	EDITOR_END
@@ -234,7 +236,7 @@ static void menu_cb_SaveAs24BitWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 static void menu_cb_SaveAs32BitWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as 32-bit WAV file", nullptr)
 		Melder_assert (! my d_longSound.data && my d_sound.data);
-		Melder_sprint (defaultName,300, my d_sound.data -> name, U".wav");
+		Melder_sprint (defaultName,300, my d_sound.data -> name.get(), U".wav");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_WAV, 32);
 	EDITOR_END
@@ -242,7 +244,7 @@ static void menu_cb_SaveAs32BitWav (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 
 static void menu_cb_WriteAiff (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as AIFF file", nullptr)
-		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".aiff");
+		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name.get() : my d_sound.data -> name.get(), U".aiff");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_AIFF, 16);
 	EDITOR_END
@@ -250,7 +252,7 @@ static void menu_cb_WriteAiff (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 
 static void menu_cb_WriteAifc (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as AIFC file", nullptr)
-		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".aifc");
+		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name.get() : my d_sound.data -> name.get(), U".aifc");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_AIFC, 16);
 	EDITOR_END
@@ -258,7 +260,7 @@ static void menu_cb_WriteAifc (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 
 static void menu_cb_WriteNextSun (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as NeXT/Sun file", nullptr)
-		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".au");
+		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name.get() : my d_sound.data -> name.get(), U".au");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_NEXT_SUN, 16);
 	EDITOR_END
@@ -266,7 +268,7 @@ static void menu_cb_WriteNextSun (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 
 static void menu_cb_WriteNist (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as NIST file", nullptr)
-		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".nist");
+		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name.get() : my d_sound.data -> name.get(), U".nist");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_NIST, 16);
 	EDITOR_END
@@ -274,7 +276,7 @@ static void menu_cb_WriteNist (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 
 static void menu_cb_WriteFlac (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save selected sound as FLAC file", nullptr)
-		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name : my d_sound.data -> name, U".flac");
+		Melder_sprint (defaultName,300, my d_longSound.data ? my d_longSound.data -> name.get() : my d_sound.data -> name.get(), U".flac");
 	EDITOR_DO_SAVE
 		do_write (me, file, Melder_FLAC, 16);
 	EDITOR_END
@@ -282,71 +284,71 @@ static void menu_cb_WriteFlac (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 
 void structTimeSoundEditor :: v_createMenuItems_file_draw (EditorMenu menu) {
 	EditorMenu_addCommand (menu, U"Draw to picture window:", GuiMenu_INSENSITIVE, menu_cb_DrawVisibleSound /* dummy */);
-	if (d_sound.data || d_longSound.data) {
+	if (our d_sound.data || our d_longSound.data) {
 		EditorMenu_addCommand (menu, U"Draw visible sound...", 0, menu_cb_DrawVisibleSound);
-		drawButton = EditorMenu_addCommand (menu, U"Draw selected sound...", 0, menu_cb_DrawSelectedSound);
+		our drawButton = EditorMenu_addCommand (menu, U"Draw selected sound...", 0, menu_cb_DrawSelectedSound);
 	}
 }
 
 void structTimeSoundEditor :: v_createMenuItems_file_extract (EditorMenu menu) {
 	EditorMenu_addCommand (menu, U"Extract to objects window:", GuiMenu_INSENSITIVE, menu_cb_ExtractSelectedSound_preserveTimes /* dummy */);
-	if (d_sound.data || d_longSound.data) {
-		publishPreserveButton = EditorMenu_addCommand (menu, U"Extract selected sound (preserve times)", 0, menu_cb_ExtractSelectedSound_preserveTimes);
+	if (our d_sound.data || our d_longSound.data) {
+		our publishPreserveButton = EditorMenu_addCommand (menu, U"Extract selected sound (preserve times)", 0, menu_cb_ExtractSelectedSound_preserveTimes);
 			EditorMenu_addCommand (menu, U"Extract sound selection (preserve times)", Editor_HIDDEN, menu_cb_ExtractSelectedSound_preserveTimes);
 			EditorMenu_addCommand (menu, U"Extract selection (preserve times)", Editor_HIDDEN, menu_cb_ExtractSelectedSound_preserveTimes);
-		publishButton = EditorMenu_addCommand (menu, U"Extract selected sound (time from 0)", 0, menu_cb_ExtractSelectedSound_timeFromZero);
+		our publishButton = EditorMenu_addCommand (menu, U"Extract selected sound (time from 0)", 0, menu_cb_ExtractSelectedSound_timeFromZero);
 			EditorMenu_addCommand (menu, U"Extract sound selection (time from 0)", Editor_HIDDEN, menu_cb_ExtractSelectedSound_timeFromZero);
 			EditorMenu_addCommand (menu, U"Extract selection (time from 0)", Editor_HIDDEN, menu_cb_ExtractSelectedSound_timeFromZero);
 			EditorMenu_addCommand (menu, U"Extract selection", Editor_HIDDEN, menu_cb_ExtractSelectedSound_timeFromZero);
-		if (d_sound.data) {
-			publishWindowButton = EditorMenu_addCommand (menu, U"Extract selected sound (windowed)...", 0, menu_cb_ExtractSelectedSound_windowed);
+		if (our d_sound.data) {
+			our publishWindowButton = EditorMenu_addCommand (menu, U"Extract selected sound (windowed)...", 0, menu_cb_ExtractSelectedSound_windowed);
 				EditorMenu_addCommand (menu, U"Extract windowed sound selection...", Editor_HIDDEN, menu_cb_ExtractSelectedSound_windowed);
 				EditorMenu_addCommand (menu, U"Extract windowed selection...", Editor_HIDDEN, menu_cb_ExtractSelectedSound_windowed);
-			publishOverlapButton = EditorMenu_addCommand (menu, U"Extract selected sound for overlap...", 0, menu_cb_ExtractSelectedSoundForOverlap);
+			our publishOverlapButton = EditorMenu_addCommand (menu, U"Extract selected sound for overlap...", 0, menu_cb_ExtractSelectedSoundForOverlap);
 		}
 	}
 }
 
 void structTimeSoundEditor :: v_createMenuItems_file_write (EditorMenu menu) {
 	EditorMenu_addCommand (menu, U"Save to disk:", GuiMenu_INSENSITIVE, menu_cb_WriteWav /* dummy */);
-	if (d_sound.data || d_longSound.data) {
-		writeWavButton = EditorMenu_addCommand (menu, U"Save selected sound as WAV file...", 0, menu_cb_WriteWav);
+	if (our d_sound.data || our d_longSound.data) {
+		our writeWavButton = EditorMenu_addCommand (menu, U"Save selected sound as WAV file...", 0, menu_cb_WriteWav);
 			EditorMenu_addCommand (menu, U"Write selected sound to WAV file...", Editor_HIDDEN, menu_cb_WriteWav);
 			EditorMenu_addCommand (menu, U"Write sound selection to WAV file...", Editor_HIDDEN, menu_cb_WriteWav);
 			EditorMenu_addCommand (menu, U"Write selection to WAV file...", Editor_HIDDEN, menu_cb_WriteWav);
-		if (d_sound.data) {
-			d_saveAs24BitWavButton = EditorMenu_addCommand (menu, U"Save selected sound as 24-bit WAV file...", 0, menu_cb_SaveAs24BitWav);
-			d_saveAs32BitWavButton = EditorMenu_addCommand (menu, U"Save selected sound as 32-bit WAV file...", 0, menu_cb_SaveAs32BitWav);
+		if (our d_sound.data) {
+			our saveAs24BitWavButton = EditorMenu_addCommand (menu, U"Save selected sound as 24-bit WAV file...", 0, menu_cb_SaveAs24BitWav);
+			our saveAs32BitWavButton = EditorMenu_addCommand (menu, U"Save selected sound as 32-bit WAV file...", 0, menu_cb_SaveAs32BitWav);
 		}
-		writeAiffButton = EditorMenu_addCommand (menu, U"Save selected sound as AIFF file...", 0, menu_cb_WriteAiff);
+		our writeAiffButton = EditorMenu_addCommand (menu, U"Save selected sound as AIFF file...", 0, menu_cb_WriteAiff);
 			EditorMenu_addCommand (menu, U"Write selected sound to AIFF file...", Editor_HIDDEN, menu_cb_WriteAiff);
 			EditorMenu_addCommand (menu, U"Write sound selection to AIFF file...", Editor_HIDDEN, menu_cb_WriteAiff);
 			EditorMenu_addCommand (menu, U"Write selection to AIFF file...", Editor_HIDDEN, menu_cb_WriteAiff);
-		writeAifcButton = EditorMenu_addCommand (menu, U"Save selected sound as AIFC file...", 0, menu_cb_WriteAifc);
+		our writeAifcButton = EditorMenu_addCommand (menu, U"Save selected sound as AIFC file...", 0, menu_cb_WriteAifc);
 			EditorMenu_addCommand (menu, U"Write selected sound to AIFC file...", Editor_HIDDEN, menu_cb_WriteAifc);
 			EditorMenu_addCommand (menu, U"Write sound selection to AIFC file...", Editor_HIDDEN, menu_cb_WriteAifc);
 			EditorMenu_addCommand (menu, U"Write selection to AIFC file...", Editor_HIDDEN, menu_cb_WriteAifc);
-		writeNextSunButton = EditorMenu_addCommand (menu, U"Save selected sound as Next/Sun file...", 0, menu_cb_WriteNextSun);
+		our writeNextSunButton = EditorMenu_addCommand (menu, U"Save selected sound as Next/Sun file...", 0, menu_cb_WriteNextSun);
 			EditorMenu_addCommand (menu, U"Write selected sound to Next/Sun file...", Editor_HIDDEN, menu_cb_WriteNextSun);
 			EditorMenu_addCommand (menu, U"Write sound selection to Next/Sun file...", Editor_HIDDEN, menu_cb_WriteNextSun);
 			EditorMenu_addCommand (menu, U"Write selection to Next/Sun file...", Editor_HIDDEN, menu_cb_WriteNextSun);
-		writeNistButton = EditorMenu_addCommand (menu, U"Save selected sound as NIST file...", 0, menu_cb_WriteNist);
+		our writeNistButton = EditorMenu_addCommand (menu, U"Save selected sound as NIST file...", 0, menu_cb_WriteNist);
 			EditorMenu_addCommand (menu, U"Write selected sound to NIST file...", Editor_HIDDEN, menu_cb_WriteNist);
 			EditorMenu_addCommand (menu, U"Write sound selection to NIST file...", Editor_HIDDEN, menu_cb_WriteNist);
 			EditorMenu_addCommand (menu, U"Write selection to NIST file...", Editor_HIDDEN, menu_cb_WriteNist);
-		writeFlacButton = EditorMenu_addCommand (menu, U"Save selected sound as FLAC file...", 0, menu_cb_WriteFlac);
+		our writeFlacButton = EditorMenu_addCommand (menu, U"Save selected sound as FLAC file...", 0, menu_cb_WriteFlac);
 			EditorMenu_addCommand (menu, U"Write selected sound to FLAC file...", Editor_HIDDEN, menu_cb_WriteFlac);
 			EditorMenu_addCommand (menu, U"Write sound selection to FLAC file...", Editor_HIDDEN, menu_cb_WriteFlac);
 	}
 }
 
 void structTimeSoundEditor :: v_createMenuItems_file (EditorMenu menu) {
-	TimeSoundEditor_Parent :: v_createMenuItems_file (menu);
-	v_createMenuItems_file_draw (menu);
+	our TimeSoundEditor_Parent :: v_createMenuItems_file (menu);
+	our v_createMenuItems_file_draw (menu);
 	EditorMenu_addCommand (menu, U"-- after file draw --", 0, nullptr);
-	v_createMenuItems_file_extract (menu);
+	our v_createMenuItems_file_extract (menu);
 	EditorMenu_addCommand (menu, U"-- after file extract --", 0, nullptr);
-	v_createMenuItems_file_write (menu);
+	our v_createMenuItems_file_write (menu);
 	EditorMenu_addCommand (menu, U"-- after file write --", 0, nullptr);
 }
 
@@ -362,9 +364,9 @@ static void menu_cb_LongSoundInfo (TimeSoundEditor me, EDITOR_ARGS_DIRECT) {
 
 void structTimeSoundEditor :: v_createMenuItems_query_info (EditorMenu menu) {
 	TimeSoundEditor_Parent :: v_createMenuItems_query_info (menu);
-	if (d_sound.data && d_sound.data != data) {
+	if (our d_sound.data && our d_sound.data != data) {
 		EditorMenu_addCommand (menu, U"Sound info", 0, menu_cb_SoundInfo);
-	} else if (d_longSound.data && d_longSound.data != data) {
+	} else if (our d_longSound.data && our d_longSound.data != data) {
 		EditorMenu_addCommand (menu, U"LongSound info", 0, menu_cb_LongSoundInfo);
 	}
 }
@@ -373,7 +375,8 @@ void structTimeSoundEditor :: v_createMenuItems_query_info (EditorMenu menu) {
 
 static void menu_cb_soundScaling (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Sound scaling", nullptr)
-		OPTIONMENU_ENUM (scalingStrategy, U"Scaling strategy", kTimeSoundEditor_scalingStrategy, my default_sound_scalingStrategy ())
+		OPTIONMENU_ENUM (kTimeSoundEditor_scalingStrategy, scalingStrategy,
+				U"Scaling strategy", my default_sound_scalingStrategy ())
 		LABEL (U"For \"fixed height\":")
 		POSITIVE (height, U"Height", my default_sound_scaling_height ())
 		LABEL (U"For \"fixed range\":")
@@ -399,24 +402,20 @@ static void menu_cb_soundMuteChannels (TimeSoundEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_OK
 	EDITOR_DO
 		integer numberOfChannels = my d_longSound.data ? my d_longSound.data -> numberOfChannels : my d_sound.data -> ny;
-		integer numberOfElements;
-		autoNUMvector<integer> channelNumber (NUMstring_getElementsOfRanges (channels_string, 5 * numberOfChannels, & numberOfElements, nullptr, U"channel", false), 1);
-		bool *muteChannels = my d_sound.muteChannels;
-		for (integer i = 1; i <= numberOfChannels; i ++) {
-			muteChannels [i] = false;
-		}
-		for (integer i = 1; i <= numberOfElements; i++) {
-			if (channelNumber [i] > 0 && channelNumber [i] <= numberOfChannels) {
-				muteChannels [channelNumber [i]] = true;
-			}
+		autoINTVEC channelNumber = NUMstring_getElementsOfRanges (channels_string, 5 * numberOfChannels, U"channel", false);
+		for (integer i = 1; i <= numberOfChannels; i ++)
+			my d_sound.muteChannels [i] = false;
+		for (integer i = 1; i <= channelNumber.size; i++) {
+			if (channelNumber [i] > 0 && channelNumber [i] <= numberOfChannels)
+				my d_sound.muteChannels [channelNumber [i]] = true;
 		}
 		FunctionEditor_redraw (me);
 	EDITOR_END
 }
 
 void structTimeSoundEditor :: v_createMenuItems_view (EditorMenu menu) {
-	if (d_sound.data || d_longSound.data)
-		v_createMenuItems_view_sound (menu);
+	if (our d_sound.data || our d_longSound.data)
+		our v_createMenuItems_view_sound (menu);
 	TimeSoundEditor_Parent :: v_createMenuItems_view (menu);
 }
 
@@ -427,37 +426,39 @@ void structTimeSoundEditor :: v_createMenuItems_view_sound (EditorMenu menu) {
 
 void structTimeSoundEditor :: v_updateMenuItems_file () {
 	Sampled sound;
-	if (d_sound.data) {
-		sound = d_sound.data;
+	if (our d_sound.data) {   // cannot do this with "?:", because d_sound.data and d_longSound.data have differemt types
+		sound = our d_sound.data;
 	} else {
-		sound = d_longSound.data;
+		sound = our d_longSound.data;
 	}
 	if (! sound) return;
 	integer first, last, selectedSamples = Sampled_getWindowSamples (sound, our startSelection, our endSelection, & first, & last);
-	if (drawButton) {
-		GuiThing_setSensitive (drawButton, selectedSamples != 0);
-		GuiThing_setSensitive (publishButton, selectedSamples != 0);
-		GuiThing_setSensitive (publishPreserveButton, selectedSamples != 0);
-		if (publishWindowButton) GuiThing_setSensitive (publishWindowButton, selectedSamples != 0);
-		if (publishOverlapButton) GuiThing_setSensitive (publishOverlapButton, selectedSamples != 0);
+	if (our drawButton) {
+		GuiThing_setSensitive (our drawButton, selectedSamples != 0);
+		GuiThing_setSensitive (our publishButton, selectedSamples != 0);
+		GuiThing_setSensitive (our publishPreserveButton, selectedSamples != 0);
+		if (our publishWindowButton)
+			GuiThing_setSensitive (our publishWindowButton, selectedSamples != 0);
+		if (our publishOverlapButton)
+			GuiThing_setSensitive (our publishOverlapButton, selectedSamples != 0);
 	}
-	GuiThing_setSensitive (writeWavButton, selectedSamples != 0);
-	if (d_saveAs24BitWavButton)
-		GuiThing_setSensitive (d_saveAs24BitWavButton, selectedSamples != 0);
-	if (d_saveAs32BitWavButton)
-		GuiThing_setSensitive (d_saveAs32BitWavButton, selectedSamples != 0);
-	GuiThing_setSensitive (writeAiffButton, selectedSamples != 0);
-	GuiThing_setSensitive (writeAifcButton, selectedSamples != 0);
-	GuiThing_setSensitive (writeNextSunButton, selectedSamples != 0);
-	GuiThing_setSensitive (writeNistButton, selectedSamples != 0);
-	GuiThing_setSensitive (writeFlacButton, selectedSamples != 0);
+	GuiThing_setSensitive (our writeWavButton, selectedSamples != 0);
+	if (our saveAs24BitWavButton)
+		GuiThing_setSensitive (our saveAs24BitWavButton, selectedSamples != 0);
+	if (our saveAs32BitWavButton)
+		GuiThing_setSensitive (our saveAs32BitWavButton, selectedSamples != 0);
+	GuiThing_setSensitive (our writeAiffButton, selectedSamples != 0);
+	GuiThing_setSensitive (our writeAifcButton, selectedSamples != 0);
+	GuiThing_setSensitive (our writeNextSunButton, selectedSamples != 0);
+	GuiThing_setSensitive (our writeNistButton, selectedSamples != 0);
+	GuiThing_setSensitive (our writeFlacButton, selectedSamples != 0);
 }
 
 void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double globalMaximum) {
 	Sound sound = my d_sound.data;
 	LongSound longSound = my d_longSound.data;
 	Melder_assert (!! sound != !! longSound);
-	int nchan = sound ? sound -> ny : longSound -> numberOfChannels;
+	integer numberOfChannels = ( sound ? sound -> ny : longSound -> numberOfChannels );
 	bool cursorVisible = my startSelection == my endSelection && my startSelection >= my startWindow && my startSelection <= my endWindow;
 	Graphics_setColour (my graphics.get(), Graphics_BLACK);
 	bool fits;
@@ -484,17 +485,18 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 		Graphics_text (my graphics.get(), 0.5, 0.5, U"(zoom out to see the data)");
 		return;
 	}
-	const int numberOfVisibleChannels = nchan > 8 ? 8 : nchan;
-	const int firstVisibleChannel = my d_sound.channelOffset + 1;
-	int lastVisibleChannel = my d_sound.channelOffset + numberOfVisibleChannels;
-	if (lastVisibleChannel > nchan) lastVisibleChannel = nchan;
+	const integer numberOfVisibleChannels = ( numberOfChannels > 8 ? 8 : numberOfChannels );
+	const integer firstVisibleChannel = my d_sound.channelOffset + 1;
+	integer lastVisibleChannel = my d_sound.channelOffset + numberOfVisibleChannels;
+	if (lastVisibleChannel > numberOfChannels)
+		lastVisibleChannel = numberOfChannels;
 	double maximumExtent = 0.0, visibleMinimum = 0.0, visibleMaximum = 0.0;
 	if (my p_sound_scalingStrategy == kTimeSoundEditor_scalingStrategy::BY_WINDOW) {
 		if (longSound)
 			LongSound_getWindowExtrema (longSound, my startWindow, my endWindow, firstVisibleChannel, & visibleMinimum, & visibleMaximum);
 		else
 			Matrix_getWindowExtrema (sound, first, last, firstVisibleChannel, firstVisibleChannel, & visibleMinimum, & visibleMaximum);
-		for (int ichan = firstVisibleChannel + 1; ichan <= lastVisibleChannel; ichan ++) {
+		for (integer ichan = firstVisibleChannel + 1; ichan <= lastVisibleChannel; ichan ++) {
 			double visibleChannelMinimum, visibleChannelMaximum;
 			if (longSound)
 				LongSound_getWindowExtrema (longSound, my startWindow, my endWindow, ichan, & visibleChannelMinimum, & visibleChannelMaximum);
@@ -507,7 +509,7 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 		}
 		maximumExtent = visibleMaximum - visibleMinimum;
 	}
-	for (int ichan = firstVisibleChannel; ichan <= lastVisibleChannel; ichan ++) {
+	for (integer ichan = firstVisibleChannel; ichan <= lastVisibleChannel; ichan ++) {
 		double cursorFunctionValue = longSound ? 0.0 :
 			Vector_getValueAtX (sound, 0.5 * (my startSelection + my endSelection), ichan, 70);
 		/*
@@ -519,7 +521,7 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 		bool horizontal = false;
 		double minimum = sound ? globalMinimum : -1.0, maximum = sound ? globalMaximum : 1.0;
 		if (my p_sound_scalingStrategy == kTimeSoundEditor_scalingStrategy::BY_WINDOW) {
-			if (nchan > 2) {
+			if (numberOfChannels > 2) {
 				if (longSound) {
 					LongSound_getWindowExtrema (longSound, my startWindow, my endWindow, ichan, & minimum, & maximum);
 				} else {
@@ -554,7 +556,11 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 			minimum = my p_sound_scaling_minimum;
 			maximum = my p_sound_scaling_maximum;
 		}
-		if (minimum == maximum) { horizontal = true; minimum -= 1.0; maximum += 1.0;}
+		if (minimum == maximum) {
+			horizontal = true;
+			minimum -= 1.0;
+			maximum += 1.0;
+		}
 		Graphics_setWindow (my graphics.get(), my startWindow, my endWindow, minimum, maximum);
 		if (horizontal) {
 			Graphics_setTextAlignment (my graphics.get(), Graphics_RIGHT, Graphics_HALF);
@@ -588,10 +594,10 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 		Graphics_setColour (my graphics.get(), Graphics_CYAN);
 		Graphics_innerRectangle (my graphics.get(), 0.0, 1.0, 0.0, 1.0);
 		Graphics_setColour (my graphics.get(), Graphics_BLACK);
-		if (nchan > 1) {
+		if (numberOfChannels > 1) {
 			Graphics_setTextAlignment (my graphics.get(), Graphics_LEFT, Graphics_HALF);
 			Graphics_setTextAlignment (my graphics.get(), Graphics_LEFT, Graphics_HALF);
-			const char32 *channelName = my v_getChannelName (ichan);
+			conststring32 channelName = my v_getChannelName (ichan);
 			static MelderString channelLabel { };
 			MelderString_copy (& channelLabel, ( channelName ? U"ch" : U"Ch " ), ichan);
 			if (channelName)
@@ -601,7 +607,7 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 				( my d_sound.muteChannels [ichan] ? UNITEXT_SPEAKER_WITH_CANCELLATION_STROKE : UNITEXT_SPEAKER ));
 			if (ichan > 8 && ichan - my d_sound.channelOffset == 1) {
 				MelderString_append (& channelLabel, U"      " UNITEXT_UPWARDS_ARROW);
-			} else if (ichan >= 8 && ichan - my d_sound.channelOffset == 8 && ichan < nchan) {
+			} else if (numberOfChannels >= 8 && ichan - my d_sound.channelOffset == 8 && ichan < numberOfChannels) {
 				MelderString_append (& channelLabel, U"      " UNITEXT_DOWNWARDS_ARROW);
 			}
 			Graphics_text (my graphics.get(), 1.0, 0.5, channelLabel.string);
@@ -609,7 +615,7 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 		/*
 		 * Draw a very thin separator line underneath.
 		 */
-		if (ichan < nchan) {
+		if (ichan < numberOfChannels) {
 			/*Graphics_setColour (my graphics.get(), Graphics_BLACK);*/
 			Graphics_line (my graphics.get(), 0.0, 0.0, 1.0, 0.0);
 		}
@@ -622,12 +628,12 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 			if (cursorVisible && isdefined (cursorFunctionValue))
 				FunctionEditor_drawCursorFunctionValue (me, cursorFunctionValue, Melder_float (Melder_half (cursorFunctionValue)), U"");
 			Graphics_setColour (my graphics.get(), Graphics_BLACK);
-			Graphics_function (my graphics.get(), sound -> z [ichan], first, last,
+			Graphics_function (my graphics.get(), & sound -> z [ichan] [0], first, last,
 				Sampled_indexToX (sound, first), Sampled_indexToX (sound, last));
 		} else {
 			Graphics_setWindow (my graphics.get(), my startWindow, my endWindow, minimum * 32768, maximum * 32768);
 			Graphics_function16 (my graphics.get(),
-				longSound -> buffer - longSound -> imin * nchan + (ichan - 1), nchan - 1, first, last,
+				longSound -> buffer - longSound -> imin * numberOfChannels + (ichan - 1), numberOfChannels, first, last,
 				Sampled_indexToX (longSound, first), Sampled_indexToX (longSound, last));
 		}
 		Graphics_resetViewport (my graphics.get(), vp);
@@ -637,19 +643,19 @@ void TimeSoundEditor_drawSound (TimeSoundEditor me, double globalMinimum, double
 }
 
 bool structTimeSoundEditor :: v_click (double xbegin, double ybegin, bool shiftKeyPressed) {
-	Sound sound = d_sound.data;
-	LongSound longSound = d_longSound.data;
+	Sound sound = our d_sound.data;
+	LongSound longSound = our d_longSound.data;
 	if (!! sound != !! longSound) {
 		ybegin = (ybegin - v_getBottomOfSoundArea ()) / (1.0 - v_getBottomOfSoundArea ());
-		int nchan = sound ? sound -> ny : longSound -> numberOfChannels;
-		if (nchan > 8) {
-			trace (xbegin, U" ", ybegin, U" ", nchan, U" ", d_sound.channelOffset);
-			if (xbegin >= our endWindow && ybegin > 0.875 && ybegin <= 1.000 && d_sound.channelOffset > 0) {
-				d_sound.channelOffset -= 8;
+		integer numberOfChannels = ( sound ? sound -> ny : longSound -> numberOfChannels );
+		if (numberOfChannels > 8) {
+			trace (xbegin, U" ", ybegin, U" ", numberOfChannels, U" ", our d_sound.channelOffset);
+			if (xbegin >= our endWindow && ybegin > 0.875 && ybegin <= 1.000 && our d_sound.channelOffset > 0) {
+				our d_sound.channelOffset -= 8;
 				return FunctionEditor_UPDATE_NEEDED;
 			}
-			if (xbegin >= our endWindow && ybegin > 0.000 && ybegin <= 0.125 && d_sound.channelOffset < nchan - 8) {
-				d_sound.channelOffset += 8;
+			if (xbegin >= our endWindow && ybegin > 0.000 && ybegin <= 0.125 && our d_sound.channelOffset < numberOfChannels - 8) {
+				our d_sound.channelOffset += 8;
 				return FunctionEditor_UPDATE_NEEDED;
 			}
 		}
@@ -658,29 +664,31 @@ bool structTimeSoundEditor :: v_click (double xbegin, double ybegin, bool shiftK
 }
 
 bool structTimeSoundEditor :: v_clickB (double xbegin, double ybegin) {
-	Sound sound = d_sound.data;
-	LongSound longSound = d_longSound.data;
+	Sound sound = our d_sound.data;
+	LongSound longSound = our d_longSound.data;
 	if (!! sound != !! longSound) {
 		ybegin = (ybegin - v_getBottomOfSoundArea ()) / (1.0 - v_getBottomOfSoundArea ());
-		int numberOfChannels = sound ? sound -> ny : longSound -> numberOfChannels;
+		integer numberOfChannels = ( sound ? sound -> ny : longSound -> numberOfChannels );
 		if (numberOfChannels > 1) {
-			int numberOfVisibleChannels = numberOfChannels > 8 ? 8 : numberOfChannels;
-			bool *muteChannels = d_sound . muteChannels;
+			integer numberOfVisibleChannels = ( numberOfChannels > 8 ? 8 : numberOfChannels );
+			bool *muteChannels = our d_sound. muteChannels;
 			trace (xbegin, U" ", ybegin, U" ", numberOfChannels, U" ", d_sound.channelOffset);
-			int box = ybegin * numberOfVisibleChannels + 1;
-			box = box < 1 ? 1 : box > numberOfVisibleChannels ? numberOfVisibleChannels : box; // top: numberOfVisibleChannels, bottom: 1
-			int channel = numberOfVisibleChannels - box + 1 + d_sound.channelOffset;
-			if (Melder_debug == 24) {
+			integer box = ybegin * numberOfVisibleChannels + 1;
+			if (box < 1)
+				box = 1;
+			if (box > numberOfVisibleChannels)
+				box = numberOfVisibleChannels;
+			integer channel = numberOfVisibleChannels - box + 1 + d_sound.channelOffset;
+			if (Melder_debug == 24)
 				Melder_casual (U"structTimeSoundEditor :: v_clickB ", ybegin, U" ", channel);
-			}
-			muteChannels [channel] = not muteChannels [channel];
+			muteChannels [channel] = ! muteChannels [channel];
 			return FunctionEditor_UPDATE_NEEDED;
 		}
 	}
 	return TimeSoundEditor_Parent :: v_clickB (xbegin, ybegin);
 }
 
-void TimeSoundEditor_init (TimeSoundEditor me, const char32 *title, Function data, Sampled sound, bool ownSound) {
+void TimeSoundEditor_init (TimeSoundEditor me, conststring32 title, Function data, Sampled sound, bool ownSound) {
 	my d_ownSound = ownSound;
 	if (sound) {
 		integer numberOfChannels = 1;
