@@ -1,6 +1,6 @@
 /* Transition.cpp
  *
- * Copyright (C) 1997-2012,2015,2016,2017 Paul Boersma
+ * Copyright (C) 1997-2012,2015-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,13 +45,13 @@ void structTransition :: v_info () {
 }
 
 void structTransition :: v_writeText (MelderFile file) {
-	texputi32 (file, numberOfStates, U"numberOfStates", 0,0,0,0,0);
+	texputi32 (file, numberOfStates, U"numberOfStates");
 	MelderFile_write (file, U"\nstateLabels []: ");
 	if (numberOfStates < 1) MelderFile_write (file, U"(empty)");
 	MelderFile_write (file, U"\n");
 	for (integer i = 1; i <= numberOfStates; i ++) {
 		MelderFile_write (file, U"\"");
-		if (stateLabels [i]) MelderFile_write (file, stateLabels [i]);
+		if (stateLabels [i]) MelderFile_write (file, stateLabels [i].get());
 		MelderFile_write (file, U"\"\t");
 	}
 	for (integer i = 1; i <= numberOfStates; i ++) {
@@ -66,8 +66,8 @@ void Transition_init (Transition me, integer numberOfStates) {
 	if (numberOfStates < 1)
 		Melder_throw (U"Cannot create empty matrix.");
 	my numberOfStates = numberOfStates;
-	my stateLabels = NUMvector <char32 *> (1, numberOfStates);
-	my data = NUMmatrix <double> (1, my numberOfStates, 1, my numberOfStates);
+	my stateLabels = autostring32vector (numberOfStates);
+	my data = newMATzero (my numberOfStates, my numberOfStates);
 }
 
 autoTransition Transition_create (integer numberOfStates) {
@@ -113,13 +113,13 @@ static void print4 (char *buffer, double value, int iformat, int width, int prec
 void Transition_drawAsNumbers (Transition me, Graphics g, int iformat, int precision) {
 	double maxTextWidth = 0, maxTextHeight = 0;
 	Graphics_setInner (g);
-	Graphics_setWindow (g, 0.5, my numberOfStates + 0.5, 0, 1);
-	double leftMargin = Graphics_dxMMtoWC (g, 1);
-	double lineSpacing = Graphics_dyMMtoWC (g, 1.5 * Graphics_inqFontSize (g) * 25.4 / 72);
+	Graphics_setWindow (g, 0.5, my numberOfStates + 0.5, 0.0, 1.0);
+	double leftMargin = Graphics_dxMMtoWC (g, 1.0);
+	double lineSpacing = Graphics_dyMMtoWC (g, 1.5 * Graphics_inqFontSize (g) * 25.4 / 72.0);
 	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_BOTTOM);
 	for (integer col = 1; col <= my numberOfStates; col ++) {
 		if (my stateLabels && my stateLabels [col] && my stateLabels [col] [0]) {
-			Graphics_text (g, col, 1, my stateLabels [col]);
+			Graphics_text (g, col, 1, my stateLabels [col].get());
 			if (maxTextHeight == 0.0) maxTextHeight = lineSpacing;
 		}
 	}
@@ -127,9 +127,9 @@ void Transition_drawAsNumbers (Transition me, Graphics g, int iformat, int preci
 		double y = 1 - lineSpacing * (row - 1 + 0.7);
 		Graphics_setTextAlignment (g, Graphics_RIGHT, Graphics_HALF);
 		if (my stateLabels && my stateLabels [row]) {
-			double textWidth = Graphics_textWidth (g, my stateLabels [row]);
+			double textWidth = Graphics_textWidth (g, my stateLabels [row].get());
 			if (textWidth > maxTextWidth) maxTextWidth = textWidth;
-			Graphics_text (g, 0.5 - leftMargin, y, my stateLabels [row]);
+			Graphics_text (g, 0.5 - leftMargin, y, my stateLabels [row].get());
 		}
 		Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
 		for (integer col = 1; col <= my numberOfStates; col ++) {
@@ -160,7 +160,7 @@ void Transition_eigen (Transition me, autoMatrix *out_eigenvectors, autoMatrix *
 	try {
 		autoEigen eigen = Thing_new (Eigen);
 		Transition_transpose (me);
-		Eigen_initFromSymmetricMatrix (eigen.get(), my data, my numberOfStates);
+		Eigen_initFromSymmetricMatrix (eigen.get(), my data.get());
 		Transition_transpose (me);
 		transposed = true;
 		autoMatrix eigenvectors = Matrix_createSimple (my numberOfStates, my numberOfStates);
@@ -184,7 +184,8 @@ autoTransition Transition_power (Transition me, integer power) {
 		autoTransition thee = Data_copy (me);
 		autoTransition him = Data_copy (me);
 		for (integer ipow = 2; ipow <= power; ipow ++) {
-			double **tmp = his data; his data = thy data; thy data = tmp;   // OPTIMIZE
+			std::swap (his data.at_deprecated, thy data.at_deprecated);   // OPTIMIZE
+			std::swap (his data.cells, thy data.cells);   // OPTIMIZE
 			for (integer irow = 1; irow <= my numberOfStates; irow ++) {
 				for (integer icol = 1; icol <= my numberOfStates; icol ++) {
 					thy data [irow] [icol] = 0.0;
