@@ -17,7 +17,7 @@
  */
 
 #include "WordList.h"
-#include "longchar.h"
+#include "../kar/longchar.h"
 
 #include "oo_DESTROY.h"
 #include "WordList_def.h"
@@ -40,7 +40,7 @@ Thing_implement (WordList, Daata, 0);
 
 static integer WordList_count (WordList me) {
 	integer n = 0;
-	for (char32 *p = my string; *p; p ++) {
+	for (char32 *p = & my string [0]; *p; p ++) {
 		if (*p == '\n') n += 1;
 	}
 	return n;
@@ -49,7 +49,8 @@ static integer WordList_count (WordList me) {
 void structWordList :: v_info () {
 	structDaata :: v_info ();
 	integer n = WordList_count (this);
-	if (! our length) our length = str32len (our string);
+	if (our length == 0)
+		our length = str32len (our string.get());
 	MelderInfo_writeLine (U"Number of words: ", n);
 	MelderInfo_writeLine (U"Number of characters: ", length - n);
 }
@@ -60,21 +61,21 @@ void structWordList :: v_readBinary (FILE *f, int /*formatVersion*/) {
 	our length = bingeti32 (f);
 	if (our length < 0)
 		Melder_throw (U"Wrong length ", our length, U".");
-	string = Melder_calloc (char32, our length + 1);
-	p = current = string;
+	our string = autostring32 (our length);
+	p = current = & our string [0];
 	if (our length > 0) {
 		/*
 		 * Read first word.
 		 */
 		for (;;) {
-			if (p - string >= length - 1) break;
+			if (p - & string [0] >= length - 1) break;
 			kar = fgetc (f);
 			if (kar == EOF)
 				Melder_throw (U"Early end of file.");
 			if (kar >= 128) break;
 			*p ++ = kar;
 		}
-		*p ++ = '\n';
+		*p ++ = U'\n';
 		/*
 		 * Read following words.
 		 */
@@ -85,28 +86,29 @@ void structWordList :: v_readBinary (FILE *f, int /*formatVersion*/) {
 			str32ncpy (current, previous, numberOfSame);
 			p += numberOfSame;
 			for (;;) {
-				if (p - string >= length - 1) break;
+				if (p - & string [0] >= length - 1) break;
 				kar = fgetc (f);
 				if (kar == EOF)
 					Melder_throw (U"Early end of file.");
 				if (kar >= 128) break;
 				*p ++ = kar;
 			}
-			*p ++ = '\n';
-			if (p - string >= length) break;
+			*p ++ = U'\n';
+			if (p - & string [0] >= our length) break;
 		}
 	}
-	*p = '\0';
-	if (p - string != length)
-		Melder_throw (U"Length in header (", length, U") does not match lenth of string (", (integer) (p - string), U").");
+	*p = U'\0';
+	if (p - & our string [0] != our length)
+		Melder_throw (U"Length in header (", our length, U") does not match lenth of string (", (integer) (p - & our string [0]), U").");
 }
 
 void structWordList :: v_writeBinary (FILE *f) {
 	integer currentLength, previousLength;
-	if (! length) length = str32len (string);
-	binputi32 (length, f);
-	if (length > 0) {
-		char32 *current = string, *kar = current;
+	if (our length == 0)
+		our length = str32len (our string.get());
+	binputi32 (our length, f);
+	if (our length > 0) {
+		char32 *current = & our string [0], *kar = current;
 		for (kar = current; *kar != U'\n'; kar ++) { }
 		currentLength = kar - current;
 		for (integer i = 0; i < currentLength; i ++)
@@ -139,31 +141,31 @@ autoWordList Strings_to_WordList (Strings me) {
 		 * Check whether the strings are generic and sorted.
 		 */
 		for (integer i = 1; i <= my numberOfStrings; i ++) {
-			char32 *string = my strings [i], *p;
+			char32 *string = my strings [i].get(), *p;
 			for (p = & string [0]; *p; p ++) {
 				if (*p > 126)
 					Melder_throw (U"String \"", string, U"\" not generic.\nPlease convert to backslash trigraphs first.");
 			}
-			if (i > 1 && str32cmp (my strings [i - 1], string) > 0) {
+			if (i > 1 && str32cmp (my strings [i - 1].get(), string) > 0) {
 				Melder_throw (U"String \"", string, U"\" not sorted.\nPlease sort first.");
 			}
 			totalLength += str32len (string);
 		}
 		autoWordList thee = Thing_new (WordList);
 		thy length = totalLength + my numberOfStrings;
-		thy string = Melder_calloc (char32, thy length + 1);
+		thy string = autostring32 (thy length);
 		/*
 		 * Concatenate the strings into the word list.
 		 */
-		char32 *q = thy string;
+		char32 *q = & thy string [0];
 		for (integer i = 1; i <= my numberOfStrings; i ++) {
-			integer length = str32len (my strings [i]);
-			str32cpy (q, my strings [i]);
+			integer length = str32len (my strings [i].get());
+			str32cpy (q, my strings [i].get());
 			q += length;
 			*q ++ = '\n';
 		}
 		*q = U'\0';
-		Melder_assert (q - thy string == thy length);
+		Melder_assert (q - & thy string [0] == thy length);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": not converted to WordList.");
@@ -172,18 +174,18 @@ autoWordList Strings_to_WordList (Strings me) {
 
 autoStrings WordList_to_Strings (WordList me) {
 	try {
-		unsigned char *word = (unsigned char *) my string;   // BUG: explain this
+		unsigned char *word = (unsigned char *) & my string [0];   // BUG: explain this
 		autoStrings thee = Thing_new (Strings);
 		thy numberOfStrings = WordList_count (me);
 		if (thy numberOfStrings > 0) {
-			thy strings = NUMvector <char32 *> (1, thy numberOfStrings);
+			thy strings = autostring32vector (thy numberOfStrings);
 		}
 		for (integer i = 1; i <= thy numberOfStrings; i ++) {
 			unsigned char *kar = word;
 			for (; *kar != '\n'; kar ++) { }
 			integer length = kar - word;
-			thy strings [i] = Melder_calloc (char32, length + 1);
-			str32ncpy (thy strings [i], Melder_peek8to32 ((const char *) word), length);
+			thy strings [i] = autostring32 (length);
+			str32ncpy (thy strings [i].get(), Melder_peek8to32 ((const char *) word), length);
 			thy strings [i] [length] = U'\0';
 			word += length + 1;
 		}
@@ -215,7 +217,7 @@ static integer gotoPrevious (WordList me, integer p) {
 	return p + 1;
 }
 
-static int compare (const char32 *word, const char32 *p) {
+static int compare (conststring32 word, conststring32 p) {
 	for (;;) {
 		if (*word == U'\0') {
 			if (*p == U'\n') return 0;
@@ -231,26 +233,28 @@ static int compare (const char32 *word, const char32 *p) {
 
 static char32 buffer [3333+1];
 
-bool WordList_hasWord (WordList me, const char32 *word) {
-	if (str32len (word) > 3333) return false;
+bool WordList_hasWord (WordList me, conststring32 word) {
+	if (str32len (word) > 3333)
+		return false;
 	Longchar_genericize32 (word, buffer);
-	if (! my length) my length = str32len (my string);
+	if (! my length)
+		my length = str32len (my string.get());
 	integer p = my length / 2, d = p / 2;
 	while (d > 20) {
 		p = gotoStart (me, p);
-		int cf = compare (buffer, my string + p);
+		int cf = compare (buffer, my string.get() + p);
 		if (cf == 0) return true;
 		if (cf < 0) p -= d; else p += d;
 		d /= 2;
 	}
 	p = gotoStart (me, p);
-	int cf = compare (buffer, my string + p);
+	int cf = compare (buffer, my string.get() + p);
 	if (cf == 0) return true;
 	if (cf > 0) {
 		for (;;) {
 			p = gotoNext (me, p);
 			if (p >= my length) return false;
-			cf = compare (buffer, my string + p);
+			cf = compare (buffer, my string.get() + p);
 			if (cf == 0) return true;
 			if (cf < 0) return false;
 		}
@@ -258,7 +262,7 @@ bool WordList_hasWord (WordList me, const char32 *word) {
 		for (;;) {
 			p = gotoPrevious (me, p);
 			if (p < 0) return false;
-			cf = compare (buffer, my string + p);
+			cf = compare (buffer, my string.get() + p);
 			if (cf == 0) return true;
 			if (cf > 0) return false;
 		}
