@@ -1,6 +1,6 @@
 /* Net.cpp
  *
- * Copyright (C) 2017 Paul Boersma
+ * Copyright (C) 2017,2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
 
 //#include <OpenCL/OpenCL.h>
 #include "Net.h"
-#include "tensor.h"
-#include "PAIRWISE_SUM.h"
 
 #include "oo_DESTROY.h"
 #include "Net_def.h"
@@ -50,14 +48,14 @@ static autoRBMLayer RBMLayer_create (integer numberOfInputNodes, integer numberO
 	try {
 		autoRBMLayer me = Thing_new (RBMLayer);
 		my numberOfInputNodes = numberOfInputNodes;
-		my inputBiases = NUMvector <double> (1, numberOfInputNodes);
-		my inputActivities = NUMvector <double> (1, numberOfInputNodes);
-		my inputReconstruction = NUMvector <double> (1, numberOfInputNodes);
+		my inputBiases = newVECzero (numberOfInputNodes);
+		my inputActivities = newVECzero (numberOfInputNodes);
+		my inputReconstruction = newVECzero (numberOfInputNodes);
 		my numberOfOutputNodes = numberOfOutputNodes;
-		my outputBiases = NUMvector <double> (1, numberOfOutputNodes);
-		my outputActivities = NUMvector <double> (1, numberOfOutputNodes);
-		my outputReconstruction = NUMvector <double> (1, numberOfOutputNodes);
-		my weights = NUMmatrix <double> (1, numberOfInputNodes, 1, numberOfOutputNodes);
+		my outputBiases = newVECzero (numberOfOutputNodes);
+		my outputActivities = newVECzero (numberOfOutputNodes);
+		my outputReconstruction = newVECzero (numberOfOutputNodes);
+		my weights = newMATzero (numberOfInputNodes, numberOfOutputNodes);
 		my inputsAreBinary = inputsAreBinary;
 		return me;
 	} catch (MelderError) {
@@ -66,6 +64,10 @@ static autoRBMLayer RBMLayer_create (integer numberOfInputNodes, integer numberO
 	}
 }
 
+void Net_initEmpty (Net me) {
+	my layers = LayerList_create ();
+}
+
 autoNet Net_createEmpty (integer numberOfInputNodes) {
 	try {
 		autoNet me = Thing_new (Net);
@@ -77,11 +79,25 @@ autoNet Net_createEmpty (integer numberOfInputNodes) {
 	}
 }
 
-void Net_initAsDeepBeliefNet (Net me, numvec numbersOfNodes, bool inputsAreBinary) {
+/*
+void Net_addRBMLayer (Net me, integer numberOfOutputNodes) {
+	Melder_require (my layers.size >= 1,
+		U"Cannot add an RBM layer if there is no input layer.");
+	integer layerNumber = my layers.size + 1;
+	autoRBMLayer layer = RBMLayer_create (
+		layerNumber == 1 ? my numberOfInputNodes : Melder_iround (numbersOfNodes [ilayer]),
+		Melder_iround (numbersOfNodes [ilayer + 1]),
+		ilayer == 1 ? inputsAreBinary : true
+	);
+	my layers -> addItem_move (layer.move());
+}
+*/
+
+void Net_initAsDeepBeliefNet (Net me, constVEC numbersOfNodes, bool inputsAreBinary) {
 	if (numbersOfNodes.size < 2)
 		Melder_throw (U"A deep belief net should have at least two levels of nodes.");
 	integer numberOfLayers = numbersOfNodes.size - 1;
-	my layers = LayerList_create ();
+	Net_initEmpty (me);
 	for (integer ilayer = 1; ilayer <= numberOfLayers; ilayer ++) {
 		autoRBMLayer layer = RBMLayer_create (
 			Melder_iround (numbersOfNodes [ilayer]),
@@ -92,7 +108,7 @@ void Net_initAsDeepBeliefNet (Net me, numvec numbersOfNodes, bool inputsAreBinar
 	}
 }
 
-autoNet Net_createAsDeepBeliefNet (numvec numbersOfNodes, bool inputsAreBinary) {
+autoNet Net_createAsDeepBeliefNet (constVEC numbersOfNodes, bool inputsAreBinary) {
 	try {
 		autoNet me = Thing_new (Net);
 		Net_initAsDeepBeliefNet (me.get(), numbersOfNodes, inputsAreBinary);
@@ -103,34 +119,41 @@ autoNet Net_createAsDeepBeliefNet (numvec numbersOfNodes, bool inputsAreBinary)
 }
 
 static void copyOutputsToInputs (Layer me, Layer you) {
-	Melder_assert (my numberOfOutputNodes == your numberOfInputNodes);
-	for (integer inode = 1; inode <= my numberOfOutputNodes; inode ++) {
-		your inputActivities [inode] = my outputActivities [inode];
-	}
+	your inputActivities.all() <<= my outputActivities.all();
 }
 
 inline static double logistic (double excitation) {
 	return 1.0 / (1.0 + exp (- excitation));
 }
 
+inline static double inverseLogistic (double activation) {
+	/*
+		y = 1 / (1 + exp (x))
+		1 + exp (x) = 1 / y
+		exp (x) = 1 / y - 1 = (1 - y) / y
+		x = ln ((1 - y) / y)
+	*/
+	return log ((1.0 - activation) / activation);
+}
+
 static void Layer_sampleOutput (Layer me) {
 	for (integer jnode = 1; jnode <= my numberOfOutputNodes; jnode ++) {
-		double probability = my outputActivities [jnode];
-		my outputActivities [jnode] = (double) NUMrandomBernoulli (probability);
+		const double probability = my outputActivities [jnode];
+		my outputActivities [jnode] = double (NUMrandomBernoulli (probability));
 	}
 }
 
 void structRBMLayer :: v_spreadUp (kLayer_activationType activationType) {
-	integer numberOfOutputNodes = our numberOfOutputNodes;
-	for (integer jnode = 1; jnode <= numberOfOutputNodes; jnode ++) {
+	const integer _numberOfOutputNodes = our numberOfOutputNodes;
+	for (integer jnode = 1; jnode <= _numberOfOutputNodes; jnode ++) {
 		PAIRWISE_SUM (longdouble, excitation, integer, our numberOfInputNodes,
- 			double *p_inputActivity = & our inputActivities [0];
- 			double *p_weight = & our weights [1] [jnode] - numberOfOutputNodes,
- 			( p_inputActivity += 1, p_weight += numberOfOutputNodes ),
- 			(longdouble) *p_inputActivity * (longdouble) *p_weight
-		);
+ 			const double *p_inputActivity = & our inputActivities [1];
+ 			const double *p_weight = & our weights [1] [jnode],
+ 			longdouble (*p_inputActivity) * longdouble (*p_weight),
+ 			( p_inputActivity += 1, p_weight += _numberOfOutputNodes )
+		)
 		excitation += our outputBiases [jnode];
-		our outputActivities [jnode] = logistic ((double) excitation);
+		our outputActivities [jnode] = logistic (double (excitation));
 	}
 	if (activationType == kLayer_activationType::STOCHASTIC)
 		Layer_sampleOutput (this);
@@ -138,7 +161,7 @@ void structRBMLayer :: v_spreadUp (kLayer_activationType activationType) {
 
 void Net_spreadUp (Net me, kLayer_activationType activationType) {
 	for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++) {
-		Layer layer = my layers->at [ilayer];
+		Layer const layer = my layers->at [ilayer];
 		if (ilayer > 1)
 			copyOutputsToInputs (my layers->at [ilayer - 1], layer);
 		layer -> v_spreadUp (activationType);
@@ -148,10 +171,10 @@ void Net_spreadUp (Net me, kLayer_activationType activationType) {
 void structRBMLayer :: v_sampleInput () {
 	for (integer inode = 1; inode <= our numberOfInputNodes; inode ++) {
 		if (our inputsAreBinary) {
-			double probability = our inputActivities [inode];
-			our inputActivities [inode] = (double) NUMrandomBernoulli (probability);
+			const double probability = our inputActivities [inode];
+			our inputActivities [inode] = double (NUMrandomBernoulli (probability));
 		} else {   // Gaussian
-			double excitation = our inputActivities [inode];
+			const double excitation = our inputActivities [inode];
 			our inputActivities [inode] = NUMrandomGauss (excitation, 1.0);
 		}
 	}
@@ -166,25 +189,22 @@ void Net_sampleOutput (Net me) {
 }
 
 static void copyInputsToOutputs (Layer me, Layer you) {
-	Melder_assert (my numberOfInputNodes == your numberOfOutputNodes);
-	for (integer inode = 1; inode <= my numberOfInputNodes; inode ++) {
-		your outputActivities [inode] = my inputActivities [inode];
-	}
+	your outputActivities.all() <<= my inputActivities.all();
 }
 
 void structRBMLayer :: v_spreadDown (kLayer_activationType activationType) {
 	for (integer inode = 1; inode <= our numberOfInputNodes; inode ++) {
 		PAIRWISE_SUM (longdouble, excitation, integer, our numberOfOutputNodes,
- 			double *p_weight = & our weights [inode] [0];
- 			double *p_outputActivity = & our outputActivities [0],
- 			( p_weight += 1, p_outputActivity += 1 ),
- 			(longdouble) *p_weight * (longdouble) *p_outputActivity
-		);
+ 			const double *p_weight = & our weights [inode] [1];
+ 			const double *p_outputActivity = & our outputActivities [1],
+ 			longdouble (*p_weight) * longdouble (*p_outputActivity),
+ 			( p_weight += 1, p_outputActivity += 1 )
+		)
 		excitation += our inputBiases [inode];
 		if (our inputsAreBinary) {
-			our inputActivities [inode] = logistic ((double) excitation);
+			our inputActivities [inode] = logistic (double (excitation));
 		} else {   // linear
-			our inputActivities [inode] = (double) excitation;
+			our inputActivities [inode] = double (excitation);
 		}
 	}
 	if (activationType == kLayer_activationType::STOCHASTIC)
@@ -193,7 +213,7 @@ void structRBMLayer :: v_spreadDown (kLayer_activationType activationType) {
 
 void Net_spreadDown (Net me, kLayer_activationType activationType) {
 	for (integer ilayer = my layers->size; ilayer > 0; ilayer --) {
-		Layer layer = my layers->at [ilayer];
+		Layer const layer = my layers->at [ilayer];
 		if (ilayer < my layers->size)
 			copyInputsToOutputs (my layers->at [ilayer + 1], layer);
 		layer -> v_spreadDown (activationType);
@@ -203,11 +223,11 @@ void Net_spreadDown (Net me, kLayer_activationType activationType) {
 void structRBMLayer :: v_spreadDown_reconstruction () {
 	for (integer inode = 1; inode <= our numberOfInputNodes; inode ++) {
 		PAIRWISE_SUM (longdouble, excitation, integer, our numberOfOutputNodes,
- 			double *p_weight = & our weights [inode] [0];
- 			double *p_outputActivity = & our outputActivities [0],
- 			( p_weight += 1, p_outputActivity += 1 ),
- 			(longdouble) *p_weight * (longdouble) *p_outputActivity
-		);
+ 			double *p_weight = & our weights [inode] [1];
+ 			double *p_outputActivity = & our outputActivities [1],
+ 			(longdouble) *p_weight * (longdouble) *p_outputActivity,
+ 			( p_weight += 1, p_outputActivity += 1 )
+		)
 		excitation += our inputBiases [inode];
 		if (our inputsAreBinary) {
 			our inputReconstruction [inode] = logistic ((double) excitation);
@@ -218,49 +238,66 @@ void structRBMLayer :: v_spreadDown_reconstruction () {
 }
 
 void Net_spreadDown_reconstruction (Net me) {
-	for (integer ilayer = my layers->size; ilayer > 0; ilayer --) {
+	for (integer ilayer = my layers->size; ilayer > 0; ilayer --)
 		my layers->at [ilayer] -> v_spreadDown_reconstruction ();
-	}
 }
 
 void structRBMLayer :: v_spreadUp_reconstruction () {
 	integer numberOfOutputNodes = our numberOfOutputNodes;
 	for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++) {
 		PAIRWISE_SUM (longdouble, excitation, integer, our numberOfInputNodes,
- 			double *p_inputActivity = & our inputReconstruction [0];
- 			double *p_weight = & our weights [1] [jnode] - numberOfOutputNodes,
- 			( p_inputActivity += 1, p_weight += numberOfOutputNodes ),
- 			(longdouble) *p_inputActivity * (longdouble) *p_weight
-		);
+ 			double *p_inputActivity = & our inputReconstruction [1];
+ 			double *p_weight = & our weights [1] [jnode],
+ 			(longdouble) *p_inputActivity * (longdouble) *p_weight,
+ 			( p_inputActivity += 1, p_weight += numberOfOutputNodes )
+		)
 		excitation += our outputBiases [jnode];
 		our outputReconstruction [jnode] = logistic ((double) excitation);
 	}
 }
 
 void Net_spreadUp_reconstruction (Net me) {
-	for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++) {
+	for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++)
 		my layers->at [ilayer] -> v_spreadUp_reconstruction ();
-	}
 }
 
 void structRBMLayer :: v_update (double learningRate) {
-	for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++) {
+	for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++)
 		our outputBiases [jnode] += learningRate * (our outputActivities [jnode] - our outputReconstruction [jnode]);
-	}
 	for (integer inode = 1; inode <= our numberOfInputNodes; inode ++) {
 		our inputBiases [inode] += learningRate * (our inputActivities [inode] - our inputReconstruction [inode]);
-		for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++) {
+		for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++)
 			our weights [inode] [jnode] += learningRate *
-				(our inputActivities [inode] * our outputActivities [jnode] -
-				 our inputReconstruction [inode] * our outputReconstruction [jnode]);
-		}
+					(our inputActivities [inode] * our outputActivities [jnode] -
+					 our inputReconstruction [inode] * our outputReconstruction [jnode]);
+	}
+}
+
+void structRBMLayer :: v_updateFirstPhase (double learningRate) {
+	for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++)
+		our outputBiases [jnode] += learningRate * our outputActivities [jnode];
+	for (integer inode = 1; inode <= our numberOfInputNodes; inode ++) {
+		our inputBiases [inode] += learningRate * our inputActivities [inode];
+		for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++)
+			our weights [inode] [jnode] += learningRate *
+					our inputActivities [inode] * our outputActivities [jnode];
+	}
+}
+
+void structRBMLayer :: v_updateSecondPhase (double learningRate) {
+	for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++)
+		our outputBiases [jnode] -= learningRate * our outputActivities [jnode];
+	for (integer inode = 1; inode <= our numberOfInputNodes; inode ++) {
+		our inputBiases [inode] -= learningRate * our inputActivities [inode];
+		for (integer jnode = 1; jnode <= our numberOfOutputNodes; jnode ++)
+			our weights [inode] [jnode] -= learningRate *
+					our inputActivities [inode] * our outputActivities [jnode];
 	}
 }
 
 void Net_update (Net me, double learningRate) {
-	for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++) {
+	for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++)
 		my layers->at [ilayer] -> v_update (learningRate);
-	}
 }
 
 static void Layer_PatternList_applyToInput (Layer me, PatternList thee, integer rowNumber) {
@@ -331,6 +368,106 @@ void Net_PatternList_learnByLayer (Net me, PatternList thee, double learningRate
 	}
 }
 
+void Net_PatternList_learn_twoPhases_old (Net me, PatternList thee, double learningRate) {
+	try {
+		for (integer ipattern = 1; ipattern <= thy ny; ipattern ++) {
+			Net_PatternList_applyToInput (me, thee, ipattern);
+			Net_spreadUp (me, kLayer_activationType::STOCHASTIC);   // no contrasts will develop if this is deterministic
+			for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++) {
+				Layer layer = my layers->at [ilayer];
+				layer -> v_updateFirstPhase (learningRate);
+			}
+			#if 0
+			for (integer isweep = 1; isweep <= 1; isweep ++) {
+				Net_spreadDown (me, kLayer_activationType::DETERMINISTIC);
+				Net_spreadUp (me, kLayer_activationType::DETERMINISTIC);
+			}
+			#else
+			for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++) {
+				Layer layer = my layers->at [ilayer];
+				for (integer isweep = 1; isweep <= 1; isweep ++) {
+					layer -> v_spreadDown (kLayer_activationType::DETERMINISTIC);
+					layer -> v_spreadUp (kLayer_activationType::DETERMINISTIC);
+				}
+			}
+			#endif
+			for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++) {
+				Layer layer = my layers->at [ilayer];
+				layer -> v_updateSecondPhase (learningRate);
+			}
+		}
+	} catch (MelderError) {
+		Melder_throw (me, U" & ", thee, U": not learned.");
+	}
+}
+void Net_PatternList_learn_twoPhases_old2 (Net me, PatternList thee, double learningRate) {
+	try {
+		for (integer ipattern = 1; ipattern <= thy ny; ipattern ++) {
+			Net_PatternList_applyToInput (me, thee, ipattern);
+			Net_spreadUp (me, kLayer_activationType::STOCHASTIC);   // no contrasts will develop if this is deterministic
+			for (integer ilayer = 1; ilayer <= my layers->size; ilayer ++) {
+				Layer layer = my layers->at [ilayer];
+				layer -> v_updateFirstPhase (learningRate);
+				for (integer isweep = 1; isweep <= 1; isweep ++) {
+					layer -> v_spreadDown (kLayer_activationType::DETERMINISTIC);
+					layer -> v_spreadUp (kLayer_activationType::DETERMINISTIC);
+				}
+				layer -> v_updateSecondPhase (learningRate);
+			}
+		}
+	} catch (MelderError) {
+		Melder_throw (me, U" & ", thee, U": not learned.");
+	}
+}
+
+static void combineOutputsAndInputs (Layer me_any, Layer you_any) {
+	RBMLayer me = static_cast <RBMLayer> (me_any);
+	RBMLayer you = static_cast <RBMLayer> (you_any);
+	integer numberOfNodes = my numberOfOutputNodes;
+	Melder_assert (your numberOfInputNodes == numberOfNodes);
+	for (integer inode = 1; inode <= my numberOfOutputNodes; inode ++) {
+		PAIRWISE_SUM (longdouble, myExcitation, integer, my numberOfInputNodes,
+ 			double *p_inputActivity = & my inputActivities [1];
+ 			double *p_weight = & my weights [1] [inode],
+ 			(longdouble) *p_inputActivity * (longdouble) *p_weight,
+ 			( p_inputActivity += 1, p_weight += numberOfNodes )
+		)
+		PAIRWISE_SUM (longdouble, yourExcitation, integer, your numberOfOutputNodes,
+ 			double *p_weight = & your weights [inode] [1];
+ 			double *p_outputActivity = & your outputActivities [1],
+ 			(longdouble) *p_weight * (longdouble) *p_outputActivity,
+ 			( p_weight += 1, p_outputActivity += 1 )
+		)
+		double totalExcitation = 0.5 * double (myExcitation + yourExcitation);
+		totalExcitation += my outputBiases [inode];
+		my outputActivities [inode] = your inputActivities [inode] = logistic (totalExcitation);
+	}
+}
+
+void Net_PatternList_learn_twoPhases (Net me, PatternList thee, double learningRate) {
+	try {
+		for (integer ipattern = 1; ipattern <= thy ny; ipattern ++) {
+			Net_PatternList_applyToInput (me, thee, ipattern);
+			Layer layer1 = my layers->at [1];
+			Layer layer2 = my layers->at [2];
+			Net_spreadUp (me, kLayer_activationType::STOCHASTIC);   // no contrasts will develop if this is deterministic
+			layer1 -> v_updateFirstPhase (learningRate);
+			layer2 -> v_updateFirstPhase (learningRate);
+			for (integer isweep = 1; isweep <= 10; isweep ++) {
+				layer1 -> v_spreadDown (kLayer_activationType::DETERMINISTIC);
+				layer2 -> v_spreadUp (kLayer_activationType::DETERMINISTIC);
+				//layer1 -> v_spreadUp (kLayer_activationType::DETERMINISTIC);
+				//layer2 -> v_spreadDown (kLayer_activationType::DETERMINISTIC);
+				combineOutputsAndInputs (layer1, layer2);
+			}
+			layer1 -> v_updateSecondPhase (learningRate);
+			layer2 -> v_updateSecondPhase (learningRate);
+		}
+	} catch (MelderError) {
+		Melder_throw (me, U" & ", thee, U": not learned.");
+	}
+}
+
 autoActivationList Net_PatternList_to_ActivationList (Net me, PatternList thee, kLayer_activationType activationType) {
 	try {
 		Layer outputLayer = my layers->at [my layers->size];
@@ -338,7 +475,7 @@ autoActivationList Net_PatternList_to_ActivationList (Net me, PatternList thee,
 		for (integer ipattern = 1; ipattern <= thy ny; ipattern ++) {
 			Net_PatternList_applyToInput (me, thee, ipattern);
 			Net_spreadUp (me, activationType);
-			NUMvector_copyElements <double> (outputLayer -> outputActivities, activations -> z [ipattern], 1, outputLayer -> numberOfOutputNodes);
+			activations -> z.row (ipattern) <<= outputLayer -> outputActivities;
 		}
 		return activations;
 	} catch (MelderError) {
@@ -349,7 +486,7 @@ autoActivationList Net_PatternList_to_ActivationList (Net me, PatternList thee,
 static autoMatrix Layer_extractInputActivities (Layer me) {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, my numberOfInputNodes);
-		NUMvector_copyElements <double> (my inputActivities, thy z [1], 1, my numberOfInputNodes);
+		thy z.row (1) <<= my inputActivities;
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": input activities not extracted.");
@@ -363,7 +500,7 @@ autoMatrix Net_extractInputActivities (Net me) {
 static autoMatrix Layer_extractOutputActivities (Layer me) {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, my numberOfOutputNodes);
-		NUMvector_copyElements <double> (my outputActivities, thy z [1], 1, my numberOfOutputNodes);
+		thy z.row (1) <<= my outputActivities;
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": output activities not extracted.");
@@ -377,7 +514,7 @@ autoMatrix Net_extractOutputActivities (Net me) {
 autoMatrix structRBMLayer :: v_extractInputReconstruction () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfInputNodes);
-		NUMvector_copyElements <double> (our inputReconstruction, thy z [1], 1, our numberOfInputNodes);
+		thy z.row (1) <<= our inputReconstruction;
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": input reconstruction not extracted.");
@@ -391,7 +528,7 @@ autoMatrix Net_extractInputReconstruction (Net me) {
 autoMatrix structRBMLayer :: v_extractOutputReconstruction () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfOutputNodes);
-		NUMvector_copyElements <double> (our outputReconstruction, thy z [1], 1, our numberOfOutputNodes);
+		thy z.row (1) <<= our outputReconstruction;
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": output reconstruction not extracted.");
@@ -405,7 +542,7 @@ autoMatrix Net_extractOutputReconstruction (Net me) {
 autoMatrix structRBMLayer :: v_extractInputBiases () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfInputNodes);
-		NUMvector_copyElements <double> (our inputBiases, thy z [1], 1, our numberOfInputNodes);
+		thy z.row (1) <<= our inputBiases;
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": input biases not extracted.");
@@ -431,7 +568,7 @@ autoMatrix Net_extractInputBiases (Net me, integer layerNumber) {
 autoMatrix structRBMLayer :: v_extractOutputBiases () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfOutputNodes);
-		NUMvector_copyElements <double> (our outputBiases, thy z [1], 1, our numberOfOutputNodes);
+		thy z.row (1) <<= our outputBiases;
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": input biases not extracted.");
@@ -450,7 +587,7 @@ autoMatrix Net_extractOutputBiases (Net me, integer layerNumber) {
 autoMatrix structRBMLayer :: v_extractWeights () {
 	try {
 		autoMatrix thee = Matrix_createSimple (our numberOfInputNodes, our numberOfOutputNodes);
-		NUMmatrix_copyElements <double> (our weights, thy z, 1, our numberOfInputNodes, 1, our numberOfOutputNodes);
+		thy z.all() <<= our weights.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": weights not extracted.");
@@ -466,12 +603,12 @@ autoMatrix Net_extractWeights (Net me, integer layerNumber) {
 	}
 }
 
-autonummat structRBMLayer :: v_getWeights_nummat () {
-	return copy_nummat ({ our weights, our numberOfInputNodes, our numberOfOutputNodes });
+autoMAT structRBMLayer :: v_getWeights () {
+	return newMATcopy (our weights.get());
 }
 
-autonummat Net_getWeights_nummat (Net me, integer layerNumber) {
-	return my layers->at [layerNumber] -> v_getWeights_nummat ();
+autoMAT Net_getWeights (Net me, integer layerNumber) {
+	return my layers->at [layerNumber] -> v_getWeights ();
 }
 
 /* End of file Net.cpp */
