@@ -1,6 +1,6 @@
 /* Network_def.h
  *
- * Copyright (C) 2009-2017 Paul Boersma
+ * Copyright (C) 2009-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,14 +39,14 @@ oo_DEFINE_STRUCT (NetworkConnection)
 	oo_INTEGER (nodeFrom)
 	oo_INTEGER (nodeTo)
 	oo_DOUBLE (weight)
+
 	oo_FROM (1)
 		oo_DOUBLE (plasticity)
 	oo_ENDFROM
-
 	#if oo_READING
-		if (formatVersion < 1) {
-			plasticity = 1.0;
-		}
+		oo_VERSION_UNTIL (1)
+			our plasticity = 1.0;
+		oo_VERSION_END
 	#endif
 
 oo_END_STRUCT (NetworkConnection)
@@ -80,15 +80,15 @@ oo_DEFINE_CLASS (Network, Daata)
 	oo_DOUBLE (weightLeak)
 
 	#if oo_READING
-		if (formatVersion < 6) {
-			if (formatVersion < 5) {
-				if (our learningRate != 0.0) our weightLeak /= our learningRate;
-				if (our dummyWeightUpdateRule == 1) our instar = 1.0, our outstar = 0.0;
-				if (our dummyWeightUpdateRule == 2) our instar = 0.0, our outstar = 1.0;
-				if (our dummyWeightUpdateRule == 3) our instar = 0.5, our outstar = 0.5;
-			}
+		oo_VERSION_UNTIL (5)
+			if (our learningRate != 0.0) our weightLeak /= our learningRate;
+			if (our dummyWeightUpdateRule == 1) our instar = 1.0, our outstar = 0.0;
+			if (our dummyWeightUpdateRule == 2) our instar = 0.0, our outstar = 1.0;
+			if (our dummyWeightUpdateRule == 3) our instar = 0.5, our outstar = 0.5;
+		oo_VERSION_END
+		oo_VERSION_UNTIL (6)
 			our activityLeak = - our activityLeak;   // convert self-excitation to activity leak
-		}
+		oo_VERSION_END
 	#endif
 
 	oo_DOUBLE (xmin)
