@@ -1,6 +1,6 @@
 /* OTGrammarEditor.cpp
  *
- * Copyright (C) 1997-2005,2007-2017 Paul Boersma
+ * Copyright (C) 1997-2005,2007-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@ static void menu_cb_editConstraint (OTGrammarEditor me, EDITOR_ARGS_FORM) {
 		if (my selected < 1 || my selected > ot -> numberOfConstraints)
 			Melder_throw (U"Select a constraint first.");
 		constraint = & ot -> constraints [ot -> index [my selected]];
-		SET_STRING (constraintLabel, constraint -> name)
+		SET_STRING (constraintLabel, constraint -> name.get())
 		SET_REAL (rankingValue, constraint -> ranking)
 		SET_REAL (disharmony, constraint -> disharmony)
 		SET_REAL (plasticity, constraint -> plasticity)
@@ -96,7 +96,8 @@ static void menu_cb_learnOne (OTGrammarEditor me, EDITOR_ARGS_FORM) {
 		LABEL (U"Adult surface form:")
 		SENTENCE (outputString, U"Output string", U"")
 		REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-		OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
+		OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+				U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 		REAL (plasticity, U"Plasticity", U"0.1")
 		REAL (relativePlasticitySpreading, U"Rel. plasticity spreading", U"0.1")
 		BOOLEAN (honourLocalRankings, U"Honour local rankings", true)
@@ -117,7 +118,8 @@ static void menu_cb_learnOneFromPartialOutput (OTGrammarEditor me, EDITOR_ARGS_F
 		LABEL (U"Partial adult surface form (e.g. overt form):")
 		SENTENCE (partialOutput, U"Partial output", U"")
 		REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-		OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
+		OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+				U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 		REAL (plasticity, U"Plasticity", U"0.1")
 		REAL (relativePlasticitySpreading, U"Rel. plasticity spreading", U"0.1")
 		BOOLEAN (honourLocalRankings, U"Honour local rankings", true)
@@ -141,7 +143,7 @@ static void menu_cb_removeConstraint (OTGrammarEditor me, EDITOR_ARGS_DIRECT) {
 		Melder_throw (U"Select a constraint first.");
 	constraint = & ot -> constraints [ot -> index [my selected]];
 	Editor_save (me, U"Remove constraint");
-	OTGrammar_removeConstraint (ot, constraint -> name);
+	OTGrammar_removeConstraint (ot, constraint -> name.get());
 	Graphics_updateWs (my graphics.get());
 	Editor_broadcastDataChanged (me);
 }
@@ -185,7 +187,7 @@ void structOTGrammarEditor :: v_createHelpMenuItems (EditorMenu menu) {
 }
 
 static OTGrammar drawTableau_ot;
-static const char32 *drawTableau_input;
+static conststring32 drawTableau_input;
 static bool drawTableau_constraintsAreDrawnVertically;
 static void drawTableau (Graphics g) {
 	OTGrammar_drawTableau (drawTableau_ot, g, drawTableau_constraintsAreDrawnVertically, drawTableau_input);
@@ -210,7 +212,7 @@ void structOTGrammarEditor :: v_draw () {
 			Melder_sprint (text,1000,
 				U"\t", icons == selected ? U"♠︎ " : U"   ",
 				U"@@", icons,
-				U"|", constraint -> name,
+				U"|", constraint -> name.get(),
 				U"@\t      ", Melder_fixed (constraint -> ranking, 3),
 				U"\t      ", Melder_fixed (constraint -> disharmony, 3),
 				U"\t      ", Melder_fixed (constraint -> plasticity, 6),
@@ -219,7 +221,7 @@ void structOTGrammarEditor :: v_draw () {
 			Melder_sprint (text,1000,
 				U"\t", icons == selected ? U"♠︎ " : U"   ",
 				U"@@", icons,
-				U"|", constraint -> name,
+				U"|", constraint -> name.get(),
 				U"@\t      ", Melder_fixed (constraint -> ranking, 3),
 				U"\t      ", Melder_fixed (constraint -> disharmony, 3),
 				U"\t      ", Melder_fixed (constraint -> plasticity, 6));
@@ -232,20 +234,20 @@ void structOTGrammarEditor :: v_draw () {
 		double rowHeight = 0.25;
 		double tableauHeight = rowHeight * (tableau -> numberOfCandidates + 2);
 		drawTableau_ot = ot;
-		drawTableau_input = tableau -> input;
+		drawTableau_input = tableau -> input.get();
 		drawTableau_constraintsAreDrawnVertically = d_constraintsAreDrawnVertically;
 		HyperPage_picture (this, 20, tableauHeight, drawTableau);
 	}
 	Graphics_setAtSignIsLink (graphics.get(), true);
 }
 
-int structOTGrammarEditor :: v_goToPage (const char32 *title) {
+int structOTGrammarEditor :: v_goToPage (conststring32 title) {
 	if (! title) return 1;
 	selected = Melder_atoi (title);
 	return 1;
 }
 
-autoOTGrammarEditor OTGrammarEditor_create (const char32 *title, OTGrammar ot) {
+autoOTGrammarEditor OTGrammarEditor_create (conststring32 title, OTGrammar ot) {
 	try {
 		autoOTGrammarEditor me = Thing_new (OTGrammarEditor);
 		my data = ot;
