@@ -1,6 +1,6 @@
 /* OTGrammar_def.h
  *
- * Copyright (C) 1997-2011,2015-2017 Paul Boersma
+ * Copyright (C) 1997-2011,2015-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,19 +23,19 @@ oo_DEFINE_STRUCT (OTGrammarConstraint)
 	oo_STRING (name)
 	oo_DOUBLE (ranking)
 	oo_DOUBLE (disharmony)
+
 	oo_FROM (2)
 		oo_DOUBLE (plasticity)
 	oo_ENDFROM
-
 	#if oo_READING
-		if (formatVersion < 2) {
-			plasticity = 1.0;
-		}
+		oo_VERSION_UNTIL (2)
+			our plasticity = 1.0;
+		oo_VERSION_END
 	#endif
 
-	#if !oo_READING && !oo_WRITING
-		oo_INT16 (tiedToTheLeft)
-		oo_INT16 (tiedToTheRight)
+	#if ! oo_READING && ! oo_WRITING
+		oo_BOOLEAN (tiedToTheLeft)
+		oo_BOOLEAN (tiedToTheRight)
 	#endif
 
 oo_END_STRUCT (OTGrammarConstraint)
@@ -57,9 +57,9 @@ oo_DEFINE_STRUCT (OTGrammarCandidate)
 
 	oo_STRING (output)
 	oo_INTEGER (numberOfConstraints)
-	oo_INT_VECTOR (marks, numberOfConstraints)
+	oo_INTVEC16 (marks, numberOfConstraints)
 
-	#if !oo_READING && !oo_WRITING
+	#if ! oo_READING && ! oo_WRITING
 		oo_DOUBLE (harmony)
 		oo_DOUBLE (probability)
 		oo_INTEGER (numberOfPotentialPartialOutputsMatching)
@@ -92,7 +92,7 @@ oo_DEFINE_CLASS (OTGrammar, Daata)
 	oo_ENDFROM
 	oo_INTEGER (numberOfConstraints)
 	oo_STRUCT_VECTOR (OTGrammarConstraint, constraints, numberOfConstraints)
-	oo_INTEGER_VECTOR (index, numberOfConstraints)   // not read or written in text files
+	oo_INTVEC (index, numberOfConstraints)   // not read or written in text files
 	oo_INTEGER (numberOfFixedRankings)
 	oo_STRUCT_VECTOR (OTGrammarFixedRanking, fixedRankings, numberOfFixedRankings)
 	oo_INTEGER (numberOfTableaus)
