@@ -1,6 +1,6 @@
 /* OTMultiEditor.cpp
  *
- * Copyright (C) 2005-2017 Paul Boersma
+ * Copyright (C) 2005-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ static void menu_cb_editRanking (OTMultiEditor me, EDITOR_ARGS_FORM) {
 		if (my selectedConstraint < 1 || my selectedConstraint > grammar -> numberOfConstraints)
 			Melder_throw (U"Select a constraint first.");
 		constraint = & grammar -> constraints [grammar -> index [my selectedConstraint]];
-		SET_STRING (constraintLabel, constraint -> name)
+		SET_STRING (constraintLabel, constraint -> name.get())
 		SET_REAL (rankingValue, constraint -> ranking)
 		SET_REAL (disharmony, constraint -> disharmony)
 	EDITOR_DO
@@ -83,7 +83,8 @@ static void menu_cb_editRanking (OTMultiEditor me, EDITOR_ARGS_FORM) {
 
 static void menu_cb_learnOne (OTMultiEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Learn one", U"OTGrammar: Learn one...")
-		OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
+		OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+				U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 		OPTIONMENU (direction, U"Direction", 3)
 			OPTION (U"forward")
 			OPTION (U"backward")
@@ -93,11 +94,9 @@ static void menu_cb_learnOne (OTMultiEditor me, EDITOR_ARGS_FORM) {
 	EDITOR_OK
 	EDITOR_DO
 		Editor_save (me, U"Learn one");
-		Melder_free (my form1);
-		Melder_free (my form2);
 		my form1 = GuiText_getString (my form1Text);
 		my form2 = GuiText_getString (my form2Text);
-		OTMulti_learnOne ((OTMulti) my data, my form1, my form2,
+		OTMulti_learnOne ((OTMulti) my data, my form1.get(), my form2.get(),
 			updateRule, direction,
 			plasticity, relativePlasticitySpreading);
 		Graphics_updateWs (my graphics.get());
@@ -111,7 +110,7 @@ static void menu_cb_removeConstraint (OTMultiEditor me, EDITOR_ARGS_DIRECT) {
 		Melder_throw (U"Select a constraint first.");
 	OTConstraint constraint = & grammar -> constraints [grammar -> index [my selectedConstraint]];
 	Editor_save (me, U"Remove constraint");
-	OTMulti_removeConstraint (grammar, constraint -> name);
+	OTMulti_removeConstraint (grammar, constraint -> name.get());
 	Graphics_updateWs (my graphics.get());
 	Editor_broadcastDataChanged (me);
 }
@@ -133,8 +132,6 @@ static void menu_cb_OTLearningTutorial (OTMultiEditor, EDITOR_ARGS_DIRECT) {
 }
 
 static void do_limit (OTMultiEditor me) {
-	Melder_free (my form1);
-	Melder_free (my form2);
 	my form1 = GuiText_getString (my form1Text);
 	my form2 = GuiText_getString (my form2Text);
 	Graphics_updateWs (my graphics.get());
@@ -179,7 +176,7 @@ void structOTMultiEditor :: v_createHelpMenuItems (EditorMenu menu) {
 }
 
 static OTMulti drawTableau_grammar;
-static const char32 *drawTableau_form1, *drawTableau_form2;
+static conststring32 drawTableau_form1, drawTableau_form2;
 static bool drawTableau_constraintsAreDrawnVertically;
 static void drawTableau (Graphics g) {
 	OTMulti_drawTableau (drawTableau_grammar, g, drawTableau_form1, drawTableau_form2, drawTableau_constraintsAreDrawnVertically, true);
@@ -194,7 +191,7 @@ void structOTMultiEditor :: v_draw () {
 	for (integer icons = 1; icons <= grammar -> numberOfConstraints; icons ++) {
 		OTConstraint constraint = & grammar -> constraints [grammar -> index [icons]];
 		MelderString_copy (& buffer, U"\t", ( icons == selectedConstraint ? U"♠︎ " : U"   " ), U"@@", icons,
-			U"|", constraint -> name, U"@\t      ", Melder_fixed (constraint -> ranking, 3),
+			U"|", constraint -> name.get(), U"@\t      ", Melder_fixed (constraint -> ranking, 3),
 			U"\t      ", Melder_fixed (constraint -> disharmony, 3),
 			U"\t      ", Melder_fixed (constraint -> plasticity, 6)
 		);
@@ -203,24 +200,24 @@ void structOTMultiEditor :: v_draw () {
 	Graphics_setAtSignIsLink (graphics.get(), false);
 	drawTableau_grammar = grammar;
 	for (integer icand = 1; icand <= grammar -> numberOfCandidates; icand ++) {
-		if (OTMulti_candidateMatches (grammar, icand, form1, form2)) {
+		if (OTMulti_candidateMatches (grammar, icand, our form1.get(), our form2.get())) {
 			tableauHeight += rowHeight;
 		}
 	}
-	drawTableau_form1 = form1;
-	drawTableau_form2 = form2;
+	drawTableau_form1 = our form1.get();   // BUG: dangle
+	drawTableau_form2 = our form2.get();
 	drawTableau_constraintsAreDrawnVertically = d_constraintsAreDrawnVertically;
 	HyperPage_picture (this, 20, tableauHeight, drawTableau);
 	Graphics_setAtSignIsLink (graphics.get(), true);
 }
 
-int structOTMultiEditor :: v_goToPage (const char32 *title) {
+int structOTMultiEditor :: v_goToPage (conststring32 title) {
 	if (! title) return 1;
 	selectedConstraint = Melder_atoi (title);
 	return 1;
 }
 
-autoOTMultiEditor OTMultiEditor_create (const char32 *title, OTMulti grammar) {
+autoOTMultiEditor OTMultiEditor_create (conststring32 title, OTMulti grammar) {
 	try {
 		autoOTMultiEditor me = Thing_new (OTMultiEditor);
 		my data = grammar;
