@@ -36,7 +36,8 @@
 #define UiForm_addNetworkFields  \
 	LABEL (U"Activity spreading settings:") \
 	REAL (spreadingRate, U"Spreading rate", U"0.01") \
-	OPTIONMENU_ENUM (activityClippingRule, U"Activity clipping rule", kNetwork_activityClippingRule, DEFAULT) \
+	OPTIONMENU_ENUM (kNetwork_activityClippingRule, activityClippingRule, \
+			U"Activity clipping rule", kNetwork_activityClippingRule::DEFAULT) \
 	REAL (minimumActivity, U"left Activity range", U"0.0") \
 	REAL (maximumActivity, U"right Activity range", U"1.0") \
 	REAL (activityLeak, U"Activity leak", U"1.0") \
@@ -57,10 +58,10 @@ FORM (NEW1_Create_empty_Network, U"Create empty Network", nullptr) {
 	OK
 DO
 	CREATE_ONE
-		autoNetwork result = Network_create (spreadingRate,
-			(kNetwork_activityClippingRule) activityClippingRule,
+		autoNetwork result = Network_create (spreadingRate, activityClippingRule,
 			minimumActivity, maximumActivity, activityLeak, learningRate, minimumWeight, maximumWeight, weightLeak,
-			fromX, toX, fromY, toY, 0, 0);
+			fromX, toX, fromY, toY, 0, 0
+		);
 	CREATE_ONE_END (name)
 }
 
@@ -76,10 +77,10 @@ FORM (NEW1_Create_rectangular_Network, U"Create rectangular Network", nullptr) {
 	OK
 DO
 	CREATE_ONE
-		autoNetwork result = Network_create_rectangle (spreadingRate,
-			(kNetwork_activityClippingRule) activityClippingRule,
+		autoNetwork result = Network_create_rectangle (spreadingRate, activityClippingRule,
 			minimumActivity, maximumActivity, activityLeak, learningRate, minimumWeight, maximumWeight, weightLeak,
-			numberOfRows, numberOfColumns, bottomRowClamped, minimumInitialWeight, maximumInitialWeight);
+			numberOfRows, numberOfColumns, bottomRowClamped, minimumInitialWeight, maximumInitialWeight
+		);
 	CREATE_ONE_END (U"rectangle_", numberOfRows, U"_", numberOfColumns)
 }
 
@@ -98,7 +99,8 @@ DO
 		autoNetwork result = Network_create_rectangle_vertical (spreadingRate,
 			(kNetwork_activityClippingRule) activityClippingRule,
 			minimumActivity, maximumActivity, activityLeak, learningRate, minimumWeight, maximumWeight, weightLeak,
-			numberOfRows, numberOfColumns, bottomRowClamped, minimumInitialWeight, maximumInitialWeight);
+			numberOfRows, numberOfColumns, bottomRowClamped, minimumInitialWeight, maximumInitialWeight
+		);
 	CREATE_ONE_END (U"rectangle_", numberOfRows, U"_", numberOfColumns)
 }
 
@@ -131,7 +133,8 @@ DO
 	INFO_ONE (Network)
 		Network_listNodes (me, fromNodeNumber, toNodeNumber,
 			includeNodeNumbers, includeX, includeY, positionDecimals,
-			includeClamped, includeActivity, includeExcitation, activityDecimals);
+			includeClamped, includeActivity, includeExcitation, activityDecimals
+		);
 	INFO_ONE_END
 }
 
@@ -151,8 +154,9 @@ DO
 	CONVERT_EACH (Network)
 		autoTable result = Network_nodes_downto_Table (me, fromNodeNumber, toNodeNumber,
 			includeNodeNumbers, includeX, includeY, positionDecimals,
-			includeClamped, includeActivity, includeExcitation, activityDecimals);
-	CONVERT_EACH_END (my name)
+			includeClamped, includeActivity, includeExcitation, activityDecimals
+		);
+	CONVERT_EACH_END (my name.get())
 }
 
 // MARK: Query
@@ -235,7 +239,8 @@ DO
 }
 
 FORM (MODIFY_Network_setActivityClippingRule, U"Network: Set activity clipping rule", nullptr) {
-	RADIO_ENUM (activityClippingRule, U"Activity clipping rule", kNetwork_activityClippingRule, DEFAULT)
+	RADIO_ENUM (kNetwork_activityClippingRule, activityClippingRule,
+			U"Activity clipping rule", kNetwork_activityClippingRule::DEFAULT)
 	OK
 DO
 	MODIFY_EACH (Network)
@@ -360,8 +365,10 @@ DIRECT (NEW1_Create_NPA_distribution) {
 }
 
 FORM (NEW1_Create_tongue_root_grammar, U"Create tongue-root grammar", U"Create tongue-root grammar...") {
-	RADIO_ENUM (constraintSet, U"Constraint set", kOTGrammar_createTongueRootGrammar_constraintSet, DEFAULT)
-	RADIO_ENUM (ranking, U"Ranking", kOTGrammar_createTongueRootGrammar_ranking, DEFAULT)
+	RADIO_ENUM (kOTGrammar_createTongueRootGrammar_constraintSet, constraintSet,
+			U"Constraint set", kOTGrammar_createTongueRootGrammar_constraintSet::DEFAULT)
+	RADIO_ENUM (kOTGrammar_createTongueRootGrammar_ranking, ranking,
+			U"Ranking", kOTGrammar_createTongueRootGrammar_ranking::DEFAULT)
 	OK
 DO
 	CREATE_ONE
@@ -370,7 +377,8 @@ DO
 }
 
 FORM (NEW1_Create_metrics_grammar, U"Create metrics grammar", nullptr) {
-	OPTIONMENU_ENUM (initialRanking, U"Initial ranking", kOTGrammar_createMetricsGrammar_initialRanking, DEFAULT)
+	OPTIONMENU_ENUM (kOTGrammar_createMetricsGrammar_initialRanking, initialRanking,
+			U"Initial ranking", kOTGrammar_createMetricsGrammar_initialRanking::DEFAULT)
 	OPTIONMENU (trochaicityConstraint, U"Trochaicity constraint", 1)
 		OPTION (U"FtNonfinal")
 		OPTION (U"Trochaic")
@@ -453,7 +461,7 @@ DO
 	STRING_ONE (OTGrammar)
 		if (constraintNumber > my numberOfConstraints)
 			Melder_throw (U"The specified constraint number should not exceed the number of constraints.");
-		const char32 *result = my constraints [constraintNumber]. name;
+		const conststring32 result = my constraints [constraintNumber]. name.get();
 	STRING_ONE_END
 }
 
@@ -492,7 +500,7 @@ DO
 	STRING_ONE (OTGrammar)
 		if (tableauNumber > my numberOfTableaus)
 			Melder_throw (U"The specified tableau number should not exceed the number of tableaus.");
-		const char32 *result = my tableaus [tableauNumber]. input;
+		const conststring32 result = my tableaus [tableauNumber]. input.get();
 	STRING_ONE_END
 }
 
@@ -518,7 +526,7 @@ DO
 		OTGrammarTableau tableau = & my tableaus [tableauNumber];
 		if (candidateNumber > tableau -> numberOfCandidates)
 			Melder_throw (U"The specified candidate should not exceed the number of candidates.");
-		const char32 *result = tableau -> candidates [candidateNumber]. output;
+		const conststring32 result = tableau -> candidates [candidateNumber]. output.get();
 	STRING_ONE_END
 }
 
@@ -619,8 +627,8 @@ DO
 	FIND_ONE (OTGrammar)
 		integer bestInput, bestOutput;
 		OTGrammar_getInterpretiveParse (me, partialOutput, & bestInput, & bestOutput);
-		Melder_information (U"Best input = ", bestInput, U": ", my tableaus [bestInput]. input,
-			U"\nBest output = ", bestOutput, U": ", my tableaus [bestInput]. candidates [bestOutput]. output);
+		Melder_information (U"Best input = ", bestInput, U": ", my tableaus [bestInput]. input.get(),
+			U"\nBest output = ", bestOutput, U": ", my tableaus [bestInput]. candidates [bestOutput]. output.get());
 	END
 }
 
@@ -650,20 +658,20 @@ FORM (NEW_OTGrammar_generateInputs, U"Generate inputs", U"OTGrammar: Generate in
 DO
 	CONVERT_EACH (OTGrammar)
 		autoStrings result = OTGrammar_generateInputs (me, numberOfTrials);
-	CONVERT_EACH_END (my name, U"_in")
+	CONVERT_EACH_END (my name.get(), U"_in")
 }
 
 DIRECT (NEW_OTGrammar_getInputs) {
 	CONVERT_EACH (OTGrammar)
 		autoStrings result = OTGrammar_getInputs (me);
-	CONVERT_EACH_END (my name, U"_in")
+	CONVERT_EACH_END (my name.get(), U"_in")
 }
 
 DIRECT (NEW_MODIFY_OTGrammar_measureTypology) {
 	LOOP try {
 		iam (OTGrammar);
 		autoDistributions thee = OTGrammar_measureTypology_WEAK (me);
-		praat_new (thee.move(), my name, U"_out");
+		praat_new (std::move (thee), my name.get(), U"_out");
 		praat_dataChanged (me);
 	} catch (MelderError) {
 		praat_dataChanged (OBJECT);
@@ -688,9 +696,8 @@ FORM (STRING_MODIFY_OTGrammar_inputToOutput, U"OTGrammar: Input to output", U"OT
 	OK
 DO
 	FIND_ONE (OTGrammar)
-		char32 output [100];
-		OTGrammar_inputToOutput (me, inputForm, output, evaluationNoise);
-		Melder_information (output);
+		autostring32 output = OTGrammar_inputToOutput (me, inputForm, evaluationNoise);
+		Melder_information (output.get());
 		praat_dataChanged (me);
 	END
 }
@@ -703,7 +710,7 @@ FORM (NEW1_MODIFY_OTGrammar_inputToOutputs, U"OTGrammar: Input to outputs", U"OT
 DO
 	FIND_ONE (OTGrammar)
 		autoStrings thee = OTGrammar_inputToOutputs (me, inputForm, trials, evaluationNoise);
-		praat_new (thee.move(), my name, U"_out");
+		praat_new (thee.move(), my name.get(), U"_out");
 		praat_dataChanged (me);
 	END
 }
@@ -717,7 +724,7 @@ DO
 		iam (OTGrammar);
 		try {
 			autoDistributions thee = OTGrammar_to_Distribution (me, trialsPerInput, evaluationNoise);
-			praat_new (thee.move(), my name, U"_out");
+			praat_new (thee.move(), my name.get(), U"_out");
 			praat_dataChanged (me);
 		} catch (MelderError) {
 			praat_dataChanged (me);
@@ -734,7 +741,7 @@ DO
 	LOOP try {
 		iam (OTGrammar);
 		autoPairDistribution thee = OTGrammar_to_PairDistribution (me, trialsPerInput, evaluationNoise);
-		praat_new (thee.move(), my name, U"_out");
+		praat_new (thee.move(), my name.get(), U"_out");
 		praat_dataChanged (me);
 	} catch (MelderError) {
 		praat_dataChanged (OBJECT);
@@ -788,15 +795,15 @@ FORM (MODIFY_OTGrammar_learnOne, U"OTGrammar: Learn one", U"OTGrammar: Learn one
 	SENTENCE (inputString, U"Input string", U"")
 	SENTENCE (outputString, U"Output string", U"")
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (plasticity, U"Plasticity", U"0.1")
 	REAL (relativePlasticitySpreading, U"Rel. plasticity spreading", U"0.1")
 	BOOLEAN (honourLocalRankings, U"Honour local rankings", 1)
 	OK
 DO
 	MODIFY_EACH_WEAK (OTGrammar)
-		OTGrammar_learnOne (me, inputString, outputString, evaluationNoise,
-			(kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+		OTGrammar_learnOne (me, inputString, outputString, evaluationNoise, updateRule, honourLocalRankings,
 			plasticity, relativePlasticitySpreading, true, true, nullptr);
 	MODIFY_EACH_WEAK_END
 }
@@ -805,7 +812,8 @@ FORM (MODIFY_OTGrammar_learnOneFromPartialOutput, U"OTGrammar: Learn one from pa
 	LABEL (U"Partial adult surface form (e.g. overt form):")
 	SENTENCE (partialOutput, U"Partial output", U"")
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (plasticity, U"Plasticity", U"0.1")
 	REAL (relativePlasticitySpreading, U"Rel. plasticity spreading", U"0.1")
 	BOOLEAN (honourLocalRankings, U"Honour local rankings", 1)
@@ -813,8 +821,7 @@ FORM (MODIFY_OTGrammar_learnOneFromPartialOutput, U"OTGrammar: Learn one from pa
 	OK
 DO
 	MODIFY_EACH_WEAK (OTGrammar)
-		OTGrammar_learnOneFromPartialOutput (me, partialOutput, evaluationNoise,
-			(kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+		OTGrammar_learnOneFromPartialOutput (me, partialOutput, evaluationNoise, updateRule, honourLocalRankings,
 			plasticity, relativePlasticitySpreading, numberOfChews, true);
 	MODIFY_EACH_WEAK_END
 }
@@ -822,7 +829,8 @@ DO
 // MARK: Modify behaviour
 
 FORM (MODIFY_OTGrammar_setDecisionStrategy, U"OTGrammar: Set decision strategy", nullptr) {
-	RADIO_ENUM (decisionStrategy, U"Decision strategy", kOTGrammar_decisionStrategy, DEFAULT)
+	RADIO_ENUM (kOTGrammar_decisionStrategy, decisionStrategy,
+			U"Decision strategy", kOTGrammar_decisionStrategy::DEFAULT)
 OK
 	FIND_ONE (OTGrammar)
 		SET_ENUM (decisionStrategy, kOTGrammar_decisionStrategy, my decisionStrategy);
@@ -881,7 +889,7 @@ FORM (NEW1_MODIFY_OTGrammar_Strings_inputsToOutputs, U"OTGrammar: Inputs to outp
 DO
 	FIND_TWO (OTGrammar, Strings)
 		autoStrings result = OTGrammar_inputsToOutputs (me, you, evaluationNoise);
-		praat_new (result.move(), my name, U"_out");
+		praat_new (result.move(), my name.get(), U"_out");
 		praat_dataChanged (me);
 	END
 }
@@ -900,7 +908,8 @@ DIRECT (BOOLEAN_OTGrammar_Strings_areAllPartialOutputsSinglyGrammatical) {
 
 FORM (MODIFY_OTGrammar_Stringses_learn, U"OTGrammar: Learn", U"OTGrammar & 2 Strings: Learn...") {
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (plasticity, U"Plasticity", U"0.1")
 	REAL (relativePlasticitySpreading, U"Rel. plasticity spreading", U"0.1")
 	BOOLEAN (honourLocalRankings, U"Honour local rankings", 1)
@@ -908,14 +917,15 @@ FORM (MODIFY_OTGrammar_Stringses_learn, U"OTGrammar: Learn", U"OTGrammar & 2 Str
 	OK
 DO
 	MODIFY_FIRST_OF_ONE_AND_COUPLE_WEAK (OTGrammar, Strings)
-		OTGrammar_learn (me, you, him, evaluationNoise, (enum kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+		OTGrammar_learn (me, you, him, evaluationNoise, updateRule, honourLocalRankings,
 			plasticity, relativePlasticitySpreading, numberOfChews);
 	MODIFY_FIRST_OF_ONE_AND_COUPLE_WEAK_END
 }
 
 FORM (MODIFY_OTGrammar_Strings_learnFromPartialOutputs, U"OTGrammar: Learn from partial adult outputs", nullptr) {
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (plasticity, U"Plasticity", U"0.1")
 	REAL (relativePlasticitySpreading, U"Rel. plasticity spreading", U"0.1")
 	BOOLEAN (honourLocalRankings, U"Honour local rankings", 1)
@@ -926,8 +936,7 @@ DO
 	FIND_TWO (OTGrammar, Strings)
 		autoOTHistory history;
 		try {
-			OTGrammar_learnFromPartialOutputs (me, you, evaluationNoise,
-				(kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+			OTGrammar_learnFromPartialOutputs (me, you, evaluationNoise, updateRule, honourLocalRankings,
 				plasticity, relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history);
 			praat_dataChanged (me);
 		} catch (MelderError) {
@@ -935,7 +944,7 @@ DO
 			Melder_flushError ();
 			// trickle down to save history
 		}
-		if (history) praat_new (history.move(), my name);
+		if (history) praat_new (history.move(), my name.get());
 	END
 }
 
@@ -958,7 +967,8 @@ DO
 FORM (MODIFY_OTGrammar_Distributions_learnFromPartialOutputs, U"OTGrammar & Distributions: Learn from partial outputs", U"OT learning 6. Shortcut to grammar learning") {
 	NATURAL (columnNumber, U"Column number", U"1")
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (initialPlasticity, U"Initial plasticity", U"1.0")
 	NATURAL (replicationsPerPlasticity, U"Replications per plasticity", U"100000")
 	REAL (plasticityDecrement, U"Plasticity decrement", U"0.1")
@@ -973,22 +983,25 @@ DO
 		autoOTHistory history;
 		try {
 			OTGrammar_Distributions_learnFromPartialOutputs (me, you, columnNumber, evaluationNoise,
-				(kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+				updateRule, honourLocalRankings,
 				initialPlasticity, replicationsPerPlasticity, plasticityDecrement, numberOfPlasticities,
-				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, false, false, 0);
+				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, false, false, 0
+			);
 			praat_dataChanged (me);
 		} catch (MelderError) {
 			praat_dataChanged (me);
 			Melder_flushError ();
 		}
-		if (history) praat_new (history.move(), my name);
+		if (history)
+			praat_new (history.move(), my name.get());
 	END
 }
 
 FORM (MODIFY_OTGrammar_Distributions_learnFromPartialOutputs_rrip, U"OTGrammar & Distributions: Learn from partial outputs (rrip)", U"OT learning 6. Shortcut to grammar learning") {
 	NATURAL (columnNumber, U"Column number", U"1")
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (initialPlasticity, U"Initial plasticity", U"1.0")
 	NATURAL (replicationsPerPlasticity, U"Replications per plasticity", U"100000")
 	REAL (plasticityDecrement, U"Plasticity decrement", U"0.1")
@@ -1003,22 +1016,25 @@ DO
 		autoOTHistory history;
 		try {
 			OTGrammar_Distributions_learnFromPartialOutputs (me, you, columnNumber, evaluationNoise,
-				(kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+				updateRule, honourLocalRankings,
 				initialPlasticity, replicationsPerPlasticity, plasticityDecrement, numberOfPlasticities,
-				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, true, true, 0);
+				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, true, true, 0
+			);
 			praat_dataChanged (me);
 		} catch (MelderError) {
 			praat_dataChanged (me);
 			Melder_flushError ();
 		}
-		if (history) praat_new (history.move(), my name);
+		if (history)
+			praat_new (history.move(), my name.get());
 	END
 }
 
 FORM (MODIFY_OTGrammar_Distributions_learnFromPartialOutputs_eip, U"OTGrammar & Distributions: Learn from partial outputs (eip)", U"OT learning 6. Shortcut to grammar learning") {
 	NATURAL (columnNumber, U"Column number", U"1")
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (initialPlasticity, U"Initial plasticity", U"1.0")
 	NATURAL (replicationsPerPlasticity, U"Replications per plasticity", U"100000")
 	REAL (plasticityDecrement, U"Plasticity decrement", U"0.1")
@@ -1033,22 +1049,25 @@ DO
 		autoOTHistory history;
 		try {
 			OTGrammar_Distributions_learnFromPartialOutputs (me, you, columnNumber, evaluationNoise,
-				(kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+				updateRule, honourLocalRankings,
 				initialPlasticity, replicationsPerPlasticity, plasticityDecrement, numberOfPlasticities,
-				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, true, true, 1000);
+				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, true, true, 1000
+			);
 			praat_dataChanged (me);
 		} catch (MelderError) {
 			praat_dataChanged (me);
 			Melder_flushError ();
 		}
-		if (history) praat_new (history.move(), my name);
+		if (history)
+			praat_new (history.move(), my name.get());
 	END
 }
 
 FORM (MODIFY_OTGrammar_Distributions_learnFromPartialOutputs_wrip, U"OTGrammar & Distributions: Learn from partial outputs (wrip)", U"OT learning 6. Shortcut to grammar learning") {
 	NATURAL (columnNumber, U"Column number", U"1")
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	REAL (initialPlasticity, U"Initial plasticity", U"1.0")
 	NATURAL (replicationsPerPlasticity, U"Replications per plasticity", U"100000")
 	REAL (plasticityDecrement, U"Plasticity decrement", U"0.1")
@@ -1063,15 +1082,17 @@ DO
 		autoOTHistory history;
 		try {
 			OTGrammar_Distributions_learnFromPartialOutputs (me, you, columnNumber, evaluationNoise,
-				(kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+				updateRule, honourLocalRankings,
 				initialPlasticity, replicationsPerPlasticity, plasticityDecrement, numberOfPlasticities,
-				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, true, true, 1);
+				relativePlasticitySpreading, numberOfChews, storeHistoryEvery, & history, true, true, 1
+			);
 			praat_dataChanged (me);
 		} catch (MelderError) {
 			praat_dataChanged (me);
 			Melder_flushError ();
 		}
-		if (history) praat_new (history.move(), my name);
+		if (history)
+			praat_new (history.move(), my name.get());
 	END
 }
 
@@ -1092,7 +1113,7 @@ FORM (MODIFY_OTGrammar_PairDistribution_findPositiveWeights, U"OTGrammar & PairD
 	OK
 DO
 	MODIFY_FIRST_OF_TWO (OTGrammar, PairDistribution)
-		OTGrammar_PairDistribution_findPositiveWeights_e (me, you, weightFloor, marginOfSeparation);
+		OTGrammar_PairDistribution_findPositiveWeights (me, you, weightFloor, marginOfSeparation);
 	MODIFY_FIRST_OF_TWO_END
 }
 
@@ -1135,7 +1156,8 @@ DO
 
 FORM (MODIFY_OTGrammar_PairDistribution_learn, U"OTGrammar & PairDistribution: Learn", U"OT learning 6. Shortcut to grammar learning") {
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	POSITIVE (initialPlasticity, U"Initial plasticity", U"1.0")
 	NATURAL (replicationsPerPlasticity, U"Replications per plasticity", U"100000")
 	REAL (plasticityDecrement, U"Plasticity decrement", U"0.1")
@@ -1147,7 +1169,7 @@ FORM (MODIFY_OTGrammar_PairDistribution_learn, U"OTGrammar & PairDistribution: L
 DO
 	MODIFY_FIRST_OF_TWO_WEAK (OTGrammar, PairDistribution)
 		OTGrammar_PairDistribution_learn (me, you,
-			evaluationNoise, (enum kOTGrammar_rerankingStrategy) updateRule, honourLocalRankings,
+			evaluationNoise, updateRule, honourLocalRankings,
 			initialPlasticity, replicationsPerPlasticity,
 			plasticityDecrement, numberOfPlasticities, relativePlasticitySpreading, numberOfChews);
 	MODIFY_FIRST_OF_TWO_WEAK_END
@@ -1164,7 +1186,8 @@ DIRECT (LIST_OTGrammar_PairDistribution_listObligatoryRankings) {
 // MARK: New
 
 FORM (NEW1_Create_multi_level_metrics_grammar, U"Create multi-level metrics grammar", nullptr) {
-	OPTIONMENU_ENUM (initialRanking, U"Initial ranking", kOTGrammar_createMetricsGrammar_initialRanking, DEFAULT)
+	OPTIONMENU_ENUM (kOTGrammar_createMetricsGrammar_initialRanking, initialRanking,
+			U"Initial ranking", kOTGrammar_createMetricsGrammar_initialRanking::DEFAULT)
 	OPTIONMENU (trochaicityConstraint, U"Trochaicity constraint", 1)
 		OPTION (U"FtNonfinal")
 		OPTION (U"Trochaic")
@@ -1237,7 +1260,7 @@ DO
 	STRING_ONE (OTMulti)
 		if (constraintNumber > my numberOfConstraints)
 			Melder_throw (U"Your constraint number should not exceed the number of constraints.");
-		const char32 *result = my constraints [constraintNumber]. name;
+		const conststring32 result = my constraints [constraintNumber]. name.get();
 	STRING_ONE_END
 }
 
@@ -1285,7 +1308,7 @@ DO
 	STRING_ONE (OTMulti)
 		if (candidateNumber > my numberOfCandidates)
 			Melder_throw (U"Your candidate number should not exceed the number of candidates.");
-		const char32 *result = my candidates [candidateNumber]. string;
+		const conststring32 result = my candidates [candidateNumber]. string.get();
 	STRING_ONE_END
 }
 
@@ -1331,9 +1354,8 @@ FORM (STRING_MODIFY_OTMulti_generateOptimalForm, U"OTMulti: Generate optimal for
 	OK
 DO
 	FIND_ONE (OTMulti)
-		char32 output [100];
-		OTMulti_generateOptimalForm (me, partialForm1, partialForm2, output, evaluationNoise);
-		Melder_information (output);
+		autostring32 output = OTMulti_generateOptimalForm (me, partialForm1, partialForm2, evaluationNoise);
+		Melder_information (output.get());
 		praat_dataChanged (me);
 	END
 }
@@ -1348,7 +1370,7 @@ DO
 	FIND_ONE (OTMulti)
 		autoStrings thee = OTMulti_generateOptimalForms (me, partialForm1, partialForm2,
 			numberOfTrials, evaluationNoise);
-		praat_new (thee.move(), my name, U"_out");
+		praat_new (thee.move(), my name.get(), U"_out");
 		praat_dataChanged (me);
 	END
 }
@@ -1365,7 +1387,7 @@ DO
 		try {
 			autoDistributions result = OTMulti_to_Distribution (me, partialForm1, partialForm2,
 				numberOfTrials, evaluationNoise);
-			praat_new (result.move(), my name, U"_out");
+			praat_new (result.move(), my name.get(), U"_out");
 			praat_dataChanged (me);
 		} catch (MelderError) {
 			praat_dataChanged (me);
@@ -1399,7 +1421,8 @@ DO
 FORM (MODIFY_OTMulti_learnOne, U"OTMulti: Learn one", nullptr) {
 	SENTENCE (partialForm1, U"Partial form 1", U"")
 	SENTENCE (partialForm2, U"Partial form 2", U"")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	OPTIONMENU (direction, U"Direction", 3)
 		OPTION (U"forward")
 		OPTION (U"backward")
@@ -1417,7 +1440,8 @@ DO
 // MARK: Modify behaviour
 
 FORM (MODIFY_OTMulti_setDecisionStrategy, U"OTMulti: Set decision strategy", nullptr) {
-	RADIO_ENUM (decisionStrategy, U"Decision strategy", kOTGrammar_decisionStrategy, DEFAULT)
+	RADIO_ENUM (kOTGrammar_decisionStrategy, decisionStrategy,
+			U"Decision strategy", kOTGrammar_decisionStrategy::DEFAULT)
 OK
 	FIND_ONE (OTMulti)
 		SET_ENUM (decisionStrategy, kOTGrammar_decisionStrategy, my decisionStrategy);
@@ -1463,7 +1487,8 @@ DO
 
 FORM (DANGEROUS_MODIFY_OTMulti_PairDistribution_learn, U"OTMulti & PairDistribution: Learn", nullptr) {
 	REAL (evaluationNoise, U"Evaluation noise", U"2.0")
-	OPTIONMENU_ENUM (updateRule, U"Update rule", kOTGrammar_rerankingStrategy, SYMMETRIC_ALL)
+	OPTIONMENU_ENUM (kOTGrammar_rerankingStrategy, updateRule,
+			U"Update rule", kOTGrammar_rerankingStrategy::SYMMETRIC_ALL)
 	OPTIONMENU (direction, U"Direction", 3)
 		OPTION (U"forward")
 		OPTION (U"backward")
@@ -1489,7 +1514,7 @@ DO
 			Melder_flushError ();
 			// trickle down to save history
 		}
-		if (history) praat_new (history.move(), my name);
+		if (history) praat_new (history.move(), my name.get());
 	END
 }
 
@@ -1501,7 +1526,7 @@ FORM (NEW1_MODIFY_OTMulti_Strings_generateOptimalForms, U"OTGrammar: Inputs to o
 DO
 	FIND_TWO (OTMulti, Strings)
 		autoStrings result = OTMulti_Strings_generateOptimalForms (me, you, evaluationNoide);
-		praat_new (result.move(), my name, U"_out");
+		praat_new (result.move(), my name.get(), U"_out");
 		praat_dataChanged (me);
 	END
 }
@@ -1524,7 +1549,8 @@ DO
 // MARK: Modify
 
 FORM (MODIFY_Net_spreadUp, U"Net: Spread up", nullptr) {
-	RADIO_ENUM (activationType, U"Activation type", kLayer_activationType, STOCHASTIC)
+	RADIO_ENUM (kLayer_activationType, activationType,
+			U"Activation type", kLayer_activationType::STOCHASTIC)
 	OK
 DO
 	MODIFY_EACH (Net)
@@ -1533,7 +1559,8 @@ DO
 }
 
 FORM (MODIFY_Net_spreadDown, U"Net: Spread down", nullptr) {
-	RADIO_ENUM (activationType, U"Activation type", kLayer_activationType, DETERMINISTIC)
+	RADIO_ENUM (kLayer_activationType, activationType,
+			U"Activation type", kLayer_activationType::DETERMINISTIC)
 	OK
 DO
 	MODIFY_EACH (Net)
@@ -1579,25 +1606,25 @@ DO
 DIRECT (NEW_Net_extractInputActivities) {
 	CONVERT_EACH (Net)
 		autoMatrix result = Net_extractInputActivities (me);
-	CONVERT_EACH_END (my name, U"_inputActivities")
+	CONVERT_EACH_END (my name.get(), U"_inputActivities")
 }
 
 DIRECT (NEW_Net_extractOutputActivities) {
 	CONVERT_EACH (Net)
 		autoMatrix result = Net_extractOutputActivities (me);
-	CONVERT_EACH_END (my name, U"_outputActivities")
+	CONVERT_EACH_END (my name.get(), U"_outputActivities")
 }
 
 DIRECT (NEW_Net_extractInputReconstruction) {
 	CONVERT_EACH (Net)
 		autoMatrix result = Net_extractInputReconstruction (me);
-	CONVERT_EACH_END (my name, U"_inputReconstruction")
+	CONVERT_EACH_END (my name.get(), U"_inputReconstruction")
 }
 
 DIRECT (NEW_Net_extractOutputReconstruction) {
 	CONVERT_EACH (Net)
 		autoMatrix result = Net_extractOutputReconstruction (me);
-	CONVERT_EACH_END (my name, U"_outputReconstruction")
+	CONVERT_EACH_END (my name.get(), U"_outputReconstruction")
 }
 
 FORM (NEW_Net_extractInputBiases, U"Net: Extract input biases", nullptr) {
@@ -1606,7 +1633,7 @@ FORM (NEW_Net_extractInputBiases, U"Net: Extract input biases", nullptr) {
 DO
 	CONVERT_EACH (Net)
 		autoMatrix result = Net_extractInputBiases (me, layerNumber);
-	CONVERT_EACH_END (my name, U"_inputBiases")
+	CONVERT_EACH_END (my name.get(), U"_inputBiases")
 }
 
 FORM (NEW_Net_extractOutputBiases, U"Net: Extract output biases", nullptr) {
@@ -1615,7 +1642,7 @@ FORM (NEW_Net_extractOutputBiases, U"Net: Extract output biases", nullptr) {
 DO
 	CONVERT_EACH (Net)
 		autoMatrix result = Net_extractOutputBiases (me, layerNumber);
-	CONVERT_EACH_END (my name, U"_outputBiases")
+	CONVERT_EACH_END (my name.get(), U"_outputBiases")
 }
 
 FORM (NEW_Net_extractWeights, U"Net: Extract weights", nullptr) {
@@ -1624,7 +1651,7 @@ FORM (NEW_Net_extractWeights, U"Net: Extract weights", nullptr) {
 DO
 	CONVERT_EACH (Net)
 		autoMatrix result = Net_extractWeights (me, layerNumber);
-	CONVERT_EACH_END (my name, U"_weights")
+	CONVERT_EACH_END (my name.get(), U"_weights")
 }
 
 FORM (NUMMAT_Net_getWeights, U"Net: Get weigths", nullptr) {
@@ -1632,7 +1659,7 @@ FORM (NUMMAT_Net_getWeights, U"Net: Get weigths", nullptr) {
 	OK
 DO
 	NUMMAT_ONE (Net)
-		autonummat result = Net_getWeights_nummat (me, layerNumber);
+		autoMAT result = Net_getWeights (me, layerNumber);
 	NUMMAT_ONE_END
 }
 
@@ -1674,13 +1701,23 @@ DO
 	MODIFY_FIRST_OF_TWO_END
 }
 
+FORM (MODIFY_Net_PatternList_learn_twoPhases, U"Net & PatternList: Learn (two phases)", nullptr) {
+	POSITIVE (learningRate, U"Learning rate", U"0.001")
+	OK
+DO
+	MODIFY_FIRST_OF_TWO (Net, PatternList)
+		Net_PatternList_learn_twoPhases (me, you, learningRate);
+	MODIFY_FIRST_OF_TWO_END
+}
+
 FORM (NEW1_Net_PatternList_to_ActivationList, U"Net & PatternList: To ActivationList", nullptr) {
-	RADIO_ENUM (activationType, U"Activation type", kLayer_activationType, DETERMINISTIC)
+	RADIO_ENUM (kLayer_activationType, activationType,
+			U"Activation type", kLayer_activationType::DETERMINISTIC)
 	OK
 DO
 	CONVERT_TWO (Net, PatternList)
 		autoActivationList result = Net_PatternList_to_ActivationList (me, you, activationType);
-	CONVERT_TWO_END (my name, U"_", your name)
+	CONVERT_TWO_END (my name.get(), U"_", your name.get())
 }
 
 // MARK: - NOULLIGRID
@@ -1707,6 +1744,8 @@ void praat_uvafon_gram_init () {
 		nullptr);
 	Thing_recognizeClassByOtherName (classOTGrammar, U"OTCase");
 
+	structNoulliGridEditor :: f_preferences ();
+
 	praat_addMenuCommand (U"Objects", U"New", U"Constraint grammars", nullptr, 0, nullptr);
 		praat_addMenuCommand (U"Objects", U"New", U"OT learning tutorial", nullptr, praat_DEPTH_1 | praat_NO_API, HELP_OT_learning_tutorial);
 		praat_addMenuCommand (U"Objects", U"New", U"-- tableau grammars --", nullptr, 1, nullptr);
@@ -1879,6 +1918,7 @@ void praat_uvafon_gram_init () {
 	praat_addAction2 (classNet, 1, classPatternList, 1, U"Apply to output...", nullptr, 0, MODIFY_Net_PatternList_applyToOutput);
 	praat_addAction2 (classNet, 1, classPatternList, 1, U"Learn...", nullptr, 0, MODIFY_Net_PatternList_learn);
 	praat_addAction2 (classNet, 1, classPatternList, 1, U"Learn by layer...", nullptr, 0, MODIFY_Net_PatternList_learnByLayer);
+	praat_addAction2 (classNet, 1, classPatternList, 1, U"Learn (two phases)...", nullptr, 0, MODIFY_Net_PatternList_learn_twoPhases);
 	praat_addAction2 (classNet, 1, classPatternList, 1, U"To ActivationList", nullptr, 0, NEW1_Net_PatternList_to_ActivationList);
 
 	praat_addAction1 (classNoulliGrid, 1, U"View & Edit", nullptr, praat_ATTRACTIVE, WINDOW_NoulliGrid_viewAndEdit);
