@@ -36,7 +36,6 @@
  */
 
 #include "longchar.h"
-#include <stdio.h>   /* For error message. */
 #include "UnicodeData.h"
 
 static struct structLongchar_Info Longchar_database [] = {
@@ -626,9 +625,13 @@ void Longchar_init () {
 	for (; data -> first != '\0'; i ++, data ++) {
 		short *location = & where [data -> first - 32] [data -> second - 32];
 		if (*location) {
-			/* Doubly defined symbol; an error! */
-			/* We may not be able to use Melder_error yet, so just write a warning to stderr. */
-			fprintf (stderr, "Longchar init: symbol \"%c%c\" doubly defined.\n", data -> first, data -> second);
+			/*
+				Doubly defined symbol; an error!
+				We may not be able to use Melder_error yet,
+				so just write a warning to stderr.
+			*/
+			fprintf (stderr, "Longchar init: symbol \"%c%c\" doubly defined.\n",
+					data -> first, data -> second);
 		}
 		*location = i;
 		if (data -> unicode <= kUCD_TOP_OF_LIST) {
@@ -639,7 +642,7 @@ void Longchar_init () {
 	inited = 1;
 }
 
-char32 * Longchar_nativize32 (const char32 *generic, char32 *native, int educateQuotes) {
+char32 * Longchar_nativize32 (conststring32 generic, char32 *native, int educateQuotes) {
 	integer nquote = 0;
 	char32_t kar, kar1, kar2;
 	if (! inited) Longchar_init ();
@@ -675,7 +678,7 @@ char32 * Longchar_nativize32 (const char32 *generic, char32 *native, int educate
 	return native;
 }
 
-char32_t *Longchar_genericize32 (const char32 *native, char32 *g) {
+char32_t *Longchar_genericize32 (conststring32 native, char32 *g) {
 	char32_t kar;
 	if (! inited) Longchar_init ();
 	while ((kar = *native++) != U'\0') {
