@@ -1,6 +1,6 @@
 /* LogisticRegression.cpp
  *
- * Copyright (C) 2005-2012,2015,2016,2017 Paul Boersma
+ * Copyright (C) 2005-2012,2015-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  */
 
 #include "LogisticRegression.h"
-#include "UnicodeData.h"
+#include "../kar/UnicodeData.h"
 
 #include "oo_DESTROY.h"
 #include "LogisticRegression_def.h"
@@ -42,28 +42,28 @@ Thing_implement (LogisticRegression, Regression, 0);
 
 void structLogisticRegression :: v_info () {
 	LogisticRegression_Parent :: v_info ();
-	MelderInfo_writeLine (U"Dependent 1: ", our dependent1);
-	MelderInfo_writeLine (U"Dependent 2: ", our dependent2);
+	MelderInfo_writeLine (U"Dependent 1: ", our dependent1.get());
+	MelderInfo_writeLine (U"Dependent 2: ", our dependent2.get());
 	MelderInfo_writeLine (U"Interpretation:");
-	MelderInfo_write (U"   ln (P(", dependent2, U")/P(", dependent1, U")) " UNITEXT_ALMOST_EQUAL_TO U" ", Melder_fixed (intercept, 6));
+	MelderInfo_write (U"   ln (P(", our dependent2.get(), U")/P(", our dependent1.get(), U")) " UNITEXT_ALMOST_EQUAL_TO U" ", Melder_fixed (intercept, 6));
 	for (integer ivar = 1; ivar <= parameters.size; ivar ++) {
 		RegressionParameter parm = parameters.at [ivar];
-		MelderInfo_write (parm -> value < 0.0 ? U" - " : U" + ", Melder_fixed (fabs (parm -> value), 6), U" * ", parm -> label);
+		MelderInfo_write (parm -> value < 0.0 ? U" - " : U" + ", Melder_fixed (fabs (parm -> value), 6), U" * ", parm -> label.get());
 	}
 	MelderInfo_writeLine (U"");
 	MelderInfo_writeLine (U"Log odds ratios:");
 	for (integer ivar = 1; ivar <= parameters.size; ivar ++) {
 		RegressionParameter parm = parameters.at [ivar];
-		MelderInfo_writeLine (U"   Log odds ratio of factor ", parm -> label, U": ", Melder_fixed ((parm -> maximum - parm -> minimum) * parm -> value, 6));
+		MelderInfo_writeLine (U"   Log odds ratio of factor ", parm -> label.get(), U": ", Melder_fixed ((parm -> maximum - parm -> minimum) * parm -> value, 6));
 	}
 	MelderInfo_writeLine (U"Odds ratios:");
 	for (integer ivar = 1; ivar <= parameters.size; ivar ++) {
 		RegressionParameter parm = parameters.at [ivar];
-		MelderInfo_writeLine (U"   Odds ratio of factor ", parm -> label, U": ", exp ((parm -> maximum - parm -> minimum) * parm -> value));
+		MelderInfo_writeLine (U"   Odds ratio of factor ", parm -> label.get(), U": ", exp ((parm -> maximum - parm -> minimum) * parm -> value));
 	}
 }
 
-autoLogisticRegression LogisticRegression_create (const char32 *dependent1, const char32 *dependent2) {
+autoLogisticRegression LogisticRegression_create (conststring32 dependent1, conststring32 dependent2) {
 	try {
 		autoLogisticRegression me = Thing_new (LogisticRegression);
 		Regression_init (me.get());
@@ -75,26 +75,28 @@ autoLogisticRegression LogisticRegression_create (const char32 *dependent1, cons
 	}
 }
 
-static autoLogisticRegression _Table_to_LogisticRegression (Table me, integer *factors, integer numberOfFactors, integer dependent1, integer dependent2) {
-	integer numberOfParameters = numberOfFactors + 1;
-	integer numberOfCells = my rows.size, numberOfY0 = 0, numberOfY1 = 0, numberOfData = 0;
+static autoLogisticRegression _Table_to_LogisticRegression (Table me, constINTVEC factors, integer dependent1, integer dependent2) {
+	const integer numberOfFactors = factors.size;
+	const integer numberOfParameters = numberOfFactors + 1;
+	const integer numberOfCells = my rows.size;
+	integer numberOfY0 = 0, numberOfY1 = 0, numberOfData = 0;
 	double logLikelihood = 1e307, previousLogLikelihood = 1e308;
 	if (numberOfParameters < 1)   // includes intercept
 		Melder_throw (U"Not enough columns (has to be more than 1).");
 	/*
-	 * Divide up the contents of the table into a number of independent variables (x) and two dependent variables (y0 and y1).
-	 */
-	autoNUMmatrix <double> x (1, numberOfCells, 0, numberOfFactors);   // column 0 is the intercept
+		Divide up the contents of the table into a number of independent variables (x) and two dependent variables (y0 and y1).
+	*/
+	autoMAT x = newMATzero (numberOfCells, 1+numberOfFactors);   // column 1 is the intercept
 	autoNUMvector <double> y0 (1, numberOfCells);
 	autoNUMvector <double> y1 (1, numberOfCells);
 	autoNUMvector <double> meanX (1, numberOfFactors);
 	autoNUMvector <double> stdevX (1, numberOfFactors);
-	autoNUMmatrix <double> smallMatrix (0, numberOfFactors, 0, numberOfParameters);
-	autoLogisticRegression thee = LogisticRegression_create (my columnHeaders [dependent1]. label, my columnHeaders [dependent2]. label);
+	autoMAT smallMatrix = newMATzero (1+numberOfFactors, 1+numberOfParameters);
+	autoLogisticRegression thee = LogisticRegression_create (my columnHeaders [dependent1]. label.get(), my columnHeaders [dependent2]. label.get());
 	for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
 		double minimum = Table_getMinimum (me, factors [ivar]);
 		double maximum = Table_getMaximum (me, factors [ivar]);
-		Regression_addParameter (thee.get(), my columnHeaders [factors [ivar]]. label, minimum, maximum, 0.0);
+		Regression_addParameter (thee.get(), my columnHeaders [factors [ivar]]. label.get(), minimum, maximum, 0.0);
 	}
 	for (integer icell = 1; icell <= numberOfCells; icell ++) {
 		y0 [icell] = Table_getNumericValue_Assert (me, icell, dependent1);
@@ -102,36 +104,36 @@ static autoLogisticRegression _Table_to_LogisticRegression (Table me, integer *f
 		numberOfY0 += y0 [icell];
 		numberOfY1 += y1 [icell];
 		numberOfData += y0 [icell] + y1 [icell];
-		x [icell] [0] = 1.0;   // intercept
+		x [icell] [1+0] = 1.0;   // intercept
 		for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
-			x [icell] [ivar] = Table_getNumericValue_Assert (me, icell, factors [ivar]);
-			meanX [ivar] += x [icell] [ivar] * (y0 [icell] + y1 [icell]);
+			x [icell] [1+ivar] = Table_getNumericValue_Assert (me, icell, factors [ivar]);
+			meanX [ivar] += x [icell] [1+ivar] * (y0 [icell] + y1 [icell]);
 		}
 	}
 	if (numberOfY0 == 0 && numberOfY1 == 0)
 		Melder_throw (U"No data in either class. Cannot determine result.");
 	if (numberOfY0 == 0)
-		Melder_throw (U"No data in class ", my columnHeaders [dependent1]. label, U". Cannot determine result.");
+		Melder_throw (U"No data in class ", my columnHeaders [dependent1]. label.get(), U". Cannot determine result.");
 	if (numberOfY1 == 0)
-		Melder_throw (U"No data in class ", my columnHeaders [dependent2]. label, U". Cannot determine result.");
+		Melder_throw (U"No data in class ", my columnHeaders [dependent2]. label.get(), U". Cannot determine result.");
 	/*
 	 * Normalize the data.
 	 */
 	for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
 		meanX [ivar] /= numberOfData;
 		for (integer icell = 1; icell <= numberOfCells; icell ++) {
-			x [icell] [ivar] -= meanX [ivar];
+			x [icell] [1+ivar] -= meanX [ivar];
 		}
 	}
 	for (integer icell = 1; icell <= numberOfCells; icell ++) {
 		for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
-			stdevX [ivar] += x [icell] [ivar] * x [icell] [ivar] * (y0 [icell] + y1 [icell]);
+			stdevX [ivar] += x [icell] [1+ivar] * x [icell] [1+ivar] * (y0 [icell] + y1 [icell]);
 		}
 	}
 	for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
 		stdevX [ivar] = sqrt (stdevX [ivar] / numberOfData);
 		for (integer icell = 1; icell <= numberOfCells; icell ++) {
-			x [icell] [ivar] /= stdevX [ivar];
+			x [icell] [1+ivar] /= stdevX [ivar];
 		}
 	}
 	/*
@@ -147,7 +149,7 @@ static autoLogisticRegression _Table_to_LogisticRegression (Table me, integer *f
 		previousLogLikelihood = logLikelihood;
 		for (integer ivar = 0; ivar <= numberOfFactors; ivar ++) {
 			for (integer jvar = ivar; jvar <= numberOfParameters; jvar ++) {
-				smallMatrix [ivar] [jvar] = 0.0;
+				smallMatrix [1+ivar] [1+jvar] = 0.0;
 			}
 		}
 		/*
@@ -158,7 +160,7 @@ static autoLogisticRegression _Table_to_LogisticRegression (Table me, integer *f
 			double fittedLogit = thy intercept, fittedP, fittedQ, fittedLogP, fittedLogQ, fittedPQ, fittedVariance;
 			for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
 				RegressionParameter parm = thy parameters.at [ivar];
-				fittedLogit += parm -> value * x [icell] [ivar];
+				fittedLogit += parm -> value * x [icell] [1+ivar];
 			}
 			/*
 			 * Basically we have fittedP = 1.0 / (1.0 + exp (- fittedLogit)),
@@ -206,10 +208,9 @@ static autoLogisticRegression _Table_to_LogisticRegression (Table me, integer *f
 				/*
 				 * The last column gets the gradient of LL: dLL/da, dLL/db, dLL/dc.
 				 */
-				smallMatrix [ivar] [numberOfParameters] += x [icell] [ivar] * (y1 [icell] * fittedQ - y0 [icell] * fittedP);
-				for (integer jvar = ivar; jvar <= numberOfFactors; jvar ++) {
-					smallMatrix [ivar] [jvar] += x [icell] [ivar] * x [icell] [jvar] * fittedVariance;
-				}
+				smallMatrix [1+ivar] [1+numberOfParameters] += x [icell] [1+ivar] * (y1 [icell] * fittedQ - y0 [icell] * fittedP);
+				for (integer jvar = ivar; jvar <= numberOfFactors; jvar ++)
+					smallMatrix [1+ivar] [1+jvar] += x [icell] [1+ivar] * x [icell] [1+jvar] * fittedVariance;
 			}
 		}
 		if (fabs (logLikelihood - previousLogLikelihood) < 1e-11) {
@@ -218,37 +219,33 @@ static autoLogisticRegression _Table_to_LogisticRegression (Table me, integer *f
 		/*
 		 * Make matrix symmetric.
 		 */
-		for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
-			for (integer jvar = 0; jvar < ivar; jvar ++) {
-				smallMatrix [ivar] [jvar] = smallMatrix [jvar] [ivar];
-			}
-		}
+		for (integer ivar = 1; ivar <= numberOfFactors; ivar ++)
+			for (integer jvar = 0; jvar < ivar; jvar ++)
+				smallMatrix [1+ivar] [1+jvar] = smallMatrix [1+jvar] [1+ivar];
 		/*
 		 * Invert matrix in the simplest way, and shift and wipe the last column with it.
 		 */
 		for (integer ivar = 0; ivar <= numberOfFactors; ivar ++) {
-			double pivot = smallMatrix [ivar] [ivar];   /* Save diagonal. */
-			smallMatrix [ivar] [ivar] = 1.0;
-			for (integer jvar = 0; jvar <= numberOfParameters; jvar ++) {
-				smallMatrix [ivar] [jvar] /= pivot;
-			}
+			double pivot = smallMatrix [1+ivar] [1+ivar];   /* Save diagonal. */
+			smallMatrix [1+ivar] [1+ivar] = 1.0;
+			for (integer jvar = 0; jvar <= numberOfParameters; jvar ++)
+				smallMatrix [1+ivar] [1+jvar] /= pivot;
 			for (integer jvar = 0; jvar <= numberOfFactors; jvar ++) {
 				if (jvar != ivar) {
-					double temp = smallMatrix [jvar] [ivar];
-					smallMatrix [jvar] [ivar] = 0.0;
-					for (integer kvar = 0; kvar <= numberOfParameters; kvar ++) {
-						smallMatrix [jvar] [kvar] -= temp * smallMatrix [ivar] [kvar];
-					}
+					double temp = smallMatrix [1+jvar] [1+ivar];
+					smallMatrix [1+jvar] [1+ivar] = 0.0;
+					for (integer kvar = 0; kvar <= numberOfParameters; kvar ++)
+						smallMatrix [1+jvar] [1+kvar] -= temp * smallMatrix [1+ivar] [1+kvar];
 				}
 			}
 		}
 		/*
 		 * Update the parameters from the last column of smallMatrix.
 		 */
-		thy intercept += smallMatrix [0] [numberOfParameters];
+		thy intercept += smallMatrix [1+0] [1+numberOfParameters];
 		for (integer ivar = 1; ivar <= numberOfFactors; ivar ++) {
 			RegressionParameter parm = thy parameters.at [ivar];
-			parm -> value += smallMatrix [ivar] [numberOfParameters];
+			parm -> value += smallMatrix [1+ivar] [1+numberOfParameters];
 		}
 	}
 	if (iteration > 100) {
@@ -262,15 +259,14 @@ static autoLogisticRegression _Table_to_LogisticRegression (Table me, integer *f
 	return thee;
 }
 
-autoLogisticRegression Table_to_LogisticRegression (Table me, const char32 *factors_columnLabelString,
-	const char32 *dependent1_columnLabel, const char32 *dependent2_columnLabel)
+autoLogisticRegression Table_to_LogisticRegression (Table me, conststring32 factors_columnLabelString,
+	conststring32 dependent1_columnLabel, conststring32 dependent2_columnLabel)
 {
 	try {
-		integer numberOfFactors;
-		autoNUMvector <integer> factors_columnIndices (Table_getColumnIndicesFromColumnLabelString (me, factors_columnLabelString, & numberOfFactors), 1);
+		auto factors_columnIndices = Table_getColumnIndicesFromColumnLabelString (me, factors_columnLabelString);
 		integer dependent1_columnIndex = Table_getColumnIndexFromColumnLabel (me, dependent1_columnLabel);
 		integer dependent2_columnIndex = Table_getColumnIndexFromColumnLabel (me, dependent2_columnLabel);
-		autoLogisticRegression thee = _Table_to_LogisticRegression (me, factors_columnIndices.peek(), numberOfFactors, dependent1_columnIndex, dependent2_columnIndex);
+		autoLogisticRegression thee = _Table_to_LogisticRegression (me, factors_columnIndices.get(), dependent1_columnIndex, dependent2_columnIndex);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": logistic regression not performed.");
@@ -334,9 +330,9 @@ void LogisticRegression_drawBoundary (LogisticRegression me, Graphics graphics,
 	Graphics_unsetInner (graphics);
 	if (garnish) {
 		Graphics_drawInnerBox (graphics);
-		Graphics_textBottom (graphics, true, parmx -> label);
+		Graphics_textBottom (graphics, true, parmx -> label.get());
 		Graphics_marksBottom (graphics, 2, true, true, false);
-		Graphics_textLeft (graphics, true, parmy -> label);
+		Graphics_textLeft (graphics, true, parmy -> label.get());
 		Graphics_marksLeft (graphics, 2, true, true, false);
 	}
 }
