@@ -46,7 +46,7 @@ void structPairDistribution :: v_info () {
 	MelderInfo_writeLine (U"Number of pairs: ", pairs.size);
 }
 
-autoPairProbability PairProbability_create (const char32 *string1, const char32 *string2, double weight) {
+autoPairProbability PairProbability_create (conststring32 string1, conststring32 string2, double weight) {
 	autoPairProbability me = Thing_new (PairProbability);
 	my string1 = Melder_dup (string1);
 	my string2 = Melder_dup (string2);
@@ -64,7 +64,7 @@ autoPairDistribution PairDistribution_create () {
 }
 
 int PairProbability_compare (PairProbability me, PairProbability thee) noexcept {
-	return str32cmp (my string1, thy string1);
+	return str32cmp (my string1.get(), thy string1.get());
 }
 
 static void PairDistribution_checkSpecifiedPairNumber (PairDistribution me, integer pairNumber) {
@@ -74,21 +74,21 @@ static void PairDistribution_checkSpecifiedPairNumber (PairDistribution me, inte
 		Melder_throw (me, U": the specified pair number is ", pairNumber, U", but should be at most my number of pairs (", my pairs.size, U").");
 }
 
-const char32 * PairDistribution_getString1 (PairDistribution me, integer pairNumber) {
+conststring32 PairDistribution_getString1 (PairDistribution me, integer pairNumber) {
 	try {
 		PairDistribution_checkSpecifiedPairNumber (me, pairNumber);
 		PairProbability prob = my pairs.at [pairNumber];
-		return prob -> string1;
+		return prob -> string1.get();
 	} catch (MelderError) {
 		Melder_throw (me, U": string1 not retrieved.");
 	}
 }
 
-const char32 * PairDistribution_getString2 (PairDistribution me, integer pairNumber) {
+conststring32 PairDistribution_getString2 (PairDistribution me, integer pairNumber) {
 	try {
 		PairDistribution_checkSpecifiedPairNumber (me, pairNumber);
 		PairProbability prob = my pairs.at [pairNumber];
-		return prob -> string2;
+		return prob -> string2.get();
 	} catch (MelderError) {
 		Melder_throw (me, U": string2 not retrieved.");
 	}
@@ -104,7 +104,7 @@ double PairDistribution_getWeight (PairDistribution me, integer pairNumber) {
 	}
 }
 
-void PairDistribution_add (PairDistribution me, const char32 *string1, const char32 *string2, double weight) {
+void PairDistribution_add (PairDistribution me, conststring32 string1, conststring32 string2, double weight) {
 	autoPairProbability pair = PairProbability_create (string1, string2, weight);
 	my pairs.addItem_move (pair.move());
 }
@@ -121,52 +121,53 @@ void PairDistribution_removeZeroWeights (PairDistribution me) {
 void PairDistribution_swapInputsAndOutputs (PairDistribution me) {
 	for (integer ipair = my pairs.size; ipair > 0; ipair --) {
 		PairProbability prob = my pairs.at [ipair];
-		char32 *tmp = prob -> string1;
-		prob -> string1 = prob -> string2;
-		prob -> string2 = tmp;
+		std::swap (prob -> string1, prob -> string2);
 	}
 }
 
-static double PairDistributions_getTotalWeight_checkPositive (PairDistribution me) throw (MelderError) {
+static double PairDistributions_getTotalWeight_checkPositive (PairDistribution me) {
 	longdouble totalWeight = 0.0;
 	for (integer ipair = 1; ipair <= my pairs.size; ipair ++) {
 		PairProbability prob = my pairs.at [ipair];
 		totalWeight += prob -> weight;
 	}
-	if (totalWeight <= 0.0) {
-		Melder_throw (me, U": the total probability weight is ", Melder_half (totalWeight), U" but should be greater than zero for this operation.");
-	}
-	return (double) totalWeight;
+	Melder_require (totalWeight > 0.0,
+		me, U": the total probability weight is ", Melder_half (double (totalWeight)),
+		U" but should be greater than zero for this operation."
+	);
+	return double (totalWeight);
 }
 
 void PairDistribution_to_Stringses (PairDistribution me, integer nout, autoStrings *strings1_out, autoStrings *strings2_out) {
 	try {
-		integer nin = my pairs.size, iin;
+		const integer nin = my pairs.size;
 		if (nin < 1)
 			Melder_throw (U"No candidates.");
 		if (nout < 1)
 			Melder_throw (U"Number of generated string pairs should be positive.");
-		double total = PairDistributions_getTotalWeight_checkPositive (me);
+		const double total = PairDistributions_getTotalWeight_checkPositive (me);
 		autoStrings strings1 = Thing_new (Strings);
 		strings1 -> numberOfStrings = nout;
-		strings1 -> strings = NUMvector <char32 *> (1, nout);
+		strings1 -> strings = autostring32vector (nout);
 		autoStrings strings2 = Thing_new (Strings);
 		strings2 -> numberOfStrings = nout;
-		strings2 -> strings = NUMvector <char32 *> (1, nout);
+		strings2 -> strings = autostring32vector (nout);
 		for (integer iout = 1; iout <= nout; iout ++) {
+			integer iin;
 			do {
-				double rand = NUMrandomUniform (0, total), sum = 0.0;
+				const double rand = NUMrandomUniform (0, total);
+				longdouble sum = 0.0;
 				for (iin = 1; iin <= nin; iin ++) {
 					PairProbability prob = my pairs.at [iin];
 					sum += prob -> weight;
 					if (rand <= sum) break;
 				}
 			} while (iin > nin);   /* Guard against rounding errors. */
-			PairProbability prob = my pairs.at [iin];
+			PairProbability const prob = my pairs.at [iin];
 			if (! prob -> string1 || ! prob -> string2)
 				Melder_throw (U"No string in probability pair ", iin, U".");
-			strings1 -> strings [iout] = Melder_dup (prob -> string1);
-			strings2 -> strings [iout] = Melder_dup (prob -> string2);
+			strings1 -> strings [iout] = Melder_dup (prob -> string1.get());
+			strings2 -> strings [iout] = Melder_dup (prob -> string2.get());
 		}
 		*strings1_out = strings1.move();
 		*strings2_out = strings2.move();
@@ -175,19 +176,21 @@ void PairDistribution_to_Stringses (PairDistribution me, integer nout, autoStrin
 	}
 }
 
-void PairDistribution_peekPair (PairDistribution me, char32 **string1, char32 **string2) {
+void PairDistribution_peekPair (PairDistribution me, conststring32 *out_string1, conststring32 *out_string2) {
 	try {
-		*string1 = *string2 = nullptr;
-		double total = 0.0;
-		integer nin = my pairs.size, iin;
+		*out_string1 = *out_string2 = nullptr;
+		longdouble total = 0.0;
+		const integer nin = my pairs.size;
 		PairProbability prob;
 		if (nin < 1) Melder_throw (U"No candidates.");
+		integer iin;
 		for (iin = 1; iin <= nin; iin ++) {
 			prob = my pairs.at [iin];
 			total += prob -> weight;
 		}
 		do {
-			double rand = NUMrandomUniform (0, total), sum = 0.0;
+			const double rand = NUMrandomUniform (0.0, double (total));
+			longdouble sum = 0.0;
 			for (iin = 1; iin <= nin; iin ++) {
 				prob = my pairs.at [iin];
 				sum += prob -> weight;
@@ -196,8 +199,8 @@ void PairDistribution_peekPair (PairDistribution me, char32 **string1, char32 **
 		} while (iin > nin);   // guard against rounding errors
 		prob = my pairs.at [iin];
 		if (! prob -> string1 || ! prob -> string2) Melder_throw (U"No string in probability pair ", iin, U".");
-		*string1 = prob -> string1;
-		*string2 = prob -> string2;
+		*out_string1 = prob -> string1.get();
+		*out_string2 = prob -> string2.get();
 	} catch (MelderError) {
 		Melder_throw (me, U": pair not peeked.");
 	}
@@ -205,17 +208,17 @@ void PairDistribution_peekPair (PairDistribution me, char32 **string1, char32 **
 
 static double PairDistribution_getFractionCorrect (PairDistribution me, int which) {
 	try {
-		double correct = 0.0;
+		longdouble correct = 0.0;
 		integer pairmin = 1, ipair;
 		autoPairDistribution thee = Data_copy (me);
 		thy pairs.sort (PairProbability_compare);
 		double total = PairDistributions_getTotalWeight_checkPositive (thee.get());
 		do {
 			integer pairmax = pairmin;
-			char32 *firstInput = thy pairs.at [pairmin] -> string1;
+			const conststring32 firstInput = thy pairs.at [pairmin] -> string1.get();
 			for (ipair = pairmin + 1; ipair <= thy pairs.size; ipair ++) {
-				PairProbability prob = thy pairs.at [ipair];
-				if (! str32equ (prob -> string1, firstInput)) {
+				PairProbability const prob = thy pairs.at [ipair];
+				if (! str32equ (prob -> string1.get(), firstInput)) {
 					pairmax = ipair - 1;
 					break;
 				}
@@ -224,16 +227,16 @@ static double PairDistribution_getFractionCorrect (PairDistribution me, int whic
 			if (which == 0) {
 				double pmax = 0.0;
 				for (ipair = pairmin; ipair <= pairmax; ipair ++) {
-					PairProbability prob = thy pairs.at [ipair];
-					double p = prob -> weight / total;
+					PairProbability const prob = thy pairs.at [ipair];
+					const double p = prob -> weight / total;
 					if (p > pmax) pmax = p;
 				}
 				correct += pmax;
 			} else {
-				double sum = 0.0, p2 = 0.0;
+				longdouble sum = 0.0, p2 = 0.0;
 				for (ipair = pairmin; ipair <= pairmax; ipair ++) {
-					PairProbability prob = thy pairs.at [ipair];
-					double p = prob -> weight / total;
+					PairProbability const prob = thy pairs.at [ipair];
+					const double p = prob -> weight / total;
 					sum += p;
 					p2 += p * p;
 				}
@@ -241,7 +244,7 @@ static double PairDistribution_getFractionCorrect (PairDistribution me, int whic
 			}
 			pairmin = pairmax + 1;
 		} while (pairmin <= thy pairs.size);
-		return correct;
+		return double (correct);
 	} catch (MelderError) {
 		Melder_throw (me, U": could not compute my fraction correct.");
 	}
@@ -257,7 +260,7 @@ double PairDistribution_getFractionCorrect_probabilityMatching (PairDistribution
 
 double PairDistribution_Distributions_getFractionCorrect (PairDistribution me, Distributions dist, integer column) {
 	try {
-		double correct = 0.0;
+		longdouble correct = 0.0;
 		integer pairmin = 1;
 		char32 string [1000];
 		Distributions_checkSpecifiedColumnNumberWithinRange (dist, column);
@@ -265,12 +268,12 @@ double PairDistribution_Distributions_getFractionCorrect (PairDistribution me, D
 		thy pairs.sort (PairProbability_compare);
 		double total = PairDistributions_getTotalWeight_checkPositive (thee.get());
 		do {
-			integer pairmax = pairmin, length, ipair;
-			double sum = 0.0, sumDist = 0.0;
-			char32 *firstInput = thy pairs.at [pairmin] -> string1;
+			integer pairmax = pairmin, ipair;
+			longdouble sum = 0.0, sumDist = 0.0;
+			const conststring32 firstInput = thy pairs.at [pairmin] -> string1.get();
 			for (ipair = pairmin + 1; ipair <= thy pairs.size; ipair ++) {
 				PairProbability prob = thy pairs.at [ipair];
-				if (! str32equ (prob -> string1, firstInput)) {
+				if (! str32equ (prob -> string1.get(), firstInput)) {
 					pairmax = ipair - 1;
 					break;
 				}
@@ -278,10 +281,11 @@ double PairDistribution_Distributions_getFractionCorrect (PairDistribution me, D
 			if (ipair > thy pairs.size) pairmax = thy pairs.size;
 			for (ipair = pairmin; ipair <= pairmax; ipair ++) {
 				PairProbability prob = thy pairs.at [ipair];
-				double p = prob -> weight / total, pout = 0.0;
-				Melder_sprint (string, 1000, prob -> string1, U" \\-> ", prob -> string2);
+				const double p = prob -> weight / total;
+				double pout = 0.0;
+				Melder_sprint (string, 1000, prob -> string1.get(), U" \\-> ", prob -> string2.get());
 				for (integer idist = 1; idist <= dist -> numberOfRows; idist ++) {
-					if (str32equ (string, dist -> rowLabels [idist])) {
+					if (str32equ (string, dist -> rowLabels [idist].get())) {
 						pout = dist -> data [idist] [column];
 						break;
 					}
@@ -289,16 +293,14 @@ double PairDistribution_Distributions_getFractionCorrect (PairDistribution me, D
 				sum += p * pout;
 			}
 			Melder_sprint (string, 1000, firstInput, U" \\-> ");
-			length = str32len (string);
-			for (integer idist = 1; idist <= dist -> numberOfRows; idist ++) {
-				if (str32nequ (string, dist -> rowLabels [idist], length)) {
+			const integer length = str32len (string);
+			for (integer idist = 1; idist <= dist -> numberOfRows; idist ++)
+				if (str32nequ (string, dist -> rowLabels [idist].get(), length))
 					sumDist += dist -> data [idist] [column];
-				}
-			}
 			if (sumDist != 0.0) correct += sum / sumDist;
 			pairmin = pairmax + 1;
 		} while (pairmin <= thy pairs.size);
-		return correct;
+		return double (correct);
 	} catch (MelderError) {
 		Melder_throw (me, U" & ", dist, U": could not compute our fraction correct.");
 	}
@@ -309,8 +311,8 @@ autoTable PairDistribution_to_Table (PairDistribution me) {
 		autoTable thee = Table_createWithColumnNames (my pairs.size, U"string1 string2 weight");
 		for (integer ipair = 1; ipair <= my pairs.size; ipair ++) {
 			PairProbability prob = my pairs.at [ipair];
-			Table_setStringValue (thee.get(), ipair, 1, prob -> string1);
-			Table_setStringValue (thee.get(), ipair, 2, prob -> string2);
+			Table_setStringValue (thee.get(), ipair, 1, prob -> string1.get());
+			Table_setStringValue (thee.get(), ipair, 2, prob -> string2.get());
 			Table_setNumericValue (thee.get(), ipair, 3, prob -> weight);
 		}
 		return thee;
