@@ -46,18 +46,18 @@ void structRegression :: v_info () {
 	MelderInfo_writeLine (U"   Number of factors: ", our parameters.size);
 	for (integer ivar = 1; ivar <= our parameters.size; ivar ++) {
 		RegressionParameter parm = our parameters.at [ivar];
-		MelderInfo_writeLine (U"   Factor ", ivar, U": ", parm -> label);
+		MelderInfo_writeLine (U"   Factor ", ivar, U": ", parm -> label.get());
 	}
 	MelderInfo_writeLine (U"Fitted coefficients:");
 	MelderInfo_writeLine (U"   Intercept: ", intercept);
 	for (integer ivar = 1; ivar <= our parameters.size; ivar ++) {
 		RegressionParameter parm = our parameters.at [ivar];
-		MelderInfo_writeLine (U"   Coefficient of factor ", parm -> label, U": ", parm -> value);
+		MelderInfo_writeLine (U"   Coefficient of factor ", parm -> label.get(), U": ", parm -> value);
 	}
 	MelderInfo_writeLine (U"Ranges of values:");
 	for (integer ivar = 1; ivar <= our parameters.size; ivar ++) {
 		RegressionParameter parm = our parameters.at [ivar];
-		MelderInfo_writeLine (U"   Range of factor ", parm -> label, U": minimum ",
+		MelderInfo_writeLine (U"   Range of factor ", parm -> label.get(), U": minimum ",
 			parm -> minimum, U", maximum ", parm -> maximum);
 	}
 }
@@ -68,7 +68,7 @@ void Regression_init (Regression me) {
 	//my parameters = Ordered_create ();
 }
 
-void Regression_addParameter (Regression me, const char32 *label, double minimum, double maximum, double value) {
+void Regression_addParameter (Regression me, conststring32 label, double minimum, double maximum, double value) {
 	try {
 		autoRegressionParameter thee = Thing_new (RegressionParameter);
 		thy label = Melder_dup (label);
@@ -81,10 +81,10 @@ void Regression_addParameter (Regression me, const char32 *label, double minimum
 	}
 }
 
-integer Regression_getFactorIndexFromFactorName_e (Regression me, const char32 *factorName) {
+integer Regression_getFactorIndexFromFactorName_e (Regression me, conststring32 factorName) {
 	for (integer iparm = 1; iparm <= my parameters.size; iparm ++) {
 		RegressionParameter parm = my parameters.at [iparm];
-		if (Melder_equ (factorName, parm -> label)) return iparm;
+		if (Melder_equ (factorName, parm -> label.get())) return iparm;
 	}
 	Melder_throw (me, U" has no parameter named \"", factorName, U"\".");
 }
@@ -112,14 +112,13 @@ autoLinearRegression Table_to_LinearRegression (Table me) {
 		if (numberOfCells < numberOfParameters) {
 			Melder_warning (U"Solution is not unique (more parameters than cases).");
 		}
-		autoNUMmatrix <double> u (1, numberOfCells, 1, numberOfParameters);
-		autoNUMvector <double> b (1, numberOfCells);
-		autoNUMvector <double> x (1, numberOfParameters);
+		autoMAT u = newMATraw (numberOfCells, numberOfParameters);
+		autoVEC b = newVECraw (numberOfCells);
 		autoLinearRegression thee = LinearRegression_create ();
 		for (integer ivar = 1; ivar <= numberOfIndependentVariables; ivar ++) {
 			double minimum = Table_getMinimum (me, ivar);
 			double maximum = Table_getMaximum (me, ivar);
-			Regression_addParameter (thee.get(), my columnHeaders [ivar]. label, minimum, maximum, 0.0);
+			Regression_addParameter (thee.get(), my columnHeaders [ivar]. label.get(), minimum, maximum, 0.0);
 		}
 		for (integer icell = 1; icell <= numberOfCells; icell ++) {
 			for (integer ivar = 1; ivar < numberOfParameters; ivar ++) {
@@ -128,7 +127,7 @@ autoLinearRegression Table_to_LinearRegression (Table me) {
 			u [icell] [numberOfParameters] = 1.0;   // for the intercept
 			b [icell] = Table_getNumericValue_Assert (me, icell, my numberOfColumns);   // the dependent variable
 		}
-		NUMsolveEquation (u.peek(), numberOfCells, numberOfParameters, b.peek(), NUMeps * numberOfCells, x.peek());
+		autoVEC x = NUMsolveEquation (u.get(), b.get(), NUMeps * numberOfCells);
 		thy intercept = x [numberOfParameters];
 		for (integer ivar = 1; ivar <= numberOfIndependentVariables; ivar ++) {
 			RegressionParameter parm = thy parameters.at [ivar];
