@@ -40,76 +40,76 @@
 Thing_implement (TableOfReal, Daata, 0);
 Thing_implement (TableOfRealList, Ordered, 0);
 
-static void fprintquotedstring (MelderFile file, const char32 *s) {
+static void fprintquotedstring (MelderFile file, conststring32 s) {
 	MelderFile_writeCharacter (file, U'\"');
 	if (s) { char32 c; while ((c = *s ++) != U'\0') { MelderFile_writeCharacter (file, c); if (c == U'\"') MelderFile_writeCharacter (file, c); } }
 	MelderFile_writeCharacter (file, U'\"');
 }
 
 void structTableOfReal :: v_writeText (MelderFile file) {
-	texputi32 (file, numberOfColumns, U"numberOfColumns", 0,0,0,0,0);
+	texputi32 (file, our numberOfColumns, U"numberOfColumns");
 	MelderFile_write (file, U"\ncolumnLabels []: ");
-	if (numberOfColumns < 1) MelderFile_write (file, U"(empty)");
+	if (our numberOfColumns < 1) MelderFile_write (file, U"(empty)");
 	MelderFile_write (file, U"\n");
-	for (integer i = 1; i <= numberOfColumns; i ++) {
-		fprintquotedstring (file, columnLabels [i]);
+	for (integer i = 1; i <= our numberOfColumns; i ++) {
+		fprintquotedstring (file, our columnLabels [i].get());
 		MelderFile_writeCharacter (file, U'\t');
 	}
-	texputi32 (file, numberOfRows, U"numberOfRows", 0,0,0,0,0);
-	for (integer i = 1; i <= numberOfRows; i ++) {
+	texputi32 (file, our numberOfRows, U"numberOfRows");
+	for (integer i = 1; i <= our numberOfRows; i ++) {
 		MelderFile_write (file, U"\nrow [", i, U"]: ");
-		fprintquotedstring (file, rowLabels [i]);
-		for (integer j = 1; j <= numberOfColumns; j ++) {
-			double x = data [i] [j];
+		fprintquotedstring (file, our rowLabels [i].get());
+		for (integer j = 1; j <= our numberOfColumns; j ++) {
+			double x = our data [i] [j];
 			MelderFile_write (file, U"\t", x);
 		}
 	}
 }
 
 void structTableOfReal :: v_readText (MelderReadText a_text, int /*formatVersion*/) {
-	numberOfColumns = texgeti32 (a_text);
-	if (numberOfColumns >= 1) {
-		columnLabels = NUMvector <char32*> (1, numberOfColumns);
-		for (integer i = 1; i <= numberOfColumns; i ++)
-			columnLabels [i] = texgetw16 (a_text);
+	our numberOfColumns = texgeti32 (a_text);
+	if (our numberOfColumns >= 1) {
+		our columnLabels = autostring32vector (our numberOfColumns);
+		for (integer i = 1; i <= our numberOfColumns; i ++)
+			our columnLabels [i] = texgetw16 (a_text);
 	}
-	numberOfRows = texgeti32 (a_text);
-	if (numberOfRows >= 1) {
-		rowLabels = NUMvector <char32*> (1, numberOfRows);
+	our numberOfRows = texgeti32 (a_text);
+	if (our numberOfRows >= 1) {
+		our rowLabels = autostring32vector (our numberOfRows);
 	}
-	if (numberOfRows >= 1 && numberOfColumns >= 1) {
-		data = NUMmatrix <double> (1, numberOfRows, 1, numberOfColumns);
-		for (integer i = 1; i <= numberOfRows; i ++) {
-			rowLabels [i] = texgetw16 (a_text);
-			for (integer j = 1; j <= numberOfColumns; j ++)
-				data [i] [j] = texgetr64 (a_text);
+	if (our numberOfRows >= 1 && our numberOfColumns >= 1) {
+		our data = newMATzero (our numberOfRows, our numberOfColumns);
+		for (integer i = 1; i <= our numberOfRows; i ++) {
+			our rowLabels [i] = texgetw16 (a_text);
+			for (integer j = 1; j <= our numberOfColumns; j ++)
+				our data [i] [j] = texgetr64 (a_text);
 		}
 	}
 }
 
 void structTableOfReal :: v_info () {
 	structDaata :: v_info ();
-	MelderInfo_writeLine (U"Number of rows: ", numberOfRows);
-	MelderInfo_writeLine (U"Number of columns: ", numberOfColumns);
+	MelderInfo_writeLine (U"Number of rows: ", our numberOfRows);
+	MelderInfo_writeLine (U"Number of columns: ", our numberOfColumns);
 }
 
-const char32 * structTableOfReal :: v_getRowStr (integer irow) {
-	if (irow < 1 || irow > numberOfRows) return nullptr;
-	return rowLabels [irow] ? rowLabels [irow] : U"";
+conststring32 structTableOfReal :: v_getRowStr (integer irow) {
+	if (irow < 1 || irow > our numberOfRows) return nullptr;
+	return our rowLabels [irow] ? our rowLabels [irow].get() : U"";
 }
-const char32 * structTableOfReal :: v_getColStr (integer icol) {
-	if (icol < 1 || icol > numberOfColumns) return nullptr;
-	return columnLabels [icol] ? columnLabels [icol] : U"";
+conststring32 structTableOfReal :: v_getColStr (integer icol) {
+	if (icol < 1 || icol > our numberOfColumns) return nullptr;
+	return our columnLabels [icol] ? our columnLabels [icol].get() : U"";
 }
 double structTableOfReal :: v_getMatrix (integer irow, integer icol) {
-	if (irow < 1 || irow > numberOfRows) return undefined;
-	if (icol < 1 || icol > numberOfColumns) return undefined;
-	return data [irow] [icol];
+	if (irow < 1 || irow > our numberOfRows) return undefined;
+	if (icol < 1 || icol > our numberOfColumns) return undefined;
+	return our data [irow] [icol];
 }
-double structTableOfReal :: v_getRowIndex (const char32 *rowLabel) {
+double structTableOfReal :: v_getRowIndex (conststring32 rowLabel) {
 	return TableOfReal_rowLabelToIndex (this, rowLabel);
 }
-double structTableOfReal :: v_getColIndex (const char32 *columnLabel) {
+double structTableOfReal :: v_getColIndex (conststring32 columnLabel) {
 	return TableOfReal_columnLabelToIndex (this, columnLabel);
 }
 
@@ -118,9 +118,10 @@ void TableOfReal_init (TableOfReal me, integer numberOfRows, integer numberOfCol
 		Melder_throw (U"Cannot create cell-less table.");
 	my numberOfRows = numberOfRows;
 	my numberOfColumns = numberOfColumns;
-	my rowLabels = NUMvector <char32*> (1, numberOfRows);
-	my columnLabels = NUMvector <char32*> (1, numberOfColumns);
-	my data = NUMmatrix <double> (1, my numberOfRows, 1, my numberOfColumns);
+	my rowLabels = autostring32vector (numberOfRows);
+	Melder_assert (my rowLabels.size == numberOfRows);   // probably captured by test script
+	my columnLabels = autostring32vector (numberOfColumns);
+	my data = newMATzero (my numberOfRows, my numberOfColumns);
 }
 
 autoTableOfReal TableOfReal_create (integer numberOfRows, integer numberOfColumns) {
@@ -135,16 +136,16 @@ autoTableOfReal TableOfReal_create (integer numberOfRows, integer numberOfColumn
 
 /***** QUERY *****/
 
-integer TableOfReal_rowLabelToIndex (TableOfReal me, const char32 *label) {
+integer TableOfReal_rowLabelToIndex (TableOfReal me, conststring32 label) {
 	for (integer irow = 1; irow <= my numberOfRows; irow ++)
-		if (my rowLabels [irow] && str32equ (my rowLabels [irow], label))
+		if (my rowLabels [irow] && str32equ (my rowLabels [irow].get(), label))
 			return irow;
 	return 0;
 }
 
-integer TableOfReal_columnLabelToIndex (TableOfReal me, const char32 *label) {
+integer TableOfReal_columnLabelToIndex (TableOfReal me, conststring32 label) {
 	for (integer icol = 1; icol <= my numberOfColumns; icol ++)
-		if (my columnLabels [icol] && str32equ (my columnLabels [icol], label))
+		if (my columnLabels [icol] && str32equ (my columnLabels [icol].get(), label))
 			return icol;
 	return 0;
 }
@@ -174,25 +175,24 @@ double TableOfReal_getColumnStdev (TableOfReal me, integer columnNumber) {
 
 void TableOfReal_removeRow (TableOfReal me, integer rowNumber) {
 	try {
-		if (my numberOfRows == 1)
-			Melder_throw (me, U" has only one row, and a TableOfReal without rows cannot exist.");
-		if (rowNumber < 1 || rowNumber > my numberOfRows)
-			Melder_throw (U"No row ", rowNumber, U".");
-		autoNUMmatrix <double> data (1, my numberOfRows - 1, 1, my numberOfColumns);
+		Melder_require (my numberOfRows > 1,
+			me, U" has only one row, and a TableOfReal without rows cannot exist.");
+		Melder_require (rowNumber > 0 && rowNumber <= my numberOfRows,
+			U"No row ", rowNumber, U".");
+		autoMAT newData = newMATraw (my numberOfRows - 1, my numberOfColumns);
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			for (integer irow = 1; irow < rowNumber; irow ++)
-				data [irow] [icol] = my data [irow] [icol];
+				newData [irow] [icol] = my data [irow] [icol];
 			for (integer irow = rowNumber; irow < my numberOfRows; irow ++)
-				data [irow] [icol] = my data [irow + 1] [icol];
+				newData [irow] [icol] = my data [irow + 1] [icol];
 		}
 		/*
 			Change without error.
 		*/
-		Melder_free (my rowLabels [rowNumber]);
 		for (integer irow = rowNumber; irow < my numberOfRows; irow ++)
-			my rowLabels [irow] = my rowLabels [irow + 1];
-		NUMmatrix_free (my data, 1, 1);
-		my data = data.transfer();
+			my rowLabels [irow] = my rowLabels [irow + 1]. move();
+		my rowLabels [my numberOfRows]. reset();
+		my data = newData.move();
 		my numberOfRows --;
 	} catch (MelderError) {
 		Melder_throw (me, U": row ", rowNumber, U" not removed.");
@@ -203,25 +203,26 @@ void TableOfReal_insertRow (TableOfReal me, integer rowNumber) {
 	try {
 		if (rowNumber < 1 || rowNumber > my numberOfRows + 1)
 			Melder_throw (U"Cannot create row ", rowNumber, U".");
-		autoNUMmatrix <double> data (1, my numberOfRows + 1, 1, my numberOfColumns);
-		autoNUMvector <char32 *> rowLabels (1, my numberOfRows + 1);   // not autostringvector...
+		/*
+			Create without change.
+		*/
+		autoMAT newData = newMATzero (my numberOfRows + 1, my numberOfColumns);
+		autostring32vector newRowLabels (my numberOfRows + 1);
 		for (integer irow = 1; irow < rowNumber; irow ++)	{
-			rowLabels [irow] = my rowLabels [irow];   // ...because this is a dangling copy
+			newRowLabels [irow] = my rowLabels [irow]. move();
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++)
-				data [irow] [icol] = my data [irow] [icol];
+				newData [irow] [icol] = my data [irow] [icol];
 		}
 		for (integer irow = my numberOfRows + 1; irow > rowNumber; irow --) {
-			rowLabels [irow] = my rowLabels [irow - 1];
+			newRowLabels [irow] = my rowLabels [irow - 1]. move();
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++)
-				data [irow] [icol] = my data [irow - 1] [icol];
+				newData [irow] [icol] = my data [irow - 1] [icol];
 		}
 		/*
 			Change without error.
 		*/
-		NUMvector_free (my rowLabels, 1);
-		my rowLabels = rowLabels.transfer();
-		NUMmatrix_free (my data, 1, 1);
-		my data = data.transfer();
+		my rowLabels = std::move (newRowLabels);
+		my data = newData.move();
 		my numberOfRows ++;
 	} catch (MelderError) {
 		Melder_throw (me, U": row at position ", rowNumber, U" not inserted.");
@@ -234,21 +235,23 @@ void TableOfReal_removeColumn (TableOfReal me, integer columnNumber) {
 			Melder_throw (U"Cannot remove the only column.");
 		if (columnNumber < 1 || columnNumber > my numberOfColumns)
 			Melder_throw (U"No column ", columnNumber, U".");
-		autoNUMmatrix <double> data (1, my numberOfRows, 1, my numberOfColumns - 1);
+		/*
+			Create without change.
+		*/
+		autoMAT newData = newMATraw (my numberOfRows, my numberOfColumns - 1);
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
 			for (integer icol = 1; icol < columnNumber; icol ++)
-				data [irow] [icol] = my data [irow] [icol];
+				newData [irow] [icol] = my data [irow] [icol];
 			for (integer icol = columnNumber; icol < my numberOfColumns; icol ++)
-				data [irow] [icol] = my data [irow] [icol + 1];
+				newData [irow] [icol] = my data [irow] [icol + 1];
 		}
 		/*
 			Change without error.
 		*/
-		Melder_free (my columnLabels [columnNumber]);
 		for (integer icol = columnNumber; icol < my numberOfColumns; icol ++)
-			my columnLabels [icol] = my columnLabels [icol + 1];
-		NUMmatrix_free (my data, 1, 1);
-		my data = data.transfer();
+			my columnLabels [icol] = my columnLabels [icol + 1]. move();
+		my columnLabels [my numberOfColumns]. reset();
+		my data = newData.move();
 		my numberOfColumns --;
 	} catch (MelderError) {
 		Melder_throw (me, U": column at position ", columnNumber, U" not inserted.");
@@ -259,64 +262,59 @@ void TableOfReal_insertColumn (TableOfReal me, integer columnNumber) {
 	try {
 		if (columnNumber < 1 || columnNumber > my numberOfColumns + 1)
 			Melder_throw (U"Cannot create column ", columnNumber, U".");
-		autoNUMmatrix <double> data (1, my numberOfRows, 1, my numberOfColumns + 1);
-		autoNUMvector <char32*> columnLabels (1, my numberOfColumns + 1);   // not autostringvector...
+		/*
+			Create without change.
+		*/
+		autoMAT newData = newMATzero (my numberOfRows, my numberOfColumns + 1);
+		autostring32vector newColumnLabels (my numberOfColumns + 1);
 		for (integer j = 1; j < columnNumber; j ++) {
-			columnLabels [j] = my columnLabels [j];   // ...because this is a dangling copy
-			for (integer i = 1; i <= my numberOfRows; i ++) data [i] [j] = my data [i] [j];
+			newColumnLabels [j] = my columnLabels [j]. move();
+			for (integer i = 1; i <= my numberOfRows; i ++)
+				newData [i] [j] = my data [i] [j];
 		}
 		for (integer j = my numberOfColumns + 1; j > columnNumber; j --) {
-			columnLabels [j] = my columnLabels [j - 1];
-			for (integer i = 1; i <= my numberOfRows; i ++) data [i] [j] = my data [i] [j - 1];
+			newColumnLabels [j] = my columnLabels [j - 1]. move();
+			for (integer i = 1; i <= my numberOfRows; i ++)
+				newData [i] [j] = my data [i] [j - 1];
 		}
 		/*
 			Change without error.
 		*/
-		NUMvector_free (my columnLabels, 1);
-		my columnLabels = columnLabels.transfer();
-		NUMmatrix_free (my data, 1, 1);
-		my data = data.transfer();
+		my columnLabels = std::move (newColumnLabels);
+		my data = newData.move();
 		my numberOfColumns ++;
 	} catch (MelderError) {
 		Melder_throw (me, U": column at position ", columnNumber, U" not inserted.");
 	}
 }
 
-void TableOfReal_setRowLabel (TableOfReal me, integer rowNumber, const char32 *label) {
+void TableOfReal_setRowLabel (TableOfReal me, integer rowNumber, conststring32 label) {
 	try {
 		if (rowNumber < 1 || rowNumber > my numberOfRows) return;
-		autostring32 newLabel = Melder_dup (label);
-		/*
-			Change without error.
-		*/
-		Melder_free (my rowLabels [rowNumber]);
-		my rowLabels [rowNumber] = newLabel.transfer();
+		my rowLabels [rowNumber] = Melder_dup (label);
 	} catch (MelderError) {
 		Melder_throw (me, U": label of row ", rowNumber, U" not set.");
 	}
 }
 
-void TableOfReal_setColumnLabel (TableOfReal me, integer columnNumber, const char32 *label) {
+void TableOfReal_setColumnLabel (TableOfReal me, integer columnNumber, conststring32 label) {
 	try {
-		if (columnNumber < 1 || columnNumber > my numberOfColumns) return;
-		autostring32 newLabel = Melder_dup (label);
-		/*
-			Change without error.
-		*/
-		Melder_free (my columnLabels [columnNumber]);
-		my columnLabels [columnNumber] = newLabel.transfer();
+		if (columnNumber < 1 || columnNumber > my numberOfColumns)
+			return;
+		my columnLabels [columnNumber] = Melder_dup (label);
 	} catch (MelderError) {
 		Melder_throw (me, U": label of column ", columnNumber, U" not set.");
 	}
 }
 
-void TableOfReal_formula (TableOfReal me, const char32 *expression, Interpreter interpreter, TableOfReal thee) {
+void TableOfReal_formula (TableOfReal me, conststring32 expression, Interpreter interpreter, TableOfReal thee) {
 	try {
 		Formula_compile (interpreter, me, expression, kFormula_EXPRESSION_TYPE_NUMERIC, true);
-		if (! thee) thee = me;
+		Formula_Result result;
+		if (! thee)
+			thee = me;
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-				Formula_Result result;
 				Formula_run (irow, icol, & result);
 				thy data [irow] [icol] = result. numericResult;
 			}
@@ -331,35 +329,31 @@ void TableOfReal_formula (TableOfReal me, const char32 *expression, Interpreter
 static void copyRowLabels (TableOfReal me, TableOfReal thee) {
 	Melder_assert (me != thee);
 	Melder_assert (my numberOfRows == thy numberOfRows);
-	for (integer irow = 1; irow <= my numberOfRows; irow ++) {
-		thy rowLabels [irow] = Melder_dup (my rowLabels [irow]);
-	}
+	for (integer irow = 1; irow <= my numberOfRows; irow ++)
+		thy rowLabels [irow] = Melder_dup (my rowLabels [irow].get());
 }
 
 static void copyColumnLabels (TableOfReal me, TableOfReal thee) {
 	Melder_assert (me != thee);
 	Melder_assert (my numberOfColumns == thy numberOfColumns);
-	for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-		thy columnLabels [icol] = Melder_dup (my columnLabels [icol]);
-	}
+	for (integer icol = 1; icol <= my numberOfColumns; icol ++)
+		thy columnLabels [icol] = Melder_dup (my columnLabels [icol].get());
 }
 
 static void copyRow (TableOfReal me, integer myRow, TableOfReal thee, integer thyRow) {
 	Melder_assert (me != thee);
 	Melder_assert (my numberOfColumns == thy numberOfColumns);
-	thy rowLabels [thyRow] = Melder_dup (my rowLabels [myRow]);
-	for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
+	thy rowLabels [thyRow] = Melder_dup (my rowLabels [myRow].get());
+	for (integer icol = 1; icol <= my numberOfColumns; icol ++)
 		thy data [thyRow] [icol] = my data [myRow] [icol];
-	}
 }
 
 static void copyColumn (TableOfReal me, integer myCol, TableOfReal thee, integer thyCol) {
 	Melder_assert (me != thee);
 	Melder_assert (my numberOfRows == thy numberOfRows);
-	thy columnLabels [thyCol] = Melder_dup (my columnLabels [myCol]);
-	for (integer irow = 1; irow <= my numberOfRows; irow ++) {
+	thy columnLabels [thyCol] = Melder_dup (my columnLabels [myCol].get());
+	for (integer irow = 1; irow <= my numberOfRows; irow ++)
 		thy data [irow] [thyCol] = my data [irow] [myCol];
-	}
 }
 
 autoTableOfReal TableOfReal_extractRowsWhereColumn (TableOfReal me, integer column, kMelder_number which, double criterion) {
@@ -385,11 +379,11 @@ autoTableOfReal TableOfReal_extractRowsWhereColumn (TableOfReal me, integer colu
 	}
 }
 
-autoTableOfReal TableOfReal_extractRowsWhereLabel (TableOfReal me, kMelder_string which, const char32 *criterion) {
+autoTableOfReal TableOfReal_extractRowsWhereLabel (TableOfReal me, kMelder_string which, conststring32 criterion) {
 	try {
 		integer n = 0;
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
-			if (Melder_stringMatchesCriterion (my rowLabels [irow], which, criterion, true)) {
+			if (Melder_stringMatchesCriterion (my rowLabels [irow].get(), which, criterion, true)) {
 				n ++;
 			}
 		}
@@ -399,7 +393,7 @@ autoTableOfReal TableOfReal_extractRowsWhereLabel (TableOfReal me, kMelder_strin
 		copyColumnLabels (me, thee.get());
 		n = 0;
 		for (integer irow = 1; irow <= my numberOfRows; irow ++)
-			if (Melder_stringMatchesCriterion (my rowLabels [irow], which, criterion, true))
+			if (Melder_stringMatchesCriterion (my rowLabels [irow].get(), which, criterion, true))
 				copyRow (me, irow, thee.get(), ++ n);
 		return thee;
 	} catch (MelderError) {
@@ -431,11 +425,11 @@ autoTableOfReal TableOfReal_extractColumnsWhereRow (TableOfReal me, integer row,
 	}
 }
 
-autoTableOfReal TableOfReal_extractColumnsWhereLabel (TableOfReal me, kMelder_string which, const char32 *criterion) {
+autoTableOfReal TableOfReal_extractColumnsWhereLabel (TableOfReal me, kMelder_string which, conststring32 criterion) {
 	try {
 		integer n = 0;
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-			if (Melder_stringMatchesCriterion (my columnLabels [icol], which, criterion, true)) {
+			if (Melder_stringMatchesCriterion (my columnLabels [icol].get(), which, criterion, true)) {
 				n ++;
 			}
 		}
@@ -445,7 +439,7 @@ autoTableOfReal TableOfReal_extractColumnsWhereLabel (TableOfReal me, kMelder_st
 		copyRowLabels (me, thee.get());
 		n = 0;
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-			if (Melder_stringMatchesCriterion (my columnLabels [icol], which, criterion, true)) {
+			if (Melder_stringMatchesCriterion (my columnLabels [icol].get(), which, criterion, true)) {
 				copyColumn (me, icol, thee.get(), ++ n);
 			}
 		}
@@ -460,7 +454,7 @@ autoTableOfReal TableOfReal_extractColumnsWhereLabel (TableOfReal me, kMelder_st
  * 1, 4, 2, 3, 4, 5, 6, 7, 4, 3, 3, 4, 5, 4, 3, 2
  * Overlap is allowed. Ranges can go up and down.
  */
-static integer *getElementsOfRanges (const char32 *ranges, integer maximumElement, integer *numberOfElements, const char32 *elementType) {
+static integer *getElementsOfRanges (conststring32 ranges, integer maximumElement, integer *numberOfElements, conststring32 elementType) {
 	/*
 		Count the elements.
 	*/
@@ -539,7 +533,7 @@ static integer *getElementsOfRanges (const char32 *ranges, integer maximumElemen
 	return elements.transfer();
 }
 
-autoTableOfReal TableOfReal_extractRowRanges (TableOfReal me, const char32 *ranges) {
+autoTableOfReal TableOfReal_extractRowRanges (TableOfReal me, conststring32 ranges) {
 	try {
 		integer numberOfElements;
 		autoNUMvector <integer> elements (getElementsOfRanges (ranges, my numberOfRows, & numberOfElements, U"row"), 1);
@@ -553,7 +547,7 @@ autoTableOfReal TableOfReal_extractRowRanges (TableOfReal me, const char32 *rang
 	}
 }
 
-autoTableOfReal TableOfReal_extractColumnRanges (TableOfReal me, const char32 *ranges) {
+autoTableOfReal TableOfReal_extractColumnRanges (TableOfReal me, conststring32 ranges) {
 	try {
 		integer numberOfElements;
 		autoNUMvector <integer> elements (getElementsOfRanges (ranges, my numberOfColumns, & numberOfElements, U"column"), 1);
@@ -567,16 +561,16 @@ autoTableOfReal TableOfReal_extractColumnRanges (TableOfReal me, const char32 *r
 	}
 }
 
-autoTableOfReal TableOfReal_extractRowsWhere (TableOfReal me, const char32 *condition, Interpreter interpreter) {
+autoTableOfReal TableOfReal_extractRowsWhere (TableOfReal me, conststring32 condition, Interpreter interpreter) {
 	try {
 		Formula_compile (interpreter, me, condition, kFormula_EXPRESSION_TYPE_NUMERIC, true);
+		Formula_Result result;
 		/*
 			Count the new number of rows.
 		*/
 		integer numberOfElements = 0;
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-				Formula_Result result;
 				Formula_run (irow, icol, & result);
 				if (result. numericResult != 0.0) {
 					numberOfElements ++;
@@ -584,7 +578,8 @@ autoTableOfReal TableOfReal_extractRowsWhere (TableOfReal me, const char32 *cond
 				}
 			}
 		}
-		if (numberOfElements < 1) Melder_throw (U"No rows match this condition.");
+		if (numberOfElements < 1)
+			Melder_throw (U"No rows match this condition.");
 
 		/*
 			Create room for the result.
@@ -597,7 +592,6 @@ autoTableOfReal TableOfReal_extractRowsWhere (TableOfReal me, const char32 *cond
 		numberOfElements = 0;
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-				Formula_Result result;
 				Formula_run (irow, icol, & result);
 				if (result. numericResult != 0.0) {
 					copyRow (me, irow, thee.get(), ++ numberOfElements);
@@ -611,16 +605,16 @@ autoTableOfReal TableOfReal_extractRowsWhere (TableOfReal me, const char32 *cond
 	}
 }
 
-autoTableOfReal TableOfReal_extractColumnsWhere (TableOfReal me, const char32 *condition, Interpreter interpreter) {
+autoTableOfReal TableOfReal_extractColumnsWhere (TableOfReal me, conststring32 condition, Interpreter interpreter) {
 	try {
 		Formula_compile (interpreter, me, condition, kFormula_EXPRESSION_TYPE_NUMERIC, true);
+		Formula_Result result;
 		/*
 			Count the new number of columns.
 		*/
 		integer numberOfElements = 0;
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			for (integer irow = 1; irow <= my numberOfRows; irow ++) {
-				Formula_Result result;
 				Formula_run (irow, icol, & result);
 				if (result. numericResult != 0.0) {
 					numberOfElements ++;
@@ -628,7 +622,8 @@ autoTableOfReal TableOfReal_extractColumnsWhere (TableOfReal me, const char32 *c
 				}
 			}
 		}
-		if (numberOfElements < 1) Melder_throw (U"No columns match this condition.");
+		if (numberOfElements < 1)
+			Melder_throw (U"No columns match this condition.");
 
 		/*
 			Create room for the result.
@@ -641,7 +636,6 @@ autoTableOfReal TableOfReal_extractColumnsWhere (TableOfReal me, const char32 *c
 		numberOfElements = 0;
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			for (integer irow = 1; irow <= my numberOfRows; irow ++) {
-				Formula_Result result;
 				Formula_run (irow, icol, & result);
 				if (result. numericResult != 0.0) {
 					copyColumn (me, icol, thee.get(), ++ numberOfElements);
@@ -660,10 +654,10 @@ autoTableOfReal TableOfReal_extractColumnsWhere (TableOfReal me, const char32 *c
 autoStrings TableOfReal_extractRowLabelsAsStrings (TableOfReal me) {
 	try {
 		autoStrings thee = Thing_new (Strings);
-		thy strings = NUMvector <char32 *> (1, my numberOfRows);
+		thy strings = autostring32vector (my numberOfRows);
 		thy numberOfStrings = my numberOfRows;
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
-			thy strings [irow] = Melder_dup (my rowLabels [irow] ? my rowLabels [irow] : U"");
+			thy strings [irow] = Melder_dup (my rowLabels [irow] ? my rowLabels [irow].get() : U"");
 		}
 		return thee;
 	} catch (MelderError) {
@@ -674,10 +668,10 @@ autoStrings TableOfReal_extractRowLabelsAsStrings (TableOfReal me) {
 autoStrings TableOfReal_extractColumnLabelsAsStrings (TableOfReal me) {
 	try {
 		autoStrings thee = Thing_new (Strings);
-		thy strings = NUMvector <char32 *> (1, my numberOfColumns);
+		thy strings = autostring32vector (my numberOfColumns);
 		thy numberOfStrings = my numberOfColumns;
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-			thy strings [icol] = Melder_dup (my columnLabels [icol] ? my columnLabels [icol] : U"");
+			thy strings [icol] = Melder_dup (my columnLabels [icol] ? my columnLabels [icol].get() : U"");
 		}
 		return thee;
 	} catch (MelderError) {
@@ -733,7 +727,7 @@ static double getMaxRowLabelWidth (TableOfReal me, Graphics graphics, integer ro
 	if (! my rowLabels) return 0.0;
 	fixRows (me, & rowmin, & rowmax);
 	for (integer irow = rowmin; irow <= rowmax; irow ++) if (my rowLabels [irow] && my rowLabels [irow] [0]) {
-		double textWidth = Graphics_textWidth_ps (graphics, my rowLabels [irow], true);   // SILIPA is bigger than XIPA
+		double textWidth = Graphics_textWidth_ps (graphics, my rowLabels [irow].get(), true);   // SILIPA is bigger than XIPA
 		if (textWidth > maxWidth) maxWidth = textWidth;
 	}
 	return maxWidth;
@@ -766,13 +760,13 @@ void TableOfReal_drawAsNumbers (TableOfReal me, Graphics graphics, integer rowmi
 	Graphics_setTextAlignment (graphics, Graphics_CENTRE, Graphics_BOTTOM);
 	for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 		if (my columnLabels && my columnLabels [icol] && my columnLabels [icol] [0])
-			Graphics_text (graphics, icol, 1, my columnLabels [icol]);
+			Graphics_text (graphics, icol, 1, my columnLabels [icol].get());
 	}
 	for (integer irow = rowmin; irow <= rowmax; irow ++) {
 		double y = 1.0 - lineSpacing * (irow - rowmin + 0.6);
 		Graphics_setTextAlignment (graphics, Graphics_RIGHT, Graphics_HALF);
 		if (my rowLabels && my rowLabels [irow] && my rowLabels [irow] [0])
-			Graphics_text (graphics, 0.5 - leftMargin, y, my rowLabels [irow]);
+			Graphics_text (graphics, 0.5 - leftMargin, y, my rowLabels [irow].get());
 		Graphics_setTextAlignment (graphics, Graphics_CENTRE, Graphics_HALF);
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			char text [40];
@@ -789,7 +783,7 @@ void TableOfReal_drawAsNumbers (TableOfReal me, Graphics graphics, integer rowmi
 }
 
 void TableOfReal_drawAsNumbers_if (TableOfReal me, Graphics graphics, integer rowmin, integer rowmax, int iformat, int precision,
-	const char32 *conditionFormula, Interpreter interpreter)
+	conststring32 conditionFormula, Interpreter interpreter)
 {
 	try {
 		autoMatrix original = TableOfReal_to_Matrix (me);
@@ -806,13 +800,13 @@ void TableOfReal_drawAsNumbers_if (TableOfReal me, Graphics graphics, integer ro
 		Graphics_setTextAlignment (graphics, Graphics_CENTRE, Graphics_BOTTOM);
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			if (my columnLabels && my columnLabels [icol] && my columnLabels [icol] [0])
-				Graphics_text (graphics, icol, 1, my columnLabels [icol]);
+				Graphics_text (graphics, icol, 1, my columnLabels [icol].get());
 		}
 		for (integer irow = rowmin; irow <= rowmax; irow ++) {
 			double y = 1.0 - lineSpacing * (irow - rowmin + 0.6);
 			Graphics_setTextAlignment (graphics, Graphics_RIGHT, Graphics_HALF);
 			if (my rowLabels && my rowLabels [irow] && my rowLabels [irow] [0])
-				Graphics_text (graphics, 0.5 - leftMargin, y, my rowLabels [irow]);
+				Graphics_text (graphics, 0.5 - leftMargin, y, my rowLabels [irow].get());
 			Graphics_setTextAlignment (graphics, Graphics_CENTRE, Graphics_HALF);
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++) if (conditions -> z [irow] [icol] != 0.0) {
 				char text [40];
@@ -938,9 +932,9 @@ void TableOfReal_drawAsSquares (TableOfReal me, Graphics graphics, integer rowmi
 	Graphics_unsetInner (graphics);
 	if (garnish) {
 		for (integer irow = rowmin; irow <= rowmax; irow ++) if (my rowLabels [irow])
-			Graphics_markLeft (graphics, rowmax + rowmin - irow, false, false, false, my rowLabels [irow]);
+			Graphics_markLeft (graphics, rowmax + rowmin - irow, false, false, false, my rowLabels [irow].get());
 		for (integer icol = colmin; icol <= colmax; icol ++) if (my columnLabels [icol])
-			Graphics_markTop (graphics, icol, false, false, false, my columnLabels [icol]);
+			Graphics_markTop (graphics, icol, false, false, false, my columnLabels [icol].get());
 	}
 }
 
@@ -952,16 +946,16 @@ autoTableOfReal TablesOfReal_append (TableOfReal me, TableOfReal thee) {
 		TableOfReal_init (him.get(), my numberOfRows + thy numberOfRows, my numberOfColumns);
 		/* Unsafe: new attributes not initialized. */
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-			TableOfReal_setColumnLabel (him.get(), icol, my columnLabels [icol]);
+			TableOfReal_setColumnLabel (him.get(), icol, my columnLabels [icol].get());
 		}
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
-			TableOfReal_setRowLabel (him.get(), irow, my rowLabels [irow]);
+			TableOfReal_setRowLabel (him.get(), irow, my rowLabels [irow].get());
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++)
 				his data [irow] [icol] = my data [irow] [icol];
 		}
 		for (integer irow = 1; irow <= thy numberOfRows; irow ++) {
 			integer hisRow = irow + my numberOfRows;
-			TableOfReal_setRowLabel (him.get(), hisRow, thy rowLabels [irow]);
+			TableOfReal_setRowLabel (him.get(), hisRow, thy rowLabels [irow].get());
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++)
 				his data [hisRow] [icol] = thy data [irow] [icol];
 		}
@@ -986,14 +980,14 @@ autoTableOfReal TablesOfReal_appendMany (OrderedOf<structTableOfReal>* me) {
 		TableOfReal_init (him.get(), totalNumberOfRows, numberOfColumns);
 		/* Unsafe: new attributes not initialized. */
 		for (integer icol = 1; icol <= numberOfColumns; icol ++) {
-			TableOfReal_setColumnLabel (him.get(), icol, thy columnLabels [icol]);
+			TableOfReal_setColumnLabel (him.get(), icol, thy columnLabels [icol].get());
 		}
 		totalNumberOfRows = 0;
 		for (integer itab = 1; itab <= my size; itab ++) {
 			thee = my at [itab];
 			for (integer irow = 1; irow <= thy numberOfRows; irow ++) {
 				totalNumberOfRows ++;
-				TableOfReal_setRowLabel (him.get(), totalNumberOfRows, thy rowLabels [irow]);
+				TableOfReal_setRowLabel (him.get(), totalNumberOfRows, thy rowLabels [irow].get());
 				for (integer icol = 1; icol <= numberOfColumns; icol ++)
 					his data [totalNumberOfRows] [icol] = thy data [irow] [icol];
 			}
@@ -1007,35 +1001,40 @@ autoTableOfReal TablesOfReal_appendMany (OrderedOf<structTableOfReal>* me) {
 
 static void TableOfReal_sort (TableOfReal me, bool useLabels, integer column1, integer column2) {
 	for (integer irow = 1; irow < my numberOfRows; irow ++) for (integer jrow = irow + 1; jrow <= my numberOfRows; jrow ++) {
-		char32 *tmpString;
 		if (useLabels) {
 			if (my rowLabels [irow]) {
 				if (my rowLabels [jrow]) {
-					int compare = str32cmp (my rowLabels [irow], my rowLabels [jrow]);
-					if (compare < 0) continue;
-					if (compare > 0) goto swap;
-				} else goto swap;
+					int compare = str32cmp (my rowLabels [irow].get(), my rowLabels [jrow].get());
+					if (compare < 0)
+						continue;
+					if (compare > 0)
+						goto swap;
+				} else {
+					goto swap;
+				}
 			} else if (my rowLabels [jrow]) continue;
 		}
 		/*
 		 * If we arrive here, the two labels are equal or both null (or useLabels is `false`).
 		 */
 		if (column1 > 0 && column1 <= my numberOfColumns) {
-			if (my data [irow] [column1] < my data [jrow] [column1]) continue;
-			if (my data [irow] [column1] > my data [jrow] [column1]) goto swap;
+			if (my data [irow] [column1] < my data [jrow] [column1])
+				continue;
+			if (my data [irow] [column1] > my data [jrow] [column1])
+				goto swap;
 		}
 		if (column2 > 0 && column2 <= my numberOfColumns) {
-			if (my data [irow] [column2] < my data [jrow] [column2]) continue;
-			if (my data [irow] [column2] > my data [jrow] [column2]) goto swap;
+			if (my data [irow] [column2] < my data [jrow] [column2])
+				continue;
+			if (my data [irow] [column2] > my data [jrow] [column2])
+				goto swap;
 		}
 		/*
 		 * If we arrive here, everything is equal.
 		 */
 		continue;
 	swap:
-		tmpString = my rowLabels [irow];
-		my rowLabels [irow] = my rowLabels [jrow];
-		my rowLabels [jrow] = tmpString;
+		std::swap (my rowLabels [irow], my rowLabels [jrow]);
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			double tmpValue = my data [irow] [icol];
 			my data [irow] [icol] = my data [jrow] [icol];
@@ -1061,14 +1060,14 @@ autoTableOfReal Table_to_TableOfReal (Table me, integer labelColumn) {
 		}
 		if (labelColumn) {
 			for (integer icol = 1; icol < labelColumn; icol ++) {
-				TableOfReal_setColumnLabel (thee.get(), icol, my columnHeaders [icol]. label);
+				TableOfReal_setColumnLabel (thee.get(), icol, my columnHeaders [icol]. label.get());
 			}
 			for (integer icol = labelColumn + 1; icol <= my numberOfColumns; icol ++) {
-				TableOfReal_setColumnLabel (thee.get(), icol - 1, my columnHeaders [icol]. label);
+				TableOfReal_setColumnLabel (thee.get(), icol - 1, my columnHeaders [icol]. label.get());
 			}
 			for (integer irow = 1; irow <= my rows.size; irow ++) {
 				TableRow row = my rows.at [irow];
-				char32 *string = row -> cells [labelColumn]. string;
+				char32 *string = row -> cells [labelColumn]. string.get();
 				TableOfReal_setRowLabel (thee.get(), irow, string ? string : U"");
 				for (integer icol = 1; icol < labelColumn; icol ++) {
 					thy data [irow] [icol] = row -> cells [icol]. number;   // Optimization.
@@ -1081,7 +1080,7 @@ autoTableOfReal Table_to_TableOfReal (Table me, integer labelColumn) {
 			}
 		} else {
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-				TableOfReal_setColumnLabel (thee.get(), icol, my columnHeaders [icol]. label);
+				TableOfReal_setColumnLabel (thee.get(), icol, my columnHeaders [icol]. label.get());
 			}
 			for (integer irow = 1; irow <= my rows.size; irow ++) {
 				TableRow row = my rows.at [irow];
@@ -1097,16 +1096,16 @@ autoTableOfReal Table_to_TableOfReal (Table me, integer labelColumn) {
 	}
 }
 
-autoTable TableOfReal_to_Table (TableOfReal me, const char32 *labelOfFirstColumn) {
+autoTable TableOfReal_to_Table (TableOfReal me, conststring32 labelOfFirstColumn) {
 	try {
 		autoTable thee = Table_createWithoutColumnNames (my numberOfRows, my numberOfColumns + 1);
 		Table_setColumnLabel (thee.get(), 1, labelOfFirstColumn);
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
-			char32 *columnLabel = my columnLabels [icol];
+			conststring32 columnLabel = my columnLabels [icol].get();
 			thy columnHeaders [icol + 1]. label = Melder_dup (columnLabel && columnLabel [0] ? columnLabel : U"?");
 		}
 		for (integer irow = 1; irow <= thy rows.size; irow ++) {
-			char32 *stringValue = my rowLabels [irow];
+			conststring32 stringValue = my rowLabels [irow].get();
 			TableRow row = thy rows.at [irow];
 			row -> cells [1]. string = Melder_dup (stringValue && stringValue [0] ? stringValue : U"?");
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
@@ -1126,12 +1125,12 @@ void TableOfReal_writeToHeaderlessSpreadsheetFile (TableOfReal me, MelderFile fi
 		MelderString_copy (& buffer, U"rowLabel");
 		for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 			MelderString_appendCharacter (& buffer, U'\t');
-			char32 *s = my columnLabels [icol];
+			conststring32 s = my columnLabels [icol].get();
 			MelderString_append (& buffer, ( s && s [0] != U'\0' ? s : U"?" ));
 		}
 		MelderString_appendCharacter (& buffer, U'\n');
 		for (integer irow = 1; irow <= my numberOfRows; irow ++) {
-			char32 *s = my rowLabels [irow];
+			conststring32 s = my rowLabels [irow].get();
 			MelderString_append (& buffer, ( s && s [0] != U'\0' ? s : U"?" ));
 			for (integer icol = 1; icol <= my numberOfColumns; icol ++) {
 				MelderString_appendCharacter (& buffer, U'\t');
