@@ -1,6 +1,6 @@
 /* DataEditor.cpp
  *
- * Copyright (C) 1995-2012,2015,2016,2017 Paul Boersma
+ * Copyright (C) 1995-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,17 +35,17 @@ static Data_Description Class_getDescription (ClassInfo table) {
 	return ((Daata) _Thing_dummyObject (table)) -> v_description ();
 }
 
-static void VectorEditor_create (DataEditor root, const char32 *title, void *address,
+static void VectorEditor_create (DataEditor root, conststring32 title, void *address,
 	Data_Description description, integer minimum, integer maximum);
 
-static void MatrixEditor_create (DataEditor root, const char32 *title, void *address,
+static void MatrixEditor_create (DataEditor root, conststring32 title, void *address,
 	Data_Description description, integer min1, integer max1, integer min2, integer max2);
 
-static void StructEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description);
+static void StructEditor_create (DataEditor root, conststring32 title, void *address, Data_Description description);
 
-static void ClassEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description);
+static void ClassEditor_create (DataEditor root, conststring32 title, void *address, Data_Description description);
 
-static inline const char32 * strip_d (const char32 *s) {
+static inline conststring32 strip_d (conststring32 s) {
 	return s && s [0] == U'd' && s [1] == U'_' ? & s [2] : & s [0];
 }
 
@@ -54,8 +54,8 @@ static inline const char32 * strip_d (const char32 *s) {
 Thing_implement (DataSubEditor, Editor, 0);
 
 void structDataSubEditor :: v_destroy () noexcept {
-	for (int i = 1; i <= kDataSubEditor_MAXNUM_ROWS; i ++)
-		Melder_free (d_fieldData [i]. history);
+	//for (int i = 1; i <= kDataSubEditor_MAXNUM_ROWS; i ++)
+	//	Melder_free (d_fieldData [i]. history);
 	if (our root)
 		for (int i = our root -> children.size; i > 0; i --)
 			if (our root -> children.at [i] == this)
@@ -79,7 +79,7 @@ static void update (DataSubEditor me) {
 	my v_showMembers ();
 }
 
-static Data_Description DataSubEditor_findNumberUse (DataSubEditor me, const char32 *number) {
+static Data_Description DataSubEditor_findNumberUse (DataSubEditor me, conststring32 number) {
 	Data_Description structDescription, result;
 	char32 string [100];
 	if (my classInfo == classMatrixEditor) return nullptr;   // no structs inside
@@ -111,12 +111,11 @@ static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */)
 		#endif
 		if (visible) {
 			int type = my d_fieldData [irow]. description -> type;
-			char32 *text;
 			if (type > maxsingletypewa) continue;
-			text = GuiText_getString (my d_fieldData [irow]. text);
+			autostring32 text = GuiText_getString (my d_fieldData [irow]. text);
 			switch (type) {
 				case bytewa: {
-					signed char oldValue = * (signed char *) my d_fieldData [irow]. address, newValue = (signed char) Melder_atoi (text);
+					signed char oldValue = * (signed char *) my d_fieldData [irow]. address, newValue = (signed char) Melder_atoi (text.get());
 					if (newValue != oldValue) {
 						Data_Description numberUse = DataSubEditor_findNumberUse (me, my d_fieldData [irow]. description -> name);
 						if (numberUse) {
@@ -129,7 +128,7 @@ static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */)
 				} break;
 				case int16wa: {
 					int16 oldValue = * (int16 *) my d_fieldData [irow]. address;
-					int64 newValue = Melder_atoi (text);
+					int64 newValue = Melder_atoi (text.get());
 					if (newValue != oldValue) {
 						Data_Description numberUse = DataSubEditor_findNumberUse (me, my d_fieldData [irow]. description -> name);
 						if (numberUse) {
@@ -144,7 +143,7 @@ static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */)
 					}
 				} break;
 				case intwa: {
-					int oldValue = * (int *) my d_fieldData [irow]. address, newValue = Melder_atoi (text);
+					int oldValue = * (int *) my d_fieldData [irow]. address, newValue = Melder_atoi (text.get());
 					if (newValue != oldValue) {
 						Data_Description numberUse = DataSubEditor_findNumberUse (me, my d_fieldData [irow]. description -> name);
 						if (numberUse) {
@@ -156,7 +155,7 @@ static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */)
 					}
 				} break;
 				case integerwa: {
-					integer oldValue = * (integer *) my d_fieldData [irow]. address, newValue = Melder_atoi (text);
+					integer oldValue = * (integer *) my d_fieldData [irow]. address, newValue = Melder_atoi (text.get());
 					if (newValue != oldValue) {
 						Data_Description numberUse = DataSubEditor_findNumberUse (me, my d_fieldData [irow]. description -> name);
 						if (numberUse) {
@@ -167,32 +166,32 @@ static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */)
 						}
 					}
 				} break;
-				case ubytewa: { * (unsigned char *) my d_fieldData [irow]. address = (uint8) Melder_atoi (text); } break;
-				case uintwa: { * (unsigned int *) my d_fieldData [irow]. address = Melder_atoi (text); } break;
-				case uintegerwa: { * (uinteger *) my d_fieldData [irow]. address = (uinteger) Melder_atoi (text); } break;
-				case floatwa: { * (double *) my d_fieldData [irow]. address = Melder_atof (text); } break;
-				case doublewa: { * (double *) my d_fieldData [irow]. address = Melder_atof (text); } break;
+				case ubytewa: { * (unsigned char *) my d_fieldData [irow]. address = (uint8) Melder_atoi (text.get()); } break;
+				case uintwa: { * (unsigned int *) my d_fieldData [irow]. address = Melder_atoi (text.get()); } break;
+				case uintegerwa: { * (uinteger *) my d_fieldData [irow]. address = (uinteger) Melder_atoi (text.get()); } break;
+				case floatwa: { * (double *) my d_fieldData [irow]. address = Melder_atof (text.get()); } break;
+				case doublewa: { * (double *) my d_fieldData [irow]. address = Melder_atof (text.get()); } break;
 				case complexwa: { dcomplex *x = (dcomplex *) my d_fieldData [irow]. address;
-					sscanf (Melder_peek32to8 (text), "%lf + %lf i", & x -> re, & x -> im); } break;
+					sscanf (Melder_peek32to8 (text.get()), "%lf + %lf i", & x -> re, & x -> im); } break;
 				case enumwa: {
-					if (str32len (text) < 3) goto error;
-					text [str32len (text) - 1] = '\0';   // remove trailing ">"
-					int value = ((int (*) (const char32 *)) (my d_fieldData [irow]. description -> tagType)) (text + 1);   // skip leading "<"
+					if (str32len (text.get()) < 3) goto error;
+					text [str32len (text.get()) - 1] = '\0';   // remove trailing ">"
+					int value = ((int (*) (conststring32)) (my d_fieldData [irow]. description -> tagType)) (text.get() + 1);   // skip leading "<"
 					if (value < 0) goto error;
 					* (signed char *) my d_fieldData [irow]. address = (signed char) value;
 				} break;
 				case lenumwa: {
-					if (str32len (text) < 3) goto error;
-					text [str32len (text) - 1] = '\0';   // remove trailing ">"
-					int value = ((int (*) (const char32 *)) (my d_fieldData [irow]. description -> tagType)) (text + 1);   // skip leading "<"
+					if (str32len (text.get()) < 3) goto error;
+					text [str32len (text.get()) - 1] = '\0';   // remove trailing ">"
+					int value = ((int (*) (conststring32)) (my d_fieldData [irow]. description -> tagType)) (text.get() + 1);   // skip leading "<"
 					if (value < 0) goto error;
 					* (signed short *) my d_fieldData [irow]. address = (signed short) value;
 				} break;
 				case booleanwa: {
 					bool value;
-					if (str32nequ (text, U"<true>", 6)) {
+					if (str32nequ (text.get(), U"<true>", 6)) {
 						value = true;
-					} else if (str32nequ (text, U"<false>", 7)) {
+					} else if (str32nequ (text.get(), U"<false>", 7)) {
 						value = false;
 					} else {
 						goto error;
@@ -201,9 +200,9 @@ static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */)
 				} break;
 				case questionwa: {
 					bool value;
-					if (str32nequ (text, U"<yes>", 5)) {
+					if (str32nequ (text.get(), U"<yes>", 5)) {
 						value = true;
-					} else if (str32nequ (text, U"<no>", 4)) {
+					} else if (str32nequ (text.get(), U"<no>", 4)) {
 						value = false;
 					} else {
 						goto error;
@@ -214,11 +213,10 @@ static void gui_button_cb_change (DataSubEditor me, GuiButtonEvent /* event */)
 				case lstringwa: {
 					char32 *old = * (char32 **) my d_fieldData [irow]. address;
 					Melder_free (old);
-					* (char32 **) my d_fieldData [irow]. address = Melder_dup_f (text);
+					* (char32 **) my d_fieldData [irow]. address = Melder_dup_f (text.get()).transfer();
 				} break;
 				default: break;
 			}
-			Melder_free (text);
 		}
 	}
 	/* Several collaborators have to be notified of this change:
@@ -265,29 +263,28 @@ static void gui_button_cb_open (DataSubEditor me, GuiButtonEvent event) {
 	}
 
 	if (fieldData -> description -> rank == 1 || fieldData -> description -> rank == 3 || fieldData -> description -> rank < 0) {
-		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name),
+		MelderString_append (& name, fieldData -> history.get(), U". ", strip_d (fieldData -> description -> name),
 			U" [", fieldData -> minimum, U"..", fieldData -> maximum, U"]");
 		VectorEditor_create (my root, name.string, fieldData -> address,
 			fieldData -> description, fieldData -> minimum, fieldData -> maximum);
 	} else if (fieldData -> description -> rank == 2) {
-		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name),
+		MelderString_append (& name, fieldData -> history.get(), U". ", strip_d (fieldData -> description -> name),
 			U" [", fieldData -> minimum, U"..", fieldData -> maximum, U"]");
 		MelderString_append (& name, U" [", fieldData -> min2, U"..", fieldData -> max2, U"]");
 		MatrixEditor_create (my root, name.string, fieldData -> address, fieldData -> description,
 			fieldData -> minimum, fieldData -> maximum, fieldData -> min2, fieldData -> max2);
 	} else if (fieldData -> description -> type == structwa) {
-		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name));
+		MelderString_append (& name, fieldData -> history.get(), U". ", strip_d (fieldData -> description -> name));
 		StructEditor_create (my root, name.string, fieldData -> address,
 			* (Data_Description *) fieldData -> description -> tagType);
 	} else if (fieldData -> description -> type == objectwa ||
-	           fieldData -> description -> type == autoobjectwa ||
 	           fieldData -> description -> type == collectionofwa ||
-			   fieldData -> description -> type == autocollectionwa) {
-		MelderString_append (& name, fieldData -> history, U". ", strip_d (fieldData -> description -> name));
+			   fieldData -> description -> type == collectionwa) {
+		MelderString_append (& name, fieldData -> history.get(), U". ", strip_d (fieldData -> description -> name));
 		ClassEditor_create (my root, name.string, fieldData -> address,
 			Class_getDescription ((ClassInfo) fieldData -> description -> tagType));
 	} else /*if (fieldData -> description -> type == inheritwa)*/ {
-		ClassEditor_create (my root, fieldData -> history, fieldData -> address,
+		ClassEditor_create (my root, fieldData -> history.get(), fieldData -> address,
 			fieldData -> description);
 /*	} else {
 		Melder_casual (
@@ -332,7 +329,7 @@ void structDataSubEditor :: v_createHelpMenuItems (EditorMenu menu) {
 	EditorMenu_addCommand (menu, U"DataEditor help", '?', menu_cb_help);
 }
 
-static void DataSubEditor_init (DataSubEditor me, DataEditor root, const char32 *title, void *address, Data_Description description) {
+static void DataSubEditor_init (DataSubEditor me, DataEditor root, conststring32 title, void *address, Data_Description description) {
 	my root = root;
 	if (me != root) {
 		root -> children.addItem_ref (me);
@@ -353,7 +350,7 @@ integer structStructEditor :: v_countFields () {
 	return Data_Description_countMembers (d_description);
 }
 
-static const char32 * singleTypeToText (void *address, int type, void *tagType, MelderString *buffer) {
+static conststring32 singleTypeToText (void *address, int type, void *tagType, MelderString *buffer) {
 	switch (type) {
 		case bytewa:     MelderString_append (buffer, Melder_integer  (* (signed char *)    address)); break;
 		case int16wa:    MelderString_append (buffer, Melder_integer  (* (int16 *)          address)); break;
@@ -364,9 +361,9 @@ static const char32 * singleTypeToText (void *address, int type, void *tagType,
 		case uintegerwa: MelderString_append (buffer, Melder_integer  (* (uinteger *)       address)); break;
 		case floatwa:    MelderString_append (buffer, Melder_single   (* (double *)         address)); break;
 		case doublewa:   MelderString_append (buffer, Melder_double   (* (double *)         address)); break;
-		case complexwa: MelderString_append (buffer, Melder_dcomplex (* (dcomplex *)       address)); break;
-		case enumwa:  MelderString_append (buffer, U"<", ((const char32 * (*) (int)) tagType) (* (signed char *)  address), U">"); break;
-		case lenumwa: MelderString_append (buffer, U"<", ((const char32 * (*) (int)) tagType) (* (signed short *) address), U">"); break;
+		case complexwa:  MelderString_append (buffer, Melder_dcomplex (* (dcomplex *)       address)); break;
+		case enumwa:     MelderString_append (buffer, U"<", ((conststring32 (*) (int)) tagType) (* (signed char *)  address), U">"); break;
+		case lenumwa:    MelderString_append (buffer, U"<", ((conststring32 (*) (int)) tagType) (* (signed short *) address), U">"); break;
 		case booleanwa:  MelderString_append (buffer, * (bool *) address ? U"<true>" : U"<false>"); break;
 		case questionwa: MelderString_append (buffer, * (bool *) address ? U"<yes>"  : U"<no>"   ); break;
 		case stringwa:
@@ -406,7 +403,7 @@ static void showStructMember (
 			XtVaSetValues (fieldData -> text -> d_widget, XmNcolumns, 60, nullptr);   // TODO: change to GuiObject_size
 		#endif
 		autoMelderString buffer;
-		const char32 *text = singleTypeToText (memberAddress, type, memberDescription -> tagType, & buffer);
+		conststring32 text = singleTypeToText (memberAddress, type, memberDescription -> tagType, & buffer);
 		GuiText_setString (fieldData -> text, text);
 		GuiThing_show (fieldData -> text);
 		fieldData -> address = memberAddress;
@@ -426,7 +423,7 @@ static void showStructMember (
 		fieldData -> minimum = minimum;   // normally 1
 		fieldData -> maximum = maximum;
 		fieldData -> rank = 1;
-		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
 	} else if (rank < 0) {
 		/*
@@ -441,7 +438,7 @@ static void showStructMember (
 		fieldData -> minimum = 0;   /* In-line arrays start with index 0. */
 		fieldData -> maximum = maximum;   /* Probably between -1 and capacity - 1. */
 		fieldData -> rank = rank;
-		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
 	} else if (rank == 3) {
 		/*
@@ -449,10 +446,10 @@ static void showStructMember (
 		 */
 		fieldData -> address = memberAddress;   /* Direct. */
 		fieldData -> description = memberDescription;
-		fieldData -> minimum = str32equ (((const char32 * (*) (int)) memberDescription -> min1) (0), U"_") ? 1 : 0;
-		fieldData -> maximum = ((int (*) (const char32 *)) memberDescription -> max1) (U"\n");
+		fieldData -> minimum = str32equ (((conststring32 (*) (int)) memberDescription -> min1) (0), U"_") ? 1 : 0;
+		fieldData -> maximum = ((int (*) (conststring32)) memberDescription -> max1) (U"\n");
 		fieldData -> rank = rank;
-		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
 	} else if (rank == 2) {
 		void *arrayAddress = * (void **) memberAddress;
@@ -474,20 +471,20 @@ static void showStructMember (
 		fieldData -> min2 = min2;
 		fieldData -> max2 = max2;
 		fieldData -> rank = 2;
-		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
 	} else if (type == structwa) {   // in-line struct
 		fieldData -> address = memberAddress;   // direct
 		fieldData -> description = memberDescription;
 		fieldData -> rank = 0;
-		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
-	} else if (type == objectwa || type == autoobjectwa || type == autocollectionwa) {
+	} else if (type == objectwa || type == collectionwa) {
 		fieldData -> address = * (Daata *) memberAddress;   // indirect  // FIXME: not guaranteed for auto objects
 		if (! fieldData -> address) return;   // no button if no object
 		fieldData -> description = memberDescription;
 		fieldData -> rank = 0;
-		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
 	} else if (type == collectionofwa) {
 		fieldData -> address = (Daata) memberAddress;   // direct  // FIXME: not guaranteed for auto objects
@@ -496,7 +493,7 @@ static void showStructMember (
 		if (! fieldData -> address) return;   // no button if no object
 		fieldData -> description = memberDescription;
 		fieldData -> rank = 0;
-		Melder_free (fieldData -> history); fieldData -> history = Melder_dup_f (history);
+		fieldData -> history = Melder_dup_f (history);
 		GuiThing_show (fieldData -> button);
 	}
 }
@@ -513,14 +510,14 @@ static void showStructMembers (DataSubEditor me, void *structAddress, Data_Descr
 }
 
 void structStructEditor :: v_showMembers () {
-	showStructMembers (this, d_address, d_description, d_topField, name);
+	showStructMembers (this, our d_address, our d_description, our d_topField, our name.get());
 }
 
-static void StructEditor_init (StructEditor me, DataEditor root, const char32 *title, void *address, Data_Description description) {
+static void StructEditor_init (StructEditor me, DataEditor root, conststring32 title, void *address, Data_Description description) {
 	DataSubEditor_init (me, root, title, address, description);
 }
 
-static void StructEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description) {
+static void StructEditor_create (DataEditor root, conststring32 title, void *address, Data_Description description) {
 	try {
 		autoStructEditor me = Thing_new (StructEditor);
 		StructEditor_init (me.get(), root, title, address, description);
@@ -543,28 +540,28 @@ integer structVectorEditor :: v_countFields () {
 }
 
 void structVectorEditor :: v_showMembers () {
-	int type = d_description -> type, isSingleType = type <= maxsingletypewa;
+	int type = our d_description -> type, isSingleType = type <= maxsingletypewa;
 	int elementSize = type == structwa ?
 		Data_Description_countMembers (* (Data_Description *) d_description -> tagType) + 1 : 1;
 	integer firstElement = d_minimum + (d_topField - 1) / elementSize;
 
 	for (integer ielement = firstElement; ielement <= d_maximum; ielement ++) {
-		unsigned char *elementAddress = (unsigned char *) d_address + ielement * d_description -> size;
-		int skip = ielement == firstElement ? (d_topField - 1) % elementSize : 0;
+		unsigned char *elementAddress = (unsigned char *) our d_address + ielement * our d_description -> size;
+		int skip = ielement == firstElement ? (our d_topField - 1) % elementSize : 0;
 
-		if (++ d_irow > kDataSubEditor_MAXNUM_ROWS) return;
-		DataSubEditor_FieldData fieldData = & d_fieldData [d_irow];
+		if (++ our d_irow > kDataSubEditor_MAXNUM_ROWS) return;
+		DataSubEditor_FieldData fieldData = & our d_fieldData [d_irow];
 
 		if (isSingleType) {
 			GuiControl_move (fieldData -> label, 0, fieldData -> y);
 			GuiLabel_setText (fieldData -> label,
-				Melder_cat (strip_d (d_description -> name), U" [",
-					( d_description -> rank == 3 ? ((const char32 * (*) (int)) d_description -> min1) (ielement) : Melder_integer (ielement) ),
+				Melder_cat (strip_d (our d_description -> name), U" [",
+					( our d_description -> rank == 3 ? ((conststring32 (*) (int)) our d_description -> min1) (ielement) : Melder_integer (ielement) ),
 					U"]"));
 			GuiThing_show (fieldData -> label);
 
 			autoMelderString buffer;
-			const char32 *text = singleTypeToText (elementAddress, type, d_description -> tagType, & buffer);
+			conststring32 text = singleTypeToText (elementAddress, type, our d_description -> tagType, & buffer);
 			#if motif
 				XtVaSetValues (fieldData -> text -> d_widget, XmNcolumns, 60, nullptr);   // TODO: change to GuiObject_size
 			#endif
@@ -574,7 +571,7 @@ void structVectorEditor :: v_showMembers () {
 			fieldData -> description = d_description;
 		} else if (type == structwa) {
 			static MelderString history { };
-			MelderString_copy (& history, name);
+			MelderString_copy (& history, our name.get());
 
 			/* Replace things like [1..100] by things like [19]. */
 
@@ -587,7 +584,7 @@ void structVectorEditor :: v_showMembers () {
 			MelderString_append (& history, U"[", ielement, U"]");
 
 			if (skip) {
-				d_irow --;
+				our d_irow --;
 			} else {
 				GuiControl_move (fieldData -> label, 0, fieldData -> y);
 				GuiLabel_setText (fieldData -> label,
@@ -595,9 +592,9 @@ void structVectorEditor :: v_showMembers () {
 				GuiThing_show (fieldData -> label);
 			}
 			showStructMembers (this, elementAddress, * (Data_Description *) d_description -> tagType, skip, history.string);
-		} else if (type == objectwa || type == autoobjectwa) {
+		} else if (type == objectwa) {
 			static MelderString history { };
-			MelderString_copy (& history, name);
+			MelderString_copy (& history, our name.get());
 			if (history.string [history.length - 1] == U']') {
 				char32 *openingBracket = str32rchr (history.string, U'[');
 				Melder_assert (openingBracket != nullptr);
@@ -607,7 +604,7 @@ void structVectorEditor :: v_showMembers () {
 			MelderString_append (& history, U"[", ielement, U"]");
 
 			GuiControl_move (fieldData -> label, 0, fieldData -> y);
-			GuiLabel_setText (fieldData -> label, Melder_cat (strip_d (d_description -> name), U" [", ielement, U"]"));
+			GuiLabel_setText (fieldData -> label, Melder_cat (strip_d (our d_description -> name), U" [", ielement, U"]"));
 			GuiThing_show (fieldData -> label);
 
 			Daata object = * (Daata *) elementAddress;
@@ -616,14 +613,13 @@ void structVectorEditor :: v_showMembers () {
 			fieldData -> address = object;
 			fieldData -> description = Class_getDescription (object -> classInfo);
 			fieldData -> rank = 0;
-			if (fieldData -> history) Melder_free (fieldData -> history);
 			fieldData -> history = Melder_dup_f (history.string);
 			GuiThing_show (fieldData -> button);
 		}
 	}
 }
 
-static void VectorEditor_create (DataEditor root, const char32 *title, void *address,
+static void VectorEditor_create (DataEditor root, conststring32 title, void *address,
 	Data_Description description, integer minimum, integer maximum)
 {
 	try {
@@ -670,7 +666,7 @@ void structMatrixEditor :: v_showMembers () {
 			GuiThing_show (fieldData -> label);
 
 			autoMelderString buffer;
-			const char32 *text = singleTypeToText (elementAddress, type, d_description -> tagType, & buffer);
+			conststring32 text = singleTypeToText (elementAddress, type, d_description -> tagType, & buffer);
 			#if motif
 				XtVaSetValues (fieldData -> text -> d_widget, XmNcolumns, 60, nullptr);   // TODO: change to GuiObject_size
 			#endif
@@ -682,7 +678,7 @@ void structMatrixEditor :: v_showMembers () {
 	}
 }
 
-static void MatrixEditor_create (DataEditor root, const char32 *title, void *address,
+static void MatrixEditor_create (DataEditor root, conststring32 title, void *address,
 	Data_Description description, integer min1, integer max1, integer min2, integer max2)
 {
 	try {
@@ -713,20 +709,20 @@ static void ClassEditor_showMembers_recursive (ClassEditor me, ClassInfo klas) {
 		classFieldsTraversed = Data_Description_countMembers (Class_getDescription (parentClass));
 		//Melder_casual (U"ClassEditor_showMembers_recursive: classFieldsTraversed = ", classFieldsTraversed);
 	}
-	showStructMembers (me, my d_address, description, my d_irow ? 1 : my d_topField - classFieldsTraversed, my name);
+	showStructMembers (me, my d_address, description, my d_irow ? 1 : my d_topField - classFieldsTraversed, my name.get());
 }
 
 void structClassEditor :: v_showMembers () {
 	ClassEditor_showMembers_recursive (this, ((Daata) d_address) -> classInfo);
 }
 
-static void ClassEditor_init (ClassEditor me, DataEditor root, const char32 *title, void *address, Data_Description description) {
+static void ClassEditor_init (ClassEditor me, DataEditor root, conststring32 title, void *address, Data_Description description) {
 	if (! description)
 		Melder_throw (U"Class ", Thing_className ((Thing) address), U" cannot be inspected.");
 	StructEditor_init (me, root, title, address, description);
 }
 
-static void ClassEditor_create (DataEditor root, const char32 *title, void *address, Data_Description description) {
+static void ClassEditor_create (DataEditor root, conststring32 title, void *address, Data_Description description) {
 	try {
 		autoClassEditor me = Thing_new (ClassEditor);
 		ClassEditor_init (me.get(), root, title, address, description);
@@ -810,7 +806,7 @@ void structDataEditor :: v_dataChanged () {
 	DataEditor_destroyAllChildren (this);
 }
 
-autoDataEditor DataEditor_create (const char32 *title, Daata data) {
+autoDataEditor DataEditor_create (conststring32 title, Daata data) {
 	try {
 		ClassInfo klas = data -> classInfo;
 		if (Class_getDescription (klas) == nullptr)
