@@ -19,7 +19,7 @@
  */
 
 #include "Thing.h"
-#include "longchar.h"
+#include "../kar/longchar.h"
 
 #include "Graphics_enums.h"
 
@@ -178,45 +178,38 @@ void Graphics_setWindow (Graphics me, double x1, double x2, double y1, double y2
 
 void Graphics_polyline (Graphics me, integer numberOfPoints, double *x, double *y);
 void Graphics_polyline_closed (Graphics me, integer numberOfPoints, double *x, double *y);
-void Graphics_text (Graphics me, double x, double y, Melder_1_ARG);
-void Graphics_text (Graphics me, double x, double y, Melder_2_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_3_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_4_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_5_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_6_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_7_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_8_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_9_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_10_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_11_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_12_OR_13_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_14_OR_15_ARGS);
-void Graphics_text (Graphics me, double x, double y, Melder_16_TO_19_ARGS);
-void Graphics_textRect (Graphics me, double x1, double x2, double y1, double y2, const char32 *text /* cattable */);
-double Graphics_textWidth       (Graphics me, const char32 *text /* cattable */);
-double Graphics_textWidth_ps    (Graphics me, const char32 *text /* cattable */, bool useSilipaPS);
-double Graphics_textWidth_ps_mm (Graphics me, const char32 *text /* cattable */, bool useSilipaPS);
-void Graphics_fillArea (Graphics me, integer numberOfPoints, double *x, double *y);
-void Graphics_cellArray (Graphics me, double **z, integer ix1, integer ix2, double x1, double x2,
-	integer iy1, integer iy2, double y1, double y2, double minimum, double maximum);
-void Graphics_cellArray_colour (Graphics me, double_rgbt **z, integer ix1, integer ix2, double x1, double x2,
-	integer iy1, integer iy2, double y1, double y2, double minimum, double maximum);
-void Graphics_cellArray8 (Graphics me, unsigned char **z, integer ix1, integer ix2, double x1, double x2,
-	integer iy1, integer iy2, double y1, double y2, unsigned char minimum, unsigned char maximum);
-void Graphics_image (Graphics me, double **z, integer ix1, integer ix2, double x1, double x2,
-	integer iy1, integer iy2, double y1, double y2, double minimum, double maximum);
-void Graphics_image_colour (Graphics me, double_rgbt **z, integer ix1, integer ix2, double x1, double x2,
-	integer iy1, integer iy2, double y1, double y2, double minimum, double maximum);
-void Graphics_image8 (Graphics me, unsigned char **z, integer ix1, integer ix2, double x1, double x2,
-	integer iy1, integer iy2, double y1, double y2, uint8 minimum, uint8 maximum);
-void Graphics_imageFromFile (Graphics me, const char32 *relativeFileName, double x1, double x2, double y1, double y2);
+
+void Graphics_text (Graphics me, double xWC, double yWC, conststring32 txt);
+template <typename... Args>
+void Graphics_text (Graphics me, double xWC, double yWC, const MelderArg& first, Args... rest) {
+	Graphics_text (me, xWC, yWC, Melder_cat (first, rest...));
+}
+
+void Graphics_textRect (Graphics me, double x1, double x2, double y1, double y2, conststring32 text /* cattable */);
+double Graphics_textWidth       (Graphics me, conststring32 text /* cattable */);
+double Graphics_textWidth_ps    (Graphics me, conststring32 text /* cattable */, bool useSilipaPS);
+double Graphics_textWidth_ps_mm (Graphics me, conststring32 text /* cattable */, bool useSilipaPS);
+void Graphics_fillArea (Graphics me, integer numberOfPoints, double const *x, double const *y);
+void Graphics_cellArray (Graphics me, constMATVU const& z,
+	double x1, double x2, double y1, double y2, double minimum, double maximum);
+void Graphics_cellArray_colour (Graphics me, constmatrixview<double_rgbt> const& z,
+	double x1, double x2, double y1, double y2, double minimum, double maximum);
+void Graphics_cellArray8 (Graphics me, constmatrixview <unsigned char> const& z,
+	double x1, double x2, double y1, double y2, unsigned char minimum, unsigned char maximum);
+void Graphics_image (Graphics me, constMATVU const& z,
+	double x1, double x2, double y1, double y2, double minimum, double maximum);
+void Graphics_image_colour (Graphics me, constmatrixview<double_rgbt> const& z,
+	double x1, double x2, double y1, double y2, double minimum, double maximum);
+void Graphics_image8 (Graphics me, constmatrixview <unsigned char> const& z,
+	double x1, double x2, double y1, double y2, uint8 minimum, uint8 maximum);
+void Graphics_imageFromFile (Graphics me, conststring32 relativeFileName, double x1, double x2, double y1, double y2);
 void Graphics_line (Graphics me, double x1, double y1, double x2, double y2);
 void Graphics_rectangle (Graphics me, double x1, double x2, double y1, double y2);
 void Graphics_fillRectangle (Graphics me, double x1, double x2, double y1, double y2);
 void Graphics_roundedRectangle (Graphics me, double x1, double x2, double y1, double y2, double r_mm);
 void Graphics_fillRoundedRectangle (Graphics me, double x1, double x2, double y1, double y2, double r_mm);
-void Graphics_function (Graphics me, double y [], integer ix1, integer ix2, double x1, double x2);   // y [ix1..ix2]
-void Graphics_function16 (Graphics me, int16_t y [], int stagger, integer ix1, integer ix2, double x1, double x2);   // y [ix1..ix2] or y [ix1*2..ix2*2]
+void Graphics_function (Graphics me, const double y [], integer ix1, integer ix2, double x1, double x2);   // y [ix1..ix2]
+void Graphics_function16 (Graphics me, const int16 y [], int stride, integer ix1, integer ix2, double x1, double x2);   // y [ix1..ix2] or y [ix1*2..ix2*2]
 void Graphics_circle (Graphics me, double x, double y, double r);
 void Graphics_fillCircle (Graphics me, double x, double y, double r);
 void Graphics_circle_mm (Graphics me, double x, double y, double d);
@@ -231,14 +224,14 @@ void Graphics_fillEllipse (Graphics me, double x1, double x2, double y1, double
 void Graphics_arrow (Graphics me, double x1, double y1, double x2, double y2);
 void Graphics_doubleArrow (Graphics me, double x1, double y1, double x2, double y2);
 void Graphics_arcArrow (Graphics me, double x, double y, double r, double fromAngle, double toAngle, int arrowAtStart, int arrowAtEnd);
-void Graphics_mark (Graphics me, double x, double y, double size_mm, const char32 *markString /* cattable */);
+void Graphics_mark (Graphics me, double x, double y, double size_mm, conststring32 markString /* cattable */);
 void Graphics_button (Graphics me, double x1, double x2, double y1, double y2);
 void Graphics_innerRectangle (Graphics me, double x1, double x2, double y1, double y2);
 
 extern Graphics_Colour Graphics_BLACK, Graphics_WHITE, Graphics_RED, Graphics_GREEN, Graphics_BLUE,
 	Graphics_CYAN, Graphics_MAGENTA, Graphics_YELLOW, Graphics_MAROON, Graphics_LIME, Graphics_NAVY, Graphics_TEAL,
 	Graphics_PURPLE, Graphics_OLIVE, Graphics_PINK, Graphics_SILVER, Graphics_GREY, Graphics_WINDOW_BACKGROUND_COLOUR;
-const char32 * Graphics_Colour_name (Graphics_Colour colour);
+conststring32 Graphics_Colour_name (Graphics_Colour colour);
 static inline bool Graphics_Colour_equal (Graphics_Colour colour1, Graphics_Colour colour2) {
 	return colour1. red == colour2. red && colour1. green == colour2. green && colour1. blue == colour2. blue;
 }
@@ -318,23 +311,23 @@ double Graphics_inqSpeckleSize (Graphics me);
 Graphics_Colour Graphics_inqColour (Graphics me);
 enum kGraphics_colourScale Graphics_inqColourScale (Graphics me);
 
-void Graphics_contour (Graphics me, double **z,
-	integer ix1, integer ix2, double x1WC, double x2WC, integer iy1, integer iy2, double y1WC, double y2WC, double height);
-void Graphics_altitude (Graphics me, double **z,
-	integer ix1, integer ix2, double x1, double x2, integer iy1, integer iy2, double y1, double y2, int numberOfBorders, double borders []);
-void Graphics_grey (Graphics me, double **z,
-	integer ix1, integer ix2, double x1, double x2, integer iy1, integer iy2, double y1, double y2, int numberOfBorders, double borders []);
+void Graphics_contour (Graphics me, constMATVU const& z,
+	double x1WC, double x2WC, double y1WC, double y2WC, double height);
+void Graphics_altitude (Graphics me, constMATVU const& z,
+	double x1WC, double x2WC, double y1WC, double y2WC, int numberOfBorders, double borders []);
+void Graphics_grey (Graphics me, constMATVU const& z,
+	double x1WC, double x2WC, double y1WC, double y2WC, int numberOfBorders, double borders []);
 #define Graphics_gray Graphics_grey
-void Graphics_surface (Graphics me, double **z, integer ix1, integer ix2, double x1, double x2,
-	integer iy1, integer iy2, double y1, double y2, double minimum, double maximum, double elevation, double azimuth);
+void Graphics_surface (Graphics me, double const * const *z, integer ix1, integer ix2, double x1WC, double x2WC,
+	integer iy1, integer iy2, double y1WC, double y2WC, double minimum, double maximum, double elevation, double azimuth);
 
 void Graphics_setInner (Graphics me);
 void Graphics_unsetInner (Graphics me);
 void Graphics_drawInnerBox (Graphics me);
-void Graphics_textLeft   (Graphics me, bool farr, const char32 *text /* cattable */);
-void Graphics_textRight  (Graphics me, bool farr, const char32 *text /* cattable */);
-void Graphics_textBottom (Graphics me, bool farr, const char32 *text /* cattable */);
-void Graphics_textTop    (Graphics me, bool farr, const char32 *text /* cattable */);
+void Graphics_textLeft   (Graphics me, bool farr, conststring32 text /* cattable */);
+void Graphics_textRight  (Graphics me, bool farr, conststring32 text /* cattable */);
+void Graphics_textBottom (Graphics me, bool farr, conststring32 text /* cattable */);
+void Graphics_textTop    (Graphics me, bool farr, conststring32 text /* cattable */);
 void Graphics_marksLeft   (Graphics me, int numberOfMarks, bool haveNumbers, bool haveTicks, bool haveDottedLines);
 void Graphics_marksRight  (Graphics me, int numberOfMarks, bool haveNumbers, bool haveTicks, bool haveDottedLines);
 void Graphics_marksBottom (Graphics me, int numberOfMarks, bool haveNumbers, bool haveTicks, bool haveDottedLines);
@@ -343,14 +336,14 @@ void Graphics_marksLeftLogarithmic   (Graphics me, int numberOfMarksPerDecade, b
 void Graphics_marksRightLogarithmic  (Graphics me, int numberOfMarksPerDecade, bool haveNumbers, bool haveTicks, bool haveDottedLines);
 void Graphics_marksBottomLogarithmic (Graphics me, int numberOfMarksPerDecade, bool haveNumbers, bool haveTicks, bool haveDottedLines);
 void Graphics_marksTopLogarithmic    (Graphics me, int numberOfMarksPerDecade, bool haveNumbers, bool haveTicks, bool haveDottedLines);
-void Graphics_markLeft   (Graphics me, double yWC, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);
-void Graphics_markRight  (Graphics me, double yWC, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);
-void Graphics_markBottom (Graphics me, double xWC, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);
-void Graphics_markTop    (Graphics me, double xWC, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);
-void Graphics_markLeftLogarithmic   (Graphics me, double y, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);   // y > 0
-void Graphics_markRightLogarithmic  (Graphics me, double y, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);
-void Graphics_markBottomLogarithmic (Graphics me, double x, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);
-void Graphics_markTopLogarithmic    (Graphics me, double x, bool hasNumber, bool hasTick, bool hasDottedLine, const char32 *text /* cattable */);
+void Graphics_markLeft   (Graphics me, double yWC, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);
+void Graphics_markRight  (Graphics me, double yWC, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);
+void Graphics_markBottom (Graphics me, double xWC, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);
+void Graphics_markTop    (Graphics me, double xWC, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);
+void Graphics_markLeftLogarithmic   (Graphics me, double y, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);   // y > 0
+void Graphics_markRightLogarithmic  (Graphics me, double y, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);
+void Graphics_markBottomLogarithmic (Graphics me, double x, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);
+void Graphics_markTopLogarithmic    (Graphics me, double x, bool hasNumber, bool hasTick, bool hasDottedLine, conststring32 text /* cattable */);
 void Graphics_marksLeftEvery   (Graphics me, double units, double distance, bool haveNumbers, bool haveTicks, bool haveDottedLines);
 void Graphics_marksRightEvery  (Graphics me, double units, double distance, bool haveNumbers, bool haveTicks, bool haveDottedLines);
 void Graphics_marksBottomEvery (Graphics me, double units, double distance, bool haveNumbers, bool haveTicks, bool haveDottedLines);
