@@ -705,7 +705,7 @@ void GuiText_cut (GuiText me) {
 	#endif
 }
 
-char32 * GuiText_getSelection (GuiText me) {
+autostring32 GuiText_getSelection (GuiText me) {
 	#if gtk
 		// first = gtk_text_iter_get_offset (& start);
 		// last = gtk_text_iter_get_offset (& end);
@@ -714,7 +714,7 @@ char32 * GuiText_getSelection (GuiText me) {
 			gtk_editable_get_selection_bounds (GTK_EDITABLE (my d_widget), & start, & end);
 			if (end > start) {   // at least one character selected?
 				gchar *text = gtk_editable_get_chars (GTK_EDITABLE (my d_widget), start, end);
-				char32 *result = Melder_8to32 (text);
+				autostring32 result = Melder_8to32 (text);
 				g_free (text);
 				return result;
 			}
@@ -724,7 +724,7 @@ char32 * GuiText_getSelection (GuiText me) {
 				GtkTextIter start, end;
 				gtk_text_buffer_get_selection_bounds (textBuffer, & start, & end);
 				gchar *text = gtk_text_buffer_get_text (textBuffer, & start, & end, true);
-				char32 *result = Melder_8to32 (text);
+				autostring32 result = Melder_8to32 (text);
 				g_free (text);
 				return result;
 			}
@@ -734,19 +734,19 @@ char32 * GuiText_getSelection (GuiText me) {
 		(void) NativeText_getSelectionRange (my d_widget, & startW, & endW);
 		if (endW > startW) {   // at least one character selected?
 			/*
-			 * Get all text.
-			 */
+				Get all text.
+			*/
 			integer lengthW = NativeText_getLength (my d_widget);   // in UTF-16 code units
 			WCHAR *bufferW = Melder_malloc_f (WCHAR, lengthW + 1);
 			GetWindowTextW (my d_widget -> window, bufferW, lengthW + 1);
 			/*
-			 * Zoom in on selection.
-			 */
+				Zoom in on selection.
+			*/
 			lengthW = endW - startW;
 			memmove (bufferW, bufferW + startW, lengthW * sizeof (WCHAR));   // not because of realloc, but because of free!
 			bufferW [lengthW] = U'\0';
-			char32 *result = Melder_dup_f (Melder_peekWto32 (bufferW));
-			(void) Melder_killReturns_inplace (result);   // AFTER zooming!
+			autostring32 result = Melder_dup_f (Melder_peekWto32 (bufferW));
+			(void) Melder_killReturns_inplace (result.get());   // AFTER zooming!
 			return result;
 		}
 	#elif cocoa
@@ -754,22 +754,22 @@ char32 * GuiText_getSelection (GuiText me) {
 		autostring32 selection = GuiText_getStringAndSelectionPosition (me, & start, & end);
 		integer length = end - start;
 		if (length > 0) {
-			char32 *result = Melder_malloc_f (char32, length + 1);
-			memcpy (result, & selection [start], length * sizeof (char32));
-			result [length] = '\0';
-			(void) Melder_killReturns_inplace (result);
+			autostring32 result (length, true);
+			memcpy (result.get(), & selection [start], integer_to_uinteger (length) * sizeof (char32));
+			result [length] = U'\0';
+			(void) Melder_killReturns_inplace (result.get());
 			return result;
 		}
 	#endif
-	return nullptr;   // zero characters selected
+	return autostring32();   // zero characters selected
 }
 
-char32 * GuiText_getString (GuiText me) {
+autostring32 GuiText_getString (GuiText me) {
 	integer first, last;
 	return GuiText_getStringAndSelectionPosition (me, & first, & last);
 }
 
-char32 * GuiText_getStringAndSelectionPosition (GuiText me, integer *first, integer *last) {
+autostring32 GuiText_getStringAndSelectionPosition (GuiText me, integer *first, integer *last) {
 	#if gtk
 		if (G_OBJECT_TYPE (G_OBJECT (my d_widget)) == GTK_TYPE_ENTRY) {
 			gint first_gint, last_gint;
@@ -783,63 +783,82 @@ char32 * GuiText_getStringAndSelectionPosition (GuiText me, integer *first, inte
 			gtk_text_buffer_get_start_iter (textBuffer, & start);
 			gtk_text_buffer_get_end_iter (textBuffer, & end);
 			gchar *text = gtk_text_buffer_get_text (textBuffer, & start, & end, true);   // TODO: Hidden chars ook maar doen he?
-			char32 *result = Melder_8to32 (text);
+			autostring32 result = Melder_8to32 (text);
 			g_free (text);
 			gtk_text_buffer_get_selection_bounds (textBuffer, & start, & end);
 			*first = gtk_text_iter_get_offset (& start);
 			*last = gtk_text_iter_get_offset (& end);
 			return result;
 		}
-		return nullptr;
+		return autostring32();
 	#elif motif
 		integer lengthW = NativeText_getLength (my d_widget);
-		WCHAR *bufferW = Melder_malloc_f (WCHAR, lengthW + 1);
-		GetWindowTextW (my d_widget -> window, bufferW, lengthW + 1);
+		autostringW bufferW (lengthW , true);
+		GetWindowTextW (my d_widget -> window, bufferW.get(), lengthW + 1);
 		integer firstW, lastW;
 		(void) NativeText_getSelectionRange (my d_widget, & firstW, & lastW);
 
 		integer differenceFirst = 0;
 		for (integer i = 0; i < firstW; i ++) {
-			if (bufferW [i] == 13 && (bufferW [i + 1] == L'\n' || bufferW [i + 1] == 0x0085)) differenceFirst ++;
-			if (bufferW [i] >= 0xDC00 && bufferW [i] <= 0xDFFF) differenceFirst ++;
+			if (bufferW [i] == 13 && (bufferW [i + 1] == L'\n' || bufferW [i + 1] == 0x0085))
+				differenceFirst ++;
+			if (bufferW [i] >= 0xDC00 && bufferW [i] <= 0xDFFF)
+				differenceFirst ++;
 		}
 		*first = firstW - differenceFirst;
 
 		integer differenceLast = differenceFirst;
 		for (integer i = firstW; i < lastW; i ++) {
-			if (bufferW [i] == 13 && (bufferW [i + 1] == L'\n' || bufferW [i + 1] == 0x0085)) differenceLast ++;
-			if (bufferW [i] >= 0xDC00 && bufferW [i] <= 0xDFFF) differenceLast ++;
+			if (bufferW [i] == 13 && (bufferW [i + 1] == L'\n' || bufferW [i + 1] == 0x0085))
+				differenceLast ++;
+			if (bufferW [i] >= 0xDC00 && bufferW [i] <= 0xDFFF)
+				differenceLast ++;
 		}
 		*last = lastW - differenceLast;
 
-		char32 *result = Melder_dup_f (Melder_peekWto32 (bufferW));
-		Melder_free (bufferW);
-		(void) Melder_killReturns_inplace (result);
+		autostring32 result = Melder_dup_f (Melder_peekWto32 (bufferW.get()));
+		(void) Melder_killReturns_inplace (result.get());
 		return result;
 	#elif cocoa
 		if (my d_cocoaTextView) {
 			NSString *nsString = [my d_cocoaTextView   string];
-			char32 *result = Melder_8to32 ([nsString UTF8String]);
-			trace (U"string ", result);
+			autostring32 result = Melder_8to32 ([nsString UTF8String]);
+			trace (U"string ", result.get());
 			NSRange nsRange = [my d_cocoaTextView   selectedRange];
-			*first = nsRange. location;
-			*last = *first + nsRange. length;
-			for (integer i = 0; i < *first; i ++) if (result [i] > 0xFFFF) { (*first) --; (*last) --; }
-			for (integer i = *first; i < *last; i ++) if (result [i] > 0xFFFF) { (*last) --; }
+			*first = uinteger_to_integer (nsRange. location);
+			*last = *first + uinteger_to_integer (nsRange. length);
+			for (integer i = 0; i < *first; i ++) {
+				if (result [i] > 0xFFFF) {
+					(*first) --;
+					(*last) --;
+				}
+			}
+			for (integer i = *first; i < *last; i ++) {
+				if (result [i] > 0xFFFF)
+					(*last) --;
+			}
 			return result;
 		} else {
 			NSString *nsString = [(NSTextField *) my d_widget   stringValue];
-			char32 *result = Melder_8to32 ([nsString UTF8String]);
-			trace (U"string ", result);
+			autostring32 result = Melder_8to32 ([nsString UTF8String]);
+			trace (U"string ", result.get());
 			NSRange nsRange = [[[(NSTextField *) my d_widget   window] fieldEditor: NO forObject: nil] selectedRange];
-			*first = nsRange. location;
-			*last = *first + nsRange. length;
-			for (integer i = 0; i < *first; i ++) if (result [i] > 0xFFFF) { (*first) --; (*last) --; }
-			for (integer i = *first; i < *last; i ++) if (result [i] > 0xFFFF) { (*last) --; }
+			*first = uinteger_to_integer (nsRange. location);
+			*last = *first + uinteger_to_integer (nsRange. length);
+			for (integer i = 0; i < *first; i ++) {
+				if (result [i] > 0xFFFF) {
+					(*first) --;
+					(*last) --;
+				}
+			}
+			for (integer i = *first; i < *last; i ++) {
+				if (result [i] > 0xFFFF)
+					(*last) --;
+			}
 			return result;
 		}
 	#else
-		return nullptr;
+		return autostring32();
 	#endif
 }
 
@@ -894,9 +913,9 @@ void GuiText_remove (GuiText me) {
 	#endif
 }
 
-void GuiText_replace (GuiText me, integer from_pos, integer to_pos, const char32 *text) {
+void GuiText_replace (GuiText me, integer from_pos, integer to_pos, conststring32 text) {
 	#if gtk
-		gchar *newText = Melder_peek32to8 (text);
+		const gchar *newText = Melder_peek32to8 (text);
 		if (G_OBJECT_TYPE (G_OBJECT (my d_widget)) == GTK_TYPE_ENTRY) {
 			gtk_editable_delete_text (GTK_EDITABLE (my d_widget), from_pos, to_pos);
 			gint from_pos_gint = from_pos;
@@ -912,22 +931,21 @@ void GuiText_replace (GuiText me, integer from_pos, integer to_pos, const char32
 				gtk_text_view_get_editable (GTK_TEXT_VIEW (my d_widget)));
 		}
 	#elif motif
-		const char32 *from;
-		char32 *winText = Melder_malloc_f (char32, 2 * str32len (text) + 1), *to;   // all new lines plus one null byte
 		Melder_assert (MEMBER (my d_widget, Text));
+		autostring32 winText (2 * str32len (text), true);   // all newlines
+		char32 *to = & winText [0];
 		/*
-		 * Replace all LF with CR/LF.
-		 */
-		for (from = text, to = winText; *from != U'\0'; from ++, to ++)
+			Replace all LF with CR/LF.
+		*/
+		for (const char32 *from = & text [0]; *from != U'\0'; from ++, to ++)
 			if (*from == U'\n') { *to = 13; * ++ to = U'\n'; } else *to = *from;
 		*to = U'\0';
 		/*
-		 * We DON'T replace any text without selecting it, so we can deselect any other text,
-		 * thus allowing ourselves to select [from_pos, to_pos] and use the REPLACESEL message.
+			We DON'T replace any text without selecting it, so we can deselect any other text,
+			thus allowing ourselves to select [from_pos, to_pos] and use the REPLACESEL message.
 		 */
 		GuiText_setSelection (me, from_pos, to_pos);
-		Edit_ReplaceSel (my d_widget -> window, Melder_peek32toW (winText));
-		Melder_free (winText);
+		Edit_ReplaceSel (my d_widget -> window, Melder_peek32toW (winText.get()));
 		UpdateWindow (my d_widget -> window);
 	#elif cocoa
 		if (my d_cocoaTextView) {
@@ -939,7 +957,7 @@ void GuiText_replace (GuiText me, integer from_pos, integer to_pos, const char32
 			}
 			from_pos += numberOfLeadingHighUnicodeValues;
 			to_pos += numberOfLeadingHighUnicodeValues + numberOfSelectedHighUnicodeValues;
-			NSRange nsRange = NSMakeRange (from_pos, to_pos - from_pos);
+			NSRange nsRange = NSMakeRange (integer_to_uinteger (from_pos), integer_to_uinteger (to_pos - from_pos));
 			NSString *nsString = (NSString *) Melder_peek32toCfstring (text);
 			[my d_cocoaTextView   shouldChangeTextInRange: nsRange   replacementString: nsString];   // ignore the returned BOOL: only interested in the side effect of having undo support
 			[[my d_cocoaTextView   textStorage] replaceCharactersInRange: nsRange   withString: nsString];
@@ -986,7 +1004,7 @@ void GuiText_setFontSize (GuiText me, int size) {
 	#elif motif
 		// a trick to update the window. BUG: why doesn't UpdateWindow seem to suffice?
 		integer first, last;
-		char32 *text = GuiText_getStringAndSelectionPosition (me, & first, & last);
+		autostring32 text = GuiText_getStringAndSelectionPosition (me, & first, & last);
 		GuiText_setString (me, U"");   // erase all
 		UpdateWindow (my d_widget -> window);
 		if (size <= 10) {
@@ -1000,8 +1018,7 @@ void GuiText_setFontSize (GuiText me, int size) {
 		} else {
 			SetWindowFont (my d_widget -> window, font24, false);
 		}
-		GuiText_setString (me, text);
-		Melder_free (text);
+		GuiText_setString (me, text.get());
 		GuiText_setSelection (me, first, last);
 		UpdateWindow (my d_widget -> window);
 	#elif cocoa
@@ -1038,10 +1055,10 @@ void GuiText_setSelection (GuiText me, integer first, integer last) {
 			gtk_text_buffer_select_range (buffer, & from_it, & to_it);
 		}
 	#elif motif
-		char32 *text = GuiText_getString (me);
+		autostring32 text = GuiText_getString (me);
 		if (first < 0) first = 0;
 		if (last < 0) last = 0;
-		integer length = str32len (text);
+		integer length = str32len (text.get());
 		if (first >= length) first = length;
 		if (last >= length) last = length;
 		/*
@@ -1052,8 +1069,8 @@ void GuiText_setSelection (GuiText me, integer first, integer last) {
 		for (integer i = 0; i < first; i ++) if (text [i] == U'\n') numberOfLeadingLineBreaks ++;
 		for (integer i = first; i < last; i ++) if (text [i] == U'\n') numberOfSelectedLineBreaks ++;
 		/*
-		 * On Windows, characters are counted in UTF-16 units, whereas 'first' and 'last' are in UTF-32 units. Convert.
-		 */
+			On Windows, characters are counted in UTF-16 units, whereas 'first' and 'last' are in UTF-32 units. Convert.
+		*/
 		integer numberOfLeadingHighUnicodeValues = 0, numberOfSelectedHighUnicodeValues = 0;
 		for (integer i = 0; i < first; i ++) if (text [i] > 0xFFFF) numberOfLeadingHighUnicodeValues ++;
 		for (integer i = first; i < last; i ++) if (text [i] > 0xFFFF) numberOfSelectedHighUnicodeValues ++;
@@ -1062,36 +1079,34 @@ void GuiText_setSelection (GuiText me, integer first, integer last) {
 		last += numberOfLeadingLineBreaks + numberOfSelectedLineBreaks;
 		first += numberOfLeadingHighUnicodeValues;
 		last += numberOfLeadingHighUnicodeValues + numberOfSelectedHighUnicodeValues;
-		Melder_free (text);
 
 		Edit_SetSel (my d_widget -> window, first, last);
 		UpdateWindow (my d_widget -> window);
 	#elif cocoa
 		/*
-		 * On Cocoa, characters are counted in UTF-16 units, whereas 'first' and 'last' are in UTF-32 units. Convert.
-		 */
-		char32 *text = GuiText_getString (me);
+			On Cocoa, characters are counted in UTF-16 units, whereas 'first' and 'last' are in UTF-32 units. Convert.
+		*/
+		autostring32 text = GuiText_getString (me);
 		integer numberOfLeadingHighUnicodeValues = 0, numberOfSelectedHighUnicodeValues = 0;
 		for (integer i = 0; i < first; i ++) if (text [i] > 0xFFFF) numberOfLeadingHighUnicodeValues ++;
 		for (integer i = first; i < last; i ++) if (text [i] > 0xFFFF) numberOfSelectedHighUnicodeValues ++;
 		first += numberOfLeadingHighUnicodeValues;
 		last += numberOfLeadingHighUnicodeValues + numberOfSelectedHighUnicodeValues;
-		Melder_free (text);
 
 		if (my d_cocoaTextView) {
-			[my d_cocoaTextView   setSelectedRange: NSMakeRange (first, last - first)];
+			[my d_cocoaTextView   setSelectedRange: NSMakeRange (integer_to_uinteger (first), integer_to_uinteger (last - first))];
 		}
 	#endif
 	}
 }
 
-void GuiText_setString (GuiText me, const char32 *text) {
+void GuiText_setString (GuiText me, conststring32 text) {
 	#if gtk
 		if (G_OBJECT_TYPE (my d_widget) == GTK_TYPE_ENTRY) {
 			gtk_entry_set_text (GTK_ENTRY (my d_widget), Melder_peek32to8 (text));
 		} else if (G_OBJECT_TYPE (my d_widget) == GTK_TYPE_TEXT_VIEW) {
 			GtkTextBuffer *textBuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (my d_widget));
-			gchar *textUtf8 = Melder_peek32to8 (text);
+			const gchar *textUtf8 = Melder_peek32to8 (text);
 			//gtk_text_buffer_set_text (textBuffer, textUtf8, strlen (textUtf8));   // length in bytes!
 			GtkTextIter start, end;
 			gtk_text_buffer_get_start_iter (textBuffer, & start);
@@ -1100,16 +1115,15 @@ void GuiText_setString (GuiText me, const char32 *text) {
 			gtk_text_buffer_insert_interactive (textBuffer, & start, textUtf8, strlen (textUtf8), gtk_text_view_get_editable (GTK_TEXT_VIEW (my d_widget)));
 		}
 	#elif motif
-		const char32 *from;
-		char32 *winText = Melder_malloc_f (char32, 2 * str32len (text) + 1), *to;   /* All new lines plus one null byte. */
+		autostring32 winText (2 * str32len (text), true);   // all new lines
+		char32 *to = & winText [0];
 		/*
-		 * Replace all LF with CR/LF.
-		 */
-		for (from = text, to = winText; *from != U'\0'; from ++, to ++)
+			Replace all LF with CR/LF.
+		*/
+		for (const char32 *from = & text [0]; *from != U'\0'; from ++, to ++)
 			if (*from == U'\n') { *to = 13; * ++ to = U'\n'; } else *to = *from;
 		*to = U'\0';
-		SetWindowTextW (my d_widget -> window, Melder_peek32toW (winText));
-		Melder_free (winText);
+		SetWindowTextW (my d_widget -> window, Melder_peek32toW (winText.get()));
 		UpdateWindow (my d_widget -> window);
 	#elif cocoa
 		trace (U"title");
