@@ -26,7 +26,7 @@ Thing_define (HyperLink, Daata) {
 	double x1DC, x2DC, y1DC, y2DC;
 };
 
-autoHyperLink HyperLink_create (const char32 *name, double x1, double x2, double y1, double y2);
+autoHyperLink HyperLink_create (conststring32 name, double x1, double x2, double y1, double y2);
 
 Thing_define (HyperPage, Editor) {
 	GuiDrawingArea drawingArea;
@@ -39,12 +39,12 @@ Thing_define (HyperPage, Editor) {
 	CollectionOf<structHyperLink> links;
 	bool printing, mirror;
 	int top;
-	char32 *insideHeader, *middleHeader, *outsideHeader;
-	char32 *insideFooter, *middleFooter, *outsideFooter;
-	char32 *entryHint; double entryPosition;
-	struct { char32 *page; int top; } history [20];
+	conststring32 insideHeader, middleHeader, outsideHeader;
+	conststring32 insideFooter, middleFooter, outsideFooter;
+	autostring32 entryHint; double entryPosition;
+	struct { autostring32 page; int top; } history [20];
 	int historyPointer;
-	char32 *currentPageTitle;
+	autostring32 currentPageTitle;
 	GuiMenuItem fontSizeButton_10, fontSizeButton_12, fontSizeButton_14, fontSizeButton_18, fontSizeButton_24;
 	void *praatApplication, *praatObjects, *praatPicture;
 	bool scriptErrorHasBeenNotified;
@@ -64,7 +64,7 @@ Thing_define (HyperPage, Editor) {
 	virtual void v_draw () { }
 	virtual integer v_getNumberOfPages () { return 0; }
 	virtual integer v_getCurrentPageNumber () { return 0; }
-	virtual int v_goToPage (const char32 * /* title */) { return 0; }
+	virtual int v_goToPage (conststring32 /* title */) { return 0; }
 	virtual void v_goToPage_i (integer /* pageNumber */) { }
 	virtual void v_defaultHeaders (EditorCommand /* cmd */) { }
 	virtual bool v_hasHistory () { return false; }
@@ -79,41 +79,41 @@ void HyperPage_clear (HyperPage me);
 #define HyperPage_ADD_BORDER  1
 #define HyperPage_USE_ENTRY_HINT  2
 
-void HyperPage_any (HyperPage me, const char32 *text, kGraphics_font font, int size, int style, double minFooterDistance,
+void HyperPage_any (HyperPage me, conststring32 text, kGraphics_font font, int size, int style, double minFooterDistance,
 	double x, double secondIndent, double topSpacing, double bottomSpacing, uint32 method);
-void HyperPage_pageTitle (HyperPage me, const char32 *title);
-void HyperPage_intro (HyperPage me, const char32 *text);
-void HyperPage_entry (HyperPage me, const char32 *title);
-void HyperPage_paragraph (HyperPage me, const char32 *text);
-void HyperPage_listItem (HyperPage me, const char32 *text);
-void HyperPage_listItem1 (HyperPage me, const char32 *text);
-void HyperPage_listItem2 (HyperPage me, const char32 *text);
-void HyperPage_listItem3 (HyperPage me, const char32 *text);
-void HyperPage_listTag (HyperPage me, const char32 *text);
-void HyperPage_listTag1 (HyperPage me, const char32 *text);
-void HyperPage_listTag2 (HyperPage me, const char32 *text);
-void HyperPage_listTag3 (HyperPage me, const char32 *text);
-void HyperPage_definition (HyperPage me, const char32 *text);
-void HyperPage_definition1 (HyperPage me, const char32 *text);
-void HyperPage_definition2 (HyperPage me, const char32 *text);
-void HyperPage_definition3 (HyperPage me, const char32 *text);
-void HyperPage_code (HyperPage me, const char32 *text);
-void HyperPage_code1 (HyperPage me, const char32 *text);
-void HyperPage_code2 (HyperPage me, const char32 *text);
-void HyperPage_code3 (HyperPage me, const char32 *text);
-void HyperPage_code4 (HyperPage me, const char32 *text);
-void HyperPage_code5 (HyperPage me, const char32 *text);
-void HyperPage_prototype (HyperPage me, const char32 *text);
-void HyperPage_formula (HyperPage me, const char32 *formula);
+void HyperPage_pageTitle (HyperPage me, conststring32 title);
+void HyperPage_intro (HyperPage me, conststring32 text);
+void HyperPage_entry (HyperPage me, conststring32 title);
+void HyperPage_paragraph (HyperPage me, conststring32 text);
+void HyperPage_listItem (HyperPage me, conststring32 text);
+void HyperPage_listItem1 (HyperPage me, conststring32 text);
+void HyperPage_listItem2 (HyperPage me, conststring32 text);
+void HyperPage_listItem3 (HyperPage me, conststring32 text);
+void HyperPage_listTag (HyperPage me, conststring32 text);
+void HyperPage_listTag1 (HyperPage me, conststring32 text);
+void HyperPage_listTag2 (HyperPage me, conststring32 text);
+void HyperPage_listTag3 (HyperPage me, conststring32 text);
+void HyperPage_definition (HyperPage me, conststring32 text);
+void HyperPage_definition1 (HyperPage me, conststring32 text);
+void HyperPage_definition2 (HyperPage me, conststring32 text);
+void HyperPage_definition3 (HyperPage me, conststring32 text);
+void HyperPage_code (HyperPage me, conststring32 text);
+void HyperPage_code1 (HyperPage me, conststring32 text);
+void HyperPage_code2 (HyperPage me, conststring32 text);
+void HyperPage_code3 (HyperPage me, conststring32 text);
+void HyperPage_code4 (HyperPage me, conststring32 text);
+void HyperPage_code5 (HyperPage me, conststring32 text);
+void HyperPage_prototype (HyperPage me, conststring32 text);
+void HyperPage_formula (HyperPage me, conststring32 formula);
 void HyperPage_picture (HyperPage me, double width_inches, double height_inches, void (*draw) (Graphics g));
-void HyperPage_script (HyperPage me, double width_inches, double height_inches, const char32 *script);
+void HyperPage_script (HyperPage me, double width_inches, double height_inches, conststring32 script);
 
-int HyperPage_goToPage (HyperPage me, const char32 *title);
+int HyperPage_goToPage (HyperPage me, conststring32 title);
 void HyperPage_goToPage_i (HyperPage me, integer i);
 
-void HyperPage_init (HyperPage me, const char32 *title, Daata data);
+void HyperPage_init (HyperPage me, conststring32 title, Daata data);
 
-void HyperPage_setEntryHint (HyperPage me, const char32 *entry);
+void HyperPage_setEntryHint (HyperPage me, conststring32 entry);
 void HyperPage_initSheetOfPaper (HyperPage me);
 
 /* End of file HyperPage.h */
