@@ -42,8 +42,7 @@ InfoEditor InfoEditor_getTheReferenceToTheOnlyInstance () {
 	return theReferenceToTheOnlyInfoEditor;
 }
 
-void gui_information (const char32 *message);   // BUG
-void gui_information (const char32 *message) {
+static void gui_information (conststring32 message) {
 	InfoEditor editor = InfoEditor_getTheReferenceToTheOnlyInstance ();
 	GuiText_setString (editor -> textWidget, message);
 	GuiThing_show (editor -> windowForm);
@@ -94,4 +93,8 @@ void gui_information (const char32 *message) {
 	#endif
 }
 
+void InfoEditor_injectInformationProc () {
+	Melder_setInformationProc (gui_information);
+}
+
 /* End of file InfoEditor.cpp */
