@@ -32,7 +32,7 @@ Thing_implement (Manual, HyperPage, 0);
 
 #define SEARCH_PAGE  0
 
-static const char32 *month [] =
+static const conststring32 month [] =
 	{ U"", U"January", U"February", U"March", U"April", U"May", U"June",
 	  U"July", U"August", U"September", U"October", U"November", U"December" };
 
@@ -40,7 +40,7 @@ static void menu_cb_writeOneToHtmlFile (Manual me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM_SAVE (U"Save as HTML file", nullptr)
 		ManPages manPages = (ManPages) my data;
 		autoMelderString buffer;
-		MelderString_copy (& buffer, manPages -> pages.at [my path] -> title);
+		MelderString_copy (& buffer, manPages -> pages.at [my path] -> title.get());
 		char32 *p = buffer.string;
 		while (*p) { if (! isalnum ((int) *p) && *p != U'_') *p = U'_'; p ++; }
 		MelderString_append (& buffer, U".html");
@@ -63,11 +63,10 @@ static void menu_cb_writeAllToHtmlDir (Manual me, EDITOR_ARGS_FORM) {
 static void menu_cb_searchForPageList (Manual me, EDITOR_ARGS_FORM) {
 	EDITOR_FORM (U"Search for page", nullptr)
 		static ManPages manPages;
-		static integer numberOfPages;
-		static const char32 **pages;
+		static conststring32vector pages;
 		manPages = (ManPages) my data;
-		pages = ManPages_getTitles (manPages, & numberOfPages);
-		LIST (page, U"Page", manPages -> pages.size, pages, 1)
+		pages = ManPages_getTitles (manPages);
+		LIST (page, U"Page", pages, 1)
 	EDITOR_OK
 	EDITOR_DO
 		HyperPage_goToPage_i (me, page);
@@ -86,14 +85,14 @@ void structManual :: v_draw () {
 		for (int i = 1; i <= our numberOfMatches; i ++) {
 			char32 link [300];
 			page = manPages -> pages.at [matches [i]];
-			Melder_sprint (link,300, U"• @@", page -> title);
+			Melder_sprint (link,300, U"• @@", page -> title.get());
 			HyperPage_listItem (this, link);
 		}
 		return;
 	}
 	page = manPages -> pages.at [path];
 	if (! our paragraphs) return;
-	HyperPage_pageTitle (this, page -> title);
+	HyperPage_pageTitle (this, page -> title.get());
 	for (ManPage_Paragraph paragraph = & page -> paragraphs [0]; (int) paragraph -> type != 0; paragraph ++) {
 		switch (paragraph -> type) {
 			case  kManPage_type::INTRO: HyperPage_intro (this, paragraph -> text); break;
@@ -131,7 +130,7 @@ void structManual :: v_draw () {
 		bool goAhead = true;
 		while ((int) page -> paragraphs [lastParagraph]. type != 0) lastParagraph ++;
 		if (lastParagraph > 0) {
-			const char32 *text = page -> paragraphs [lastParagraph - 1]. text;
+			conststring32 text = page -> paragraphs [lastParagraph - 1]. text;
 			if (! text || text [0] == U'\0' || text [str32len (text) - 1] != U':') {
 				if (our printing && our suppressLinksHither)
 					goAhead = false;
@@ -146,7 +145,7 @@ void structManual :: v_draw () {
 				if (page -> linksThither [jlink] == link)
 					alreadyShown = true;
 			if (! alreadyShown) {
-				const char32 *title = manPages -> pages.at [page -> linksHither [ilink]] -> title;
+				conststring32 title = manPages -> pages.at [page -> linksHither [ilink]] -> title.get();
 				char32 linkText [304];
 				Melder_sprint (linkText, 304, U"@@", title, U"@");
 				HyperPage_listItem (this, linkText);
@@ -159,8 +158,8 @@ void structManual :: v_draw () {
 		int imonth = date % 10000 / 100;
 		if (imonth < 0 || imonth > 12) imonth = 0;
 		Melder_sprint (signature,100,
-			U"© ", str32equ (page -> author, U"ppgb") ? U"Paul Boersma" :
-			       str32equ (page -> author, U"djmw") ? U"David Weenink" : page -> author,
+			U"© ", str32equ (page -> author.get(), U"ppgb") ? U"Paul Boersma" :
+			       str32equ (page -> author.get(), U"djmw") ? U"David Weenink" : page -> author.get(),
 			U", ", date % 100,
 			U" ", month [imonth],
 			U" ", date / 10000);
@@ -185,7 +184,7 @@ static void print (void *void_me, Graphics graphics) {
 	for (integer ipage = 1; ipage <= numberOfPages; ipage ++) {
 		ManPage page = manPages -> pages.at [ipage];
 		if (my printPagesStartingWith == nullptr ||
-		    Melder_stringMatchesCriterion (page -> title, kMelder_string::STARTS_WITH, my printPagesStartingWith, true))
+		    Melder_stringMatchesCriterion (page -> title.get(), kMelder_string::STARTS_WITH, my printPagesStartingWith, true))
 		{
 			ManPage_Paragraph par;
 			my path = ipage;
@@ -193,8 +192,7 @@ static void print (void *void_me, Graphics graphics) {
 			my numberOfParagraphs = 0;
 			par = my paragraphs;
 			while ((int) (par ++) -> type != 0) my numberOfParagraphs ++;
-			Melder_free (my currentPageTitle);
-			my currentPageTitle = Melder_dup_f (page -> title);
+			my currentPageTitle = Melder_dup_f (page -> title.get());
 			my v_goToPage_i (ipage);
 			my v_draw ();
 			my v_goToPage_i (savePage);
@@ -226,14 +224,16 @@ static void menu_cb_printRange (Manual me, EDITOR_ARGS_FORM) {
 		#else
 			strcpy (dateA, ctime (& today));
 		#endif
-		char32 *date = Melder_peek8to32 (dateA), *newline;
-		newline = str32chr (date, U'\n'); if (newline) *newline = U'\0';
-		SET_STRING (leftOrInsideHeader, date)
-		SET_STRING (rightOrOutsideHeader, my name)
+		autostring32 date = Melder_8to32 (dateA);
+		char32 *newline = str32chr (date.get(), U'\n');
+		if (newline)
+			*newline = U'\0';
+		SET_STRING (leftOrInsideHeader, date.get())
+		SET_STRING (rightOrOutsideHeader, my name.get())
 		if (my d_printingPageNumber) SET_INTEGER (firstPageNumber, my d_printingPageNumber + 1)
 		if (my path >= 1 && my path <= manPages -> pages.size) {
 			ManPage page = manPages -> pages.at [my path];
-			SET_STRING (printAllPagesWhoseTitleStartsWith, page -> title);
+			SET_STRING (printAllPagesWhoseTitleStartsWith, page -> title.get());
 		}
 	EDITOR_DO
 		my insideHeader = leftOrInsideHeader;
@@ -260,10 +260,10 @@ static double searchToken (ManPages me, integer ipage, char32 *token) {
 	struct structManPage_Paragraph *par = & page -> paragraphs [0];
 	if (! token [0]) return 1.0;
 	/*
-	 * Try to find a match in the title, case insensitively.
+	 * Try to find a match in the title, case-insensitively.
 	 */
 	static MelderString buffer { };
-	MelderString_copy (& buffer, page -> title);
+	MelderString_copy (& buffer, page -> title.get());
 	for (char32 *p = & buffer.string [0]; *p != U'\0'; p ++) *p = Melder_toLowerCase (*p);
 	if (str32str (buffer.string, token)) {
 		goodness += 300.0;   // lots of points for a match in the title!
@@ -291,7 +291,7 @@ static double searchToken (ManPages me, integer ipage, char32 *token) {
 	return goodness;
 }
 
-static void search (Manual me, const char32 *query) {
+static void search (Manual me, conststring32 query) {
 	ManPages manPages = (ManPages) my data;
 	integer numberOfPages = manPages -> pages.size;
 	static MelderString searchText { };
@@ -334,7 +334,7 @@ static void search (Manual me, const char32 *query) {
 	HyperPage_goToPage_i (me, SEARCH_PAGE);
 }
 
-void Manual_search (Manual me, const char32 *query) {
+void Manual_search (Manual me, conststring32 query) {
 	GuiText_setString (my searchText, query);
 	search (me, query);
 }
@@ -378,9 +378,8 @@ static void gui_button_cb_publish (Manual /* me */, GuiButtonEvent /* event */)
 }
 
 static void do_search (Manual me) {
-	char32 *query = GuiText_getString (my searchText);
-	search (me, query);
-	Melder_free (query);
+	autostring32 query = GuiText_getString (my searchText);
+	search (me, query.get());
 }
 
 static void gui_button_cb_search (Manual me, GuiButtonEvent /* event */) {
@@ -435,12 +434,12 @@ void structManual :: v_defaultHeaders (EditorCommand cmd) {
 	ManPages manPages = (ManPages) my data;
 	if (my path) {
 		char32 string [400];
-		static const char32 *shortMonth [] =
+		static const conststring32 shortMonth [] =
 			{ U"Jan", U"Feb", U"Mar", U"Apr", U"May", U"Jun", U"Jul", U"Aug", U"Sep", U"Oct", U"Nov", U"Dec" };
 		ManPage page = manPages -> pages.at [my path];
 		integer date = page -> date;
-		SET_STRING (my outsideHeader, page -> title)
-		SET_STRING (my insideFooter, page -> author)
+		SET_STRING (my outsideHeader, page -> title.get())
+		SET_STRING (my insideFooter, page -> author.get())
 		if (date) {
 			Melder_sprint (string,400, shortMonth [date % 10000 / 100 - 1], U" ", date % 100, U", ", date / 10000);
 			SET_STRING (my insideHeader, string)
@@ -462,7 +461,7 @@ void structManual :: v_goToPage_i (integer pageNumber) {
 	if (pageNumber < 1 || pageNumber > manPages -> pages.size) {
 		if (pageNumber == SEARCH_PAGE) {
 			our path = SEARCH_PAGE;
-			Melder_free (our currentPageTitle);
+			our currentPageTitle. reset();
 			return;
 		} else Melder_throw (U"Page ", pageNumber, U" not found.");
 	}
@@ -472,11 +471,10 @@ void structManual :: v_goToPage_i (integer pageNumber) {
 	our numberOfParagraphs = 0;
 	ManPage_Paragraph par = paragraphs;
 	while ((int) (par ++) -> type != 0) our numberOfParagraphs ++;
-	Melder_free (our currentPageTitle);
-	our currentPageTitle = Melder_dup_f (page -> title);
+	our currentPageTitle = Melder_dup_f (page -> title.get());
 }
 
-int structManual :: v_goToPage (const char32 *title) {
+int structManual :: v_goToPage (conststring32 title) {
 	ManPages manPages = (ManPages) our data;
 	if (title [0] == U'\\' && title [1] == U'F' && title [2] == U'I') {
 		structMelderFile file { };
@@ -488,7 +486,7 @@ int structManual :: v_goToPage (const char32 *title) {
 		autoPraatBackground background;
 		try {
 			autostring32 fileNameWithArguments = Melder_dup (title + 3);
-			praat_executeScriptFromFileNameWithArguments (fileNameWithArguments.peek());
+			praat_executeScriptFromFileNameWithArguments (fileNameWithArguments.get());
 		} catch (MelderError) {
 			Melder_flushError ();
 		}
@@ -502,7 +500,7 @@ int structManual :: v_goToPage (const char32 *title) {
 	}
 }
 
-void Manual_init (Manual me, const char32 *title, Daata data, bool ownData) {
+void Manual_init (Manual me, conststring32 title, Daata data, bool ownData) {
 	ManPages manPages = (ManPages) data;
 	integer i;
 	if ((i = ManPages_lookUp (manPages, title)) == 0)
@@ -527,7 +525,7 @@ void Manual_init (Manual me, const char32 *title, Daata data, bool ownData) {
 	my history [0]. page = Melder_dup_f (title);   // BAD
 }
 
-autoManual Manual_create (const char32 *title, Daata data, bool ownData) {
+autoManual Manual_create (conststring32 title, Daata data, bool ownData) {
 	try {
 		autoManual me = Thing_new (Manual);
 		Manual_init (me.get(), title, data, ownData);
