@@ -19,22 +19,22 @@
 #include "UiPause.h"
 #include "praatP.h"
 
-static UiForm thePauseForm = nullptr;
+static autoUiForm thePauseForm;
 static UiField thePauseFormRadio = nullptr;
 static int thePauseForm_clicked = 0;
 static int theCancelContinueButton = 0;
 static int theEventLoopDepth = 0;
 
 static void thePauseFormOkCallback (UiForm /* sendingForm */, integer /* narg */, Stackel /* args */,
-	const char32 * /* sendingString */, Interpreter /* interpreter */,
-	const char32 * /* invokingButtonTitle */, bool /* modified */, void *closure) {
+	conststring32 /* sendingString */, Interpreter /* interpreter */,
+	conststring32 /* invokingButtonTitle */, bool /* modified */, void *closure) {
 	/* BUG: should also restore praatP. editor. */
 	/*
 	 * Get the data from the pause form.
 	 */
-	thePauseForm_clicked = UiForm_getClickedContinueButton (thePauseForm);
+	thePauseForm_clicked = UiForm_getClickedContinueButton (thePauseForm.get());
 	if (thePauseForm_clicked != theCancelContinueButton)
-		UiForm_Interpreter_addVariables (thePauseForm, (Interpreter) closure);   // 'closure', not 'interpreter' or 'theInterpreter'!
+		UiForm_Interpreter_addVariables (thePauseForm.get(), (Interpreter) closure);   // 'closure', not 'interpreter' or 'theInterpreter'!
 }
 static void thePauseFormCancelCallback (UiForm /* dia */, void * /* closure */) {
 	if (theCancelContinueButton != 0) {
@@ -43,100 +43,99 @@ static void thePauseFormCancelCallback (UiForm /* dia */, void * /* closure */)
 		thePauseForm_clicked = -1;   // STOP button
 	}
 }
-void UiPause_begin (GuiWindow topShell, const char32 *title, Interpreter interpreter) {
+void UiPause_begin (GuiWindow topShell, conststring32 title, Interpreter interpreter) {
 	if (theEventLoopDepth > 0)
 		Melder_throw (U"Praat cannot have more than one pause form at a time.");
-	forget (thePauseForm);   // in case an earlier build-up of another pause window was interrupted
 	thePauseForm = UiForm_create (topShell, Melder_cat (U"Pause: ", title),
 		thePauseFormOkCallback, interpreter,   // pass interpreter as closure!
 		nullptr, nullptr);
 }
-void UiPause_real (const char32 *label, const char32 *defaultValue) {
+void UiPause_real (conststring32 label, conststring32 defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"real\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addReal (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addReal (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_positive (const char32 *label, const char32 *defaultValue) {
+void UiPause_positive (conststring32 label, conststring32 defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"positive\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addPositive (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addPositive (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_integer (const char32 *label, const char32 *defaultValue) {
+void UiPause_integer (conststring32 label, conststring32 defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"integer\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addInteger (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addInteger (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_natural (const char32 *label, const char32 *defaultValue) {
+void UiPause_natural (conststring32 label, conststring32 defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"natural\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addNatural (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addNatural (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_word (const char32 *label, const char32 *defaultValue) {
+void UiPause_word (conststring32 label, conststring32 defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"word\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addWord (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addWord (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_sentence (const char32 *label, const char32 *defaultValue) {
+void UiPause_sentence (conststring32 label, conststring32 defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"sentence\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addSentence (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addSentence (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_text (const char32 *label, const char32 *defaultValue) {
+void UiPause_text (conststring32 label, conststring32 defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"text\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addText (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addText (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_boolean (const char32 *label, bool defaultValue) {
+void UiPause_boolean (conststring32 label, bool defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"boolean\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addBoolean (thePauseForm, nullptr, nullptr, label, defaultValue);
+	UiForm_addBoolean (thePauseForm.get(), nullptr, nullptr, label, defaultValue);
 }
-void UiPause_choice (const char32 *label, int defaultValue) {
+void UiPause_choice (conststring32 label, int defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"choice\" has to be between a \"beginPause\" and an \"endPause\".");
-	thePauseFormRadio = UiForm_addRadio (thePauseForm, nullptr, nullptr, nullptr, label, defaultValue, 1);
+	thePauseFormRadio = UiForm_addRadio (thePauseForm.get(), nullptr, nullptr, nullptr, label, defaultValue, 1);
 }
-void UiPause_optionMenu (const char32 *label, int defaultValue) {
+void UiPause_optionMenu (conststring32 label, int defaultValue) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"optionMenu\" has to be between a \"beginPause\" and an \"endPause\".");
-	thePauseFormRadio = UiForm_addOptionMenu (thePauseForm, nullptr, nullptr, nullptr, label, defaultValue, 1);
+	thePauseFormRadio = UiForm_addOptionMenu (thePauseForm.get(), nullptr, nullptr, nullptr, label, defaultValue, 1);
 }
-void UiPause_option (const char32 *label) {
+void UiPause_option (conststring32 label) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"option\" has to be between a \"beginPause\" and an \"endPause\".");
 	if (! thePauseFormRadio) {
-		forget (thePauseForm);
+		thePauseForm. reset();
 		Melder_throw (U"Found the function \"option\" without a preceding \"choice\" or \"optionMenu\".");
 	}
 	UiOptionMenu_addButton (thePauseFormRadio, label);
 }
-void UiPause_comment (const char32 *label) {
+void UiPause_comment (conststring32 label) {
 	if (! thePauseForm)
 		Melder_throw (U"The function \"comment\" has to be between a \"beginPause\" and an \"endPause\".");
-	UiForm_addLabel (thePauseForm, nullptr, label);
+	UiForm_addLabel (thePauseForm.get(), nullptr, label);
 }
 int UiPause_end (int numberOfContinueButtons, int defaultContinueButton, int cancelContinueButton,
-	const char32 *continueText1, const char32 *continueText2, const char32 *continueText3,
-	const char32 *continueText4, const char32 *continueText5, const char32 *continueText6,
-	const char32 *continueText7, const char32 *continueText8, const char32 *continueText9,
-	const char32 *continueText10, Interpreter interpreter)
+	conststring32 continueText1, conststring32 continueText2, conststring32 continueText3,
+	conststring32 continueText4, conststring32 continueText5, conststring32 continueText6,
+	conststring32 continueText7, conststring32 continueText8, conststring32 continueText9,
+	conststring32 continueText10, Interpreter interpreter)
 {
 	if (! thePauseForm)
 		Melder_throw (U"Found the function \"endPause\" without a preceding \"beginPause\".");
-	UiForm_setPauseForm (thePauseForm, numberOfContinueButtons, defaultContinueButton, cancelContinueButton,
+	UiForm_setPauseForm (thePauseForm.get(), numberOfContinueButtons, defaultContinueButton, cancelContinueButton,
 		continueText1, continueText2, continueText3, continueText4, continueText5,
 		continueText6, continueText7, continueText8, continueText9, continueText10,
 		thePauseFormCancelCallback);
 	theCancelContinueButton = cancelContinueButton;
-	UiForm_finish (thePauseForm);
+	UiForm_finish (thePauseForm.get());
 	bool wasBackgrounding = Melder_backgrounding;
 	//if (theCurrentPraatApplication -> batch) goto end;
 	if (wasBackgrounding) praat_foreground ();
 	/*
 	 * Put the pause form on the screen.
 	 */
-	UiForm_destroyWhenUnmanaged (thePauseForm);
-	UiForm_do (thePauseForm, false);
+	UiForm_destroyWhenUnmanaged (thePauseForm.get());
+	UiForm_do (thePauseForm.get(), false);
 	/*
 	 * Wait for the user to click Stop or Continue.
 	 */
@@ -179,7 +178,7 @@ int UiPause_end (int numberOfContinueButtons, int defaultContinueButton, int can
 	}
 	if (wasBackgrounding) praat_background ();
 	/* BUG: should also restore praatP. editor. */
-	thePauseForm = nullptr;   // undangle
+	thePauseForm. releaseToUser();   // undangle
 	thePauseFormRadio = nullptr;   // undangle
 	if (thePauseForm_clicked == -1) {
 		Interpreter_stop (interpreter);
