@@ -56,28 +56,9 @@
 #define oo_FLOAT(x)  oo_SIMPLE (double, r32, x)
 #define oo_DOUBLE(x)  oo_SIMPLE (double, r64, x)
 //#define oo_COMPLEX(x)  oo_SIMPLE (dcomplex, c128, x)
-#define oo_POINTER(x)  oo_SIMPLE (void *, dummy, x)
 
-/* Arrays with compile-time allocation of capacity. Declarations like: int x [cap]; */
-/* 'cap' is a compile-time expression evaluating to a positive integer (e.g., 3). */
-/* First index is always 0, last index is 'n' - 1. */
-/* Actual number of elements 'n' may vary during run-time and while structure exists, */
-/* but must never be greater than 'cap'. */
-
-//#define oo_BYTE_ARRAY(x,cap,n)  oo_ARRAY (signed char, i8, x, cap, n)
-//#define oo_INT_ARRAY(x,cap,n)  oo_ARRAY (int, i16, x, cap, n)
-//#define oo_INTEGER_ARRAY(x,cap,n)  oo_ARRAY (integer, integer32BE, x, cap, n)
-//#define oo_UBYTE_ARRAY(x,cap,n)  oo_ARRAY (unsigned char, u8, x, cap, n)
-//#define oo_UINT_ARRAY(x,cap,n)  oo_ARRAY (unsigned int, u16, x, cap, n)
-//#define oo_UINTEGER_ARRAY(x,cap,n)  oo_ARRAY (uinteger, u32, x, cap, n)
-//#define oo_BOOL_ARRAY(x,cap,n)  oo_ARRAY (unsigned char, u8, x, cap, n)
-//#define oo_FLOAT_ARRAY(x,cap,n)  oo_ARRAY (double, r32, x, cap, n)
-#define oo_DOUBLE_ARRAY(x,cap,n)  oo_ARRAY (double, r64, x, cap, n)
-//#define oo_COMPLEX_ARRAY(x,cap,n)  oo_ARRAY (dcomplex, c128, x, cap, n)
-#define oo_POINTER_ARRAY(x,cap,n)  oo_ARRAY (void *, dummy, x, cap, n)
-
-/* Sets with compile-time allocation of capacity. Declarations like: int x [1 + setType_MAX]; */
-/* First index is always 0, last index is setType_MAX. */
+/* Sets with compile-time allocation of capacity. Declarations like: int x [1 + setType::MAX]; */
+/* The first index is always 0, the last index is setType::MAX. */
 
 //#define oo_BYTE_SET(x,setType)  oo_SET (signed char, i8, x, setType)
 //#define oo_INT_SET(x,setType)  oo_SET (int, i16, x, setType)
@@ -85,15 +66,14 @@
 //#define oo_UBYTE_SET(x,setType)  oo_SET (unsigned char, u8, x, setType)
 //#define oo_UINT_SET(x,setType)  oo_SET (unsigned int, u16, x, setType)
 //#define oo_UINTEGER_SET(x,setType)  oo_SET (uinteger, u32, x, setType)
-//#define oo_BOOL_SET(x,setType)  oo_SET (unsigned char, u8, x, setType)
 //#define oo_FLOAT_SET(x,setType)  oo_SET (double, r32, x, setType)
 #define oo_DOUBLE_SET(x,setType)  oo_SET (double, r64, x, setType)
 //#define oo_COMPLEX_SET(x,setType)  oo_SET (dcomplex, c128, x, setType)
-#define oo_POINTER_SET(x,setType)  oo_SET (void *, dummy, x, setType)
 
 /* Arrays with run-time allocation of size. Declarations like: int *x; */
-/* First index is 'min', last index is 'max'. */
-/* While the structure exists, 'max' may become less than the value it had at the time of allocation. */
+/* The first index is 'min', the last index is 'max'. */
+/* While the structure exists, 'min' and 'max' may change in any direction if the pointer changes; */
+/* if the pointer does not change, 'min' cannot change, but 'max' may become lower than the original value. */
 
 //#define oo_BYTE_VECTOR_FROM(x,min,max)  oo_VECTOR (signed char, i8, x, min, max)
 #define oo_INT_VECTOR_FROM(x,min,max)  oo_VECTOR (int, i16, x, min, max)
@@ -101,50 +81,33 @@
 #define oo_UBYTE_VECTOR_FROM(x,min,max)  oo_VECTOR (unsigned char, u8, x, min, max)
 //#define oo_UINT_VECTOR_FROM(x,min,max)  oo_VECTOR (unsigned int, u16, x, min, max)
 //#define oo_UINTEGER_VECTOR_FROM(x,min,max)  oo_VECTOR (uinteger, u32, x, min, max)
-//#define oo_BOOL_VECTOR_FROM(x,min,max)  oo_VECTOR (unsigned char, u8, x, min, max)
 #define oo_FLOAT_VECTOR_FROM(x,min,max)  oo_VECTOR (double, r32, x, min, max)
 #define oo_DOUBLE_VECTOR_FROM(x,min,max)  oo_VECTOR (double, r64, x, min, max)
 #define oo_COMPLEX_VECTOR_FROM(x,min,max)  oo_VECTOR (dcomplex, c128, x, min, max)
-#define oo_POINTER_VECTOR_FROM(x,min,max)  oo_VECTOR (void *, dummy, x, min, max)
-
-//#define oo_BYTE_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (signed char, i8, x, row1, row2, col1, col2)
-//#define oo_INT_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (int, i16, x, row1, row2, col1, col2)
-//#define oo_INTEGER_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (integer, integer32BE, x, row1, row2, col1, col2)
-//#define oo_UBYTE_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (unsigned char, u8, x, row1, row2, col1, col2)
-//#define oo_UINT_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (unsigned int, u16, x, row1, row2, col1, col2)
-//#define oo_UINTEGER_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (uinteger, u32, x, row1, row2, col1, col2)
-//#define oo_BOOL_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (unsigned char, u8, x, row1, row2, col1, col2)
-//#define oo_FLOAT_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (double, r32, x, row1, row2, col1, col2)
-#define oo_DOUBLE_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (double, r64, x, row1, row2, col1, col2)
-//#define oo_COMPLEX_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (dcomplex, c128, x, row1, row2, col1, col2)
-#define oo_POINTER_MATRIX_FROM(x,row1,row2,col1,col2)  oo_MATRIX (void *, dummy, x, row1, row2, col1, col2)
 
 /* The same arrays, with the first index fixed at 1. */
 
 //#define oo_BYTE_VECTOR(x,n)  oo_VECTOR (signed char, i8, x, 1, n)
 #define oo_INT_VECTOR(x,n)  oo_VECTOR (int, i16, x, 1, n)
-#define oo_INTEGER_VECTOR(x,n)  oo_VECTOR (integer, integer32BE, x, 1, n)
+//#define oo_INTEGER_VECTOR(x,n)  oo_VECTOR (integer, integer32BE, x, 1, n)
 //#define oo_UBYTE_VECTOR(x,n)  oo_VECTOR (unsigned char, u8, x, 1, n)
 //#define oo_UINT_VECTOR(x,n)  oo_VECTOR (unsigned int, u16, x, 1, n)
 //#define oo_UINTEGER_VECTOR(x,n)  oo_VECTOR (uinteger, u32, x, 1, n)
-//#define oo_BOOL_VECTOR(x,n)  oo_VECTOR (unsigned char, u8, x, 1, n)
-#define oo_FLOAT_VECTOR(x,n)  oo_VECTOR (double, r32, x, 1, n)
-#define oo_DOUBLE_VECTOR(x,n)  oo_VECTOR (double, r64, x, 1, n)
+//#define oo_FLOAT_VECTOR(x,n)  oo_VECTOR (double, r32, x, 1, n)
+//#define oo_DOUBLE_VECTOR(x,n)  oo_VECTOR (double, r64, x, 1, n)
 //#define oo_COMPLEX_VECTOR(x,n)  oo_VECTOR (dcomplex, c128, x, 1, n)
-#define oo_POINTER_VECTOR(x,n)  oo_VECTOR (void *, dummy, x, 1, n)
 
-//#define oo_BYTE_MATRIX(x,nrow,ncol)  oo_MATRIX (signed char, i8, x, 1, nrow, 1, ncol)
-//#define oo_INT_MATRIX(x,nrow,ncol)  oo_MATRIX (int, i16, x, 1, nrow, 1, ncol)
-#define oo_INTEGER_MATRIX(x,nrow,ncol)  oo_MATRIX (integer, integer32BE, x, 1, nrow, 1, ncol)
-#define oo_UBYTE_MATRIX(x,nrow,ncol)  oo_MATRIX (unsigned char, u8, x, 1, nrow, 1, ncol)
-//#define oo_UINT_MATRIX(x,nrow,ncol)  oo_MATRIX (unsigned int, u16, x, 1, nrow, 1, ncol)
-//#define oo_UINTEGER_MATRIX(x,nrow,ncol)  oo_MATRIX (uinteger, u32, x, 1, nrow, 1, ncol)
-//#define oo_BOOL_MATRIX(x,nrow,ncol)  oo_MATRIX (unsigned char, u8, x, 1, nrow, 1, ncol)
-#define oo_FLOAT_MATRIX(x,nrow,ncol)  oo_MATRIX (double, r32, x, 1, nrow, 1, ncol)
-#define oo_DOUBLE_MATRIX(x,nrow,ncol)  oo_MATRIX (double, r64, x, 1, nrow, 1, ncol)
-//#define oo_COMPLEX_MATRIX(x,nrow,ncol)  oo_MATRIX (dcomplex, c128, x, 1, nrow, 1, ncol)
-#define oo_POINTER_MATRIX(x,nrow,ncol)  oo_MATRIX (void *, dummy, x, 1, nrow, 1, ncol)
+#define oo_VEC(x,size)  oo_ANYVEC (double, r64, x, size)
+#define oo_INTVEC(x,size)  oo_ANYVEC (integer, integer32BE, x, size)
+#define oo_INTVEC16(x,size)  oo_ANYVEC (integer, integer16BE, x, size)
+#define oo_obsoleteVEC32(x,size)  oo_ANYVEC (double, r32, x, size)
 
+#define oo_MAT(x,nrow,ncol)  oo_ANYMAT (double, r64, x, nrow, ncol)
+#define oo_INTMAT(x,nrow,ncol)  oo_ANYMAT (integer, integer32BE, x, nrow, ncol)
+#define oo_obsoleteMAT32(x,nrow,ncol)  oo_ANYMAT (double, r32, x, nrow, ncol)
+#define oo_BYTEMAT(x,nrow,ncol)  oo_ANYMAT (byte, u8, x, nrow, ncol)
+
+#define oo_TEN3(x,ndi1,ndim2,ndim3)  oo_ANYTEN3 (double, r64, x, ndim1, ndim2, ndim3)
 
 /*** Enumerated types. ***/
 
@@ -158,8 +121,6 @@
 
 #define oo_ENUM(kType,x)  oo_ENUMx (kType, e8, x)
 //#define oo_LENUM(Type,x)  oo_ENUMx (short, e16, Type, x)
-//#define oo_ENUM_ARRAY(Type,x,cap,n)  oo_ENUMx_ARRAY (signed char, e8, Type, x, cap, n)
-//#define oo_LENUM_ARRAY(Type,x,cap,n)  oo_ENUMx_ARRAY (short, e16, Type, x, cap, n)
 //#define oo_ENUM_SET(Type,x,setType)  oo_ENUMx_SET (signed char, e8, Type, x, setType)
 //#define oo_LENUM_SET(Type,x,setType)  oo_ENUMx_SET (short, e16, Type, x, setType)
 //#define oo_ENUM_VECTOR_FROM(Type,x,min,max)  oo_ENUMx_VECTOR (signed char, e8, Type, x, min, max)
@@ -167,13 +128,9 @@
 //#define oo_ENUM_VECTOR(Type,x,n)  oo_ENUMx_VECTOR (signed char, e8, Type, x, 1, n)
 //#define oo_LENUM_VECTOR(Type,x,n)  oo_ENUMx_VECTOR (short, e16, Type, x, 1, n)
 #define oo_BOOLEAN(x)  oo_SIMPLE (bool, eb, x)
-//#define oo_BOOLEAN_ARRAY(x,cap,n)  oo_ARRAY (bool, eb, x, cap, n)
-//#define oo_BOOLEAN_SET(x,setType)  oo_ARRAY (bool, eb, x, setType)
 //#define oo_BOOLEAN_VECTOR_FROM(x,min,max)  oo_VECTOR (bool, eb, x, min, max)
 #define oo_BOOLEAN_VECTOR(x,n)  oo_VECTOR (bool, eb, x, 1, n)
 #define oo_QUESTION(x)  oo_SIMPLE (bool, eq, x)
-//#define oo_QUESTION_ARRAY(x,cap,n)  oo_ARRAY (bool, eq, x, cap, n)
-//#define oo_QUESTION_SET(x,setType)  oo_ARRAY (bool, eq, x, setType)
 //#define oo_QUESTION_VECTOR_FROM(x,min,max)  oo_VECTOR (bool, eq, x, min, max)
 //#define oo_QUESTION_VECTOR(x,n)  oo_VECTOR (bool, eq, x, 1, n)
 
@@ -188,54 +145,44 @@
 #define oo_STRING(x)  oo_STRINGx (w16, x)
 #define oo_LSTRING(x)  oo_STRINGx (w32, x)
 
-//#define oo_STRING_ARRAY(x,cap,n)  oo_STRINGx_ARRAY (w16, x, cap, n)
-//#define oo_LSTRING_ARRAY(x,cap,n)  oo_STRINGx_ARRAY (w32, x, cap, n)
-
 //#define oo_STRING_SET(x,setType)  oo_STRINGx_SET (w16, x, setType)
 //#define oo_LSTRING_SET(x,setType)  oo_STRINGx_SET (w32, x, setType)
 
-//#define oo_STRING_VECTOR_FROM(x,min,max)  oo_STRINGx_VECTOR (w16, x, min, max)
-//#define oo_LSTRING_VECTOR_FROM(x,min,max)  oo_STRINGx_VECTOR (w32, x, min, max)
-
-#define oo_STRING_VECTOR(x,n)  oo_STRINGx_VECTOR (w16, x, 1, n)
-//#define oo_LSTRING_VECTOR(x,n)  oo_STRINGx_VECTOR (w32, x, 1, n)
+#define oo_STRING_VECTOR(x,n)  oo_STRINGx_VECTOR (w16, x, n)
+//#define oo_LSTRING_VECTOR(x,n)  oo_STRINGx_VECTOR (w32, x, n)
 
 /*** Structs. ***/
 
 #define oo_STRUCT_VECTOR(Type,x,n)  oo_STRUCT_VECTOR_FROM (Type, x, 1, n)
-#define oo_STRUCT_MATRIX(Type,x,nrow,ncol)  oo_STRUCT_MATRIX_FROM (Type, x, 1, nrow, 1, ncol)
 
 /********** Definitions for header files only. **********/
 /* These are undef'ed and redefined in the header files that implement methods, */
-/* like oo_DESTROY.h, oo_COPY.h, oo_EQUAL.h, oo_WRITE_TEXT.h, etc. */
+/* such as oo_DESTROY.h, oo_COPY.h, oo_EQUAL.h, oo_WRITE_TEXT.h, etc. */
 
 /* Types. */
 
 #define oo_SIMPLE(type,storage,x)  type x;
-#define oo_ARRAY(type,storage,x,cap,n)  type x [cap];
 #define oo_SET(type,storage,x,setType)  type x [1 + (int) setType::MAX];
 #define oo_VECTOR(type,storage,x,min,max)  type *x;
-#define oo_MATRIX(type,storage,x,row1,row2,col1,col2)  type **x;
+#define oo_ANYVEC(type,storage,x,size)  autovector <type> x;
+#define oo_ANYMAT(type,storage,x,nrow,ncol)  automatrix <type> x;
+#define oo_ANYTEN3(type,storage,x,ndim1,ndim2,ndim3)  autotensor3 <type> x;
 
 #define oo_ENUMx(kType,storage,x)  kType x;
-//#define oo_ENUMx_ARRAY(kType,storage,x,cap,n)  kType x [cap];
 //#define oo_ENUMx_SET(kType,storage,x,setType)  kType x [1 + (int) setType::MAX];
 //#define oo_ENUMx_VECTOR(kType,storage,x,min,max)  kType *x;
 
-#define oo_STRINGx(storage,x)  char32 *x;
-#define oo_STRINGx_ARRAY(storage,x,cap,n)  char32 *x [cap];
-#define oo_STRINGx_SET(storage,x,setType)  char32 *x [1 + setType::MAX];
-#define oo_STRINGx_VECTOR(storage,x,min,max)  char32 **x;
+#define oo_STRINGx(storage,x)  autostring32 x;
+#define oo_STRINGx_SET(storage,x,setType)  autostring32 x [1 + setType::MAX];
+#define oo_STRINGx_VECTOR(storage,x,size)  autostring32vector x;
 
 #define oo_STRUCT(Type,x)  struct struct##Type x;
-#define oo_STRUCT_ARRAY(Type,x,cap,n)  struct struct##Type x [cap];
 #define oo_STRUCT_SET(Type,x,setType)  struct struct##Type x [1 + (int) setType::MAX];
 #define oo_STRUCT_VECTOR_FROM(Type,x,min,max)  Type x;
-#define oo_STRUCT_MATRIX_FROM(Type,x,row1,row2,col1,col2)  struct struct##Type **x;
 
-#define oo_AUTO_OBJECT(Class,version,x)  auto##Class x;
+#define oo_OBJECT(Class,version,x)  auto##Class x;
 #define oo_COLLECTION_OF(Class,x,ItemClass,version)  Class<struct##ItemClass> x;
-#define oo_AUTO_COLLECTION(Class,x,ItemClass,version)  auto##Class x;
+#define oo_COLLECTION(Class,x,ItemClass,version)  auto##Class x;
 #define oo_FILE(x)  structMelderFile x;
 #define oo_DIR(x)  structMelderDir x;
 
@@ -259,7 +206,7 @@
 
 #define oo_DEFINE_CLASS(klas,parent) \
 	typedef struct struct##klas *klas; \
-	typedef _Thing_auto <struct##klas> auto##klas; \
+	typedef autoSomeThing <struct##klas> auto##klas; \
 	typedef struct##parent klas##_Parent; \
 	extern struct structClassInfo theClassInfo_##klas; \
 	extern ClassInfo class##klas; \
@@ -280,14 +227,19 @@
 
 /*** Miscellaneous. ***/
 
-/* For fields that should only be destroyed, copied, compared, read, or written if 'condition': */
-#define oo_IF(condition)
-#define oo_ENDIF
-
 /* For fields that should not be read in older versions: */
 #define oo_FROM(from)
 #define oo_ENDFROM
 
+#define oo_VERSION_UNTIL(version)  \
+	if (_formatVersion_ < version) {
+#define oo_VERSION_ELSE_UNTIL(version)  \
+	} else if (_formatVersion_ < version) {
+#define oo_VERSION_ELSE  \
+	} else {
+#define oo_VERSION_END  \
+	}
+
 /* Examples of the usage of the following macros:
 	For code that should only appear in reading methods (consistency checks etc.),
 		use:  #if oo_READING
