@@ -17,30 +17,27 @@
  */
 
 #undef oo_SIMPLE
-#undef oo_ARRAY
 #undef oo_SET
 #undef oo_VECTOR
-#undef oo_MATRIX
+#undef oo_ANYVEC
+#undef oo_ANYMAT
+#undef oo_ANYTEN3
 
 #undef oo_ENUMx
-//#undef oo_ENUMx_ARRAY
 //#undef oo_ENUMx_SET
 //#undef oo_ENUMx_VECTOR
 
 #undef oo_STRINGx
-#undef oo_STRINGx_ARRAY
 #undef oo_STRINGx_SET
 #undef oo_STRINGx_VECTOR
 
 #undef oo_STRUCT
-#undef oo_STRUCT_ARRAY
 #undef oo_STRUCT_SET
 #undef oo_STRUCT_VECTOR_FROM
-#undef oo_STRUCT_MATRIX_FROM
 
-#undef oo_AUTO_OBJECT
+#undef oo_OBJECT
 #undef oo_COLLECTION_OF
-#undef oo_AUTO_COLLECTION
+#undef oo_COLLECTION
 #undef oo_FILE
 #undef oo_DIR
 
@@ -50,9 +47,6 @@
 #undef oo_DEFINE_CLASS
 #undef oo_END_CLASS
 
-#undef oo_IF
-#undef oo_ENDIF
-
 #undef oo_FROM
 #undef oo_ENDFROM
 
