@@ -1,6 +1,6 @@
 /* praat_objectMenus.cpp
  *
- * Copyright (C) 1992-2012,2013,2014,2015,2016,2017 Paul Boersma
+ * Copyright (C) 1992-2018 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ DO
 	static MelderString fullName { };
 	MelderString_copy (& fullName, Thing_className (OBJECT), U" ", string.string);
 	if (! str32equ (fullName.string, FULL_NAME)) {
-		Melder_free (FULL_NAME), FULL_NAME = Melder_dup_f (fullName.string);
+		theCurrentPraatObjects -> list [IOBJECT]. name = Melder_dup_f (fullName.string);
 		autoMelderString listName;
 		MelderString_append (& listName, ID, U". ", fullName.string);
 		praat_list_renameAndSelect (IOBJECT, listName.string);
@@ -105,7 +105,7 @@ END }
 
 static GuiMenu praatMenu, editMenu, windowMenu, newMenu, readMenu, goodiesMenu, preferencesMenu, technicalMenu, applicationHelpMenu, helpMenu;
 
-GuiMenu praat_objects_resolveMenu (const char32 *menu) {
+GuiMenu praat_objects_resolveMenu (conststring32 menu) {
 	return
 		str32equ (menu, U"Praat") || str32equ (menu, U"Control") ? praatMenu :
 		#if cocoa
@@ -229,27 +229,30 @@ END }
 /********** Callbacks of the Preferences menu. **********/
 
 FORM (PREFS_TextInputEncodingSettings, U"Text reading preferences", U"Unicode") {
-	RADIO_ENUM (encodingOf8BitTextFiles, U"Encoding of 8-bit text files", kMelder_textInputEncoding, DEFAULT)
+	RADIO_ENUM (kMelder_textInputEncoding, encodingOf8BitTextFiles,
+			U"Encoding of 8-bit text files", kMelder_textInputEncoding::DEFAULT)
 OK
 	SET_ENUM (encodingOf8BitTextFiles, kMelder_textInputEncoding, Melder_getInputEncoding ())
 DO
-	Melder_setInputEncoding ((kMelder_textInputEncoding) encodingOf8BitTextFiles);
+	Melder_setInputEncoding (encodingOf8BitTextFiles);
 END }
 
 FORM (PREFS_TextOutputEncodingSettings, U"Text writing preferences", U"Unicode") {
-	RADIO_ENUM (outputEncoding, U"Output encoding", kMelder_textOutputEncoding, DEFAULT)
+	RADIO_ENUM (kMelder_textOutputEncoding, outputEncoding,
+			U"Output encoding", kMelder_textOutputEncoding::DEFAULT)
 OK
 	SET_ENUM (outputEncoding, kMelder_textOutputEncoding, Melder_getOutputEncoding ())
 DO
-	Melder_setOutputEncoding ((kMelder_textOutputEncoding) outputEncoding);
+	Melder_setOutputEncoding (outputEncoding);
 END }
 
 FORM (PREFS_GraphicsCjkFontStyleSettings, U"CJK font style preferences", nullptr) {
-	OPTIONMENU_ENUM (cjkFontStyle, U"CJK font style", kGraphics_cjkFontStyle, DEFAULT)
+	OPTIONMENU_ENUM (kGraphics_cjkFontStyle, cjkFontStyle,
+			U"CJK font style", kGraphics_cjkFontStyle::DEFAULT)
 OK
 	SET_ENUM (cjkFontStyle, kGraphics_cjkFontStyle, theGraphicsCjkFontStyle)
 DO
-	theGraphicsCjkFontStyle = (kGraphics_cjkFontStyle) cjkFontStyle;
+	theGraphicsCjkFontStyle = cjkFontStyle;
 END }
 
 /********** Callbacks of the Goodies menu. **********/
@@ -274,16 +277,13 @@ DO
 			Melder_information (result. numericResult);
 		} break;
 		case kFormula_EXPRESSION_TYPE_STRING: {
-			Melder_information (result. stringResult);
-			Melder_free (result. stringResult);
+			Melder_information (result. stringResult.get());
 		} break;
 		case kFormula_EXPRESSION_TYPE_NUMERIC_VECTOR: {
-			Melder_information (result. numericVectorResult);
-			if (result. owned) result. numericVectorResult. reset();
+			Melder_information (constVEC (result. numericVectorResult));
 		} break;
 		case kFormula_EXPRESSION_TYPE_NUMERIC_MATRIX: {
-			Melder_information (result. numericMatrixResult);
-			if (result. owned) result. numericMatrixResult. reset();
+			Melder_information (constMAT (result. numericMatrixResult));
 		}
 	}
 END }
@@ -411,11 +411,11 @@ END }
 
 static void readFromFile (MelderFile file) {
 	autoDaata object = Data_readFromFile (file);
-	if (! object.get()) return;
+	if (! object) return;
 	if (Thing_isa (object.get(), classManPages) && ! Melder_batch) {
 		ManPages manPages = (ManPages) object.get();
 		ManPage firstPage = manPages -> pages.at [1];
-		autoManual manual = Manual_create (firstPage -> title, object.releaseToAmbiguousOwner(), true);
+		autoManual manual = Manual_create (firstPage -> title.get(), object.releaseToAmbiguousOwner(), true);
 		if (manPages -> executable)
 			Melder_warning (U"These manual pages contain links to executable scripts.\n"
 				"Only navigate these pages if you trust their author!");
@@ -495,7 +495,7 @@ DIRECT (WINDOW_ManPages_view) {
 	LOOP {
 		iam (ManPages);
 		ManPage firstPage = my pages.at [1];
-		autoManual manual = Manual_create (firstPage -> title, me, false);
+		autoManual manual = Manual_create (firstPage -> title.get(), me, false);
 		if (my executable)
 			Melder_warning (U"These manual pages contain links to executable scripts.\n"
 				"Only navigate these pages if you trust their author!");
@@ -518,9 +518,9 @@ DO
 END }
 
 FORM (HELP_GoToManualPage, U"Go to manual page", nullptr) {
-	static integer numberOfPages;
-	static const char32 **pages = ManPages_getTitles (theCurrentPraatApplication -> manPages, & numberOfPages);
-	LIST (pageNumber, U"Page", numberOfPages, pages, 1)
+	static conststring32vector pages;
+	pages = ManPages_getTitles (theCurrentPraatApplication -> manPages);
+	LIST (pageNumber, U"Page", pages, 1)
 	OK
 DO
 	if (theCurrentPraatApplication -> batch)
@@ -573,7 +573,7 @@ static void searchProc () {
 static MelderString itemTitle_about { };
 
 static autoDaata scriptRecognizer (integer nread, const char *header, MelderFile file) {
-	const char32 *name = MelderFile_name (file);
+	conststring32 name = MelderFile_name (file);
 	if (nread < 2) return autoDaata ();
 	if ((header [0] == '#' && header [1] == '!') || str32str (name, U".praat") == name + str32len (name) - 6
 	    || str32str (name, U".html") == name + str32len (name) - 5)
@@ -639,9 +639,9 @@ void praat_addMenus (GuiWindow window) {
 		helpMenu = GuiMenu_createInWindow (window, U"Help", 0);
 	}
 	
-	MelderString_append (& itemTitle_about, U"About ", praatP.title, U"...");
+	MelderString_append (& itemTitle_about, U"About ", praatP.title.get(), U"...");
+	praat_addMenuCommand (U"Objects", U"Praat", itemTitle_about.string, nullptr, praat_UNHIDABLE, WINDOW_About);
 	#ifdef macintosh
-		praat_addMenuCommand (U"Objects", U"Praat", itemTitle_about.string, nullptr, praat_UNHIDABLE, WINDOW_About);
 		#if cocoa
 			/*
 			 * HACK: give the following command weird names,
@@ -656,9 +656,6 @@ void praat_addMenus (GuiWindow window) {
 			praat_addMenuCommand (U"Objects", U"Window", U"Close   ", nullptr, 'W' | praat_NO_API, PRAAT_close);
 		#endif
 	#endif
-	#ifdef UNIX
-		praat_addMenuCommand (U"Objects", U"Praat", itemTitle_about.string, nullptr, praat_UNHIDABLE, WINDOW_About);
-	#endif
 	praat_addMenuCommand (U"Objects", U"Praat", U"-- script --", nullptr, 0, nullptr);
 	praat_addMenuCommand (U"Objects", U"Praat", U"New Praat script", nullptr, praat_NO_API, WINDOW_praat_newScript);
 	praat_addMenuCommand (U"Objects", U"Praat", U"Open Praat script...", nullptr, praat_NO_API, WINDOW_praat_openScript);
@@ -713,12 +710,10 @@ void praat_addMenus2 () {
 	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"Go to manual page...", nullptr, 0, HELP_GoToManualPage);
 	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"Write manual to HTML directory...", nullptr, praat_HIDDEN, HELP_WriteManualToHtmlDirectory);
 	praat_addMenuCommand (U"Objects", U"ApplicationHelp",
-		Melder_cat (U"Search ", praatP.title, U" manual..."),
+		Melder_cat (U"Search ", praatP.title.get(), U" manual..."),
 		nullptr, 'M' | praat_NO_API, HELP_SearchManual);
-	#ifdef _WIN32
-		praat_addMenuCommand (U"Objects", U"Help", U"-- about --", nullptr, 0, nullptr);
-		praat_addMenuCommand (U"Objects", U"Help", itemTitle_about.string, nullptr, praat_UNHIDABLE, WINDOW_About);
-	#endif
+	praat_addMenuCommand (U"Objects", U"ApplicationHelp", U"-- about --", nullptr, 0, nullptr);
+	praat_addMenuCommand (U"Objects", U"ApplicationHelp", itemTitle_about.string, nullptr, praat_UNHIDABLE, WINDOW_About);
 
 	#if defined (macintosh) || defined (_WIN32)
 		Gui_setOpenDocumentCallback (cb_openDocument);
