@@ -116,6 +116,9 @@ void praat_reportSystemProperties () {
 		MelderInfo_writeLine (U"linux is \"" xstr (linux) "\".");
 	#endif
 	MelderInfo_writeLine (U"The number of processors is ", std::thread::hardware_concurrency(), U".");
+	#ifdef macintosh
+		MelderInfo_writeLine (U"system version is ", Melder_systemVersion, U".");
+	#endif
 	MelderInfo_close ();
 }
 
@@ -184,4 +187,13 @@ void praat_reportMemoryUse () {
 	MelderInfo_close ();
 }
 
+void MelderCasual_memoryUse (integer message) {
+	integer numberOfStrings = MelderString_allocationCount () - MelderString_deallocationCount ();
+	integer numberOfArrays = NUM_getTotalNumberOfArrays ();
+	integer numberOfThings = theTotalNumberOfThings;
+	integer numberOfOther = Melder_allocationCount () - Melder_deallocationCount () - numberOfStrings - numberOfArrays - numberOfThings;
+	Melder_casual (U"Memory ", message, U": ",
+		numberOfStrings, U" strings, ", numberOfArrays, U" arrays, ", numberOfThings, U" things, ", numberOfOther, U" other.");
+}
+
 /* End of file praat_statistics.cpp */
